@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2023-08-23
+ *  Updated   : 2024-11-17
  *  Notes     :
  *
- * Copyright (c) 1997-2023 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2025 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,19 +48,13 @@
 #	include "options_menu.h"
 #endif /* !OPTIONS_MENU_H */
 
-/* This fixes ambiguities on platforms that don't distinguish extern case */
-#ifdef CASE_PROBLEM
-#	define Raw tin_raw
-#	define EndWin tin_EndWin
-#endif /* CASE_PROBLEM */
-
 /* active.c */
 extern char group_flag(char ch);
 extern int find_newnews_index(const char *cur_newnews_host);
-extern int read_news_active_file(void);
+extern int read_news_active_file(t_bool check_any_unread);
 extern t_bool match_group_list(const char *group, const char *group_list);
 extern t_bool parse_active_line(char *line, t_artnum *max, t_artnum *min, char *moderated);
-extern t_bool process_bogus(char *name);
+extern t_bool process_bogus(const char *name);
 extern t_bool need_reread_active_file(void);
 extern t_bool resync_active_file(void);
 extern void create_save_active_file(void);
@@ -74,6 +68,7 @@ extern t_bool global_look_for_multipart(int aindex, char start, char stop);
 extern t_bool index_group(struct t_group *group);
 extern void do_update(t_bool catchup);
 extern void find_base(struct t_group *group);
+extern void free_mailbox_list(struct t_mailbox *mb);
 extern void make_threads(struct t_group *group, t_bool rethread);
 extern void set_article(struct t_article *art);
 extern void show_art_msg(const char *group);
@@ -81,9 +76,9 @@ extern void sort_arts(unsigned /* int */ sort_art_type);
 
 /* attrib.c */
 extern int add_scope(const char *scope);
+extern t_bool read_attributes_file(t_bool global_file);
 extern void assign_attributes_to_groups(void);
 extern void build_news_headers_array(struct t_attribute *scope, t_bool header_to_display);
-extern void read_attributes_file(t_bool global_file);
 extern void write_attributes_file(const char *file);
 
 /* auth.c */
@@ -93,12 +88,19 @@ extern void write_attributes_file(const char *file);
 
 /* charset.c */
 extern char *convert_to_printable(char *buf, t_bool keep_tab);
+#ifdef CHARSET_CONVERSION
+#	ifdef USE_ICU_UCSDET
+	extern char *guess_charset(const char *sample, int32_t confidence);
+#	endif /* USE_ICU_UCSDET */
+#endif /* CHARSET_CONVERSION */
+extern t_bool charset_unsupported(const char *charset);
 extern t_bool is_art_tex_encoded(FILE *fp);
 extern void convert_iso2asc(char *iso, char **asc_buffer, size_t *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern wchar_t *wconvert_to_printable(wchar_t *wbuf, t_bool keep_tab);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+extern const char *validate_charset(const char *charset);
 
 /* color.c */
 extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
@@ -111,20 +113,22 @@ extern void draw_pager_line(const char *str, int flags, t_bool raw_data);
 #endif /* HAVE_COLOR */
 
 /* config.c */
-extern char **ulBuildArgv(char *cmd, int *new_argc);
-extern char *quote_space_to_dash(char *str);
+extern char **ulBuildArgv(const char *cmd, int *new_argc);
+extern char *quote_space_to_dash(const char *str);
 extern const char *print_boolean(t_bool value);
 extern t_bool match_boolean(char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer(char *line, const char *pat, int *dst, int maxval);
 extern t_bool match_list(char *line, constext *pat, constext *const *table, int *dst);
 extern t_bool match_long(char *line, const char *pat, long *dst);
 extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen);
+extern t_bool match_string_ptr(char *line, const char *pat, char **dst);
 extern t_bool read_config_file(char *file, t_bool global_file);
 extern void quote_dash_to_space(char *str);
 extern void read_server_config(void);
 extern void write_config_file(char *file);
 
 /* cook.c */
+extern char *build_attach_line(t_part *part, int depth, int max_len, int is_uue, const char *name, const char *charset);
 extern const char *get_filename(t_param *ptr);
 extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int hide_uue, t_bool show_all_headers);
 extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t lcook_width);
@@ -181,9 +185,9 @@ extern void word_highlight_string(int row, int col, int size, int color);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
 	extern void debug_print_malloc(t_bool is_malloc, const char *xfile, int line, size_t size);
-#	ifdef DEBUG
+#	ifdef NNTP_ABLE
 	extern const char *logtime(void);
-#	endif /* DEBUG */
+#	endif /* NNTP_ABLE */
 #endif /* DEBUG */
 
 /* envarg.c */
@@ -250,9 +254,9 @@ extern void toggle_mini_help(int level);
 extern const char *get_domain_name(void);
 extern const char *get_fqdn(const char *host);
 extern const char *get_host_name(void);
-#ifndef FORGERY
+#if !defined(FORGERY) && defined(NNTP_ABLE)
 	extern char *build_sender(void);
-#endif /* !FORGERY */
+#endif /* !FORGERY && NNTP_ABLE */
 
 /* heapsort.c */
 #ifndef HAVE_HEAPSORT
@@ -262,7 +266,6 @@ extern const char *get_host_name(void);
 /* inews.c */
 extern t_bool submit_news_file(char *name, struct t_group *group, char *a_message_id);
 extern void get_from_name(char *from_name, struct t_group *thisgrp);
-extern void get_user_info(char *user_name, char *full_name);
 
 /* init.c */
 extern void init_selfinfo(void);
@@ -318,6 +321,7 @@ extern void grp_del_mail_art(struct t_article *article);
 /* main.c */
 extern int main(int argc, char *argv[]);
 extern int read_cmd_line_groups(void);
+extern void handle_cmdargs(t_bool init);
 extern _Noreturn void giveup(void);
 
 /* memory.c */
@@ -347,6 +351,7 @@ extern void lookup_mimetype(const char *ext, t_part *part);
 extern t_bool lookup_extension(char *extension, size_t ext_len, const char *major, const char *minor);
 
 /* misc.c */
+extern FILE *tin_fopen(const char *pathname, const char *mode);
 extern char *buffer_to_ascii(char *c);
 extern char *escape_shell_meta(const char *source, int quote_area);
 extern char *get_tmpfilename(const char *filename);
@@ -354,50 +359,52 @@ extern char *idna_decode(char *in);
 extern char *quote_wild(char *str);
 extern char *quote_wild_whitespace(char *str);
 extern char *strip_line(char *line);
+extern char *split_mailbox_list(char *from);
 extern const char *eat_re(char *s, t_bool eat_was);
 extern const char *get_val(const char *env, const char *def);
 extern const char *gnksa_strerror(int errcode);
-extern const char *validate_charset(const char *charset);
-extern int gnksa_check_from(char *from);
+extern int gnksa_check_from(const char *from);
 extern int gnksa_split_from(const char *from, char *address, char *realname, int *addrtype);
 extern int get_initials(struct t_article *art, char *s, int maxsize);
 extern int gnksa_do_check_from(const char *from, char *address, char *realname);
-extern int my_mkdir(char *path, mode_t mode);
+extern int my_mkdir(const char *path, mode_t mode);
 extern int parse_from(const char *from, char *address, char *realname);
+extern int rndm(void);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group, t_bool expand_all);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
-extern int tin_version_info(FILE *fp);
+extern int tin_version_info(FILE *fp, int verb);
 extern int tin_gettime(struct t_tintime *tt);
 extern long file_mtime(const char *file);
 extern long file_size(const char *file);
 extern t_bool backup_file(const char *filename, const char *backupname);
-extern t_bool copy_fp(FILE *fp_ip, FILE *fp_op);
+extern int copy_fp(FILE *fp_ip, FILE *fp_op);
 extern t_bool invoke_cmd(const char *nam);
 extern t_bool invoke_editor(const char *filename, int lineno, struct t_group *group);
 extern t_bool mail_check(const char *mailbox_name);
-extern void append_file(char *old_filename, char *new_filename);
+extern int append_file(const char *old_filename, const char *new_filename);
 #ifndef NDEBUG
 	extern _Noreturn void asfail(const char *file, int line, const char *cond);
-#endif /* ! NDEBUG */
+#endif /* !NDEBUG */
 extern void base_name(const char *fullpath, char *file);
 extern void cleanup_tmp_files(void);
-extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
+extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, const char *initl, t_bool raw_data);
 extern void create_index_lock_file(char *the_lock_file);
 extern void dir_name(const char *fullpath, char *dir);
 extern void draw_mark_selected(int i);
 extern void get_author(t_bool thread, struct t_article *art, char *str, size_t len);
 extern void get_cwd(char *buf);
-extern void show_connection_page(const int level, const char *title);
+extern void show_connection_page(void);
 extern void make_base_group_path(const char *base_dir, const char *group_name, char *group_path, size_t group_path_len);
 extern void make_group_path(const char *name, char *path);
-extern void process_charsets(char **line, size_t *max_line_len, const char *network_charset, const char *local_charset, t_bool conv_tex2iso);
+extern void process_charsets(char **line, size_t *max_line_len, const char *from_charset, const char *to_charset, t_bool conv_tex2iso);
 extern void read_input_history_file(void);
-extern void rename_file(const char *old_filename, const char *new_filename);
+extern int rename_file(const char *old_filename, const char *new_filename);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern void remove_soft_hyphens(char *line);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 extern void show_inverse_video_status(void);
+extern void srndm(void);
 extern void strip_name(const char *from, char *address);
 extern _Noreturn void tin_done(int ret, const char *fmt, ...);
 extern void toggle_inverse_video(void);
@@ -408,7 +415,7 @@ extern void toggle_inverse_video(void);
 	extern int my_isprint(int c);
 #endif /* NO_LOCALE || !MULTIBYTE_ABLE */
 #ifdef CHARSET_CONVERSION
-	extern t_bool buffer_to_network(char *line, int mmnwcharset);
+	extern t_bool buffer_to_network(char **line, int mmnwcharset);
 #endif /* CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
 	extern t_bool toggle_color(void);
@@ -422,13 +429,18 @@ extern void toggle_inverse_video(void);
 	extern void do_shell_escape(void);
 #endif /* !NO_SHELL_ESCAPE */
 
+/* my_tmpfile.c */
+extern int my_mktmp(char *filename, size_t name_size, const char *base_dir);
+extern FILE *my_tmpfile(void);
+
 /* newsrc.c */
-extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, t_artnum *art_count, t_artnum *art_max, t_artnum *art_min);
+extern int group_get_art_info(const char *tin_spooldir, const char *groupname, int grouptype, t_artnum *art_count, t_artnum *art_max, t_artnum *art_min);
 extern signed long int read_newsrc(char *newsrc_file, t_bool allgroups);
 extern signed long int write_newsrc(void);
 extern t_bool pos_group_in_newsrc(struct t_group *group, int pos);
 extern void art_mark(struct t_group *group, struct t_article *art, int flag);
 extern void backup_newsrc(void);
+extern void bulk_subscribe(struct t_group **groups, int groups_cnt, int sub_state, t_bool get_info);
 extern void catchup_newsrc_file(void);
 extern void delete_group(char *group);
 extern void expand_bitmap(struct t_group *group, t_artnum min);
@@ -452,7 +464,7 @@ extern void nntp_close(t_bool send_no_quit);
 	extern int get_respcode(char *, size_t);
 	extern int get_only_respcode(char *, size_t);
 	extern int new_nntp_command(const char *command, int success, char *message, size_t mlen);
-	extern void put_server(const char *string);
+	extern void put_server(const char *string, t_bool hide_from_log);
 	extern void u_put_server(const char *string);
 	extern int fgetc_server(FILE *stream);
 	extern int ungetc_server(int c, FILE *stream);
@@ -463,15 +475,15 @@ extern void nntp_close(t_bool send_no_quit);
 #endif /* NNTP_ABLE */
 
 /* nntps.c */
-	extern int tintls_init(void);
-	extern void tintls_exit(void);
+extern int tintls_init(void);
+extern void tintls_exit(void);
 #if defined(NNTP_ABLE) && defined(NNTPS_ABLE)
 	extern int tintls_open(const char *servername, int fd, void **session_ctx);
 	extern int tintls_close(void *session_ctx);
 	extern int tintls_handshake(void *session_ctx);
 	extern ssize_t tintls_read(void *session_ctx, void *buf, size_t count);
 	extern ssize_t tintls_write(void *session_ctx, const void *buf, size_t count);
-	extern int tintls_conninfo(void *session_ctx, FILE *stream);
+	extern int tintls_conninfo(void *session_ctx, FILE *fp);
 #endif /* NNTP_ABLE && NNTPS_ABLE */
 
 /* nrctbl.c */
@@ -484,6 +496,7 @@ extern t_bool get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const ch
 extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
 extern void config_page(const char *grpname, enum context level);
 extern int option_row(enum option_enum option);
+extern t_bool option_is_default(enum option_enum option);
 extern t_bool option_is_visible(enum option_enum option);
 extern void check_score_defaults(void);
 extern void refresh_config_page(enum option_enum act_option);
@@ -494,7 +507,9 @@ extern int show_page(struct t_group *group, int start_respnum, int *threadnum);
 extern void display_info_page(int part);
 extern void draw_page(int part);
 extern void info_pager(FILE *info_fh, const char *title, t_bool wrap_at_ends);
+extern void log_article_info(t_openartinfo *artinfo);
 extern void resize_article(t_bool wrap_lines, t_openartinfo *artinfo);
+extern void update_hide_uue(void);
 extern void toggle_raw(void);
 
 /* parsdate.y */
@@ -504,21 +519,13 @@ extern time_t parsedate(char *p, TIMEINFO *now);
 #ifdef HAVE_PGP_GPG
 	extern t_bool pgp_check_article(t_openartinfo *artinfo);
 	extern void init_pgp(void);
-	extern void invoke_pgp_mail(const char *nam, char *mail_to);
-	extern void invoke_pgp_news(char *artfile);
+	extern void invoke_pgp_mail(const char *nam, const char *mail_to);
+	extern void invoke_pgp_news(const char *artfile);
 #endif /* HAVE_PGP_GPG */
 
-/* plp_snprintf.c */
-#ifndef HAVE_SNPRINTF
-	extern int plp_snprintf(char *, size_t, const char *, ...);
-#endif /* !HAVE_SNPRINTF */
-#ifndef HAVE_VSNPRINTF
-	extern int plp_vsnprintf(char *, size_t, const char *, va_list);
-#endif /* !HAVE_VSNPRINTF */
-
 /* post.c */
-extern char *backup_article_name(const char *the_article);
 extern char *checknadd_headers(const char *infile, struct t_group *group);
+extern int check_mailbox_list(char *line, const char *header, int charset, int *errors);
 extern int count_postponed_articles(void);
 extern int mail_to_author(const char *group, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
 extern int mail_to_someone(const char *address, t_bool confirm_to_mail, t_openartinfo *artinfo, const struct t_group *group);
@@ -543,12 +550,13 @@ extern void quick_post_article(t_bool postponed_only, int num_cmd_line_groups);
 
 /* prompt.c */
 extern char *prompt_string_default(const char *prompt, char *def, const char *failtext, int history);
+extern char *prompt_string_ptr_default(const char *prompt, char **def, const char *failtext, int history);
 extern char *sized_message(char **result, const char *format, const char *subject);
 extern int prompt_num(int ch, const char *prompt);
 extern int prompt_yn(const char *prompt, t_bool default_answer);
 extern int prompt_msgid(void);
-extern t_bool prompt_default_string(const char *prompt, char *buf, int buf_len, char *default_prompt, int which_hist);
-extern t_bool prompt_menu_string(int line, const char *prompt, char *var);
+extern t_bool prompt_default_string(const char *prompt, char *buf, int buf_len, const char *default_prompt, int which_hist);
+extern t_bool prompt_menu_string(int line, const char *prompt, char **var);
 extern t_bool prompt_option_char(enum option_enum option);
 extern t_bool prompt_option_list(enum option_enum option);
 extern t_bool prompt_option_num(enum option_enum option);
@@ -571,6 +579,7 @@ extern char *tin_fgets(FILE *fp, t_bool header);
 /* refs.c */
 extern char *get_references(struct t_msgid *refptr);
 extern struct t_msgid *find_msgid(const char *msgid);
+extern t_bool valid_msgid(char *msgid);
 extern void build_references(struct t_group *group);
 extern void clear_art_ptrs(void);
 extern void collate_subjects(void);
@@ -580,11 +589,9 @@ extern void thread_by_reference(void);
 /* regex.c */
 extern t_bool compile_regex(const char *regex, struct regex_cache *cache, REGEX_OPTIONS options);
 extern t_bool match_regex(const char *string, char *pattern, struct regex_cache *cache, t_bool icase);
-extern int match_regex_ex(const char *string, int length, int offset, REGEX_OPTIONS options, struct regex_cache *regex);
-
+extern int match_regex_ex(const char *string, REGEX_SIZE length, REGEX_SIZE offset, REGEX_OPTIONS options, struct regex_cache *regex);
 extern REGEX_NOFFSET regex_get_ovector_count(struct regex_cache *regex);
 extern REGEX_SIZE *regex_get_ovector_pointer(struct regex_cache *regex);
-
 extern void highlight_regexes(int row, struct regex_cache *regex, int color);
 extern void regex_cache_init(struct regex_cache *regex);
 extern void regex_cache_destroy(struct regex_cache *regex);
@@ -603,8 +610,9 @@ extern void rfc1521_encode(char *line, FILE *f, int e);
 extern FILE *open_art_fp(struct t_group *group, t_artnum art);
 extern const char *get_param(t_param *list, const char *name);
 extern char *parse_header(char *buf, const char *pat, t_bool decode, t_bool structured, t_bool keep_tab);
+extern char *parse_mb_list_header(char *buf, const char *pat);
 extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter, const char *pmesg);
-extern int content_type(char *type);
+extern int content_type(const char *type);
 extern int parse_rfc822_headers(struct t_header *hdr, FILE *from, FILE *to);
 extern t_param *new_params(void);
 extern t_part *new_part(t_part *part);
@@ -612,6 +620,7 @@ extern void art_close(t_openartinfo *artinfo);
 extern void free_and_init_header(struct t_header *hdr);
 extern void free_list(t_param *list);
 extern void free_parts(t_part *ptr);
+extern void remove_comments(char *source);
 extern void unfold_header(char *line);
 
 /* rfc2047.c */
@@ -653,7 +662,7 @@ extern void close_msglog(void);
 
 /* search.c */
 extern enum option_enum search_config(t_bool forward, t_bool repeat, enum option_enum current, enum option_enum last);
-extern int get_search_vectors(int *start, int *end);
+extern int get_search_vectors(REGEX_SIZE *start, REGEX_SIZE *end);
 extern int search(t_function func, int current_art, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
@@ -664,9 +673,11 @@ extern void reset_srch_offsets(void);
 /* select.c */
 extern int add_my_group(const char *group, t_bool add, t_bool ignore_case);
 extern int choose_new_group(void);
-extern int show_article_by_msgid(char *messageid);
+#ifdef NNTP_ABLE
+	extern int show_article_by_msgid(const char *messageid);
+#endif /* NNTP_ABLE */
 extern int skip_newgroups(void);
-extern void selection_page(int start_groupnum, int num_cmd_line_groups);
+extern _Noreturn void selection_page(int start_groupnum, int num_cmd_line_groups);
 extern void show_selection_page(void);
 extern void toggle_my_groups(const char *group);
 
@@ -686,15 +697,20 @@ extern void set_signal_handlers(void);
 extern size_t my_strftime(char *s, size_t maxsize, const char *format, struct tm *timeptr);
 
 /* string.c */
+extern char *append_to_string(char *dst, const char *src);
 extern char *eat_tab(char *s);
 extern char *fmt_string(const char *fmt, ...);
-extern char *my_strdup(const char *str);
+#if !defined(USE_DMALLOC) || (defined(USE_DMALLOC) && !defined(HAVE_STRDUP))
+	extern char *my_strdup(const char *str);
+	extern char *my_strndup(const char *str, size_t n);
+#endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
 extern char *str_trim(char *string);
-extern char *strunc(const char *message, int len);
-extern char *tin_ltoa(t_artnum value, int digits);
+extern char *strunc(const char *message, size_t len);
+extern char *tin_ltoa(t_artnum value, size_t digits);
 extern char *tin_strtok(char *str, const char *delim);
 extern int sh_format(char *dst, size_t len, const char *fmt, ...);
 extern int strwidth(const char *str);
+extern signed int s2i(const char *s, signed int min, signed int max);
 extern size_t mystrcat(char **t, const char *s);
 extern void my_strncpy(char *p, const char *q, size_t n);
 extern void parse_format_string(const char *fmtstr, struct t_fmt *fmt);
@@ -715,7 +731,7 @@ extern int my_toupper(int);
 #endif /* !HAVE_STRSTR */
 #ifndef HAVE_STRCASECMP
 	extern int strcasecmp(const char *p, const char *q);
-#endif /* !HAVE STRCASECMP */
+#endif /* !HAVE_STRCASECMP */
 #ifndef HAVE_STRNCASECMP
 	extern int strncasecmp(const char *p, const char *q, size_t n);
 #endif /* !HAVE_STRNCASECMP */
@@ -727,7 +743,7 @@ extern int my_toupper(int);
 #endif /* !HAVE_ATOL */
 #ifndef HAVE_STRTOL
 	extern long strtol(const char *str, char **ptr, int use_base);
-#endif /* !HAVE STRTOL */
+#endif /* !HAVE_STRTOL */
 #ifndef HAVE_STRERROR
 	extern char *my_strerror(int n);
 #	define strerror(n) my_strerror(n)
@@ -737,17 +753,18 @@ extern int my_toupper(int);
 #	define strrstr(s,p)	my_strrstr(s,p)
 #endif /* !HAVE_STRRSTR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	extern char *spart(const char *str, int columns, t_bool pad);
+	extern char *spart(const char *str, size_t columns, t_bool pad);
 	extern char *wchar_t2char(const wchar_t *wstr);
 	extern wchar_t *abbr_wcsgroupname(const wchar_t *grpname, int len);
 	extern wchar_t *char2wchar_t(const char *str);
-	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
+	extern wchar_t *my_wcsdup(const wchar_t *wstr);
+	extern wchar_t *wcspart(const wchar_t *wstr, size_t columns, t_bool pad);
 	extern wchar_t *wexpand_tab(wchar_t *wstr, size_t tab_width);
-	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
+	extern wchar_t *wstrunc(const wchar_t *wmessage, size_t len);
 #else
 	extern char *abbr_groupname(const char *grpname, size_t len);
-	extern char *expand_tab(char *str, size_t tab_width);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+extern char *expand_tab(char *str, size_t tab_width);
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
@@ -771,14 +788,6 @@ extern void undo_auto_select_arts(void);
 extern void undo_selections(void);
 extern void untag_article(long art);
 
-/* tmpfile.c */
-#ifndef HAVE_TMPFILE
-	extern FILE *tmpfile(void);
-#endif /* !HAVE_TMPFILE */
-
-/* my_tmpfile.c */
-extern int my_tmpfile(char *filename, size_t name_size, const char *base_dir);
-
 /* thread.c */
 extern int find_response(int i, int n);
 extern int get_score_of_thread(int n);
@@ -797,8 +806,8 @@ extern t_bool thread_mark_postprocess(int function, t_function feed_type, int re
 extern void fixup_thread(int respnum, t_bool redraw);
 
 /* version.c */
-extern struct t_version *check_upgrade(char *line, const char *skip, const char *version);
-extern void upgrade_prompt_quit(struct t_version *upgrade, const char *file);
+extern struct t_version *check_upgrade(const char *line, const char *skip, const char *version);
+extern void upgrade_prompt_quit(struct t_version *upgrade, const char *file, FILE *fp);
 
 /* wildmat.c */
 extern t_bool wildmat(const char *text, char *p, t_bool icase);
@@ -808,7 +817,7 @@ extern t_bool wildmatpos(const char *text, char *p, t_bool icase, REGEX_SIZE *sr
 #ifdef XFACE_ABLE
 	extern void slrnface_stop(void);
 	extern void slrnface_start(void);
-	extern void slrnface_display_xface(char *face);
+	extern void slrnface_display_xface(const char *face);
 	extern void slrnface_clear_xface(void);
 	extern void slrnface_suppress_xface(void);
 	extern void slrnface_show_xface(void);
