@@ -3,7 +3,7 @@
  *  Module    : tcurses.h
  *  Author    : Thomas Dickey
  *  Created   : 1997-03-02
- *  Updated   : 2021-12-14
+ *  Updated   : 2024-04-01
  *  Notes     : curses #include files, #defines & struct's
  *
  * Copyright (c) 1997-2024 Thomas Dickey <dickey@invisible-island.net>
@@ -44,11 +44,7 @@
 
 #	if defined(USE_CURSES) || defined(NEED_CURSES_H)
 #		ifdef HAVE_XCURSES
-#			undef bool
-#			define bool XCursesBool
 #			include <xcurses.h>
-#			undef bool
-#			define getattrs(w) (w)->_attrs
 #		else
 #			if defined(HAVE_NCURSESW_NCURSES_H)
 #				ifndef _XOPEN_SOURCE_EXTENDED
@@ -181,7 +177,7 @@ extern void write_line(int row, char *buffer);
 
 #		define HpGlitch(func)			if (_hp_glitch) func
 
-#	endif /* USE_CURSES/!USE_CURSES */
+#	endif /* USE_CURSES */
 
 extern void my_dummy(void);
 
