@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-05-10
+ *  Updated   : 2024-08-28
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,7 @@
 
 #ifdef HAVE_CONFIG_H
 #	ifndef TIN_AUTOCONF_H
-#		include <autoconf.h>	/* FIXME: normally we use 'config.h' */
+#		include "autoconf.h"	/* FIXME: normally we use 'config.h' */
 #	endif /* !TIN_AUTOCONF_H */
 #else
 #	ifndef HAVE_CONFDEFS_H
@@ -102,7 +102,9 @@
 #	define SMALL_MEMORY_MACHINE
 #endif /* __amiga__ || __amiga */
 
-#include <signal.h>
+#ifdef HAVE_SIGNAL_H
+#	include <signal.h>
+#endif /* HAVE_SIGNAL_H */
 
 enum context { cMain, cArt, cAttachment, cAttrib, cConfig, cFilter, cGroup, cInfopager, cPage, cPOSTED, cPost, cPostCancel, cPostFup, cReconnect, cScope, cSelect, cThread, cURL };
 enum icontext { cNone, cGetline, cPromptCONT, cPromptSLK, cPromptYN };
@@ -184,13 +186,15 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	include <getopt.h>
 #endif /* HAVE_GETOPT_H */
 
+#ifdef HAVE_LIMITS_H
+#	include <limits.h>
+#endif /* HAVE_LIMITS_H */
+
 #if defined(ENABLE_LONG_ARTICLE_NUMBERS) && !defined(SMALL_MEMORY_MACHINE)
 /*
  * defines and typedefs for 64 bit article numbers
  *
  * TODO: what if !CPP_DOES_CONCAT
- *       add configure check for PRIdLEAST64
- *       add configure check for SCNdLEAST64
  *       add configure check for INT64_MAX, LLONG_MAX, LONG_MAX
  */
 #	if defined(HAVE_INT_LEAST64_T) && !defined(HAVE_INTTYPES_H)
@@ -205,10 +209,10 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	endif /* HAVE_INT_LEAST64_T || HAVE_LONG_LONG || quad_t */
 #	ifdef HAVE_STDINT_H
 #		include <stdint.h>
-#	endif	/* HAVE_STDINT_H */
+#	endif /* HAVE_STDINT_H */
 #endif /* ENABLE_LONG_ARTICLE_NUMBERS && !SMALL_MEMORY_MACHINE */
 #ifdef USE_LONG_ARTICLE_NUMBERS
-#	if defined(HAVE_INT_LEAST64_T) && defined(HAVE_INT64_C)
+#	if defined(HAVE_INT_LEAST64_T) && defined(HAVE_PRIDLEAST64) && defined(HAVE_SCNDLEAST64) && defined(HAVE_INT64_C)
 #		include <inttypes.h>
 		typedef int_least64_t t_artnum;
 #		define T_ARTNUM_PFMT PRIdLEAST64
@@ -221,7 +225,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #		define T_ARTNUM_SFMT "lld"
 #		define T_ARTNUM_CONST(v) v ## LL
 #		define ARTNUM_MAX LLONG_MAX
-#	endif /* HAVE_INT_LEAST64_T && HAVE_INT64_C */
+#	endif /* HAVE_INT_LEAST64_T && HAVE_PRIDLEAST64 && HAVE_SCNDLEAST64 && HAVE_INT64_C */
 #	ifdef HAVE_ATOLL
 #		define atoartnum atoll
 #	else
@@ -283,6 +287,14 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	define IGNORE_SYSTEM_STATUS 1
 #endif /* sun|| __sun && (!__SVR4 || __svr4__) && BSD && BSD < 199306 */
 
+#if defined(__STDC__) || defined(__STDC_VERSION__) || defined(__cplusplus)
+#	define TIN_STDC 1
+#endif /* __STDC__ || __STDC_VERSION__ || __cplusplus */
+/* compiler specific fixup */
+#if !defined(TIN_STDC) && defined(__XCC)
+#	define TIN_STDC 1
+#endif /* !TIN_STDC && __XCC */
+
 #ifdef HAVE_FCNTL_H
 #	include <fcntl.h>
 #endif /* HAVE_FCNTL_H */
@@ -339,7 +351,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* !WIFEXITED */
 
 /*
- * Needed for timeout in user abort of indexing a group (BSD & SYSV variaties)
+ * Needed for timeout in user abort of indexing a group (BSD & SYSV varieties)
  */
 #ifdef HAVE_SYS_SELECT_H
 #	ifdef NEED_TIMEVAL_FIX
@@ -408,6 +420,22 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	define _PATH_TMP	"/tmp/"
 #endif /* _PATH_TMP */
 
+/* we just need INT_MAX be defined approximately fitting the system */
+#if !defined(INT_MAX)
+#	ifdef INT_WIDTH
+#		if INT_WIDTH >= 32
+#			define INT_MAX 2147483647 /* 2^31-1 */
+#		else
+#			define INT_MAX 32767 /* 2^15-1 */
+#		endif /* INT_WIDTH >= 32 */
+#	else
+#		define INT_MAX 2147483647 /* 2^31-1 */
+#	endif /* INT_WIDTH */
+#endif /* !INT_MAX */
+#if !defined(INT_MIN)
+#	define INT_MIN (-INT_MAX - 1)
+#endif /* INT_MIN */
+
 /*
  * If OS misses the isascii() function
  */
@@ -443,7 +471,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 /*
  * Setup support for reading from NNTP
  */
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+#if defined(NNTP_ABLE) || defined(NNTP_ONLY) || defined(NNTPS_ABLE)
 #	ifndef NNTP_ABLE
 #		define NNTP_ABLE	1
 #	endif /* !NNTP_ABLE */
@@ -457,7 +485,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifdef BROKEN_LISTGROUP
 #		undef BROKEN_LISTGROUP
 #	endif /* BROKEN_LISTGROUP */
-#endif /* NNTP_ABLE || NNTP_ONLY */
+#endif /* NNTP_ABLE || NNTP_ONLY || NNTPS_ABLE */
 
 #define FAKE_NNTP_FP		(FILE *) 9999
 
@@ -506,10 +534,8 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	endif /* !HAVE_STRRCHR */
 #	if defined(__386BSD__) || defined(__bsdi__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
 #		define DEFAULT_PRINTER	"/usr/bin/lpr"
-#		define DEFAULT_SUM	"/usr/bin/cksum -o 1 <" /* use tailing <, otherwise get filename output too */
 #	else
 #		define DEFAULT_PRINTER	"/usr/ucb/lpr"
-#		define DEFAULT_SUM	"sum"
 #	endif /* __386BSD__ || __bsdi__ || __NetBSD__ || __FreeBSD__ || __OpenBSD__ */
 #	ifdef DGUX
 #		define USE_INVERSE_HACK
@@ -543,9 +569,6 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	ifndef DEFAULT_PRINTER
 #		define DEFAULT_PRINTER	"/usr/bin/lp"
 #	endif /* !DEFAULT_PRINTER */
-#	ifndef PATH_SUM
-#		define DEFAULT_SUM	"sum -r"
-#	endif /* !PATH_SUM */
 #endif /* BSD */
 
 /*
@@ -577,27 +600,9 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 
 #define INTERNAL_CMD	"--internal"
 
-/*
- * Fix up the 'sum' path and parameter for './configure'd systems
- */
-#ifdef PATH_SUM
-#	ifdef DEFAULT_SUM
-#		undef DEFAULT_SUM
-#	endif /* DEFAULT_SUM */
-#	ifdef SUM_TAKES_DASH_R
-#		define DEFAULT_SUM PATH_SUM_R
-#	else
-#		define DEFAULT_SUM PATH_SUM
-#	endif /* SUM_TAKES_DASH_R */
-#endif /* PATH_SUM */
-
 #ifdef HAVE_LONG_FILE_NAMES
-#	define PATH_PART	"part"
-#	define PATH_PATCH	"patch"
 #	define INDEX_LOCK	"tin.%.256s.LCK"
 #else
-#	define PATH_PART	""
-#	define PATH_PATCH	"p"
 #	define INDEX_LOCK	"%.10s.LCK"
 #endif /* HAVE_LONG_FILE_NAMES */
 
@@ -749,6 +754,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #define POSTPONED_FILE	"postponed.articles"
 #define SUBSCRIPTIONS_FILE	"subscriptions"
 #define NEWSGROUPS_FILE	"newsgroups"
+#define MOTD_FILE	"motd"
 #define KEYMAP_FILE	"keymap"
 
 #define SIGDASHES "-- \n"
@@ -851,13 +857,20 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #endif /* !forever */
 
 /*
- * safe strcpy into fixed-legth buffer
+ * safe ('0'-terminates but may truncate) strcpy into fixed-legth buffer
  */
-#if 1
-#	define STRCPY(dst, src)	(strncpy(dst, src, sizeof(dst) - 1), dst[sizeof(dst) - 1] = '\0')
-#else
-#	define STRCPY(dst, src)	(*(dst) = '\0', strncat(dst, src, sizeof(dst) - 1))
-#endif /* 1 */
+#define STRCPY(dst, src) ((void) snprintf(dst, sizeof(dst), "%s", src))
+/*
+	#ifdef HAVE_STRLCPY
+	#	define STRCPY(dst, src) (strlcpy(dst, src, sizeof(dst)))
+	#else
+	#	if 0
+	#		define STRCPY(dst, src)	(strncpy(dst, src, sizeof(dst) - 1)[sizeof(dst) - 1] = '\0')
+	#	else
+	#		define STRCPY(dst, src)	(*(dst) = '\0', strncat(dst, src, sizeof(dst) - 1))
+	#	endif
+	#endif
+*/
 
 #define STRCMPEQ(s1, s2)	(strcmp((s1), (s2)) == 0)
 #define STRNCMPEQ(s1, s2, n)	(strncmp((s1), (s2), n) == 0)
@@ -898,6 +911,7 @@ enum rc_state { RC_IGNORE, RC_UPGRADE, RC_DOWNGRADE, RC_ERROR };
 #	define NAME_LEN	14
 #endif /* HAVE_LONG_FILE_NAMES */
 #define LEN	1024
+#define BUF_SIZE	1024
 
 #define NEWSRC_LINE	8192
 #define HEADER_LEN	1024
@@ -1094,9 +1108,11 @@ enum {
  */
 #ifdef NNTP_ABLE
 #	define TIN_FCLOSE(x)	if (x != FAKE_NNTP_FP) fclose(x)
+#	define TIN_NNTP_TIMEOUT ((cmdline.args & CMDLINE_NNTP_TIMEOUT) ? cmdline.nntp_timeout : tinrc.nntp_read_timeout_secs)
 #else
 #	define TIN_FCLOSE(x)	fclose(x)
 #endif /* NNTP_ABLE */
+#define TIN_NNTP_TIMEOUT_MAX	16383
 
 /*
  * Often used macro to point to the group we are currently in
@@ -1147,13 +1163,13 @@ enum {
 
 
 /*
- * indicate given cmd-line options
+ * indicate cmd-line options that are not strings
+ * adjust t_cmdlineopts accordingly
  */
-#define CMDLINE_GETART_LIMIT	1
-#define CMDLINE_MAILDIR			2
-#define CMDLINE_NNTPSERVER		4
-#define CMDLINE_SAVEDIR			8
-#define CMDLINE_USE_COLOR		16
+#define CMDLINE_GETART_LIMIT	(1 << 0)
+#define CMDLINE_USE_COLOR		(1 << 1)
+#define CMDLINE_NNTP_TIMEOUT	(1 << 2)
+#define CMDLINE_NO_DESCRIPTION	(1 << 3)
 
 
 /*
@@ -1299,6 +1315,8 @@ enum {
 #define UUE_NO			0		/* Don't hide uue data */
 #define UUE_YES			1		/* Hide uue data */
 #define UUE_ALL			2		/* Hide uue data harder */
+#define UUE_COMPLETE	3		/* uue part is complete */
+#define UUE_INCOMPLETE	4		/* uue part is not complete */
 
 /*
  * used in misc.c/rfc1524.c
@@ -1362,7 +1380,7 @@ enum {
 #	undef assert
 #endif /* assert */
 #ifdef NDEBUG
-#	define assert(p)        ((void) 0)
+#	define assert(p)		((void) 0)
 #else
 #	ifdef CPP_DOES_EXPAND
 #		define assert(p)	if(!(p)) asfail(__FILE__, __LINE__, #p); else (void) 0;
@@ -1441,14 +1459,17 @@ enum {
 #define FILTER_LINES_GT		3
 
 /*
- * default format strings for selection, group, thread level
- * and the date display in the page header.
+ * default format strings for selection, group, thread, attachment level,
+ * the mime header in the pager and the date display in the page header.
  * Don't change without adjusting rc_update() and the like accordingly!
  */
-#define DEFAULT_SELECT_FORMAT	"%f %n %U  %G  %d"
-#define DEFAULT_GROUP_FORMAT	"%n %m %R %L  %s  %F"
-#define DEFAULT_THREAD_FORMAT	"%n %m  [%L]  %T  %F"
-#define DEFAULT_DATE_FORMAT		"%a, %d %b %Y %H:%M:%S"
+#define DEFAULT_SELECT_FORMAT		"%f %n %U  %G  %d"
+#define DEFAULT_GROUP_FORMAT		"%n %m %R %L  %s  %F"
+#define DEFAULT_THREAD_FORMAT		"%n %m  [%L]  %T  %F"
+#define DEFAULT_ATTACHMENT_FORMAT	"%T%S%E%C%d"
+#define DEFAULT_PAGE_MIME_FORMAT	"[-- %T%S%*n%z%*l%!c%!d%*e --]"
+#define DEFAULT_PAGE_UUE_FORMAT		"[-- %T%S%*n%I%!d%*e --]"
+#define DEFAULT_DATE_FORMAT			"%a, %d %b %Y %H:%M:%S"
 
 /*
  * unicode normalization
@@ -1540,17 +1561,19 @@ typedef unsigned char	t_bitmap;
 #else
 #	define HASHNODE_TABLE_SIZE 222199
 #	define MSGID_HASH_SIZE	222199
-#endif /* MSGID_HASH_SIZE */
+#endif /* SMALL_MEMORY_MACHINE */
 
 /*
  * cmd-line options
  */
 struct t_cmdlineopts {
-	int getart_limit;			/* getart_limit */
-	char maildir[PATH_LEN];		/* maildir */
-	char nntpserver[PATH_LEN];	/* nntpserver */
-	char savedir[PATH_LEN];		/* savedir */
-	unsigned int args:5;		/* given options */
+	int getart_limit;		/* getart_limit */
+	int nntp_timeout;		/* nntp_read_timeout_secs */
+	char *maildir;			/* maildir */
+	char *nntpserver;		/* nntpserver */
+	char *savedir;			/* savedir */
+	char *msgid;			/* messageid (TODO: reduce to 250 as of RFC 3977 3.6 & RFC 5536 3.1.3 if done everywhere in the code) */
+	unsigned int args:4;	/* given options */
 };
 
 /*
@@ -1565,6 +1588,13 @@ struct t_msgid {
 	char txt[1];			/* The actual msgid */
 };
 
+struct t_mailbox {
+	char *from;			/* From: line from mail header (address) */
+	char *name;			/* From: line from mail header (full name) */
+	int gnksa_code;
+	struct t_mailbox *next;
+};
+
 /*
  * struct t_article - article header
  *
@@ -1583,17 +1613,15 @@ struct t_msgid {
 struct t_article {
 	t_artnum artnum;		/* Article number in spool directory for group */
 	char *subject;			/* Subject: line from mail header */
-	char *from;			/* From: line from mail header (address) */
-	char *name;			/* From: line from mail header (full name) */
 	char *xref;			/* Xref: cross posted article reference line */
 	char *path;			/* Path: line */
 	/* NB: The msgid and refs are only retained until the reference tree is built */
 	char *msgid;			/* Message-ID: unique message identifier */
 	char *refs;			/* References: article reference id's */
 	struct t_msgid *refptr;		/* Pointer to us in the reference tree */
+	struct t_mailbox mailbox;
 	time_t date;			/* Date: line from header in seconds */
 	int line_count;			/* Lines: number of lines in article */
-	int gnksa_code;			/* From: line from mail header (GNKSA error code) */
 	int tagged;			/* 0 = not tagged, >0 = tagged */
 	int thread;
 	int prev;
@@ -1656,37 +1684,62 @@ struct t_newsheader {
 #define BITS_OF_thread_perc		7
 #define BITS_OF_trim_article_body	3
 
+/*
+ * used as flags for t_attach_item
+ */
+#define ATTACH_SHOW_CONTENT		(1 << 0) /* content visible */
+#define ATTACH_SHOW_BOTH		(1 << 1) /* description and content visible */
+#define ATTACH_OMIT_DESC		(1 << 2) /* description can be omitted */
+#define ATTACH_OMIT_BOTH		(1 << 3) /* description and content can be omitted */
+#define ATTACH_ITEM_IS_TYPE		(1 << 4) /* content type */
+#define ATTACH_ITEM_IS_SUBTYPE	(1 << 5) /* content subtype */
+
+/*
+ * struct t_attach_item - information about a specific header part of a mime attachment
+ */
+struct t_attach_item {
+	const char *content;		/* "7bit", "US-ASCII" etc. */
+	const char *description;	/* "encoding ", "charset " etc. */
+	const char *fmt;			/* %s */
+	unsigned int flags;
+	struct t_attach_item *prev;
+	struct t_attach_item *next;
+};
+
 /*
  * struct t_attribute - configurable attributes on a per group basis
  */
 struct t_attribute {
-	char *maildir;				/* mail dir if other than ~/Mail */
-	char *savedir;				/* save dir if other than ~/News */
-	char *savefile;				/* save articles to specified file */
-	char *sigfile;				/* sig file if other than ~/.Sig */
-	char *group_format;			/* format string for group level */
-	char *thread_format;		/* format string for thread level */
-	char *date_format;			/* format string for the date display */
-	char *editor_format;		/* editor + parameters  %E +%N %F */
-	char *organization;			/* organization name */
-	char *fcc;					/* Fcc folder for mail */
-	char *followup_to;			/* where posts should be redirected */
-	char *quick_kill_scope;		/* quick filter kill scope */
-	char *quick_select_scope;	/* quick filter select scope */
-	char *mailing_list;			/* mail list email address */
-	char *news_headers_to_display;	/* which headers to display */
-	char *news_headers_to_not_display;	/* which headers to not display */
-	char *x_headers;			/* extra headers for message header */
-	char *x_body;				/* boilerplate text for message body */
-	char *from;					/* from line */
-	char *news_quote_format;	/* another way to begin a posting format */
-	char *quote_chars;			/* string to precede quoted text on each line */
-	char *mime_types_to_save;	/* MIME content major/minors we want to save */
+	char **maildir;				/* mail dir if other than ~/Mail */
+	char **savedir;				/* save dir if other than ~/News */
+	char **savefile;			/* save articles to specified file */
+	char **sigfile;				/* sig file if other than ~/.Sig */
+	char **group_format;		/* format string for group level */
+	char **thread_format;		/* format string for thread level */
+	char **date_format;			/* format string for the date display */
+	char **editor_format;		/* editor + parameters  %E +%N %F */
+	char **organization;		/* organization name */
+	char **fcc;					/* Fcc folder for mail */
+	char **followup_to;			/* where posts should be redirected */
+	char **quick_kill_scope;	/* quick filter kill scope */
+	char **quick_select_scope;	/* quick filter select scope */
+	char **mailing_list;		/* mail list email address */
+	char **news_headers_to_display;	/* which headers to display */
+	char **news_headers_to_not_display;	/* which headers to not display */
+	char **x_headers;			/* extra headers for message header */
+	char **x_body;				/* boilerplate text for message body */
+	char **from;				/* from line */
+	char **news_quote_format;	/* another way to begin a posting format */
+	char **quote_chars;			/* string to precede quoted text on each line */
+	char **mime_types_to_save;	/* MIME content major/minors we want to save */
 #ifdef HAVE_ISPELL
-	char *ispell;				/* path to ispell and options */
+	char **ispell;				/* path to ispell and options */
 #endif /* HAVE_ISPELL */
 #ifdef CHARSET_CONVERSION
-	char *undeclared_charset;		/* charset of articles without MIME charset declaration */
+	char **undeclared_charset;	/* charset of articles without MIME charset declaration */
+#	ifdef USE_ICU_UCSDET
+	BoolField(undeclared_cs_guess);
+#	endif /* USE_ICU_UCSDET */
 	IntField(mm_network_charset);		/* network charset */
 #endif /* CHARSET_CONVERSION */
 	struct t_newsheader *headers_to_display;	/* array of which headers to display */
@@ -1729,6 +1782,7 @@ struct t_attribute {
 	IntField(thread_perc);			/* percentage threading threshold */
 	IntField(show_author);			/* 0=none, 1=name, 2=addr, 3=both */
 	BoolField(show_signatures);		/* 0=none, 1=show signatures */
+	BoolField(show_art_score);		/* 0=none, 1=show score */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	BoolField(suppress_soft_hyphens);	/* set TRUE to remove soft hyphens (U+00AD) from articles */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1816,6 +1870,7 @@ struct t_attribute_state {
 	BoolField(show_author);
 	BoolField(show_only_unread_arts);
 	BoolField(show_signatures);
+	BoolField(show_art_score);
 	BoolField(sigdashes);
 	BoolField(sigfile);
 	BoolField(signature_repost);
@@ -1832,6 +1887,9 @@ struct t_attribute_state {
 	BoolField(trim_article_body);
 #ifdef CHARSET_CONVERSION
 	BoolField(undeclared_charset);
+#	ifdef USE_ICU_UCSDET
+	BoolField(undeclared_cs_guess);
+#	endif /* USE_ICU_UCSDET */
 	BoolField(mm_network_charset);
 #endif /* CHARSET_CONVERSION */
 	BoolField(verbatim_handling);
@@ -1977,16 +2035,15 @@ struct t_filter {
 	char *msgid;			/* Message-ID: line */
 	char *xref;			/* groups in xref line */
 	char *path;			/* server in path line */
-	struct t_filter *next;		/* next rule valid in group */
 	time_t time;			/* expire time in seconds */
 	int lines_num;			/* Lines: line */
 	int gnksa_num;			/* GNKSA code */
 	int score;			/* score to give if rule matches */
 	char lines_cmp;			/* Lines compare <> */
 	char gnksa_cmp;			/* GNKSA compare <> */
-	unsigned int inscope:4;		/* if group matches scope e.g. 'comp.os.*' */
-	unsigned int icase:2;		/* Case sensitive filtering */
 	unsigned int fullref:4;		/* use full references or last entry only */
+	t_bool icase:1;			/* Case sensitive filtering */
+	t_bool inscope:1;		/* if group matches scope e.g. 'comp.os.*' */
 };
 
 /*
@@ -1994,8 +2051,8 @@ struct t_filter {
  */
 struct t_filter_rule {
 	struct t_filter_comment *comment;
-	char text[PATH_LEN];
-	char scope[PATH_LEN];
+	char *text;
+	char *scope;
 	int counter;
 	int icase;
 	int fullref;
@@ -2201,11 +2258,11 @@ typedef struct urllist {
  * Determine qsort compare type
  */
 #ifdef HAVE_COMPTYPE_VOID
-#	ifdef __STDC__
+#	ifdef TIN_STDC
 		typedef const void *t_comptype;
 #	else
 		typedef void *t_comptype;
-#	endif /* __STDC__ */
+#	endif /* TIN_STDC */
 #else
 #	ifdef HAVE_COMPTYPE_CHAR
 		typedef char *t_comptype;
@@ -2239,10 +2296,27 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #define ENV_VAR_SHELL		"SHELL"
 #define TIN_EDITOR_FMT		"%E +%N %F"
 #define MAILER_FORMAT		"%M -oi -t < %F"
+#define DEFAULT_RANGE		"1-."
+#define DEFAULT_SAVE_FILE	"savefile.tin"
+#define MAIL_QUOTE_FORMAT	"In article %M you wrote:"
+#define DEFAULT_MIME_TYPES_TO_SAVE		"*/*"
+#define DEFAULT_NEWS_HEADERS_TO_DISPLAY	"Newsgroups Followup-To Summary Keywords X-Comment-To"
+#define DEFAULT_NEWS_QUOTE_FORMAT	"%F wrote:"
+#define DEFAULT_SIGFILE		".Sig"
+#define DEFAULT_XPOST_QUOTE_FORMAT	"In %G %F wrote:"
+#define DEFAULT_FILTER		"*"
 #ifdef HAVE_KEY_PREFIX
 #	define KEY_PREFIX		0x8f: case 0x9b
 #endif /* HAVE_KEY_PREFIX */
 
+/* the next two are needed for fcc 1.0 on linux */
+#ifndef S_IFMT
+#	define S_IFMT	0xF000	/* type of file */
+#endif /* S_IFMT */
+#ifndef S_IFREG
+#	define S_IFREG	0x8000	/* regular */
+#endif /* S_IFREG */
+
 #if !defined(S_ISDIR)
 #	define S_ISDIR(m)	(((m) & S_IFMT) == S_IFDIR)
 #endif /* !S_ISDIR */
@@ -2251,6 +2325,10 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #	define S_ISREG(m)	(((m) & S_IFMT) == S_IFREG)
 #endif /* !S_ISREG */
 
+#if !defined(S_ISLNK)
+#	define S_ISLNK(m)	(((m) & S_IFMT) == S_IFLNK)
+#endif /* !S_ISLNK */
+
 #ifndef S_IRWXU /* should be defined in <sys/stat.h> */
 #	define S_IRWXU	0000700	/* read, write, execute permission (owner) */
 #	define S_IRUSR	0000400	/* read permission (owner) */
@@ -2268,13 +2346,6 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #	define S_IXOTH	0000001	/* execute permission (other) */
 #endif /* !S_IRWXU */
 
-/* the next two are needed for fcc 1.0 on linux */
-#ifndef S_IFMT
-#	define S_IFMT	0xF000	/* type of file */
-#endif /* S_IFMT */
-#ifndef S_IFREG
-#	define S_IFREG	0x8000	/* regular */
-#endif /* S_IFREG */
 
 #ifndef S_IRWXUGO
 #	define S_IRWXUGO	(S_IRWXU|S_IRWXG|S_IRWXO)	/* read, write, execute permission (all) */
@@ -2335,17 +2406,16 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
  * Cast for the (few!) places where we need to examine 8-bit characters w/o
  * sign-extension, and a corresponding test-macro.
  */
-#define EIGHT_BIT(ptr)	(unsigned char *)ptr
-#define is_EIGHT_BIT(p)	((*EIGHT_BIT(p) < 32 && !isspace((int)*p)) || *EIGHT_BIT(p) > 127)
+#define is_EIGHT_BIT(p)	((*(unsigned char *) (p) < 32 && !isspace((unsigned char) *p)) || *(unsigned char *) (p) > 127)
 
 /*
  * function prototypes & extern definitions
  */
 
 #ifndef SIG_ARGS
-#	if defined(__STDC__)
+#	if defined(TIN_STDC)
 #		define SIG_ARGS	int sig
-#	endif /* __STDC__ */
+#	endif /* TIN_STDC */
 #endif /* !SIG_ARGS */
 
 #ifndef __LCLINT__ /* lclint doesn't like it */
@@ -2373,11 +2443,11 @@ typedef void (*t_sortfunc)(void *, size_t, size_t, t_compfunc);
 #	define OUTC_ARGS	int c
 #endif /* !OUTC_ARGS */
 
-#if __STDC__ || defined(__cplusplus)
+#if TIN_STDC
 #	define OUTC_FUNCTION(func)	OUTC_RETTYPE func (OUTC_ARGS)
 #else
 #	define OUTC_FUNCTION(func)	OUTC_RETTYPE func (c) int c;
-#endif /* __STDC__ || __cplusplus */
+#endif /* TIN_STDC */
 
 typedef OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
 typedef FILE TCP;
@@ -2446,15 +2516,19 @@ typedef void (*BodyPtr) (char *, FILE *, int);
  * for the stdio functions as well as the network functions.
  */
 #ifdef USE_SOCKS5
-#	define SOCKS
+#	ifndef SOCKS /* autoconf.h */
+#		define SOCKS
+#	endif /* !SOCKS */
 #	include <socks.h>
 /* socks.h doesn't define prototypes for use */
-extern size_t read(int, char *, size_t);
 extern int dup(int);
 extern int close(int);
 extern int fprintf(FILE *, const char *, ...);
 extern int fclose(FILE *);
-extern struct tm *localtime(time_t *);
+#if 0
+	extern size_t read(int, char *, size_t);
+	extern struct tm *localtime(const time_t *);
+#endif /* 0 */
 #endif /* USE_SOCKS5 */
 
 #ifdef SETVBUF_REVERSED
@@ -2499,9 +2573,9 @@ extern struct tm *localtime(time_t *);
 #endif /* USE_CANLOCK */
 
 /* gsasl */
-#ifdef USE_SASL
+#ifdef USE_GSASL
 #	include <gsasl.h>
-#endif /* USE_SASL */
+#endif /* USE_GSASL */
 
 /*
  * adapted from ncurses curses.priv.h:
@@ -2530,14 +2604,6 @@ extern struct tm *localtime(time_t *);
 #	endif /* va_copy || HAVE_VA_COPY */
 #endif /* HAVE___VA_COPY */
 
-/* snprintf(), vsnprintf() */
-#ifndef HAVE_SNPRINTF
-#	define snprintf	plp_snprintf
-#endif /* HAVE_SNPRINTF */
-#ifndef HAVE_VSNPRINTF
-#	define vsnprintf	plp_vsnprintf
-#endif /* HAVE_VSNPRINTF */
-
 #ifdef HAVE_MEMMOVE
 #	define my_memmove memmove
 #else
@@ -2558,28 +2624,15 @@ extern struct tm *localtime(time_t *);
 #	define UNUSED(x) x
 #endif /* __GNUC__ && !__cplusplus && !__APPLE_CC__ && !__NeXT__ */
 
-/* #if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L */
 #ifdef STDC_NORETURN
 #	ifdef HAVE_STDNORETURN_H
 #		include <stdnoreturn.h>
 #	endif /* HAVE_STDNORETURN_H */
 #else
-#	undef _Noreturn
-	/*
-	 * gcc says "The attribute noreturn is not implemented in GCC
-	 * versions earlier than 2.5" but gcc 2.7.2.3 on m68k-sony-newsos
-	 * or mips-dec-osf1 still doesn't know __attribute__((noreturn)).
-	 * __GNUC__ > 2 should be fine
-	 */
-#	if defined(__GNUC__) && !defined(__cplusplus) && !defined(__APPLE_CC__) && !defined(__NeXT__)
-#		if __GNUC__ > 2
-#			define _Noreturn __attribute__((noreturn))
-#		else
-#			define _Noreturn /**/
-#		endif /* __GNUC__ > 2 */
-#	else
-#		define _Noreturn /**/
-#	endif /* __GNUC__ && !__cplusplus && !__APPLE_CC__ && !__NeXT__ */
+#	if defined(_Noreturn)
+#		undef _Noreturn
+#	endif /* _Noreturn */
+#	define _Noreturn /**/
 #endif /* STDC_NORETURN */
 
 #ifndef __CPROTO__
@@ -2624,10 +2677,24 @@ struct t_overview_fmt {
 	enum f_type type;
 };
 
+/* TODO:
+ *       switch to version = (rc_majorv << 16) + (rc_minorv << 8) + rc_subv
+ *       that is v2.6.4 == 0x020604
+ */
 struct t_version {
 	enum rc_state state;
 	int file_version;		/* rc_majorv * 10000 + rc_minorv * 100 + rc_subv */
 /*	int current_version;*/	/* c_majorv * 10000 + c_minorv * 100 + c_subv */
 };
 
+/* used in strunc(), wstrunc() and draw_page_header() */
+#define TRUNC_TAIL	"..."
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		/* U+2026 (HORIZONTAL ELLIPSIS) */
+#		define WTRUNC_TAIL	L"\x2026"
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+/* C90: isxdigit(3) */
+#define IS_XDIGIT(c) (((c) >= '0' && (c) <= '9') || ((c) >= 'a' && (c) <= 'f') || ((c) >= 'A' && (c) <= 'F'))
+
 #endif /* !TIN_H */
