@@ -3,7 +3,7 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2023-11-27
+ *  Updated   : 2024-06-21
  *  Notes     :
  *
  * Copyright (c) 1992-2024 Iain Lea <iain@bricbrac.de>
@@ -357,10 +357,17 @@ do_read_newsrc_active_file(
 	t_artnum count = T_ARTNUM_CONST(-1), min = T_ARTNUM_CONST(1), max = T_ARTNUM_CONST(0);
 	static char ngname[NNTP_GRPLEN + 1]; /* RFC 3977 3.1 limits group names to 497 octets */
 	struct t_group *grpptr;
+	t_bool changed;
 #ifdef NNTP_ABLE
 	t_bool need_auth = FALSE;
 	char *ngnames[NUM_SIMULTANEOUS_GROUP_COMMAND] = { NULL };
 	int index_i = 0, index_o = 0;
+	int respcode, i, j;
+	char fmt[25];
+	char buf[NNTP_STRLEN];
+	char line[NNTP_STRLEN];
+
+	snprintf(fmt, sizeof(fmt), "%%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%%ds", NNTP_GRPLEN);
 #endif /* NNTP_ABLE */
 
 	rewind(fp);
@@ -385,11 +392,8 @@ do_read_newsrc_active_file(
 			ptr = ngname;
 		}
 
-		if (read_news_via_nntp && !read_saved_news) {
+		if (read_news_via_nntp && !read_saved_news) { /* this should be limited to GROUP_TYPE_NEWS, but ... */
 #ifdef NNTP_ABLE
-			char buf[NNTP_STRLEN];
-			char line[NNTP_STRLEN];
-
 			if (window < NUM_SIMULTANEOUS_GROUP_COMMAND && ptr && (!list_active || (newsrc_active && list_active && group_find(ptr, FALSE)))) {
 				ngnames[index_i] = my_strdup(ptr);
 				snprintf(buf, sizeof(buf), "GROUP %s", ngnames[index_i]);
@@ -397,12 +401,12 @@ do_read_newsrc_active_file(
 				if ((debug & DEBUG_NNTP) && verbose > 1)
 					debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
-				put_server(buf);
+				put_server(buf, FALSE);
 				index_i = (index_i + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 				window++;
 			}
 			if (window == NUM_SIMULTANEOUS_GROUP_COMMAND || ptr == NULL) {
-				int respcode = get_only_respcode(line, sizeof(line));
+				respcode = get_only_respcode(line, sizeof(line));
 
 				if (reconnected_in_last_get_server) {
 					/*
@@ -411,8 +415,7 @@ do_read_newsrc_active_file(
 					 * We resend all buffered command except for last window_i.
 					 * And rotate buffer to use data received.
 					 */
-					int i;
-					int j = index_o;
+					j = index_o;
 
 					for (i = 0; i < window - 1; i++) {
 						snprintf(buf, sizeof(buf), "GROUP %s", ngnames[j]);
@@ -420,7 +423,7 @@ do_read_newsrc_active_file(
 						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "read_newsrc_active_file() %s", buf);
 #	endif /* DEBUG */
-						put_server(buf);
+						put_server(buf, FALSE);
 						j = (j + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
 					}
 					if (--index_o < 0)
@@ -432,12 +435,8 @@ do_read_newsrc_active_file(
 				}
 
 				switch (respcode) {
-
 					case OK_GROUP:
 						{
-							char fmt[25];
-
-							snprintf(fmt, sizeof(fmt), "%%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%"T_ARTNUM_SFMT" %%%ds", NNTP_GRPLEN);
 							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4) {
 #	ifdef DEBUG
 								if ((debug & DEBUG_NNTP) && verbose > 1) {
@@ -467,6 +466,9 @@ do_read_newsrc_active_file(
 						continue;
 
 					case ERR_ACCESS:
+						for (index_i = 0; index_i < NUM_SIMULTANEOUS_GROUP_COMMAND - 1; index_i++) {
+							FreeIfNeeded(ngnames[index_i]);
+						}
 						tin_done(NNTP_ERROR_EXIT, "%s", line);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
@@ -500,7 +502,7 @@ do_read_newsrc_active_file(
 		 * This call may implicitly ++num_active
 		 */
 		if ((grpptr = group_add(ptr)) == NULL) {
-			t_bool changed = FALSE;
+			changed = FALSE;
 
 			if ((grpptr = group_find(ptr, FALSE)) == NULL)
 				continue;
@@ -548,31 +550,28 @@ read_newsrc_active_file(
 	 * return immediately if no .newsrc can be found or .newsrc is empty
 	 * when function asked to use .newsrc
 	 */
-	if ((fp = fopen(newsrc, "r")) == NULL)
+	if ((fp = tin_fopen(newsrc, "r")) == NULL)
 		return;
 
-	if (file_size(newsrc) <= 0L) {
-		fclose(fp);
-		return;
-	}
-
-#ifdef NNTP_ABLE
-	need_auth = do_read_newsrc_active_file(fp);
-#else
+#ifndef NNTP_ABLE
 	do_read_newsrc_active_file(fp);
-#endif /* NNTP_ABLE */
+#else
+	need_auth = do_read_newsrc_active_file(fp);
 
-#ifdef NNTP_ABLE
 	if (need_auth) { /* delayed auth */
-		if (!authenticate(nntp_server, userid, FALSE))
+		if (!authenticate(nntp_server, userid, FALSE)) {
+			fclose(fp);
 			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
+		}
 #	if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
 		set_maxartnum(FALSE);
 #	endif /* MAXARTNUM && USE_LONG_ARTICLE_NUMBERS */
-		if (do_read_newsrc_active_file(fp))
+		if (do_read_newsrc_active_file(fp)) {
+			fclose(fp);
 			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
+		}
 	}
-#endif /* NNTP_ABLE */
+#endif /* !NNTP_ABLE */
 
 	fclose(fp);
 
@@ -636,10 +635,11 @@ read_active_file(
 			tin_done(EXIT_FAILURE, _(txt_cannot_open_active_file), news_active_file, tin_progname);
 		}
 #	endif /* !NNTP_ONLY */
-#else
-		perror_message("%s", news_active_file);
-		tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
+		{
+			perror_message("%s", news_active_file);
+			tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
+		}
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -800,16 +800,11 @@ read_news_active_file(
 	 * Ignore -n if no .newsrc can be found or .newsrc is empty
 	 */
 	if (newsrc_active) {
-		if ((fp = fopen(newsrc, "r")) == NULL) {
+		if ((fp = tin_fopen(newsrc, "r")) == NULL) {
 			list_active = TRUE;
 			newsrc_active = FALSE;
-		} else {
+		} else
 			fclose(fp);
-			if (file_size(newsrc) <= 0L) {
-				list_active = TRUE;
-				newsrc_active = FALSE;
-			}
-		}
 	}
 
 	/* Read an active file if it is allowed */
@@ -846,7 +841,7 @@ read_news_active_file(
 
 			*buff = '\0';
 			/* we can't use for_each_group(i) yet, so we have to parse the newsrc */
-			if ((fp = fopen(newsrc, "r")) != NULL) {
+			if ((fp = tin_fopen(newsrc, "r")) != NULL) {
 				while (tin_fgets(fp, FALSE) != NULL)
 					j++;
 				rewind(fp);
@@ -861,7 +856,7 @@ read_news_active_file(
 								snprintf(buff + strlen(buff), sizeof(buff) - strlen(buff), ",%s", ptr);
 							} else {
 								if (*buff) {
-									put_server(buff);
+									put_server(buff, FALSE);
 									r++;
 								}
 								snprintf(buff, sizeof(buff), "LIST %s %s", nntp_caps.list_counts ? "COUNTS" : "ACTIVE", ptr);
@@ -869,12 +864,12 @@ read_news_active_file(
 							continue;
 						} else
 							snprintf(buff, sizeof(buff), "LIST ACTIVE %s", ptr);
-						put_server(buff);
+						put_server(buff, FALSE);
 						r++;
 						*buff = '\0';
 					}
 					if (*buff) {
-						put_server(buff);
+						put_server(buff, FALSE);
 						r++;
 					}
 				} else
@@ -1170,6 +1165,7 @@ match_group_list(
 {
 	char *separator;
 	char pattern[HEADER_LEN];
+	char ngname[NNTP_GRPLEN + 1]; /* RFC 3977 3.1 limits group names to 497 octets */
 	size_t group_len, list_len;
 	t_bool negate, accept = FALSE;
 
@@ -1195,12 +1191,15 @@ match_group_list(
 		/*
 		 * copy out the entry and terminate it properly
 		 */
-		strncpy(pattern, group_list, group_len);
-		pattern[group_len] = '\0';
+		my_strncpy(pattern, group_list, group_len);
+		my_strncpy(ngname, group, sizeof(ngname) - 1);
+		str_lwr(pattern);
+		str_lwr(ngname);
+
 		/*
-		 * case-insensitive wildcard match
+		 * "case-insensitive" (str_lwr(); avoid malloc()/free() in) wildcard match
 		 */
-		if (GROUP_MATCH(group, pattern, TRUE))
+		if (GROUP_MATCH(group, pattern, FALSE))
 			accept = bool_not(negate);	/* matched! */
 
 		/*
@@ -1425,11 +1424,7 @@ append_group_line(
 				clearerr(fp);
 				fclose(fp);
 			}
-			err = rename(file_tmp, active_file);
-#ifdef DEBUG
-			if ((debug & DEBUG_MISC) && err) /* TODO: is this the right debug-level? */
-				perror_message(_(txt_rename_error), file_tmp, active_file);
-#endif /* DEBUG */
+			rename_file(file_tmp, active_file);
 		}
 		free(group_name);
 	}
