@@ -3,10 +3,10 @@
  *  Module    : art.c
  *  Author    : I.Lea & R.Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-07-26
+ *  Updated   : 2024-07-28
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,6 @@ int top_art = 0;				/* # of articles in arts[] */
 static FILE *open_art_header(char *groupname, t_artnum art, t_artnum *next);
 static FILE *open_xover_fp(struct t_group *group, const char *mode, t_artnum min, t_artnum max, t_bool local);
 static char *find_nov_file(struct t_group *group, int mode);
-static char *print_date(time_t secs);
 static char *print_from(struct t_group *group, struct t_article *article, int charset);
 static int artnum_comp(t_comptype p1, t_comptype p2);
 static int base_comp(t_comptype p1, t_comptype p2);
@@ -90,7 +89,8 @@ static t_artnum find_first_unread(struct t_group *group);
 static t_artnum setup_hard_base(struct t_group *group);
 static t_bool parse_headers(FILE *fp, struct t_article *h);
 static t_compfunc eval_sort_arts_func(unsigned int sort_art_type);
-static time_t get_last_posting_date(long n);
+static time_t get_last_posting_date(int n);
+static void build_mailbox_list(struct t_article *art, char *hdr);
 static void sort_base(unsigned int sort_threads_type);
 static void thread_by_multipart(void);
 static void thread_by_percentage(unsigned int percentage);
@@ -100,6 +100,7 @@ static void write_overview(struct t_group *group);
 	static struct t_article_range *build_range_list(t_artnum min, t_artnum max, int *range_cnt);
 	static t_bool get_path_header(int cur, int cnt, struct t_group *group, t_artnum min, t_artnum max);
 #endif /* NNTP_ABLE */
+static struct t_mailbox *add_mailbox(struct t_article *article);
 
 
 /*
@@ -295,7 +296,7 @@ setup_hard_base(
 #	ifdef DEBUG
 				/* log end of multiline response to get timing data */
 				if ((debug & DEBUG_NNTP) && !verbose)
-					debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+					debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
 #	endif /* DEBUG */
 
 				if (tin_errno)
@@ -419,8 +420,8 @@ index_group(
 	if (!batch_mode)
 		show_art_msg(group->name);
 	else {
-		if (verbose) /* -> lang.c */
-			wait_message(0, _("Reading %s\n"), group->name);
+		if (verbose > 1)
+			wait_message(0, _(txt_reading_group), group->name);
 	}
 
 	signal_context = cArt;			/* Set this only once curr_group is valid */
@@ -717,6 +718,13 @@ open_art_header(
 		}
 		return NULL;
 	}
+	/* silence compiler warning (unused parameter) */
+#	ifdef BROKEN_LISTGROUP
+	(void) groupname;
+#	endif /* BROKEN_LISTGROUP */
+#else
+	(void) groupname;
+	(void) next;
 #endif /* NNTP_ABLE */
 
 	snprintf(buf, sizeof(buf), "%"T_ARTNUM_PFMT, art);
@@ -823,6 +831,7 @@ read_art_headers(
 			FreeAndNull(arts[top_art].path);
 			FreeAndNull(arts[top_art].refs);
 			FreeAndNull(arts[top_art].msgid);
+			free_mailbox_list(arts[top_art].mailbox.next);
 			arts[top_art].tagged = 0;
 			arts[top_art].thread = ART_EXPIRED;
 			arts[top_art].prev = ART_NORMAL;
@@ -845,9 +854,8 @@ read_art_headers(
 	if (!read_news_via_nntp || group->type != GROUP_TYPE_NEWS) {
 		if (chdir(dir) == -1) {
 #ifdef DEBUG
-			int e = errno;
 			if (debug & DEBUG_MISC)
-				error_message(2, "chdir(%s): Error: %s", dir, strerror(e));
+				perror_message("chdir(%s)", dir);
 #endif /* DEBUG */
 		}
 	}
@@ -879,7 +887,7 @@ thread_by_subject(
 		/*
 		 * Get the contents of the magic marker in the hashnode
 		 */
-		h = (struct t_hashnode *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
+		h = (void *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
 
 		j = h->aptr;
 
@@ -1034,7 +1042,7 @@ global_look_for_multipart_info(
 	/* parse the message */
 	subj = arts[aindex].subject;
 	pch = strrchr(subj, start);
-	if (!pch || !isdigit((int) pch[1]))
+	if (!pch || !isdigit((unsigned char) pch[1]))
 		return 0;
 
 	tmp.arts_index = aindex;
@@ -1043,7 +1051,7 @@ global_look_for_multipart_info(
 	if (*pch != '/' && *pch != '|')
 		return 0;
 
-	if (!isdigit((int) pch[1]))
+	if (!isdigit((unsigned char) pch[1]))
 		return 0;
 
 	tmp.total = (int) strtol(pch + 1, &pch, 10);
@@ -1072,14 +1080,14 @@ global_look_for_multipart(
 	char *pch;
 
 	pch = strrchr(arts[aindex].subject, start);
-	if (!pch || !isdigit((int) pch[1]))
+	if (!pch || !isdigit((unsigned char) pch[1]))
 		return FALSE;
 
 	strtol(pch + 1, &pch, 10);
 	if (*pch != '/' && *pch != '|')
 		return FALSE;
 
-	if (!isdigit((int) pch[1]))
+	if (!isdigit((unsigned char) pch[1]))
 		return FALSE;
 
 	strtol(pch + 1, &pch, 10);
@@ -1213,8 +1221,8 @@ thread_by_multipart(
 		}
 		FreeAndNull(minfo);
 		arts[i].multipart_subj = FALSE;
-		if (i % MODULO_COUNT_NUM == 0) /* TODO: -> lang.c */
-			show_progress(_("Threading by multipart"), i, top_art);
+		if (i % MODULO_COUNT_NUM == 0)
+			show_progress(_(txt_threading_by_multipart), i, top_art);
 	}
 }
 
@@ -1292,7 +1300,7 @@ make_threads(
 					my_fprintf(stderr, "\nError  : art->refptr is NULL\n");
 					my_fprintf(stderr, "Artnum : %"T_ARTNUM_PFMT"\n", arts[i].artnum);
 					my_fprintf(stderr, "Subject: %s\n", arts[i].subject);
-					my_fprintf(stderr, "From   : %s\n", arts[i].from);
+					my_fprintf(stderr, "From   : %s\n", arts[i].mailbox.from);
 					assert(arts[i].refptr != NULL);
 				} else
 #endif /* DEBUG */
@@ -1414,6 +1422,13 @@ sort_base(
 		case SORT_THREADS_BY_LAST_POSTING_DATE_ASCEND:
 			tin_sort(base, (size_t) grpmenu.max, sizeof(t_artnum), last_date_comp_base_asc);
 			break;
+
+		/* should not happen */
+		case SORT_THREADS_BY_NOTHING:
+		default:
+			/* CONSTANTCONDITION */
+			assert(0 != 0);
+			break;
 	}
 }
 
@@ -1427,8 +1442,6 @@ parse_headers(
 	FILE *fp,
 	struct t_article *h)
 {
-	char art_from_addr[HEADER_LEN];
-	char art_full_name[HEADER_LEN];
 	char *s, *hdr, *ptr;
 	t_bool got_from, got_lines;
 
@@ -1450,18 +1463,23 @@ parse_headers(
 		switch (my_toupper((unsigned char) *ptr)) {
 			case 'D':	/* Date:  mandatory */
 				if (!h->date) {
-					if ((hdr = parse_header(ptr + 1, "ate", FALSE, FALSE, FALSE)))
-						h->date = parsedate(hdr, (struct _TIMEINFO *) 0);
+					if ((hdr = parse_header(ptr + 1, "ate", FALSE, FALSE, FALSE))) {
+						str_trim(hdr);
+						if ((h->date = parsedate(hdr, (struct _TIMEINFO *) 0)) <= 0) {
+							/* date parsing failed, cut off at last ' ' and try again */
+							if ((s = strrchr(hdr, ' ')) != NULL) {
+								*s = '\0';
+								h->date = parsedate(hdr, (struct _TIMEINFO *) 0);
+							}
+						}
+					}
 				}
 				break;
 
 			case 'F':	/* From:  mandatory */
 				if (!got_from) {
 					if ((hdr = parse_header(ptr + 1, "rom", FALSE, FALSE, FALSE))) {
-						h->gnksa_code = parse_from(hdr, art_from_addr, art_full_name);
-						h->from = hash_str(buffer_to_ascii(art_from_addr));
-						if (*art_full_name)
-							h->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name), FALSE)));
+						build_mailbox_list(h, hdr);
 						got_from = TRUE;
 					}
 				}
@@ -1470,13 +1488,13 @@ parse_headers(
 			case 'L':	/* Lines:  optional */
 				if (!got_lines) {
 					if ((hdr = parse_header(ptr + 1, "ines", FALSE, FALSE, FALSE))) {
-						h->line_count = atoi(hdr);
+						h->line_count = s2i(hdr, 0, INT_MAX);
 						got_lines = TRUE;
 					}
 				}
 				break;
 
-			case 'M':	/* Message-ID:  mandatory */
+			case 'M':	/* Message-ID:  mandatory; be aware that build_references() later on clears it! */
 				if (!h->msgid) {
 					if ((hdr = parse_header(ptr + 1, "essage-ID", FALSE, FALSE, FALSE)))
 						h->msgid = my_strdup(hdr);
@@ -1556,6 +1574,7 @@ parse_headers(
 	return FALSE;
 }
 
+
 #ifdef NNTP_ABLE
 /*
  * Loop over arts[] and find ranges without Path: header
@@ -1781,7 +1800,7 @@ get_path_header(
 	}
 
 	if (fp) {
-		int j = 0;
+		t_artnum j = 0;
 
 		prep_msg = fmt_string(_(txt_prep_for_filter_on_path), cur, cnt);
 		while ((buf = tin_fgets(fp, FALSE)) != NULL && buf[0] != '.') {
@@ -1798,7 +1817,7 @@ get_path_header(
 				if (arts[i].artnum == artnum) {
 					FreeIfNeeded(arts[i].path);
 					arts[i].path = my_strdup(ptr);
-					j = (int) i;
+					j = i;
 					break;
 				}
 			}
@@ -1808,7 +1827,7 @@ get_path_header(
 #	ifdef DEBUG
 		/* log end of multiline response to get timing data */
 		if ((debug & DEBUG_NNTP) && !verbose)
-			debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+			debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
 #	endif /* DEBUG */
 		free(prep_msg);
 		return supported;
@@ -1829,6 +1848,79 @@ get_path_header(
 #endif /* NNTP_ABLE */
 
 
+static struct t_mailbox *
+add_mailbox(
+	struct t_article *art)
+{
+	struct t_mailbox *mb;
+
+	if (!art)
+		return NULL;
+
+	if (!art->mailbox.from)
+		return &art->mailbox;
+
+	if (!(mb = art->mailbox.next)) {
+		art->mailbox.next = my_malloc(sizeof(struct t_mailbox));
+		mb = art->mailbox.next;
+	} else {
+		while (mb->next)
+			mb = mb->next;
+		mb->next = my_malloc(sizeof(struct t_mailbox));
+		mb = mb->next;
+	}
+
+	mb->from = NULL;
+	mb->name = NULL;
+	mb->next = NULL;
+
+	return mb;
+}
+
+
+static void
+build_mailbox_list(
+	struct t_article *art,
+	char *hdr)
+{
+	char art_from_addr[HEADER_LEN];
+	char art_full_name[HEADER_LEN];
+	char *tmp_from, *curr_from, *next_from;
+	struct t_mailbox *mb;
+
+	tmp_from = my_strdup(hdr);
+	curr_from = tmp_from;
+
+	do {
+		next_from = split_mailbox_list(curr_from);
+		mb = add_mailbox(art);
+		mb->gnksa_code = parse_from(curr_from, art_from_addr, art_full_name);
+		mb->from = hash_str(buffer_to_ascii(art_from_addr));
+		if (*art_full_name)
+			mb->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name), FALSE)));
+		curr_from = next_from;
+	} while (curr_from);
+
+	free(tmp_from);
+}
+
+
+void
+free_mailbox_list(
+	struct t_mailbox *mb)
+{
+	if (!mb)
+		return;
+
+	while (mb->next != NULL) {
+		free_mailbox_list(mb->next);
+		mb->next = NULL;
+	}
+
+	free(mb);
+}
+
+
 /*
  * Read in an overview index file. Fields are separated by TAB.
  * return the number of expired articles encountered or -1 if the user aborted
@@ -1862,8 +1954,6 @@ read_overview(
 	char *q;
 	char *buf;
 	char *group_msg;
-	char art_full_name[HEADER_LEN];
-	char art_from_addr[HEADER_LEN];
 	unsigned int count;
 	int expired = 0;
 	t_artnum artnum;
@@ -1914,10 +2004,10 @@ read_overview(
 	}
 
 	while ((buf = tin_fgets(fp, FALSE)) != NULL) {
-#ifdef DEBUG
+#if defined(DEBUG) && defined(NNTP_ABLE)
 		if ((debug & DEBUG_NNTP) && fp == FAKE_NNTP_FP && verbose)
 			debug_print_file("NNTP", "<<<%s%s", logtime(), buf);
-#endif /* DEBUG */
+#endif /* DEBUG && NNTP_ABLE */
 
 		if (need_resize) {
 			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
@@ -2043,13 +2133,13 @@ read_overview(
 					}
 
 					if (!strcasecmp(ofmt[count].name, "From:")) {
-						if (*ptr) {
-							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
-							art->from = hash_str(buffer_to_ascii(art_from_addr));
-							if (*art_full_name)
-								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name), FALSE)));
-						} else {
-							art->from = hash_str("");
+
+						if (*ptr)
+							build_mailbox_list(art, ptr);
+						else {
+							struct t_mailbox *mb = add_mailbox(art);
+
+							mb->from = hash_str("");
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2059,11 +2149,18 @@ read_overview(
 					}
 
 					if (!strcasecmp(ofmt[count].name, "Date:")) {
-						art->date = parsedate(ptr, (TIMEINFO *) 0);
+						str_trim(ptr);
+						if ((art->date = parsedate(ptr, (TIMEINFO *) 0)) <= 0) {
 #ifdef DEBUG
-						if ((debug & DEBUG_NNTP) && verbose > 1 && art->date == (time_t) -1)
-							debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") bogus overview-field %s %s", nntp_caps.over_cmd, artnum, ofmt[count].name, ptr);
+							if ((debug & DEBUG_NNTP) && verbose > 1)
+								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") bogus overview-field %s %s", nntp_caps.over_cmd, artnum, ofmt[count].name, ptr);
 #endif /* DEBUG */
+							/* date parsing failed, cut off at last ' ' and try again */
+							if ((q = strrchr(ptr, ' ')) != NULL) {
+								*q = '\0';
+								art->date = parsedate(ptr, (TIMEINFO *) 0);
+							}
+						}
 						continue;
 					}
 
@@ -2123,7 +2220,7 @@ read_overview(
 					if (!strcasecmp(ofmt[count].name, "Lines:")) {
 						if (*ptr) {
 							if (isdigit((unsigned char) *ptr))
-								art->line_count = atoi(ptr);
+								art->line_count = s2i(ptr, 0, INT_MAX);
 							else {
 								art->line_count = 0;
 #ifdef DEBUG
@@ -2159,13 +2256,12 @@ read_overview(
 						break;
 
 					case 2:	/* From: */
-						if (*ptr) {
-							art->gnksa_code = parse_from(ptr, art_from_addr, art_full_name);
-							art->from = hash_str(buffer_to_ascii(art_from_addr));
-							if (*art_full_name)
-								art->name = hash_str(eat_tab(convert_to_printable(rfc1522_decode(art_full_name), FALSE)));
-						} else {
-							art->from = hash_str("");
+						if (*ptr)
+							build_mailbox_list(art, ptr);
+						else {
+							struct t_mailbox *mb = add_mailbox(art);
+
+							mb->from = hash_str("");
 #ifdef DEBUG
 							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") empty overview-field %s", nntp_caps.over_cmd, artnum, ofmt[count].name);
@@ -2174,11 +2270,18 @@ read_overview(
 						break;
 
 					case 3:	/* Date: */
-						art->date = parsedate(ptr, (TIMEINFO *) 0);
+						str_trim(ptr);
+						if ((art->date = parsedate(ptr, (TIMEINFO *) 0)) <= 0) {
 #ifdef DEBUG
-						if ((debug & DEBUG_NNTP) && verbose > 1 && art->date == (time_t) -1)
-							debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") bogus overview-field %s %s", nntp_caps.over_cmd, artnum, ofmt[count].name, ptr);
+							if ((debug & DEBUG_NNTP) && verbose > 1)
+								debug_print_file("NNTP", "%s(%"T_ARTNUM_PFMT") bogus overview-field %s %s", nntp_caps.over_cmd, artnum, ofmt[count].name, ptr);
 #endif /* DEBUG */
+							/* date parsing failed, cut off at last ' ' and try again */
+							if ((q = strrchr(ptr, ' ')) != NULL) {
+								*q = '\0';
+								art->date = parsedate(ptr, (TIMEINFO *) 0);
+							}
+						}
 						break;
 
 					case 4:	/* Message-ID: */
@@ -2212,7 +2315,7 @@ read_overview(
 					case 7:	/* :lines || Lines: */
 						if (*ptr) {
 							if (isdigit((unsigned char) *ptr))
-								art->line_count = atoi(ptr);
+								art->line_count = s2i(ptr, 0, INT_MAX);
 							else {
 								art->line_count = 0;
 #ifdef DEBUG
@@ -2291,11 +2394,11 @@ read_overview(
 
 		top_art++;				/* Basically this statement commits the article */
 	}
-#	ifdef DEBUG
+#	if defined(DEBUG) && defined(NNTP_ABLE)
 	/* log end of multiline response to get timing data */
 	if ((debug & DEBUG_NNTP) && !verbose)
-		debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
-#	endif /* DEBUG */
+		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
+#	endif /* DEBUG && NNTP_ABLE */
 
 	free(group_msg);
 	TIN_FCLOSE(fp);
@@ -2353,8 +2456,9 @@ read_overview(
 
 		if (found) {
 			snprintf(cbuf, sizeof(cbuf), "%s XREF %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, min, MAX(min, max));
-			group_msg = fmt_string("%s XREF loop", nntp_caps.hdr_cmd); /* TODO: find a better message, move to lang.c */
+			group_msg = fmt_string(txt_xref_loop, nntp_caps.hdr_cmd); /* TODO: find a better message */
 			if ((fp = nntp_command(cbuf, nntp_caps.hdr ? OK_HDR : OK_HEAD, NULL, 0)) != NULL) { /* RFC 2980 (XHDR) uses 221; RFC 3977 (HDR) uses 225 */
+				top_art = 0;
 				while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
 #	ifdef DEBUG
 					if ((debug & DEBUG_NNTP) && verbose)
@@ -2365,26 +2469,29 @@ read_overview(
 					if (artnum <= 0 || artnum < group->xmin || artnum > group->xmax)
 						continue;
 					art = &arts[top_art];
-					set_article(art);
-					if (!art->xref && !strstr(ptr, "(none)")) {
+					if (artnum != art->artnum) /* try harder to find a match? while (&arts[i++].artnum != artnum) ...? */
+						continue;
+					FreeAndNull(art->xref);
+					if (!strstr(ptr, "(none)")) {
 						if ((q = strchr(ptr, ' ')) == NULL) /* skip article number */
 							continue;
 						ptr = q;
-						while (*ptr && isspace((int) *ptr))
+						while (*ptr && isspace((unsigned char) *ptr))
 							ptr++;
-						q = strchr(ptr, '\n');
-						if (q)
+						if ((q = strchr(ptr, '\n')) != NULL)
 							*q = '\0';
 						art->xref = my_strdup(ptr);
 					}
 					/* we might lose accuracy here, but that shouldn't hurt */
 					if (artnum % (MODULO_COUNT_NUM * 20) == 0)
 						show_progress(group_msg, artnum - min, max - min);
+
+					top_art++;
 				}
 #	ifdef DEBUG
 				/* log end of multiline response to get timing data */
 				if ((debug & DEBUG_NNTP) && !verbose)
-					debug_print_file("NNTP", "<<<%s%s", logtime(), ". [full data hidden, rerun with -v]");
+					debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
 #	endif /* DEBUG */
 			}
 			free(group_msg);
@@ -2395,9 +2502,9 @@ read_overview(
 	if (local) {
 #ifdef NNTP_ABLE
 		if (filter_on_path(group)) {
+			int curr_range, range_cnt;
 			struct t_article_range *ranges, *curr;
 			t_bool supported = TRUE;
-			int curr_range, range_cnt;
 
 			/*
 			 * Get the ranges without Path: header and try to fetch the
@@ -2433,12 +2540,16 @@ read_overview(
 		}
 #endif /* NNTP_ABLE */
 	} else
-		if (!path_found && filter_on_path(group)) {
+		if (!path_found && filter_on_path(group) && !batch_mode) {
 #ifdef NNTP_ABLE
 			if (!get_path_header(1, 1, group, min, *top))
 #endif /* NNTP_ABLE */
 				wait_message(2, _(txt_cannot_filter_on_path));
 		}
+#ifndef NNTP_ABLE
+	/* silence compiler warning (unused parameter) */
+	(void) rebuild_cache;
+#endif /* !NNTP_ABLE */
 	return expired;
 }
 
@@ -2515,12 +2626,11 @@ write_overview(
 	}
 #endif /* CHARSET_CONVERSION */
 
-	if (verbose && batch_mode) /* -> lang.c */
-		wait_message(0, _("Writing %s\n"), group->name);
+	if (batch_mode && verbose > 1)
+		wait_message(0, _(txt_writing_group), group->name);
 
 	for_each_art(i) {
-		char *p;
-		char *q, *ref;
+		char *p, *q, *ref;
 
 		article = &arts[i];
 
@@ -2568,19 +2678,48 @@ write_overview(
 				}
 			}
 
-			fprintf(fp, "%"T_ARTNUM_PFMT"\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
-				article->artnum,
-				p,
+			{
+				char date[30];
+#if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
+				char *old_lc_all = NULL, *old_lc_time = NULL;
+
+				/* Unlocalized date-header */
+				if (getenv("LC_ALL") != NULL) {
+					old_lc_all = my_strdup(setlocale(LC_ALL, NULL));
+					setlocale(LC_ALL, "POSIX");
+				} else {
+					old_lc_time = my_strdup(setlocale(LC_TIME, NULL));
+					setlocale(LC_TIME, "POSIX");
+				}
+#endif /* HAVE_SETLOCALE && !NO_LOCALE */
+
+				if (!my_strftime(date, sizeof(date) - 1, "%d %b %Y %H:%M:%S GMT", gmtime(&article->date)))
+					snprintf(date, sizeof(date) - 1, "01 Jan 1970 00:00:00 UTC");
+
+				fprintf(fp, "%"T_ARTNUM_PFMT"\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
+					article->artnum,
+					p,
 #ifdef CHARSET_CONVERSION
-				print_from(group, article, c),
+					print_from(group, article, c),
 #else
-				print_from(group, article, -1),
+					print_from(group, article, -1),
 #endif /* CHARSET_CONVERSION */
-				print_date(article->date),
-				BlankIfNull(article->msgid),
-				BlankIfNull(ref),
-				0,	/* bytes */
-				article->line_count);
+					date,
+					BlankIfNull(article->msgid),
+					BlankIfNull(ref),
+					0,	/* bytes */
+					article->line_count);
+#if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
+				/* change back LC_* */
+				if (old_lc_all != NULL) {
+					setlocale(LC_ALL, old_lc_all);
+					free(old_lc_all);
+				} else if (old_lc_time != NULL) {
+					setlocale(LC_TIME, old_lc_time);
+					free(old_lc_time);
+				}
+#endif /* HAVE_SETLOCALE && !NO_LOCALE */
+			}
 
 			if (article->xref)
 				fprintf(fp, "\tXref: %s", article->xref);
@@ -2589,19 +2728,20 @@ write_overview(
 				fprintf(fp, "\tPath: %s", article->path);
 
 			fprintf(fp, "\n");
+
 			free(p);
 			if (article->refs) {
 				FreeIfNeeded(ref);
 			}
 		}
-		if (i % (MODULO_COUNT_NUM * 20) == 0) /* TODO: -> lang.c */
-			show_progress(_("Writing overview cache..."), i, top_art);
+		if (i % (MODULO_COUNT_NUM * 20) == 0)
+			show_progress(_(txt_writing_overview), i, top_art);
 	}
 #ifdef HAVE_FCHMOD
 	fchmod(fileno(fp), (mode_t) (S_IWUSR|S_IRUGO));
 #else
 #	ifdef HAVE_CHMOD
-		chmod(find_nov_file(group,R_OK), (mode_t) (S_IWUSR|S_IRUGO));
+		chmod(find_nov_file(group, R_OK), (mode_t) (S_IWUSR|S_IRUGO));
 #	endif /* HAVE_CHMOD */
 #endif /* HAVE_FCHMOD */
 	fclose(fp);
@@ -2782,7 +2922,7 @@ find_nov_file(
 		snprintf(buf, sizeof(buf), "%lu.%d", hash, i);
 		joinpath(nov_file, sizeof(nov_file), dir, buf);
 
-		if ((fp = fopen(nov_file, "r")) == NULL)
+		if ((fp = tin_fopen(nov_file, "r")) == NULL)
 			break;
 
 		/*
@@ -2797,7 +2937,7 @@ find_nov_file(
 		if ((ptr = strrchr(buf, '\n')) != NULL)
 			*ptr = '\0';
 
-		if (strcmp(buf, group->name) == 0)
+		if (STRCMPEQ(buf, group->name))
 			break;
 	}
 
@@ -2937,7 +3077,7 @@ from_comp_asc(
 	const struct t_article *s1 = (const struct t_article *) p1;
 	const struct t_article *s2 = (const struct t_article *) p2;
 
-	if ((retval = strcasecmp(s1->from, s2->from))) /* != 0 */
+	if ((retval = strcasecmp(s1->mailbox.from, s2->mailbox.from))) /* != 0 */
 		return retval;
 
 	return s1->date - s2->date > 0 ? 1 : -1;
@@ -2953,7 +3093,7 @@ from_comp_desc(
 	const struct t_article *s1 = (const struct t_article *) p1;
 	const struct t_article *s2 = (const struct t_article *) p2;
 
-	if ((retval = strcasecmp(s2->from, s1->from))) /* != 0 */
+	if ((retval = strcasecmp(s2->mailbox.from, s1->mailbox.from))) /* != 0 */
 		return retval;
 
 	return s1->date - s2->date > 0 ? 1 : -1;
@@ -3103,15 +3243,15 @@ score_comp_base(
 	t_comptype p1,
 	t_comptype p2)
 {
-	int a = get_score_of_thread((int) *(const long *) p1);
-	int b = get_score_of_thread((int) *(const long *) p2);
+	int a = get_score_of_thread((int) *(const t_artnum *) p1);
+	int b = get_score_of_thread((int) *(const t_artnum *) p2);
 
 	/* If scores are equal, compare using the article sort order.
 	 * This determines the order in a group of equally scored threads.
 	 */
 	if (a == b) {
-		const struct t_article *s1 = &arts[*(const long *) p1];
-		const struct t_article *s2 = &arts[*(const long *) p2];
+		t_comptype s1 = &arts[*(const t_artnum *) p1];
+		t_comptype s2 = &arts[*(const t_artnum *) p2];
 		t_compfunc comp_func = eval_sort_arts_func(CURR_GROUP.attribute->sort_article_type);
 
 		if (comp_func)
@@ -3136,8 +3276,8 @@ last_date_comp_base_desc(
 	t_comptype p1,
 	t_comptype p2)
 {
-	time_t s1_last = get_last_posting_date(*(const long *) p1);
-	time_t s2_last = get_last_posting_date(*(const long *) p2);
+	time_t s1_last = get_last_posting_date(*(const t_artnum *) p1);
+	time_t s2_last = get_last_posting_date(*(const t_artnum *) p2);
 
 	if (s2_last < s1_last)
 		return -1;
@@ -3154,8 +3294,8 @@ last_date_comp_base_asc(
 	t_comptype p1,
 	t_comptype p2)
 {
-	time_t s1_last = get_last_posting_date(*(const long *) p1);
-	time_t s2_last = get_last_posting_date(*(const long *) p2);
+	time_t s1_last = get_last_posting_date(*(const t_artnum *) p1);
+	time_t s2_last = get_last_posting_date(*(const t_artnum *) p2);
 
 	if (s2_last > s1_last)
 		return -1;
@@ -3169,9 +3309,9 @@ last_date_comp_base_asc(
 
 static time_t
 get_last_posting_date(
-	long n)
+	int n)
 {
-	long i;
+	int i;
 	time_t last = (time_t) 0;
 
 	for (i = n; i >= 0; i = arts[i].thread) {
@@ -3188,13 +3328,14 @@ set_article(
 	struct t_article *art)
 {
 	art->subject = NULL;
-	art->from = NULL;
-	art->name = NULL;
 	art->date = (time_t) 0;
 	art->xref = NULL;
 	art->msgid = NULL;
 	art->refs = NULL;
 	art->refptr = NULL;
+	art->mailbox.from = NULL;
+	art->mailbox.name = NULL;
+	art->mailbox.next = NULL;
 	art->line_count = -1;
 	art->tagged = 0;
 	art->thread = ART_EXPIRED;
@@ -3268,36 +3409,6 @@ find_artnum(
 }
 
 
-/*----------------------------- Overview handling -----------------------*/
-/* TODO: use
- *           setlocale(LC_ALL, "POSIX"); setlocale(LC_TIME, "POSIX");
- *           my_strftime(date, sizeof(date) -1, "%d %b %Y %H:%M:%S GMT", gmtime(&secs));
- *       instead?
- */
-static char *
-print_date(
-	time_t secs)
-{
-	static char date[25];
-	struct tm *tm;
-	static const char *const months_a[] = {
-		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
-		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
-	};
-
-	if ((tm = gmtime(&secs)) != NULL)
-		snprintf(date, sizeof(date), "%02d %.3s %04d %02d:%02d:%02d GMT",
-			tm->tm_mday,
-			months_a[tm->tm_mon],
-			tm->tm_year + 1900,
-			tm->tm_hour, tm->tm_min, tm->tm_sec);
-	else
-		snprintf(date, sizeof(date), "01 Jan 1970 00:00:00 UTC");
-
-	return date;
-}
-
-
 static char *
 print_from(
 	struct t_group *group,
@@ -3306,26 +3417,41 @@ print_from(
 {
 	char *p, *q;
 	static char from[PATH_LEN];
+	char single_from[PATH_LEN];
+	struct t_mailbox *mb = &article->mailbox;
+	int c_needed = 0;
 
-	*from = '\0';
+	*from = *single_from = '\0';
 
-	if (article->name != NULL) {
-		q = my_strdup(article->name);
+	do {
+		if (mb->name != NULL) {
+			q = my_strdup(mb->name);
 #ifdef CHARSET_CONVERSION
-		if (charset != -1)
-			buffer_to_network(q, charset);
+			if (charset != -1)
+				buffer_to_network(q, charset);
+#else
+			(void) charset;
 #endif /* CHARSET_CONVERSION */
-		p = rfc1522_encode(article->name, tinrc.mm_local_charset, FALSE);
-		unfold_header(p);
-		if (strpbrk(article->name, "\".:;<>@[]()\\") != NULL && article->name[0] != '"' && article->name[strlen(article->name)] != '"')
-			snprintf(from, sizeof(from), "\"%s\" <%s>", group->attribute->post_8bit_header ? q : p, article->from);
-		else
-			snprintf(from, sizeof(from), "%s <%s>", group->attribute->post_8bit_header ? q : p, article->from);
+			p = rfc1522_encode(mb->name, tinrc.mm_local_charset, FALSE);
+			unfold_header(p);
+			if (strpbrk(mb->name, "\".:;<>@[]()\\") != NULL && mb->name[0] != '"' && mb->name[strlen(mb->name)] != '"')
+				snprintf(single_from, sizeof(single_from), "\"%s\" <%s>", group->attribute->post_8bit_header ? q : p, mb->from);
+			else
+				snprintf(single_from, sizeof(single_from), "%s <%s>", group->attribute->post_8bit_header ? q : p, mb->from);
 
-		free(p);
-		free(q);
-	} else
-		snprintf(from, sizeof(from), "<%s>", article->from);
+			free(p);
+			free(q);
+		} else
+			snprintf(single_from, sizeof(single_from), "<%s>", mb->from);
+
+		if (strlen(from) + strlen(single_from) + /* strlen(", ") */ 2 < PATH_LEN) {
+			if (c_needed++)
+				strcat(from, ", ");
+			strcat(from, single_from);
+		}
+
+		mb = mb->next;
+	} while (mb);
 
 	return from;
 }
@@ -3349,12 +3475,19 @@ open_xover_fp(
 
 		if (!max)
 			return NULL;
+
 		if (min == max)
 			snprintf(line, sizeof(line), "%s %"T_ARTNUM_PFMT, nntp_caps.over_cmd, min);
 		else
 			snprintf(line, sizeof(line), "%s %"T_ARTNUM_PFMT"-%"T_ARTNUM_PFMT, nntp_caps.over_cmd, min, MAX(min, max));
+
 		return (nntp_command(line, OK_XOVER, NULL, 0));
 	}
+#else
+	/* silence compiler warning (unused parameter) */
+	(void) min;
+	(void) max;
+	(void) local;
 #endif /* NNTP_ABLE */
 	{
 		FILE *fp;
@@ -3365,7 +3498,7 @@ open_xover_fp(
 				return fp;
 
 			if (*mode != 'r')
-				error_message(2, _(txt_cannot_open), nov_file);
+				perror_message(_(txt_cannot_open), nov_file);
 		}
 	}
 	return NULL;
