@@ -3,10 +3,10 @@
  *  Module    : charset.c
  *  Author    : M. Kuhn, T. Burmester
  *  Created   : 1993-12-10
- *  Updated   : 2021-02-23
+ *  Updated   : 2024-07-08
  *  Notes     : ISO to ascii charset conversion routines
  *
- * Copyright (c) 1993-2023 Markus Kuhn <mgk25@cl.cam.ac.uk>
+ * Copyright (c) 1993-2024 Markus Kuhn <mgk25@cl.cam.ac.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,13 @@
 #	include "tin.h"
 #endif /* !TIN_H */
 
+#if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
+#	include <unicode/utypes.h>
+#	include <unicode/localpointer.h>
+#	include <unicode/uenum.h>
+#	include <unicode/ucsdet.h>
+#endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+
 /*
  *  Table for the iso2asc conversion
  *  iso2asc  by  (unrza3@cd4680fs.rrze.uni-erlangen.de)
@@ -188,8 +195,8 @@ convert_iso2asc(
 	first = TRUE;
 	i = a = 0;
 	while (*iso != '\0') {
-		if (*EIGHT_BIT(iso) >= ISO_EXTRA) {
-			p = tab[*EIGHT_BIT(iso) - ISO_EXTRA];
+		if (*(unsigned char *) (iso) >= ISO_EXTRA) {
+			p = tab[*(unsigned char *) (iso) - ISO_EXTRA];
 			iso++;
 			i++;
 			first = TRUE;
@@ -286,7 +293,14 @@ convert_tex2iso(
 	 * code position as ISO-8859-1
 	 * DEC-MCS, Windows-1252
 	 */
-	if (IS_LOCAL_CHARSET("ISO-8859-1") ||
+	if (
+		!strcasecmp( /* ensure not to also match ISO-8859-{11,12} */
+#ifdef CHARSET_CONVERSION
+			tinrc.mm_local_charset
+#else
+			tinrc.mm_charset
+#endif /* CHARSET_CONVERSION */
+			, "ISO-8859-1") ||
 		IS_LOCAL_CHARSET("ISO-8859-2") ||
 		IS_LOCAL_CHARSET("ISO-8859-3") ||
 		IS_LOCAL_CHARSET("ISO-8859-4") ||
@@ -447,3 +461,111 @@ wconvert_to_printable(
 	return wbuf;
 }
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+
+/*
+ * Check for charsets which may contain NULL bytes and thus break string
+ * functions. Possibly incomplete.
+ *
+ * TODO: fix the other code to handle those charsets properly.
+ */
+t_bool
+charset_unsupported(
+	const char *charset)
+{
+	static const char *charsets[] = {
+		"csUnicode",	/* alias for ISO-10646-UCS-2 */
+		"csUCS4",		/* alias for ISO-10646-UCS-4 */
+		"ISO-10646-UCS-2", /* can't be shortened due to ISO-10646-UCS-Basic */
+		"ISO-10646-UCS-4",
+		"UTF-16",		/* covers also BE/LE */
+		"UTF-32",		/* covers also BE/LE */
+		NULL };
+	const char **charsetptr = charsets;
+	t_bool ret = FALSE;
+#ifdef CHARSET_CONVERSION
+	iconv_t cd;
+#endif /* CHARSET_CONVERSION */
+
+	if (!charset)
+		return ret;
+
+	do {
+		if (!strncasecmp(charset, *charsetptr, strlen(*charsetptr)))
+			ret = TRUE;
+	} while (!ret && *(++charsetptr) != NULL);
+
+#ifdef CHARSET_CONVERSION
+	if (!ret) {
+		if ((cd = iconv_open("UCS-4", charset)) == (iconv_t) (-1))
+			ret = TRUE;
+		else
+			iconv_close(cd);
+	}
+#endif /* CHARSET_CONVERSION */
+
+	return ret;
+}
+
+
+#if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
+char *
+guess_charset(
+		const char *sample,
+		int32_t confidence)
+{
+	char *guessed_charset = NULL;
+	UCharsetDetector *detector = NULL;
+	const UCharsetMatch *match;
+	UErrorCode status = 0;
+	const char *p_match = NULL;
+
+	detector = ucsdet_open(&status);
+	if (U_FAILURE(status))
+		goto failure;
+
+	ucsdet_setText(detector, sample, strlen(sample), &status);
+	if (U_FAILURE(status))
+		goto failure;
+
+	match = ucsdet_detect(detector, &status);
+	if (match == NULL || U_FAILURE(status))
+		goto failure;
+
+	p_match = ucsdet_getName(match, &status);
+	if (p_match == NULL || U_FAILURE(status))
+		goto failure;
+
+	/* badguess = 0 ... perfect = 100 */
+	if (ucsdet_getConfidence(match, &status) >= confidence)
+		guessed_charset = my_strdup(p_match);
+
+failure:
+	if (detector)
+		ucsdet_close(detector);
+
+	return guessed_charset;
+}
+#endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+
+
+/*
+ * restrict it to [a-zA-Z0-9_-]+
+ */
+const char *
+validate_charset(
+	const char *charset)
+{
+	const char *c = charset;
+
+	if (!charset)
+		return NULL;
+
+	while (*c) {
+		if (*c < 45 || *c > 122 || *c == 46 || *c == 47 || (*c >= 58 && *c <= 64) || (*c >= 91 && *c <= 94) || *c == 96)
+			return NULL;
+
+		c++;
+	}
+	return charset;
+}
