@@ -3,10 +3,10 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2023-10-29
+ *  Updated   : 2024-07-09
  *  Notes     : Configuration file routines
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,7 +65,7 @@ static void write_server_config(void);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 #	define DASH_TO_SPACE(mark)	((wchar_t) (mark == L'_' ? L' ' : mark))
-#	define SPACE_TO_DASH(mark)	((wchar_t) (mark == L' ' ? L'_' : mark))
+#	define SPACE_TO_DASH(mark)	((wint_t) (mark == L' ' ? L'_' : mark))
 #	define SET_RC_VAL(buf, rcval, defval) do { \
 		wchar_t *wbuf; \
 		if ((wbuf = char2wchar_t(buf))) { \
@@ -101,11 +101,11 @@ read_config_file(
 	t_bool is_7bit;
 #endif /* CHARSET_CONVERSION */
 
-	if ((fp = fopen(file, "r")) == NULL)
+	if ((fp = tin_fopen(file, "r")) == NULL)
 		return FALSE;
 
 	if (!batch_mode || verbose)
-		wait_message(0, _(txt_reading_config_file), (global_file) ? _(txt_global) : "");
+		wait_message(0, _(txt_reading_config_file), global_file ? _(txt_global) : "", file);
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 		if (buf[0] == '\n')
@@ -114,7 +114,7 @@ read_config_file(
 			if (upgrade == NULL && !global_file && match_string(buf, "# tin configuration file V", NULL, 0)) {
 				upgrade = check_upgrade(buf, "# tin configuration file V", TINRC_VERSION);
 				if (upgrade->state != RC_IGNORE)
-					upgrade_prompt_quit(upgrade, file); /* CONFIG_FILE */
+					upgrade_prompt_quit(upgrade, file, fp); /* CONFIG_FILE */
 				if (upgrade->state == RC_UPGRADE)
 					rc_update(fp);
 			}
@@ -283,6 +283,12 @@ read_config_file(
 			if (match_color(buf, "col_signature=", &tinrc.col_signature, MAX_COLOR))
 				break;
 
+			if (match_color(buf, "col_score_neg=", &tinrc.col_score_neg, MAX_COLOR))
+				break;
+
+			if (match_color(buf, "col_score_pos=", &tinrc.col_score_pos, MAX_COLOR))
+				break;
+
 			if (match_color(buf, "col_urls=", &tinrc.col_urls, MAX_COLOR))
 				break;
 
@@ -410,6 +416,9 @@ read_config_file(
 			if (match_string(buf, "default_shell_command=", tinrc.default_shell_command, sizeof(tinrc.default_shell_command)))
 				break;
 
+			if (match_boolean(buf, "dont_break_words=", &tinrc.dont_break_words))
+				break;
+
 			if (match_boolean(buf, "draw_arrow=", &tinrc.draw_arrow))
 				break;
 
@@ -558,7 +567,7 @@ read_config_file(
 
 #if defined(HAVE_ALARM) && defined(SIGALRM)
 			/* the number of seconds is limited on some systems (e.g. Free/OpenBSD: 100000000) */
-			if (match_integer(buf, "nntp_read_timeout_secs=", &tinrc.nntp_read_timeout_secs, 16383))
+			if (match_integer(buf, "nntp_read_timeout_secs=", &tinrc.nntp_read_timeout_secs, TIN_NNTP_TIMEOUT_MAX))
 				break;
 #endif /* HAVE_ALARM && SIGALRM */
 
@@ -690,7 +699,7 @@ read_config_file(
 				break;
 
 			if (match_boolean(buf, "show_description=", &tinrc.show_description)) {
-				if (show_description)
+				if (!(cmdline.args & CMDLINE_NO_DESCRIPTION))
 					show_description = tinrc.show_description;
 				break;
 			}
@@ -733,6 +742,9 @@ read_config_file(
 			if (match_boolean(buf, "show_signatures=", &tinrc.show_signatures))
 				break;
 
+			if (match_boolean(buf, "show_art_score=", &tinrc.show_art_score))
+				break;
+
 			if (match_string(buf, "slashes_regex=", tinrc.slashes_regex, sizeof(tinrc.slashes_regex)))
 				break;
 
@@ -973,7 +985,8 @@ write_config_file(
 		return;
 
 	/* generate tmp-filename */
-	file_tmp = get_tmpfilename(file);
+	if ((file_tmp = get_tmpfilename(file)) == NULL)
+		return;
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
 		error_message(2, _(txt_filesystem_full_backup), CONFIG_FILE);
@@ -1184,6 +1197,9 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_show_signatures.tinrc));
 	fprintf(fp, "show_signatures=%s\n\n", print_boolean(tinrc.show_signatures));
 
+	fprintf(fp, "%s", _(txt_show_art_score.tinrc));
+	fprintf(fp, "show_art_score=%s\n\n", print_boolean(tinrc.show_art_score));
+
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	fprintf(fp, "%s", _(txt_suppress_soft_hyphens.tinrc));
 	fprintf(fp, "suppress_soft_hyphens=%s\n\n", print_boolean(tinrc.suppress_soft_hyphens));
@@ -1353,6 +1369,12 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_col_signature.tinrc));
 	fprintf(fp, "col_signature=%d\n\n", tinrc.col_signature);
 
+	fprintf(fp, "%s", _(txt_col_score_neg.tinrc));
+	fprintf(fp, "col_score_neg=%d\n\n", tinrc.col_score_neg);
+
+	fprintf(fp, "%s", _(txt_col_score_pos.tinrc));
+	fprintf(fp, "col_score_pos=%d\n\n", tinrc.col_score_pos);
+
 	fprintf(fp, "%s", _(txt_col_urls.tinrc));
 	fprintf(fp, "col_urls=%d\n\n", tinrc.col_urls);
 
@@ -1400,6 +1422,9 @@ write_config_file(
 	fprintf(fp, "%s", _(txt_wrap_column.tinrc));
 	fprintf(fp, "wrap_column=%d\n\n", tinrc.wrap_column);
 
+	fprintf(fp, "%s", _(txt_dont_break_words.tinrc));
+	fprintf(fp, "dont_break_words=%s\n\n", print_boolean(tinrc.dont_break_words));
+
 	fprintf(fp, "%s", _(txt_trim_article_body.tinrc));
 	fprintf(fp, "trim_article_body=%d\n\n", tinrc.trim_article_body);
 
@@ -1782,9 +1807,10 @@ quote_dash_to_space(
  */
 char *
 quote_space_to_dash(
-	char *str)
+	const char *str)
 {
-	char *ptr, *dst;
+	char *dst;
+	const char *ptr;
 	static char buf[PATH_LEN];
 
 	dst = buf;
@@ -1823,7 +1849,7 @@ ulBuildArgv(
 		return NULL;
 	}
 
-	for (tmp = cmd; isspace((int) *tmp); tmp++)
+	for (tmp = cmd; isspace((unsigned char) *tmp); tmp++)
 		;
 
 	buf = my_strdup(tmp);
@@ -1832,9 +1858,9 @@ ulBuildArgv(
 	new_argv = my_calloc(1, sizeof(char *));
 
 	while (*tmp) {
-		if (!isspace((int) *tmp)) { /* found the beginning of a word */
+		if (!isspace((unsigned char) *tmp)) { /* found the beginning of a word */
 			new_argv[i] = tmp;
-			for (; *tmp && !isspace((int) *tmp); tmp++)
+			for (; *tmp && !isspace((unsigned char) *tmp); tmp++)
 				;
 			if (*tmp) {
 				*tmp = '\0';
@@ -2047,7 +2073,7 @@ rc_update(
 		tinrc.hide_uue = 1;
 
 	if (keep_posted_articles)
-		STRCPY(tinrc.posted_articles_file, "posted");
+		STRCPY(tinrc.posted_articles_file, POSTED_FILE);
 
 	tinrc.quote_style = (compress_quotes ? QUOTE_COMPRESS : 0) + (quote_empty_lines ? QUOTE_EMPTY : 0) + (quote_signatures ? QUOTE_SIGS : 0);
 
@@ -2111,10 +2137,11 @@ rc_post_update(
 	char buf[LEN];
 	int groupname_max_length = 0;
 
-	if (!fp)
+	if (fseek(fp, 0L, SEEK_SET) == -1) {
+		perror_message(txt_error_fseek, fp);
 		return FALSE;
+	}
 
-	rewind(fp);
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
 		if (buf[0] == '#' || buf[0] == '\n')
 			continue;
@@ -2123,7 +2150,7 @@ rc_post_update(
 			case 'c':
 #ifdef USE_CANLOCK
 				{
-					t_bool cancel_locks = TRUE;
+					t_bool cancel_locks;
 
 					if (match_boolean(buf, "cancel_locks=", &cancel_locks)) {
 						if (!cancel_locks)
@@ -2213,8 +2240,9 @@ read_server_config(
 	joinpath(serverdir, sizeof(serverdir), rcdir, file);
 	joinpath(file, sizeof(file), serverdir, SERVERCONFIG_FILE);
 	joinpath(local_newsgroups_file, sizeof(local_newsgroups_file), serverdir, NEWSGROUPS_FILE);
+	joinpath(local_motd_file, sizeof(local_motd_file), serverdir, MOTD_FILE);
 
-	if ((fp = fopen(file, "r")) == NULL)
+	if ((fp = tin_fopen(file, "r")) == NULL)
 		return;
 
 	while ((line = tin_fgets(fp, FALSE)) != NULL) {
@@ -2222,15 +2250,21 @@ read_server_config(
 			continue;
 
 		if (match_string(line, "last_newnews=", newnews_info, sizeof(newnews_info))) {
-			size_t tmp_len = strlen(nntp_server) + strlen(newnews_info) + 2;
-			char *tmp_info = my_malloc(tmp_len);
+			char *tmp_info;
+			int tmp_len;
 
-			snprintf(tmp_info, tmp_len, "%s %s", nntp_server, newnews_info);
-			load_newnews_info(tmp_info);
-			free(tmp_info);
+			if ((tmp_len = snprintf(NULL, 0, "%s %s", nntp_server, newnews_info)) > 0) {
+				tmp_info = my_malloc(++tmp_len);
+				if (snprintf(tmp_info, (size_t) tmp_len, "%s %s", nntp_server, newnews_info) == tmp_len -1)
+					load_newnews_info(tmp_info);
+				free(tmp_info);
+			}
 			continue;
 		}
 
+		if (match_long(line, "motd_hash=", &motd_hash))
+			continue;
+
 		if (match_string(line, "version=", NULL, 0)) {
 			if (upgrade != NULL) /* ignore duplicate version lines; first match counts */
 				continue;
@@ -2289,7 +2323,7 @@ write_server_config(
 				break;
 
 			default:
-				wait_message(2, "write_server_config(%s)", strerror(errno));
+				perror_message("write_server_config(%s)", serverdir);
 				break;
 		}
 		return;
@@ -2297,7 +2331,8 @@ write_server_config(
 		CLOSEDIR(dirp);
 
 	/* generate tmp-filename */
-	file_tmp = get_tmpfilename(file);
+	if ((file_tmp = get_tmpfilename(file)) == NULL)
+		return;
 
 	if ((fp = fopen(file_tmp, "w")) == NULL) {
 		error_message(2, _(txt_filesystem_full_backup), SERVERCONFIG_FILE); /* TODO: better error handling/message */
@@ -2308,6 +2343,9 @@ write_server_config(
 	fprintf(fp, _(txt_serverconfig_header), PRODUCT, tin_progname, VERSION, RELEASEDATE, RELEASENAME, PRODUCT, PRODUCT);
 	fprintf(fp, "version=%s\n", SERVERCONFIG_VERSION);
 
+	if (motd_hash != 0)
+		fprintf(fp, "motd_hash=%lu\n", (unsigned long int) motd_hash);
+
 	if ((i = find_newnews_index(nntp_server)) >= 0) {
 		if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(newnews[i].time))))
 			fprintf(fp, "last_newnews=%lu (%s)\n", (unsigned long int) newnews[i].time, timestring);
