@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   : 2023-11-26
+ *  Updated   : 2024-11-25
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2024 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2025 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,8 +43,28 @@
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
-
-
+#ifdef HAVE_LIBURIPARSER
+#	include <uriparser/Uri.h>
+#	define CHECK_URI(r,f)	do { \
+		if (MATCH_REGEX(r, line, len)) { \
+			offsets = regex_get_ovector_pointer(&r); \
+			if ((l = offsets[1] - offsets[0])) { \
+				u = my_strndup(line + offsets[0], l); \
+				if (*(u + l - 1) == '\n') \
+					*(u + l -1) = '\0'; \
+				if (*u) { \
+					state.uri = &uri; \
+					if (uriParseUriA(&state, u) == URI_SUCCESS) { \
+						if (uriNormalizeSyntaxA(&uri) == URI_SUCCESS) \
+							flags |= f; \
+						uriFreeUriMembersA(&uri); \
+					} \
+				} \
+				FreeAndNull(u); \
+			} \
+		} \
+	} while (0)
+#endif /* HAVE_LIBURIPARSER */
 /*
  * We malloc() this many t_lineinfo's at a time
  */
@@ -55,16 +75,15 @@
 			(x)->hdr.ext->type == TYPE_MULTIPART && \
 			strcasecmp("alternative", (x)->hdr.ext->subtype) == 0)
 
-#define MATCH_REGEX(x,y,z)	(match_regex_ex(y, z, 0, 0, &(x)) >= 0)
+#define MATCH_REGEX(x,y,z)	(match_regex_ex(y, (REGEX_SIZE) z, 0, 0, &(x)) >= 0)
 
 
 static char *ltobi(unsigned long i);
 static struct t_attach_item *add_attach_line_item(struct t_attach_item **item);
-static t_bool charset_unsupported(const char *charset);
 static t_bool header_wanted(const char *line);
 static t_bool shorten_attach_line(struct t_attach_item *item);
 static t_part *new_uue(t_part **part, char *name);
-static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part, int hide_uue);
+static void process_text_body_part(t_bool wrap_lines, FILE *in, const char *charset, t_part *part, int hide_uue);
 static void put_attach(t_bool wrap_lines, t_part *part, int depth, int is_uue, const char *name, const char *charset);
 static void put_cooked(size_t buf_len, t_bool wrap_lines, int flags, const char *fmt, ...);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -103,13 +122,13 @@ expand_ctrl_chars(
 	 * it should help us find problems with wide-char strings
 	 * in the development branch
 	 */
-	assert(wline != NULL);
+	assert(((void) "wline must not be NULL", wline != NULL));
 	wlen = wcslen(wline) + 1; /* add one to make coverity happy */
 	ctrl_L = wexpand_ctrl_chars(&wline, &wlen, lcook_width);
 	free(*line);
 	*line = wchar_t2char(wline);
 	free(wline);
-	assert(line != NULL);
+	assert(((void) "line must not be NULL", line != NULL));
 	*length = strlen(*line);
 #else
 	int curr_len = LEN;
@@ -140,7 +159,7 @@ expand_ctrl_chars(
 			else
 				buf[i++] = *c;
 		}
-		c++;
+		++c;
 	}
 	buf[i] = '\0';
 	*length = i + 1;
@@ -187,7 +206,7 @@ wexpand_ctrl_chars(
 			else
 				wbuf[i++] = *wc;
 		}
-		wc++;
+		++wc;
 	}
 	wbuf[i] = '\0';
 	*length = i + 1;
@@ -214,7 +233,7 @@ put_cooked(
 	const char *fmt,
 	...)
 {
-	char *p, *bufp, *buf;
+	char *p, *bufp, *buf, *last_space;
 	int wrap_column;
 	int space;
 	static int saved_flags = 0;
@@ -231,13 +250,12 @@ put_cooked(
 
 	if (tinrc.wrap_column < 0)
 		wrap_column = ((tinrc.wrap_column > -cCOLS) ? cCOLS + tinrc.wrap_column : cCOLS);
-	else
-#if 1
-		wrap_column = ((tinrc.wrap_column > 0) ? tinrc.wrap_column : cCOLS);
-#else	/* never cut off long lines */
-		wrap_column = (((tinrc.wrap_column > 0) && (tinrc.wrap_column < cCOLS)) ? tinrc.wrap_column : cCOLS);
-#endif /* 1 */
-
+	else {
+		if (tinrc.dont_break_words)
+			wrap_column = (((tinrc.wrap_column > 0) && (tinrc.wrap_column < cCOLS)) ? tinrc.wrap_column : cCOLS);
+		else
+			wrap_column = ((tinrc.wrap_column > 0) ? tinrc.wrap_column : cCOLS);
+	}
 	p = bufp = buf;
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -247,27 +265,32 @@ put_cooked(
 	while (*p) {
 		if (wrap_lines) {
 			space = wrap_column;
+			last_space = NULL;
 			while (space > 0 && *p && *p != '\n') {
+				if (tinrc.dont_break_words && !(flags & (C_VERBATIM | C_HEADER)) && (*p == ' ' || *p == '\t'))
+					last_space = p;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				if ((bytes = mbtowc((wchar_t *) wp, p, MB_CUR_MAX)) > 0) {
 					if ((space -= wcwidth((wchar_t) *wp)) < 0)
 						break;
 					p += bytes;
 				} else
-					p++;
+					++p;
 #else
-				p++;
-				space--;
+				++p;
+				--space;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			}
+			if (tinrc.dont_break_words && !(flags & (C_VERBATIM | C_HEADER)) && space <= 0 && last_space && p > last_space)
+				p = last_space + 1;
 		} else {
 			while (*p && *p != '\n')
-				p++;
+				++p;
 		}
 		fwrite(bufp, 1, (size_t) (p - bufp), art->cooked);
 		fputs("\n", art->cooked);
 		if (*p == '\n')
-			p++;
+			++p;
 		bufp = p;
 
 		if (art->cooked_lines == 0) {
@@ -403,9 +426,9 @@ get_filename(
 #define BUILD_ATTACH_ITEM() do { \
 		if (curr->flags & (ATTACH_SHOW_CONTENT | ATTACH_SHOW_BOTH)) { \
 			if (curr->flags & ATTACH_SHOW_BOTH) \
-				snprintf(buf, sizeof(buf), curr->description, curr->content); \
+				snprintf(buf, blen, curr->description, curr->content); \
 			else \
-				snprintf(buf, sizeof(buf), curr->fmt, curr->content); \
+				snprintf(buf, blen, curr->fmt, curr->content); \
 			if ((space_left -= strlen(buf) > 0)) \
 				strcat(attach_line, buf); \
 		} \
@@ -482,17 +505,25 @@ build_attach_line(
 	char *al_ptr;
 	char *fmt_ptr;
 	char *line_cnt_str = NULL;
-	char buf[BUFSIZ];
+	char *buf;
 	char *fmt;
 	int i, line_cnt_str_len;
+	size_t blen;
 	ssize_t space_left;
 	struct t_attach_item *curr = NULL;
 	struct t_attach_item *items = NULL;
-	struct t_attach_item *last = NULL;
+	struct t_attach_item *last;
 	t_bool init = TRUE;
 	t_bool excl_seen = FALSE;
 	t_bool star_seen = FALSE;
 
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	buf = my_malloc(MB_CUR_MAX * (size_t) (cCOLS + 1));
+	blen = MB_CUR_MAX * (size_t) cCOLS;
+#	else
+	buf = my_malloc(cCOLS + 1);
+	blen = (size_t) cCOLS;
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	if (is_uue)
 		fmt = tinrc.page_uue_format;
 	else if (signal_context == cAttachment)
@@ -692,7 +723,7 @@ build_attach_line(
 	star_seen = excl_seen = FALSE;
 	attach_line = my_malloc(LEN);
 	space_left = LEN - 2;
-	attach_line[0] = '\0';
+	*attach_line = '\0';
 
 	while (space_left > 0 && (init || ((strwidth(attach_line) > max_len && shorten_attach_line(last))))) {
 		init = FALSE;
@@ -824,6 +855,7 @@ build_attach_line(
 		}
 	}
 
+	free(buf);
 	return (attach_line);
 }
 
@@ -865,14 +897,14 @@ static void
 process_text_body_part(
 	t_bool wrap_lines,
 	FILE *in,
+	const char *charset,
 	t_part *part,
 	int hide_uue)
 {
 	char *rest = NULL;
 	char *line = NULL, *buf, *tmpline;
-	const char *ncharset;
-	size_t max_line_len = 0;
-	int flags, len, lines_left, len_blank;
+	size_t max_line_len = 0, len, len_blank;
+	int flags, lines_left;
 	unsigned int lines_skipped = 0;
 	t_bool in_sig = FALSE;			/* Set when in sig portion */
 	t_bool in_uue = FALSE;			/* Set when in uuencoded section */
@@ -883,6 +915,13 @@ process_text_body_part(
 	t_bool first_line_blank = TRUE;	/* Unset when first non-blank line is reached */
 	t_bool put_blank_lines = FALSE;	/* Set when previously skipped lines needs to put */
 	t_part *curruue = NULL;
+#ifdef HAVE_LIBURIPARSER
+	REGEX_SIZE *offsets;
+	char *u = NULL;
+	size_t l;
+	UriUriA uri;
+	UriParserStateA state;
+#endif /* HAVE_LIBURIPARSER */
 
 	if (part->uue) {				/* These are redone each time we recook/resize etc.. */
 		free_parts(part->uue);
@@ -893,7 +932,7 @@ process_text_body_part(
 #ifdef DEBUG
 		/*
 		 * TODO: always show to user?
-		 *       hen use something less technical and move to lang.c
+		 *       then use something less technical and move to lang.c
 		 */
 		perror_message("%s:%d process_text_body_part(fseek(in)) failed", __FILE__, __LINE__);
 #endif /* DEBUG */
@@ -939,7 +978,7 @@ process_text_body_part(
 				 */
 				strcat(line, "\n");
 
-				lines_left--;
+				--lines_left;
 				break;
 		}
 		if (!(line && strlen(line))) {
@@ -947,25 +986,18 @@ process_text_body_part(
 			break;	/* premature end of file, file error etc. */
 		}
 
-		/* convert network to local charset, tex2iso, iso2asc etc. */
-		ncharset = validate_charset(get_param(part->params, "charset"));
-		/* TODO: ok to fall back to undeclared_charset if charset is illegal or should we always hard fail to US-ASCII? */
-		process_charsets(&line, &max_line_len, ncharset ? ncharset :
-#ifdef CHARSET_CONVERSION
-			curr_group->attribute->undeclared_charset ? curr_group->attribute->undeclared_charset :
-#endif /* CHARSET_CONVERSION */
-			 "US-ASCII", tinrc.mm_local_charset, curr_group->attribute->tex2iso_conv && art->tex2iso);
+		process_charsets(&line, &max_line_len, charset, tinrc.mm_local_charset, curr_group->attribute->tex2iso_conv && art->tex2iso);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if (IS_LOCAL_CHARSET("UTF-8")) {
 			utf8_valid(line);
 
-			if (!in_verbatim && curr_group->attribute->suppress_soft_hyphens && ncharset && !strcasecmp(ncharset, "UTF-8"))
+			if (!in_verbatim && curr_group->attribute->suppress_soft_hyphens && !strcasecmp(charset, "UTF-8"))
 				remove_soft_hyphens(line);
 		}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-		len = (int) strlen(line);
+		len = strlen(line);
 
 		/*
 		 * trim article body and sig (not verbatim blocks):
@@ -979,8 +1011,8 @@ process_text_body_part(
 			tmpline = line;
 			/* check if line contains only whitespace */
 			while ((*tmpline == ' ') || (*tmpline == '\t')) {
-				len_blank++;
-				tmpline++;
+				++len_blank;
+				++tmpline;
 			}
 			if (len_blank == len) {		/* line is blank */
 				if (lines_left == 0 && (curr_group->attribute->trim_article_body & SKIP_TRAILING)) {
@@ -992,7 +1024,7 @@ process_text_body_part(
 					if (curr_group->attribute->trim_article_body & SKIP_LEADING)
 						continue;
 				} else if ((curr_group->attribute->trim_article_body & (COMPACT_MULTIPLE | SKIP_TRAILING)) && (!in_sig || curr_group->attribute->show_signatures)) {
-					lines_skipped++;
+					++lines_skipped;
 					if (lines_left == 0 && !(curr_group->attribute->trim_article_body & SKIP_TRAILING)) {
 						for (; lines_skipped > 0; lines_skipped--)
 							put_cooked(1, TRUE, in_sig ? C_SIG : C_BODY, "\n");
@@ -1055,11 +1087,12 @@ process_text_body_part(
 			 *
 			 * TODO: look for a tailing size line after end (non standard
 			 *       extension)?
+			 *       do we want to cook uue-parts in signatures?
 			 */
 
 			is_uubegin = FALSE;
 
-			if (match_regex_ex(line, len, 0, 0, &uubegin_regex) >= 0) {
+			if (MATCH_REGEX(uubegin_regex, line, len)) {
 				REGEX_SIZE *ovector = regex_get_ovector_pointer(&uubegin_regex);
 
 				if (in_uue) { /* previous uue part incomplete and the current one follows without gap */
@@ -1093,11 +1126,11 @@ process_text_body_part(
 				/* sum = 0 in a uubody only on the last line, a single ` */
 				if (sum == 0 && len == 1 + 1)			/* +1 for the \n */
 					is_uubody = TRUE;
-				else if (len == sum + 1 + 1)
+				else if (len == (unsigned) (sum + 1 + 1))
 					is_uubody = TRUE;
 #ifdef DEBUG_ART
 				if (debug & DEBUG_MISC)
-					fprintf(stderr, "%s sum=%d len=%d (%s)\n", bool_unparse(is_uubody), sum, len, line);
+					fprintf(stderr, "%s sum=%d len=%lu (%s)\n", bool_unparse(is_uubody), sum, len, line);
 #endif /* DEBUG_ART */
 			}
 
@@ -1181,12 +1214,19 @@ process_text_body_part(
 		}
 #endif /* HAVE_COLOR */
 
+#ifdef HAVE_LIBURIPARSER
+		/* find and validate URIs */
+		CHECK_URI(url_regex, C_URL);
+		CHECK_URI(mail_regex, C_MAIL);
+		CHECK_URI(news_regex, C_NEWS);
+#else
 		if (MATCH_REGEX(url_regex, line, len))
 			flags |= C_URL;
 		if (MATCH_REGEX(mail_regex, line, len))
 			flags |= C_MAIL;
 		if (MATCH_REGEX(news_regex, line, len))
 			flags |= C_NEWS;
+#endif /* HAVE_LIBURIPARSER */
 
 		if (expand_ctrl_chars(&line, &max_line_len, tabwidth))
 			flags |= C_CTRLL;				/* Line contains form-feed */
@@ -1270,42 +1310,6 @@ dump_cooked(
 #endif /* DEBUG_ART */
 
 
-/*
- * Check for charsets which may contain NULL bytes and thus break string
- * functions. Possibly incomplete.
- *
- * TODO: fix the other code to handle those charsets properly.
- */
-static t_bool
-charset_unsupported(
-	const char *charset)
-{
-	static const char *charsets[] = {
-		"csUnicode",	/* alias for ISO-10646-UCS-2 */
-		"csUCS4",		/* alias for ISO-10646-UCS-4 */
-		"ISO-10646-UCS-2",
-		"ISO-10646-UCS-4",
-		"UTF-16",		/* covers also BE/LE */
-		"UTF-32",		/* covers also BE/LE */
-		NULL };
-	const char **charsetptr = charsets;
-	t_bool ret = FALSE;
-
-	if (!charset)
-		return ret;
-
-	do {
-		if (!strncasecmp(charset, *charsetptr, strlen(*charsetptr)))
-			ret = TRUE;
-	} while (!ret && *(++charsetptr) != NULL);
-
-	if (!validate_charset(charset))
-		return FALSE;
-
-	return ret;
-}
-
-
 /*
  * 'cooks' an article, ie, prepare what will actually appear on the screen
  * It is not easy to do this in the same pass as the initial read since
@@ -1331,6 +1335,10 @@ charset_unsupported(
  * TODO:
  *      give an error-message on at least disk-full
  */
+/* set INLINE_DEBUG_MIME to 1 to inline report some errors/warnings,
+ * use PAGE_ARTICLE_INFO ('\'') for a more detailed info.
+ * strings -> lang.c?
+ */
 t_bool
 cook_article(
 	t_bool wrap_lines,
@@ -1342,15 +1350,19 @@ cook_article(
 	const char *name;
 	char *line;
 	struct t_header *hdr = &artinfo->hdr;
-	t_bool header_put = FALSE;
 	static const char *struct_header[] = {
-		"Approved: ", "From: ", "Originator: ",
-		"Reply-To: ", "Sender: ", "X-Cancelled-By: ", "X-Comment-To: ",
-		"X-Submissions-To: ", "To: ", "Cc: ", "Bcc: ", "X-Originator: ", NULL };
+		"Approved: ", "From: ", "Originator: ", "Reply-To: ",
+		"Sender: ", "To: ", "Cc: ", "Bcc: ", "X-Cancelled-By: ",
+		"X-Comment-To: ", "X-Submissions-To: ", "X-Originator: ",
+		NULL };
+	t_bool header_put = FALSE;
+#ifdef INLINE_DEBUG_MIME
+	t_bool stripped = FALSE;
+#endif /* INLINE_DEBUG_MIME */
 
 	art = artinfo;				/* Global saves lots of passing artinfo around */
 
-	if (!(art->cooked = tmpfile()))
+	if (!(art->cooked = my_tmpfile()))
 		return FALSE;
 
 	art->cooked_lines = 0;
@@ -1361,15 +1373,23 @@ cook_article(
 	 * Put down just the headers we want
 	 */
 	while ((line = tin_fgets(artinfo->raw, TRUE)) != NULL) {
-		if (line[0] == '\0') {				/* End of headers? */
+		if (!*line) {				/* End of headers? */
 			if (STRIP_ALTERNATIVE(artinfo)) {
+#ifdef INLINE_DEBUG_MIME
+				stripped = TRUE;
+#endif /* INLINE_DEBUG_MIME */
 				if (header_wanted(_(txt_info_x_conversion_note))) {
 					header_put = TRUE;
 					put_cooked(LEN, wrap_lines, C_HEADER, _(txt_info_x_conversion_note));
 				}
 			}
-			if (header_put)
+			if (header_put) {
 				put_cooked(1, TRUE, 0, "\n");		/* put a newline after headers */
+#ifdef INLINE_DEBUG_MIME
+				if (stripped)
+					put_cooked(LEN, wrap_lines, C_ATTACH, "[-- non text/plain parts from multipart/alternative stripped --]\n");
+#endif /* INLINE_DEBUG_MIME */
+			}
 			break;
 		}
 
@@ -1386,7 +1406,7 @@ cook_article(
 					if ((ptr = strchr(foo, ':'))) {
 						*ptr = '\0';
 						unfold_header(line);
-						if ((ptr = parse_header(line, foo, TRUE, TRUE, FALSE))) {
+						if ((ptr = parse_mb_list_header(line, foo))) {
 #if 0
 							/*
 							 * TODO:
@@ -1441,6 +1461,15 @@ cook_article(
 	if (hdr->mime && hdr->ext->type == TYPE_MULTIPART) {
 		t_part *ptr;
 
+#ifdef INLINE_DEBUG_MIME
+		if (hdr->ext->mime_hints.flags & MIME_VERSION_MISSING)
+			put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"MIME-Version:\"-header missing --]\n");
+		else {
+			if (hdr->ext->mime_hints.flags & MIME_VERSION_UNSUPPORTED)
+				put_cooked(LEN, wrap_lines, C_ATTACH, "[-- Unsupported \"MIME-Version:\" --]\n");
+		}
+#endif /* INLINE_DEBUG_MIME */
+
 		for (ptr = hdr->ext->next; ptr != NULL; ptr = ptr->next) {
 			/*
 			 * Ignore non text/plain sections with alternative handling
@@ -1449,41 +1478,74 @@ cook_article(
 				continue;
 
 			name = get_filename(ptr->params);
-			if (!strcmp(content_types[ptr->type], "text"))
+			if (ptr->type == TYPE_TEXT)
 				charset = get_param(ptr->params, "charset");
 			else
 				charset = NULL;
+
 			put_attach(wrap_lines, ptr, (ptr->depth - 1) * 4, 0, name, charset);
 
 			/* Try to view anything of type text, may need to review this */
 			if (IS_PLAINTEXT(ptr)) {
-				if (charset_unsupported(charset)) {
-					put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_mime_unsup_charset), (ptr->depth - 1) * 4, "", charset);
+				if (hdr->ext->mime_hints.flags & MIME_CHARSET_UNSUPPORTED) {
+					put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_mime_unsup_charset), (ptr->depth - 1) * 4, "", hdr->ext->mime_hints.charset);
 					if (ptr->next)
 						put_cooked(1, wrap_lines, C_ATTACH, "\n");
-				} else
-					process_text_body_part(wrap_lines, artinfo->raw, ptr, hide_uue);
+				} else {
+					charset = validate_charset(charset);
+					process_text_body_part(wrap_lines, artinfo->raw, charset ? charset : "US-ASCII", ptr, hide_uue);
+				}
 			}
 		}
 	} else {
-		if (!strcmp(content_types[hdr->ext->type], "text"))
-			charset = get_param(hdr->ext->params, "charset");
-		else
-			charset = NULL;
+		charset = get_param(hdr->ext->params, "charset");
+
+#ifdef INLINE_DEBUG_MIME
+		if (hdr->ext->type != TYPE_TEXT || (hdr->ext->type == TYPE_TEXT && hdr->ext->encoding != ENCODING_7BIT)) { /* don't warn on pure ascii non mime */
+			if (hdr->ext->mime_hints.flags & MIME_VERSION_MISSING)
+				put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"MIME-Version:\"-header missing --]\n");
+			else {
+				if (hdr->ext->mime_hints.flags & MIME_VERSION_UNSUPPORTED)
+					put_cooked(LEN, wrap_lines, C_ATTACH, "[-- Unsupported \"MIME-Version:\" --]\n");
+			}
+		}
+#endif /* INLINE_DEBUG_MIME */
+
+#ifdef CHARSET_CONVERSION
+#	ifdef USE_ICU_UCSDET
+		if (hdr->ext->type == TYPE_APPLICATION && hdr->ext->encoding == ENCODING_BINARY && artinfo->hdr.ext->mime_hints.flags & MIME_TRANSFER_ENCODING_UNKNOWN) {
+			put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"Content-Type: application/octet-stream\" forced --]\n");
+			put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"Content-Type-Encodig: %s\" forced --]\n", content_encodings[hdr->ext->encoding]);
+		}
+		if (hdr->ext->mime_hints.flags & MIME_CHARSET_GUESSED) {
+			charset = get_param(hdr->ext->params, "guessed_charset");
+#		ifdef INLINE_DEBUG_MIME
+			put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"charset=%s\" guessed --]\n", charset);
+			if (hdr->ext->type == TYPE_TEXT && hdr->ext->encoding != ENCODING_7BIT && ((hdr->ext->mime_hints.flags & MIME_TRANSFER_ENCODING_MISSING) || (artinfo->hdr.ext->mime_hints.flags & MIME_TRANSFER_ENCODING_UNKNOWN)))
+				put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"Content-Type-Encodig: %s\" forced --]\n", content_encodings[hdr->ext->encoding]);
+#		endif /* INLINE_DEBUG_MIME */
+		}
+#	endif /* USE_ICU_UCSDET */
+#	ifdef INLINE_DEBUG_MIME
+		if (hdr->ext->mime_hints.flags & MIME_CHARSET_UNSUPPORTED)
+			put_cooked(LEN, wrap_lines, C_ATTACH, "[-- \"charset=%s\" unsupported --]\n", hdr->ext->mime_hints.charset);
+#	endif /* INLINE_DEBUG_MIME */
+#endif /* CHARSET_CONVERSION */
+
 		/*
 		 * A regular single-body article
 		 */
-		if (IS_PLAINTEXT(hdr->ext)) {
-			if (charset_unsupported(charset))
-				put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_mime_unsup_charset), 0, "", charset);
+		if (!hdr->mime || IS_PLAINTEXT(hdr->ext)) {
+			if (hdr->ext->mime_hints.flags & MIME_CHARSET_UNSUPPORTED)
+				put_cooked(LEN, wrap_lines, C_ATTACH, _(txt_mime_unsup_charset), 0, "", hdr->ext->mime_hints.charset);
 			else
-				process_text_body_part(wrap_lines, artinfo->raw, hdr->ext, hide_uue);
+				process_text_body_part(wrap_lines, artinfo->raw, charset ? charset : "US-ASCII", hdr->ext, hide_uue);
 		} else {
 			/*
 			 * Non-textual main body
 			 */
 			name = get_filename(hdr->ext->params);
-			put_attach(wrap_lines, hdr->ext, 0, 0, name, charset);
+			put_attach(wrap_lines, hdr->ext, 0, 0, name, BlankIfNull(charset));
 		}
 	}
 
@@ -1501,27 +1563,36 @@ cook_article(
 
 /*
  * tin_ltoa() like function; IEC binary notation (base 1024)
- * with one decimal point. positive numbers only
+ * with one decimal point. positive numbers only.
  */
 #define BI_BASE 1024
 static char *
 ltobi(
 	unsigned long i)
 {
-	static const char power[] = { ' ', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', '\0' };
+	static const char power[] = { 'e', 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', '\0' };
 	static char buffer[9];
+	char r;
 	unsigned d = 0, e = 0;
 
+#if !defined(NO_LOCALE) && defined(HAVE_LANGINFO_CODESET)
+	if (!(r = nl_langinfo(RADIXCHAR)[0]))
+#endif /* !NO_LOCALE && HAVE_LANGINFO_CODESET */
+		r = '.';
+
 	while (i >= BI_BASE) {
 		d = (unsigned) (i % BI_BASE * 10 / BI_BASE);
 		i /= BI_BASE;
-		e++;
+		++e;
 	}
 
-	if (e)
-		sprintf(buffer, "%u.%u%c", (unsigned) i, d, power[e]);
-	else
-		sprintf(buffer, "0.%u%c", (unsigned) (i * 10 / BI_BASE), power[1]);
+	if (e) {
+		if (e >= sizeof(power) - 1) /* any 128bit systems around? ,-) */
+			sprintf(buffer, "%u%c%u%c", (unsigned) i, r, d, power[0]); /* omit value and use a better error string? */
+		else
+			sprintf(buffer, "%u%c%u%c", (unsigned) i, r, d, power[e]);
+	} else
+		sprintf(buffer, "0%c%u%c", r, (unsigned) (i * 10 / BI_BASE), power[1]);
 
 	return buffer;
 }
