@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2021-02-23
+ *  Updated   : 2024-07-28
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -40,7 +40,7 @@
 void my_dummy(void) { }	/* ANSI C requires non-empty file */
 t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
 
-#else	/* !USE_CURSES */
+#else /* !USE_CURSES */
 
 #	ifndef ns32000
 #	undef	sinix
@@ -187,7 +187,7 @@ setup_screen(
 #		define TGETSTR(b,bufp)    tigetstr(b)
 #		define TGETNUM(b)         tigetnum(b) /* may be tigetint() */
 #		define TGETFLAG(b)        tigetflag(b)
-#		define NO_CAP(s)          (s == 0 || s == (char *) -1)
+#		define NO_CAP(s)          (s == NULL || s == (char *) -1)
 #		if !defined(HAVE_TIGETNUM) && defined(HAVE_TIGETINT)
 #			define tigetnum tigetint
 #		endif /* !HAVE_TIGETNUM && HAVE_TIGETINT */
@@ -199,7 +199,7 @@ setup_screen(
 #		define TGETSTR(a, bufp)   tgetstr(a, bufp)
 #		define TGETNUM(a)         tgetnum(a)
 #		define TGETFLAG(a)        tgetflag(a)
-#		define NO_CAP(s)          (s == 0)
+#		define NO_CAP(s)          (s == NULL)
 #	endif /* USE_TERMINFO */
 
 #	ifdef HAVE_TPARM
@@ -521,7 +521,7 @@ ScrollScreen(
 				tputs(_scrollback, 1, outchar);
 			}
 		}
-	} else
+	} else {
 		if (_scrollfwd) {
 			i = lines_to_scroll;
 			while (i--) {
@@ -529,6 +529,7 @@ ScrollScreen(
 				tputs(_scrollfwd, 1, outchar);
 			}
 		}
+	}
 	my_flush();
 }
 
@@ -638,7 +639,7 @@ Raw(
 		cfmakeraw(&_raw_tty);
 		_raw_tty.c_lflag |= ISIG;		/* for ^Z */
 #		else
-		_raw_tty.c_lflag &= ~(ICANON | ECHO);	/* noecho raw mode */
+		_raw_tty.c_lflag &= ~((tcflag_t) ICANON | ECHO);	/* noecho raw mode */
 		_raw_tty.c_cc[VMIN] = '\01';	/* minimum # of chars to queue */
 		_raw_tty.c_cc[VTIME] = '\0';	/* minimum time to wait for input */
 #		endif /* __FreeBSD__ */
@@ -744,7 +745,7 @@ highlight_string(
 	 * In a multibyte locale we get byte offsets instead of character
 	 * offsets; calculate now the correct starting column
 	 */
-	if (col > 0) {
+	if (col > 0 && col < LEN) {
 		char tmp[LEN];
 		wchar_t *wtmp;
 
@@ -973,7 +974,7 @@ get_arrow_key(
 			usleep((unsigned long) (SECOND_CHARACTER_DELAY * 1000));
 			i++;
 		}
-#else	/* !HAVE_USLEEP */
+#else /* !HAVE_USLEEP */
 #	ifdef HAVE_SELECT
 		struct timeval tvptr;
 
