@@ -3,10 +3,10 @@
  *  Module    : feed.c
  *  Author    : I. Lea
  *  Created   : 1991-08-31
- *  Updated   : 2023-10-16
+ *  Updated   : 2024-12-04
  *  Notes     : provides same interface to mail,pipe,print,save & repost commands
  *
- * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,9 +66,9 @@ struct t_counters {
 /*
  * Local prototypes
  */
-static char *get_save_filename(struct t_group *group, int function, char *filename, int filelen);
+static char *get_save_filename(const struct t_group *group, int function, char *filename, int filelen);
 static t_bool expand_feed_filename(char *outpath, size_t outpath_len, const char *path);
-static t_bool feed_article(int art, int function, struct t_counters *counter, t_bool use_current, const char *data, struct t_group *group);
+static t_bool feed_article(int art, int function, struct t_counters *counter, t_bool use_current, const char *data, const struct t_group *group);
 static t_function get_feed_key(int function, int level, struct t_art_stat *thread);
 static t_function get_post_proc_type(void);
 static void print_save_summary(t_function type, int fed);
@@ -90,7 +90,7 @@ static void print_save_summary(t_function type, int fed);
  */
 static char *
 get_save_filename(
-	struct t_group *group,
+	const struct t_group *group,
 	int function,
 	char *filename,
 	int filelen)
@@ -102,7 +102,7 @@ get_save_filename(
 	/*
 	 * Group attribute savefile overrides tinrc default savefile
 	 */
-	my_strncpy(default_savefile, (group->attribute->savefile ? group->attribute->savefile : tinrc.default_save_file), sizeof(default_savefile) - 1);
+	my_strncpy(default_savefile, (group->attribute->savefile && *group->attribute->savefile ? *group->attribute->savefile : BlankIfNull(tinrc.default_save_file)), sizeof(default_savefile) - 1);
 
 	/*
 	 * We don't ask when auto'S'aving
@@ -118,9 +118,10 @@ get_save_filename(
 	/*
 	 * Update tinrc.default_save_file if changed
 	 */
-	if (*filename)
-		my_strncpy(tinrc.default_save_file, filename, sizeof(tinrc.default_save_file) - 1);
-	else {
+	if (*filename) {
+		FreeIfNeeded(tinrc.default_save_file);
+		tinrc.default_save_file = my_strdup(filename);
+	} else {
 		/*
 		 * None chosen (or AUTOSAVING), use tinrc default
 		 */
@@ -164,7 +165,7 @@ expand_feed_filename(
 	if ((ret == 0) || !(strrchr(outpath, '/'))) {
 		char buf[PATH_LEN];
 
-		if (!strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE))
+		if (!strfpath(cmdline.savedir ? cmdline.savedir : *curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE))
 			joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
 		joinpath(outpath, outpath_len, buf, path);
 		return FALSE;
@@ -313,7 +314,7 @@ get_feed_key(
 			{
 				char *tmp = fmt_string(_(txt_feed_pattern), tinrc.default_pattern);
 
-				if (!(prompt_string_default(tmp, tinrc.default_pattern, _(txt_no_match), HIST_REGEX_PATTERN))) {
+				if (!(prompt_string_ptr_default(tmp, &tinrc.default_pattern, _(txt_no_match), HIST_REGEX_PATTERN))) {
 					free(tmp);
 					return GLOBAL_ABORT;
 				}
@@ -412,7 +413,7 @@ feed_article(
 	struct t_counters *counter,	/* Accounting */
 	t_bool use_current,		/* Use already open pager article */
 	const char *data,		/* Extra data if needed, print command or save filename */
-	struct t_group *group)
+	const struct t_group *group)
 {
 	char *progress_mesg = NULL;
 	t_bool ok = TRUE;		/* Assume success */
@@ -469,6 +470,7 @@ feed_article(
 					break;
 
 				case POSTED_OK:
+				default:
 					break;
 			}
 			confirm = bool_not(ok);		/* Only confirm the next one after a failure */
@@ -491,9 +493,10 @@ feed_article(
 #ifndef DONT_HAVE_PIPING
 		case FEED_PIPE:
 			rewind(openartptr->raw);
-			ok = copy_fp(openartptr->raw, pipe_fp);
-			if (errno == EPIPE)	/* broken pipe in copy_fp() */
+			if (copy_fp(openartptr->raw, pipe_fp) == EPIPE) {	/* broken pipe in copy_fp() */
 				got_epipe = TRUE;
+				ok = FALSE;
+			}
 			break;
 #endif /* !DONT_HAVE_PIPING */
 
@@ -568,13 +571,13 @@ feed_articles(
 	int respnum)
 {
 	char *prompt;
-	char outpath[PATH_LEN];
+	char outpath[PATH_LEN]; /* FIXME: don't used fixed length */
 	int art;
 	int i;
 	int saved_curr_line = -1;
 	int thread_base;
 	struct t_art_stat sbuf;
-	struct t_counters counter = {0, 0, 0};
+	struct t_counters counter = { 0, 0, 0 };
 	t_bool feed_mark_function = function == FEED_MARK_READ || function == FEED_MARK_UNREAD;
 	t_bool mark_saved = FALSE;
 	t_bool no_next_unread = FALSE;
@@ -623,7 +626,7 @@ feed_articles(
 		/* Setup mail - get address to mail to */
 		case FEED_MAIL:
 			prompt = fmt_string(_(txt_mail_art_to), cCOLS - (strwidth(_(txt_mail_art_to)) > cCOLS - 30 ? cCOLS - 30 : strwidth(_(txt_mail_art_to)) + 30), tinrc.default_mail_address);
-			if (!(prompt_string_default(prompt, tinrc.default_mail_address, _(txt_no_mail_address), HIST_MAIL_ADDRESS))) {
+			if (!(prompt_string_ptr_default(prompt, &tinrc.default_mail_address, _(txt_no_mail_address), HIST_MAIL_ADDRESS))) {
 				free(prompt);
 				return -1;
 			}
@@ -634,7 +637,7 @@ feed_articles(
 		/* Setup pipe - get pipe-to command and open the pipe */
 		case FEED_PIPE:
 			prompt = fmt_string(_(txt_pipe_to_command), cCOLS - (strwidth(_(txt_pipe_to_command)) > cCOLS - 30 ? cCOLS - 30 : strwidth(_(txt_pipe_to_command)) + 30), tinrc.default_pipe_command);
-			if (!(prompt_string_default(prompt, tinrc.default_pipe_command, _(txt_no_command), HIST_PIPE_COMMAND))) {
+			if (!(prompt_string_ptr_default(prompt, &tinrc.default_pipe_command, _(txt_no_command), HIST_PIPE_COMMAND))) {
 				free(prompt);
 				return -1;
 			}
@@ -657,8 +660,16 @@ feed_articles(
 
 #ifndef DISABLE_PRINTING
 		/* Setup printing - get print command line */
+		/* FIXME: don't use fixed length buffer */
 		case FEED_PRINT:
-			snprintf(outpath, sizeof(outpath), "%s %s", tinrc.printer, REDIRECT_OUTPUT);
+			{
+				int l = sizeof(outpath) - strlen(tinrc.printer) - strlen(REDIRECT_OUTPUT) - 2;
+
+				if (l > 0)
+					snprintf(outpath, sizeof(outpath), "%.*s %s", l, tinrc.printer, REDIRECT_OUTPUT);
+				else
+					*outpath = '\0';
+			}
 			break;
 #endif /* !DISABLE_PRINTING */
 
@@ -710,14 +721,14 @@ feed_articles(
 		case FEED_REPOST:
 			{
 				char *tmp;
+				char *smsg;
 #ifndef FORGERY
 				char from_name[PATH_LEN];
 
 				get_from_name(from_name, (struct t_group *) 0);
 
-				if (strstr(from_name, arts[respnum].from)) {
+				if (strstr(from_name, arts[respnum].mailbox.from)) {
 #endif /* !FORGERY */
-					char *smsg;
 					char buf[LEN];
 					char keyrepost[MAXKEYLEN], keysupersede[MAXKEYLEN];
 					char keyquit[MAXKEYLEN];
@@ -735,12 +746,12 @@ feed_articles(
 
 					switch (func) {
 						case FEED_SUPERSEDE:
-							tmp = fmt_string(_(txt_supersede_group), tinrc.default_repost_group);
+							tmp = fmt_string(_(txt_supersede_group), BlankIfNull(tinrc.default_repost_group));
 							supersede = TRUE;
 							break;
 
 						case FEED_KEY_REPOST:
-							tmp = fmt_string(_(txt_repost_group), tinrc.default_repost_group);
+							tmp = fmt_string(_(txt_repost_group), BlankIfNull(tinrc.default_repost_group));
 							supersede = FALSE;
 							break;
 
@@ -750,14 +761,16 @@ feed_articles(
 					}
 #ifndef FORGERY
 				} else {
-					tmp = fmt_string(_(txt_repost_group), tinrc.default_repost_group);
+					tmp = fmt_string(_(txt_repost_group), BlankIfNull(tinrc.default_repost_group));
 					supersede = FALSE;
 				}
 #endif /* !FORGERY */
-				if (!(prompt_string_default(tmp, tinrc.default_repost_group, _(txt_no_group), HIST_REPOST_GROUP))) {
+				if (!(smsg = prompt_string_default(tmp, tinrc.default_repost_group, _(txt_no_group), HIST_REPOST_GROUP))) {
+					FreeIfNeeded(smsg);
 					free(tmp);
 					return -1;
 				}
+				tinrc.default_repost_group = smsg;
 				free(tmp);
 			}
 			break;
@@ -1053,8 +1066,8 @@ print_file(
 	t_openartinfo *artinfo)
 {
 	FILE *fp;
-	struct t_header *hdr = &artinfo->hdr;
-	t_bool ok;
+	const struct t_header *hdr = &artinfo->hdr;
+	t_bool ok = FALSE;
 #	ifdef DONT_HAVE_PIPING
 	char cmd[PATH_LEN], file[PATH_LEN];
 	int i;
@@ -1064,28 +1077,29 @@ print_file(
 	snprintf(file, sizeof(file), TIN_PRINTFILE, respnum);
 	if ((fp = fopen(file, "w")) == NULL) /* TODO: issue a more correct error message here */
 #	else
-	if ((fp = popen(command, "w")) == NULL)
+	if (!command || !*command || (fp = popen(command, "w")) == NULL)
 #	endif /* DONT_HAVE_PIPING */
 	{
-		perror_message(_(txt_command_failed), command);
-		return FALSE;
+		perror_message(_(txt_command_failed), BlankIfNull(command));
+		return ok;
 	}
 
 	rewind(artinfo->raw);
 	if (!curr_group->attribute->print_header && !(fseek(artinfo->raw, hdr->ext->offset, SEEK_SET))) {	/* -> start of body */
 		if (hdr->newsgroups)
 			fprintf(fp, "Newsgroups: %s\n", hdr->newsgroups);
-		if (arts[respnum].from == arts[respnum].name || arts[respnum].name == NULL)
-			fprintf(fp, "From: %s\n", arts[respnum].from);
+		if (arts[respnum].mailbox.from == arts[respnum].mailbox.name || arts[respnum].mailbox.name == NULL)
+			fprintf(fp, "From: %s\n", arts[respnum].mailbox.from);
 		else
-			fprintf(fp, "From: %s <%s>\n", arts[respnum].name, arts[respnum].from);
+			fprintf(fp, "From: %s <%s>\n", arts[respnum].mailbox.name, arts[respnum].mailbox.from);
 		if (hdr->subj)
 			fprintf(fp, "Subject: %s\n", hdr->subj);
 		if (hdr->date)
 			fprintf(fp, "Date: %s\n\n", hdr->date);
 	}
 
-	ok = copy_fp(artinfo->raw, fp);
+	if (copy_fp(artinfo->raw, fp) == 0)
+		ok = TRUE;
 
 #	ifdef DONT_HAVE_PIPING
 	fclose(fp);
