@@ -3,7 +3,7 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2023-11-22
+ *  Updated   : 2024-05-05
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
  * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
@@ -216,7 +216,7 @@ test_match(
 		if (!cache->re)
 			compile_regex(regex, cache, (nocase ? REGEX_CASELESS : 0));
 		if (cache->re) {
-			error = match_regex_ex(string, (int) strlen(string), 0, 0, cache);
+			error = match_regex_ex(string, (REGEX_SIZE) strlen(string), 0, 0, cache);
 			if (error >= 0)
 				return 1;
 			else if (error != REGEX_ERROR_NOMATCH) { /* also exclude BADUTF8 ? */
@@ -333,11 +333,11 @@ read_filter_file(
 	static t_bool first_read = TRUE;
 	struct t_version *upgrade = NULL;
 
-	if ((fp = fopen(file, "r")) == NULL)
+	if ((fp = tin_fopen(file, "r")) == NULL)
 		return FALSE;
 
 	if (!batch_mode || verbose)
-		wait_message(0, _(txt_reading_filter_file));
+		wait_message(0, _(txt_reading_filter_file), file);
 
 	(void) time(&current_secs);
 
@@ -360,7 +360,7 @@ read_filter_file(
 				no_version_line = FALSE;
 				upgrade = check_upgrade(buf, "# Filter file V", FILTER_VERSION);
 				if (upgrade->state != RC_IGNORE)
-					upgrade_prompt_quit(upgrade, file); /* FILTER_FILE */ /* TODO: do something (more) useful here */
+					upgrade_prompt_quit(upgrade, file, fp); /* FILTER_FILE */ /* TODO: do something (more) useful here */
 			}
 			continue;
 		}
@@ -428,13 +428,13 @@ read_filter_file(
 					if (ptr && !expired_time) {
 						if (gnksa[0] == '<') {
 							ptr[i].gnksa_cmp = FILTER_LINES_LT;
-							ptr[i].gnksa_num = atoi(&gnksa[1]);
+							ptr[i].gnksa_num = s2i(&gnksa[1], GNKSA_OK, GNKSA_MISSING_REALNAME);
 						} else if (gnksa[0] == '>') {
 							ptr[i].gnksa_cmp = FILTER_LINES_GT;
-							ptr[i].gnksa_num = atoi(&gnksa[1]);
+							ptr[i].gnksa_num = s2i(&gnksa[1], GNKSA_OK, GNKSA_MISSING_REALNAME);
 						} else {
 							ptr[i].gnksa_cmp = FILTER_LINES_EQ;
-							ptr[i].gnksa_num = atoi(gnksa);
+							ptr[i].gnksa_num = s2i(gnksa, GNKSA_OK, GNKSA_MISSING_REALNAME);
 						}
 					}
 				}
@@ -445,13 +445,13 @@ read_filter_file(
 					if (ptr && !expired_time) {
 						if (buffer[0] == '<') {
 							ptr[i].lines_cmp = FILTER_LINES_LT;
-							ptr[i].lines_num = atoi(&buffer[1]);
+							ptr[i].lines_num = s2i(&buffer[1], 0, INT_MAX);
 						} else if (buffer[0] == '>') {
 							ptr[i].lines_cmp = FILTER_LINES_GT;
-							ptr[i].lines_num = atoi(&buffer[1]);
+							ptr[i].lines_num = s2i(&buffer[1], 0, INT_MAX);
 						} else {
 							ptr[i].lines_cmp = FILTER_LINES_EQ;
-							ptr[i].lines_num = atoi(buffer);
+							ptr[i].lines_num = s2i(buffer, 0, INT_MAX);
 						}
 					}
 				}
@@ -562,26 +562,18 @@ read_filter_file(
 				 * read score for rule
 				 */
 				if (match_string(buf + 1, "core=", scbuf, PATH_LEN)) {
-					score = atoi(scbuf);
+					score = s2i(scbuf, -SCORE_MAX, SCORE_MAX);
 #ifdef DEBUG
 					if (debug & DEBUG_FILTER)
 						debug_print_file("FILTER", "score=[%d]", score);
 #endif /* DEBUG */
 					if (ptr && !expired_time) {
-						if (score > SCORE_MAX)
-							score = SCORE_MAX;
-						else {
-							if (score < -SCORE_MAX)
-								score = -SCORE_MAX;
+						if (!score) {
+							if (!strncasecmp(scbuf, "kill", 4))
+								score = tinrc.score_kill;
 							else {
-								if (!score) {
-									if (!strncasecmp(scbuf, "kill", 4))
-										score = tinrc.score_kill;
-									else {
-										if (!strncasecmp(scbuf, "hot", 3))
-											score = tinrc.score_select;
-									}
-								}
+								if (!strncasecmp(scbuf, "hot", 3))
+									score = tinrc.score_select;
 							}
 						}
 						ptr[i].score = score;
@@ -660,7 +652,7 @@ read_filter_file(
 		upgrade = my_malloc(sizeof(struct t_version));
 		upgrade->state = RC_UPGRADE;
 		upgrade->file_version = -1;
-		upgrade_prompt_quit(upgrade, file); /* TODO: do something (more) useful here */
+		upgrade_prompt_quit(upgrade, file, NULL); /* TODO: do something (more) useful here */
 	}
 
 	if (need_write || (upgrade && upgrade->state == RC_UPGRADE))
@@ -699,6 +691,7 @@ write_filter_file(
 	}
 
 	if ((fp = fopen(filename, "w+")) == NULL) {
+		perror_message("write_filter_file(%s)", filename);
 		free(file_tmp);
 		return;
 	}
@@ -1150,7 +1143,7 @@ filter_menu(
 
 	snprintf(text_time, sizeof(text_time), _(txt_time_default_days), tinrc.filter_days);
 	fmt_filter_menu_prompt(text_subj, sizeof(text_subj), ptr_filter_subj, len, art->subject);
-	snprintf(text_score, sizeof(text_score), _(txt_filter_score), (type == GLOBAL_MENU_FILTER_KILL ? -tinrc.score_kill : tinrc.score_select));
+	snprintf(text_score, sizeof(text_score), _(txt_filter_score), (type == GLOBAL_MENU_FILTER_KILL ? tinrc.score_kill : tinrc.score_select));
 	fmt_filter_menu_prompt(text_from, sizeof(text_from), ptr_filter_from, len, art->from);
 	fmt_filter_menu_prompt(text_msgid, sizeof(text_msgid), ptr_filter_msgid, len - 4, MSGID(art));
 
@@ -1342,7 +1335,7 @@ filter_menu(
 	}
 
 	if (*ptr)
-		rule.lines_num = abs(atoi(ptr));
+		rule.lines_num = s2i(ptr, 0, INT_MAX);
 
 	if (rule.lines_num && rule.lines_cmp == FILTER_LINES_NO)
 		rule.lines_cmp = FILTER_LINES_EQ;
@@ -1365,7 +1358,7 @@ filter_menu(
 	/* check if a score has been entered */
 	if (buf[0] != '\0')
 		/* use entered score */
-		rule.score = atoi(buf);
+		rule.score = s2i(buf, -SCORE_MAX, SCORE_MAX);
 	else {
 		/* use default score */
 		if (type == GLOBAL_MENU_FILTER_KILL)
@@ -1395,7 +1388,7 @@ filter_menu(
 	 * Expire time
 	 */
 	double_time = my_malloc(strlen(text_time) + 4); /* "2x " prefix */
-	quat_time = my_malloc(strlen(text_time) + 4); /* "4x " prefix*/
+	quat_time = my_malloc(strlen(text_time) + 4); /* "4x " prefix */
 	sprintf(double_time, "2x %s", text_time);
 	sprintf(quat_time, "4x %s", text_time);
 	list = my_malloc(sizeof(char *) * 4);
@@ -1550,7 +1543,7 @@ quick_filter(
 	rule.comment = add_filter_comment(NULL, txt);
 
 	rule.text[0] = '\0';
-	rule.icase = icase;
+	rule.icase = icase ? 1 : 0;
 	rule.expire_time = expire;
 	rule.check_string = TRUE;
 	rule.score = (type == GLOBAL_QUICK_FILTER_KILL) ? tinrc.score_kill : tinrc.score_select;
@@ -1858,7 +1851,7 @@ filter_articles(
 	struct regex_cache *regex_cache_path = NULL;
 	t_bool filtered = FALSE;
 	t_bool error = FALSE;
-	t_bool use_regex = tinrc.wildcard;
+	t_bool use_regex = tinrc.wildcard ? TRUE : FALSE;
 
 	/*
 	 * check if there are any global filter rules
@@ -1937,6 +1930,9 @@ filter_articles(
 
 				/*
 				 * Filter on From: line
+				 *
+				 * TODO: switch to angle-addr format (RFC 5322 3.4)
+				 *       as that is what we use elsewhere?
 				 */
 				if (ptr[j].from != NULL) {
 					if (arts[i].name != NULL)
@@ -2004,6 +2000,8 @@ filter_articles(
 						default: /* should not happen */
 							/* CONSTANTCONDITION */
 							assert(0 != 0);
+							myrefs = "";
+							mymsgid = "";
 							break;
 					}
 
@@ -2098,9 +2096,9 @@ filter_articles(
 
 						s = arts[i].xref;
 						if (strchr(s, ' ') || strchr(s, '\t')) {
-							while (*s && !isspace((int) *s))	/* skip server name */
+							while (*s && !isspace((unsigned char) *s))	/* skip server name */
 								s++;
-							while (*s && isspace((int) *s))
+							while (*s && isspace((unsigned char) *s))
 								s++;
 						}
 #ifdef DEBUG
@@ -2119,12 +2117,21 @@ filter_articles(
 									*e++ = ',';
 									skip = TRUE;
 								}
-								if (!skip && *s != ':' && !isspace((int) *s))
+								if (!skip && *s != ':' && !isspace((unsigned char) *s))
 									*e++ = *s;
-								if (isspace((int) *s))
+								if (isspace((unsigned char) *s))
 									skip = FALSE;
 								s++;
 							}
+							if (e == k) {
+#ifdef DEBUG
+								if (debug & DEBUG_FILTER)
+									debug_print_file("FILTER", _(txt_filter_error_skipping_xref_filter));
+#endif /* DEBUG */
+								error = TRUE;
+								free(k);
+								break;
+							}
 							*--e = '\0';
 						} else {
 #ifdef DEBUG
@@ -2233,7 +2240,7 @@ filter_on_path(
 	struct t_group *group)
 {
 	int i;
-	struct t_filter *flt;
+	const struct t_filter *flt;
 	t_bool ret = FALSE;
 
 	if (group->glob_filter->num == 0)
@@ -2290,7 +2297,7 @@ open_xhdr_fp(
 	if (read_news_via_nntp && !read_saved_news && nntp_caps.hdr_cmd) {
 		char buf[NNTP_STRLEN];
 
-		snprintf(buf, sizeof(buf), "%s %s %"T_ARTNUM_PFMT,"-%"T_ARTNUM_PFMT,, nntp_caps.hdr_cmd, header, min, max);
+		snprintf(buf, sizeof(buf), "%s %s %"T_ARTNUM_PFMT, "-%"T_ARTNUM_PFMT, nntp_caps.hdr_cmd, header, min, max);
 		return (nntp_command(buf, OK_HEAD, NULL, 0));
 	} else
 #	endif /* NNTP_ABLE */
