@@ -3,7 +3,7 @@
  *  Module    : getline.c
  *  Author    : Chris Thewalt & Iain Lea
  *  Created   : 1991-11-09
- *  Updated   : 2021-02-23
+ *  Updated   : 2024-06-04
  *  Notes     : emacs style line editing input package.
  *  Copyright : (c) Copyright 1991-99 by Chris Thewalt & Iain Lea
  *              Permission to use, copy, modify, and distribute this
@@ -22,7 +22,6 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-#define BUF_SIZE	1024
 #define TAB_SIZE		4
 
 #define CTRL_A	'\001'
@@ -72,12 +71,12 @@ static void hist_next(int w);
 static void hist_prev(int w);
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static int (*gl_in_hook) (wchar_t *) = 0;
-	static int (*gl_out_hook) (wchar_t *) = 0;
+	static int (*gl_in_hook) (wchar_t *) = NULL;
+	static int (*gl_out_hook) (wchar_t *) = NULL;
 	static int (*gl_tab_hook) (wchar_t *, int, int *) = gl_tab;
 #else
-	static int (*gl_in_hook) (char *) = 0;
-	static int (*gl_out_hook) (char *) = 0;
+	static int (*gl_in_hook) (char *) = NULL;
+	static int (*gl_out_hook) (char *) = NULL;
 	static int (*gl_tab_hook) (char *, int, int *) = gl_tab;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
@@ -474,15 +473,15 @@ gl_kill_back_word(
 		;
 #else
 	/* delete spaces */
-	for (i = gl_pos - 1; i >= 0 && isspace((int) gl_buf[i]); --i)
+	for (i = gl_pos - 1; i >= 0 && isspace((unsigned char) gl_buf[i]); --i)
 		;
 
 	/* delete not alnum characters but graph characters */
-	for (; i >= 0 && isgraph((int) gl_buf[i]) && !isalnum((int) gl_buf[i]); --i)
+	for (; i >= 0 && isgraph((unsigned char) gl_buf[i]) && !isalnum((unsigned char) gl_buf[i]); --i)
 		;
 
 	/* delete all graph characters except '/' */
-	for (; i >= 0 && gl_buf[i] != '/' && isgraph((int) gl_buf[i]); --i)
+	for (; i >= 0 && gl_buf[i] != '/' && isgraph((unsigned char) gl_buf[i]); --i)
 		;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
