@@ -3,10 +3,10 @@
  *  Module    : group.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-07-29
+ *  Updated   : 2024-02-22
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -458,7 +458,7 @@ group_page(
 				break;
 
 			case GLOBAL_CONNECTION_INFO:
-				show_connection_page(GROUP_LEVEL, _(txt_connection_info));
+				show_connection_page();
 				show_group_page();
 				break;
 
@@ -862,7 +862,7 @@ group_page(
 
 			case GROUP_SELECT_PATTERN:	/* select matching patterns */
 				if (grpmenu.curr >= 0) {
-					char pat[128];
+					char pat[LEN];
 					char *prompt;
 					struct regex_cache cache = REGEX_CACHE_INITIALIZER;
 
@@ -1175,6 +1175,10 @@ mark_screen(
 		int y, x;
 
 		getyx(stdscr, y, x);
+		if (x < 0)
+			x = 0;
+		if (y < 0)
+			y = 0;
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if ((tmp = wchar_t2char(value))) {
 			mvaddstr(INDEX2LNUM(screen_row), screen_col, tmp);
@@ -1374,11 +1378,14 @@ build_sline(
 					grp_fmt.mark_offset = (size_t) (mark_offset = strwidth(buffer) + 2);
 				if ((k = line_is_tagged(respnum))) {
 					STRCPY(tmp_buf, tin_ltoa(k, 3));
-					strcat(buffer, " ");
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+					if (art_mark_width > 1)
+						strcat(buffer, " ");
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 					tagged = TRUE;
 				} else
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-					snprintf(tmp_buf, sizeof(tmp_buf), "%s%lc", art_mark_width > wcwidth(sbuf.art_mark) ? "   " : "  ", sbuf.art_mark);
+					snprintf(tmp_buf, sizeof(tmp_buf), "%s%"T_CHAR_FMT, art_mark_width > wcwidth(sbuf.art_mark) ? "   " : "  ", (wint_t) sbuf.art_mark);
 #else
 					snprintf(tmp_buf, sizeof(tmp_buf), "  %c", sbuf.art_mark);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1518,29 +1525,29 @@ show_group_title(
 	if ((cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit)
 		snprintf(tmp, sizeof(tmp), " %d/%d%"T_CHAR_FMT,
 			(cmdline.args & CMDLINE_GETART_LIMIT) ? cmdline.getart_limit : tinrc.getart_limit, art_cnt,
-			(curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
+			(wint_t) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	else
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
 			art_cnt,
-			(curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
+			(wint_t) (curr_group->attribute->show_only_unread_arts ? tinrc.art_marked_unread : tinrc.art_marked_read));
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
 	/* selected articles */
 	if (curr_group->attribute->show_only_unread_arts)
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-			selected_art_cnt, tinrc.art_marked_selected);
+			selected_art_cnt, (wint_t) tinrc.art_marked_selected);
 	else
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT" %d%"T_CHAR_FMT,
-			selected_art_cnt, tinrc.art_marked_selected,
-			read_selected_art_cnt, tinrc.art_marked_read_selected);
+			selected_art_cnt, (wint_t) tinrc.art_marked_selected,
+			read_selected_art_cnt, (wint_t) tinrc.art_marked_read_selected);
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
 
 	/* recent articles */
 	if (tinrc.recent_time) {
 		snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-			recent_art_cnt, tinrc.art_marked_recent);
+			recent_art_cnt, (wint_t) tinrc.art_marked_recent);
 
 		if (sizeof(buf) > strlen(buf) + strlen(tmp))
 			strcat(buf, tmp);
@@ -1548,7 +1555,7 @@ show_group_title(
 
 	/* killed articles */
 	snprintf(tmp, sizeof(tmp), " %d%"T_CHAR_FMT,
-		killed_art_cnt, tinrc.art_marked_killed);
+		killed_art_cnt, (wint_t) tinrc.art_marked_killed);
 
 	if (sizeof(buf) > strlen(buf) + strlen(tmp))
 		strcat(buf, tmp);
