@@ -3,9 +3,9 @@
  *  Module    : header.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-03-10
- *  Updated   : 2023-03-15
+ *  Updated   : 2024-08-28
  *
- * Copyright (c) 1997-2023 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2024 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -52,7 +52,7 @@ const char *
 get_host_name(
 	void)
 {
-	char *ptr;
+	const char *ptr;
 	static char hostname[MAXHOSTNAMELEN + 1]; /* need space for '\0' */
 
 	hostname[0] = '\0';
@@ -65,14 +65,11 @@ get_host_name(
 		my_strncpy(hostname, system_info.nodename, sizeof(hostname) - 1);
 #endif /* HAVE_SYS_UTSNAME_H */
 	if (!*hostname) {
-		if ((ptr = getenv("HOST")) != NULL)
+		ptr = get_val("HOST", get_val("HOSTNAME", ""));
+		if (*ptr)
 			my_strncpy(hostname, ptr, sizeof(hostname) - 1);
-		else {
-			if ((ptr = getenv("HOSTNAME")) != NULL)
-				my_strncpy(hostname, ptr, sizeof(hostname) - 1);
-			else
-				hostname[0] = '\0';
-		}
+		else
+			hostname[0] = '\0';
 	}
 	hostname[MAXHOSTNAMELEN] = '\0';
 	return hostname;
@@ -100,7 +97,7 @@ get_domain_name(
 
 	if (domain[0] == '/' && domain[1]) {
 		/* read domainname from specified file */
-		if ((fp = fopen(domain, "r")) != NULL) {
+		if ((fp = tin_fopen(domain, "r")) != NULL) {
 			while (fgets(buff, (int) sizeof(buff), fp) != NULL) {
 				if (buff[0] == '#' || buff[0] == '\n')
 					continue;
@@ -110,10 +107,10 @@ get_domain_name(
 					strcpy(domain, buff);
 				}
 			}
+			fclose(fp);
+			str_trim(domain);
 			if (domain[0] == '/')	/* '/' is not allowed in domainames -> file was empty */
 				domain[0] = '\0';
-
-			fclose(fp);
 		} else
 			domain[0] = '\0';
 	}
@@ -125,7 +122,7 @@ get_domain_name(
 
 #ifdef HAVE_GETHOSTBYNAME
 #	define MAXLINELEN	1024
-#	define WS	" \f\t\v"
+#	define WSP	" \f\t\v"
 /* find FQDN - gethostbyaddr() */
 const char *
 get_fqdn(
@@ -136,7 +133,7 @@ get_fqdn(
 	char name[MAXHOSTNAMELEN + 1];
 	static char fqdn[1024];
 	struct hostent *hp;
-	struct in_addr in = {0};
+	struct in_addr in = { 0 };
 
 	*fqdn = '\0';
 	domain = NULL;
@@ -153,20 +150,22 @@ get_fqdn(
 			return NULL;
 	}
 
+	/* FIXME: this does not do IPv6 only */
 #	ifdef HAVE_INET_ADDR
 	if (*name >= '0' && *name <= '9') {
 		in_addr_t addr = inet_addr(name);
 
-		if ((hp = gethostbyaddr((char *) &addr, 4, AF_INET)))
+		if ((hp = gethostbyaddr((char *) &addr, 4, AF_INET))) {
 #		ifdef HAVE_HOSTENT_H_ADDR_LIST
 			in.s_addr = (*hp->h_addr_list[0]);
 #		else
 			in.s_addr = (*hp->h_addr);
 #		endif /* HAVE_HOSTENT_H_ADDR_LIST */
+		}
 		return (hp && strchr(hp->h_name, '.') ? hp->h_name : inet_ntoa(in));
 	}
 #	endif /* HAVE_INET_ADDR */
-	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
+	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.')) {
 #	ifdef HAVE_HOSTENT_H_ADDR_LIST
 		if ((hp = gethostbyaddr(hp->h_addr_list[0], hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr_list[0]);
@@ -174,17 +173,18 @@ get_fqdn(
 		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
 			in.s_addr = (*hp->h_addr);
 #	endif /* HAVE_HOSTENT_H_ADDR_LIST */
+	}
 	snprintf(fqdn, sizeof(fqdn), "%s", hp
 		? strchr(hp->h_name, '.')
 			? hp->h_name : inet_ntoa(in)
 		: "");
 
-	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
+	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) { /* see FIXME above about IPv6 */
 		FILE *inf;
 
 		*fqdn = '\0';
 
-		if ((inf = fopen("/etc/resolv.conf", "r")) != NULL) {
+		if ((inf = tin_fopen("/etc/resolv.conf", "r")) != NULL) {
 			char *eos;
 			int j;
 
@@ -194,10 +194,10 @@ get_fqdn(
 
 				line[MAXLINELEN] = '\0';
 
-				if ((eos = strpbrk(line, WS)) != NULL) {
+				if ((eos = strpbrk(line, WSP)) != NULL) {
 					if ((j = (int) (eos - line))) {
 						if (!strncmp(line, "domain", (size_t) j) || !strncmp(line, "search", (size_t) j)) {
-							domain = strtok(eos, WS);
+							domain = strtok(eos, WSP);
 							break;
 						}
 					}
@@ -209,7 +209,7 @@ get_fqdn(
 			fclose(inf);
 		}
 	}
-	return fqdn;
+	return *fqdn ? fqdn : get_val("HOST", get_val("HOSTNAME", ""));
 }
 #endif /* HAVE_GETHOSTBYNAME */
 
@@ -258,6 +258,7 @@ static const char *
 get_full_name(
 	void)
 {
+	const char *q;
 	char *p;
 	static char fullname[128];
 #ifndef DONT_HAVE_PW_GECOS
@@ -268,26 +269,31 @@ get_full_name(
 
 	fullname[0] = '\0';
 
-	if ((p = getenv("NAME")) != NULL) {
-		my_strncpy(fullname, p, sizeof(fullname) - 1);
-		return fullname;
-	}
-	if ((p = getenv("REALNAME")) != NULL) {
-		my_strncpy(fullname, p, sizeof(fullname) - 1);
+	/*
+	 * ignore set but empty env-var.
+	 * give mail_address in tinrc without a name instead if you
+	 * want this behaviour.
+	 */
+	if ((q = get_val("NAME", get_val("REALNAME", NULL))) != NULL) {
+		my_strncpy(fullname, q, sizeof(fullname) - 1);
 		return fullname;
 	}
 
 #ifndef DONT_HAVE_PW_GECOS
 	if ((pw = getpwuid(getuid())) != NULL) {
+		int ret;
+
 		STRCPY(buf, pw->pw_gecos);
 		if ((p = strchr(buf, ',')))
 			*p = '\0';
 		if ((p = strchr(buf, '&'))) {
 			*p++ = '\0';
 			STRCPY(tmp, pw->pw_name);
-			if (*tmp && isalpha((int)(unsigned char) *tmp) && islower((int)(unsigned char) *tmp))
-				*tmp = (char) my_toupper((int) (unsigned char) *tmp);
-			snprintf(fullname, sizeof(fullname), "%s%s%s", buf, tmp, p);
+			if (*tmp && isalpha((unsigned char) *tmp) && islower((unsigned char) *tmp))
+				*tmp = (char) my_toupper((unsigned char) *tmp);
+			ret = snprintf(fullname, sizeof(fullname), "%s%s%s", buf, tmp, p);
+			if (ret == -1 || ret > (int) sizeof(fullname))
+				error_message(2, "Fullname truncated");
 		} else
 			STRCPY(fullname, buf);
 	}
@@ -310,8 +316,8 @@ get_from_name(
 {
 	char *fromhost = domain_name;
 
-	if (thisgrp && *thisgrp->attribute->from != '\0') {
-		strcpy(from_name, thisgrp->attribute->from);
+	if (thisgrp && thisgrp->attribute->from && *thisgrp->attribute->from) {
+		strcpy(from_name, *thisgrp->attribute->from);
 		return;
 	}
 
@@ -328,26 +334,30 @@ get_from_name(
  * build_sender()
  * returns *(Full_Name <user@fq.domainna.me>)
  */
-#ifndef FORGERY
+#if !defined(FORGERY) && defined(NNTP_ABLE)
 char *
 build_sender(
 	void)
 {
-	const char *ptr;
+	const char *ptr = NULL;
 	static char sender[8192];
 
 	sender[0] = '\0';
 
-	if ((ptr = get_full_name()))
+	if ((ptr = get_full_name())) /* TODO: rfc2047 encode */
 		snprintf(sender, sizeof(sender), ((strpbrk(ptr, "\".:;<>@[]()\\")) ? "\"%s\"" : "%s "), ptr);
 	if ((ptr = get_user_name())) {
 		snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "<%s@", ptr);
 
 #	ifdef HAVE_GETHOSTBYNAME
-		if ((ptr = get_fqdn(get_host_name())))
+		ptr = get_fqdn(get_host_name());
 #	else
-		if ((ptr = get_host_name()))
+		ptr = get_host_name();
 #	endif /* HAVE_GETHOSTBYNAME */
+
+		/* intentionally do not fall back to *domain_name */
+
+		if (*ptr)
 			snprintf(sender + strlen(sender), sizeof(sender) - strlen(sender), "%s>", ptr);
 		else
 			return NULL;
@@ -356,4 +366,4 @@ build_sender(
 
 	return sender;
 }
-#endif /* !FORGERY */
+#endif /* !FORGERY && NNTP_ABLE */
