@@ -3,10 +3,10 @@
  *  Module    : help.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2022-11-01
+ *  Updated   : 2024-09-18
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -99,6 +99,8 @@ static t_help_page attachment_help_page[] = {
 #ifndef NO_SHELL_ESCAPE
 	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
 #endif /* !NO_SHELL_ESCAPE */
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
 };
 
@@ -432,6 +434,8 @@ static t_help_page thread_help_page[] = {
 	{ txt_help_article_next_unread, THREAD_READ_NEXT_ARTICLE_OR_THREAD },
 #ifndef NO_POSTING
 	{ txt_help_global_post, GLOBAL_POST },
+	{ txt_help_article_followup, THREAD_FOLLOWUP_QUOTE },
+	{ txt_help_article_followup_no_quote, THREAD_FOLLOWUP },
 	{ txt_help_global_post_postponed, GLOBAL_POSTPONED },
 	{ txt_help_article_cancel, THREAD_CANCEL },
 #endif /* NO_POSTING */
@@ -595,6 +599,7 @@ static t_help_page page_help_page[] = {
 #endif /* HAVE_PGP_GPG */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_connection_info, GLOBAL_CONNECTION_INFO },
+	{ txt_help_article_info, PAGE_ARTICLE_INFO },
 	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
 };
@@ -621,6 +626,10 @@ static t_help_page post_hist_help_page[] = {
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_disp, NOT_ASSIGNED },
 	{ txt_help_post_hist_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_global_toggle_inverse_video, GLOBAL_TOGGLE_INVERSE_VIDEO },
+#ifdef HAVE_COLOR
+	{ txt_help_global_toggle_color, GLOBAL_TOGGLE_COLOR },
+#endif /* HAVE_COLOR */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_misc, NOT_ASSIGNED },
 	{ txt_help_select_quit, GLOBAL_QUIT },
@@ -631,6 +640,8 @@ static t_help_page post_hist_help_page[] = {
 #ifndef NO_SHELL_ESCAPE
 	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
 #endif /* !NO_SHELL_ESCAPE */
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
 };
 
@@ -656,6 +667,10 @@ static t_help_page url_help_page[] = {
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_disp, NOT_ASSIGNED },
 	{ txt_help_url_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_global_toggle_inverse_video, GLOBAL_TOGGLE_INVERSE_VIDEO },
+#ifdef HAVE_COLOR
+	{ txt_help_global_toggle_color, GLOBAL_TOGGLE_COLOR },
+#endif /* HAVE_COLOR */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_misc, NOT_ASSIGNED },
 	{ txt_help_select_quit, GLOBAL_QUIT },
@@ -666,6 +681,8 @@ static t_help_page url_help_page[] = {
 #ifndef NO_SHELL_ESCAPE
 	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
 #endif /* !NO_SHELL_ESCAPE */
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
 };
 
@@ -702,7 +719,7 @@ make_help_page(
 			 */
 			buf = my_realloc(buf, LEN);
 
-			if (!strlen(helppage->helptext))	/* avoid translation of empty strings */
+			if (!*helppage->helptext)	/* avoid translation of empty strings */
 				strcpy(buf, "\n");
 			else
 				strncpy(buf, _(helppage->helptext), LEN);
@@ -717,7 +734,7 @@ make_help_page(
 					snprintf(buf, LEN, "%s\t  %s", printascii(key, (wint_t) keys.list[i].key), _(helppage->helptext));
 #else
 					snprintf(buf, LEN, "%s\t  %s", printascii(key, keys.list[i].key), _(helppage->helptext));
-#endif /* MULTIBYTE_ABLE &&! NO_LOCALE */
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 					expand_ctrl_chars(&buf, &length, 8);
 					if (strcmp(last, buf)) {
 						fprintf(fp, "%s\n", buf);
@@ -741,7 +758,7 @@ show_help_page(
 {
 	FILE *fp;
 
-	if (!(fp = tmpfile()))
+	if (!(fp = my_tmpfile()))
 		return;
 
 	switch (level) {
