@@ -3,10 +3,10 @@
  *  Module    : inews.c
  *  Author    : I. Lea
  *  Created   : 1992-03-17
- *  Updated   : 2021-02-23
+ *  Updated   : 2024-11-08
  *  Notes     : NNTP built in version of inews
  *
- * Copyright (c) 1991-2022 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,25 +50,12 @@
  * local prototypes
  */
 #ifdef NNTP_INEWS
-	static t_bool submit_inews(char *name, struct t_group *group, char *a_message_id);
+	static t_bool submit_inews(const char *name, struct t_group *group, char *a_message_id);
 #endif /* NNTP_INEWS */
 #if defined(NNTP_INEWS) && !defined(FORGERY)
-	static int sender_needed(char *from, char *sender, const char *charset);
+	static int sender_needed(const char *from, const char *sender, const char *charset);
 #endif /* NNTP_INEWS && !FORGERY */
 
-#if 0
-#	ifdef HAVE_NETDB_H
-#		include <netdb.h>
-#	endif /* HAVE_NETDB_H */
-
-#	ifdef HAVE_SYS_SOCKET_H
-#		include <sys/socket.h>
-#	endif /* HAVE_SYS_SOCKET_H */
-#	ifdef HAVE_NETINET_IN_H
-#		include <netinet/in.h>
-#	endif /* HAVE_NETINET_IN_H */
-#endif /* 0 */
-
 
 /*
  * Submit an article using the NNTP POST command
@@ -78,7 +65,7 @@
 #ifdef NNTP_INEWS
 static t_bool
 submit_inews(
-	char *name,
+	const char *name,
 	struct t_group *group,
 	char *a_message_id)
 {
@@ -95,24 +82,27 @@ submit_inews(
 	t_bool id_in_article = FALSE;
 	t_bool ret_code = FALSE;
 #	ifndef FORGERY
-	char sender_hdr[HEADER_LEN];
+	char *sender_hdr = my_malloc(HEADER_LEN + 1);
+	int charset;
 	int sender = 0;
+	int err = 0;
 	t_bool ismail = FALSE;
 #	endif /* !FORGERY */
 #	ifdef USE_CANLOCK
 	t_bool can_lock_in_article = FALSE;
 #	endif /* USE_CANLOCK */
 
-	if ((fp = fopen(name, "r")) == NULL) {
-		perror_message(_(txt_cannot_open), name);
-		return ret_code;
-	}
+	if ((fp = tin_fopen(name, "r")) == NULL)
+		goto out;
 
 	from_name[0] = '\0';
 	message_id[0] = '\0';
+#	ifndef FORGERY
+	sender_hdr[0] = '\0';
+#	endif /* !FORGERY */
 
 	while ((line = tin_fgets(fp, TRUE)) != NULL) {
-		if (line[0] == '\0') /* end of headers */
+		if (!*line) /* end of headers */
 			break;
 
 		if ((ptr = strchr(line, ':'))) {
@@ -135,10 +125,15 @@ submit_inews(
 		/* we could silently add a From: line here if we want to... */
 		error_message(2, _(txt_error_no_from));
 		fclose(fp);
-		return ret_code;
+		goto out;
 	}
 
 #	ifndef FORGERY
+#		ifdef CHARSET_CONVERSION
+	charset = group ? group->attribute->mm_network_charset : tinrc.mm_network_charset;
+#		else
+	charset = 0;
+#		endif /* CHARSET_CONVERSION */
 	/*
 	 * we should only skip the gnksa_check_from() test if we are going to
 	 * post a forged cancel, but inews.c doesn't know anything about the
@@ -149,11 +144,13 @@ submit_inews(
 	 *
 	 * check for valid From: line
 	 */
-	respcode = gnksa_check_from(from_name + 6);
-	if (!(group ? group->attribute->post_8bit_header : tinrc.post_8bit_header) && respcode > GNKSA_OK && respcode < GNKSA_MISSING_REALNAME) { /* error in address */
-		error_message(2, "inews158%s", _(txt_invalid_from), from_name + 6);
+	if (check_mailbox_list(from_name, "From:", charset, &err) > 1)
+		sender = 1;
+
+	if (err) {
+		error_message(2, _(txt_invalid_from), from_name + 6);
 		fclose(fp);
-		return ret_code;
+		goto out;
 	}
 #	endif /* !FORGERY */
 
@@ -163,36 +160,37 @@ submit_inews(
 #	ifndef FORGERY
 		if (!disable_sender && (ptr = build_sender())) {
 #		ifdef CHARSET_CONVERSION
-			const char *charset = group ? txt_mime_charsets[group->attribute->mm_network_charset] : txt_mime_charsets[tinrc.mm_network_charset];
+			const char *ccharset = group ? txt_mime_charsets[group->attribute->mm_network_charset] : txt_mime_charsets[tinrc.mm_network_charset];
 #		else
-			const char *charset = tinrc.mm_charset;
+			const char *ccharset = tinrc.mm_charset;
 #		endif /* CHARSET_CONVERSION */
 
-			sender = sender_needed(from_name + 6, ptr, charset);
+			if (!sender)
+				sender = sender_needed(from_name + 6, ptr, ccharset);
 			switch (sender) {
 				case -2: /* can't build Sender: */
 					error_message(2, _(txt_invalid_sender), ptr);
 					fclose(fp);
-					return ret_code;
+					goto out;
 					/* NOTREACHED */
 					break;
 
 				case -1: /* illegal From: (can't happen as check is done above already) */
 					error_message(2, _(txt_invalid_from), from_name + 6);
 					fclose(fp);
-					return ret_code;
+					goto out;
 					/* NOTREACHED */
 					break;
 
 				case 1:	/* insert Sender */
-					snprintf(sender_hdr, sizeof(sender_hdr), "Sender: %s", ptr);
+					snprintf(sender_hdr, HEADER_LEN + 1, "Sender: %s", ptr);
 #		ifdef CHARSET_CONVERSION
-					buffer_to_network(sender_hdr, group ? group->attribute->mm_network_charset : tinrc.mm_network_charset);
+					buffer_to_network(&sender_hdr, charset);
 #		endif /* CHARSET_CONVERSION */
 					if (!(group ? group->attribute->post_8bit_header : tinrc.post_8bit_header)) {
-						char *p = rfc1522_encode(sender_hdr, charset, ismail);
+						char *p = rfc1522_encode(sender_hdr, ccharset, ismail);
 
-						STRCPY(sender_hdr, p);
+						strcpy(sender_hdr, p);
 						free(p);
 					}
 					break;
@@ -211,7 +209,7 @@ submit_inews(
 		if (nntp_command("POST", CONT_POST, response, sizeof(response)) == NULL) {
 			error_message(2, "%s", response);
 			fclose(fp);
-			return ret_code;
+			goto out;
 		}
 
 		/*
@@ -219,7 +217,7 @@ submit_inews(
 		 * if not scan response line if it contains a Message-ID
 		 * if it's present: use it.
 		 */
-		if (message_id[0] == '\0') {
+		if (!*message_id) {
 			/* simple syntax check - locate last '<' */
 			if ((ptr = strrchr(response, '<')) != NULL) {
 				/* search next '>' */
@@ -234,14 +232,13 @@ submit_inews(
 		}
 
 #	ifndef FORGERY
-		/*
-		 * Send Path: (and Sender: if needed) headers
-		 */
+		/* Send Path: header */
 		snprintf(buf, sizeof(buf), "Path: %s", PATHMASTER);
 		u_put_server(buf);
 		u_put_server("\r\n");
 
-		if (sender == 1) {
+		/* and Sender: (if needed) */
+		if (sender == 1 && *sender_hdr) {
 			u_put_server(sender_hdr);
 			u_put_server("\r\n");
 		}
@@ -249,16 +246,18 @@ submit_inews(
 
 		/*
 		 * check if Message-ID comes from the server
+		 *
+		 * IDs must be no longer than 250 chars, but ...
 		 */
-		if (*message_id) {
+		if (*message_id && strlen(message_id) <= NNTP_STRLEN) {
 			if (!id_in_article) {
-				snprintf(buf, sizeof(buf), "Message-ID: %s", message_id);
+				snprintf(buf, sizeof(buf), "Message-ID: %.*s", NNTP_STRLEN, message_id);
 				u_put_server(buf);
 				u_put_server("\r\n");
 			}
 #	ifdef USE_CANLOCK
 			if (tinrc.cancel_lock_algo && !can_lock_in_article) {
-				char lock[1024];
+				char lock[1000];
 				char *lptr;
 
 				lock[0] = '\0';
@@ -285,7 +284,7 @@ submit_inews(
 
 #	ifdef USE_CANLOCK
 			/* skip any bogus Cancel-Locks */
-			if (!strlen(line))
+			if (!*line)
 				can_lock_in_article = FALSE;	/* don't touch the body */
 
 			if (can_lock_in_article && !id_in_article) {
@@ -304,7 +303,7 @@ submit_inews(
 		}
 
 		u_put_server(".\r\n");
-		put_server(""); /* flush */
+		put_server("", FALSE); /* flush */
 
 		/*
 		 * Receive OK_POSTED or ERROR response code from NNTP server
@@ -336,9 +335,8 @@ submit_inews(
 	 * response.)
 	 */
 	if (respcode != OK_POSTED) {
-		/* TODO: -> lang.c */
-		error_message(2, "Posting failed (%s)", str_trim(response));
-		return ret_code;
+		error_message(2, _(txt_posting_failed), str_trim(response));
+		goto out;
 	}
 
 	/*
@@ -356,10 +354,15 @@ submit_inews(
 		}
 	}
 
-#	if 0
-	if (*message_id && *a_message_id) { /* check if returned ID matches purposed ID */
-		if (strcmp(message_id, a_message_id)) {
-			; /* shouldn't happen - warn user? */
+#	if 0 /* DEBUG */
+	if ((debug & DEBUG_NNTP) && verbose > 1) {
+		if (*message_id && *a_message_id) { /* check if purposed ID matches returned ID */
+			if (strcmp(message_id, a_message_id)) {
+				if (id_in_article)
+					debug_print_file("NNTP", "Returned Message-ID: %s doesn't match given Message-ID: %s", a_message_id, message_id);
+				else
+					debug_print_file("NNTP", "Returned Message-ID: %s doesn't match purposed Message-ID: %s", a_message_id, message_id);
+			}
 		}
 	}
 #	endif /* 0 */
@@ -369,6 +372,12 @@ submit_inews(
 
 	ret_code = TRUE;
 
+out:
+#	ifndef FORGERY
+	FreeIfNeeded(sender_hdr);
+#	else
+	(void) group;
+#	endif /* !FORGERY */
 	return ret_code;
 }
 #endif /* NNTP_INEWS */
@@ -392,8 +401,8 @@ submit_news_file(
 
 	a_message_id[0] = '\0';
 
-	fcc = checknadd_headers(name, group);
-	FreeIfNeeded(fcc); /* we don't use it at the moment */
+	if ((fcc = checknadd_headers(name, group)) != NULL)
+		free(fcc); /* we don't use it at the moment */
 
 	rfc15211522_encode(name, txt_mime_encodings[(group ? group->attribute->post_mime_encoding : tinrc.post_mime_encoding)], group, (group ? group->attribute->post_8bit_header : tinrc.post_8bit_header), ismail);
 
@@ -402,33 +411,34 @@ submit_news_file(
 		ret_code = submit_inews(name, group, a_message_id);
 	else
 #endif /* NNTP_INEWS */
-		{
-			/* use tinrc.inews_prog or 'inewsdir/inews -h' 'inews -h' */
-			if (strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
-				STRCPY(buf, tinrc.inews_prog);
-			else {
-				if (*inewsdir)
-					joinpath(buf, sizeof(buf), inewsdir, "inews -h");
-				else
-					strcpy(buf, "inews -h");
-			}
-			cp += strlen(cp);
-			sh_format(cp, sizeof(buf) - (size_t) (cp - buf), " < %s", name);
+	{
+		/* use tinrc.inews_prog or 'inewsdir/inews -h' 'inews -h' */
+		if (strcasecmp(tinrc.inews_prog, INTERNAL_CMD))
+			STRCPY(buf, tinrc.inews_prog);
+		else {
+			if (*inewsdir)
+				joinpath(buf, sizeof(buf), inewsdir, "inews -h");
+			else
+				strcpy(buf, "inews -h");
+		}
+		cp += strlen(cp);
+		sh_format(cp, sizeof(buf) - (size_t) (cp - buf), " < %s", name);
 
-			ret_code = invoke_cmd(buf);
+		ret_code = invoke_cmd(buf);
 
 #ifdef NNTP_INEWS
-			if (!ret_code && read_news_via_nntp && !read_saved_news && strcasecmp(tinrc.inews_prog, INTERNAL_CMD)) {
-				if (prompt_yn(_(txt_post_via_builtin_inews), TRUE)) {
-					ret_code = submit_inews(name, group, a_message_id);
-					if (ret_code) {
-						if (prompt_yn(_(txt_post_via_builtin_inews_only), TRUE) == 1)
-							strcpy(tinrc.inews_prog, INTERNAL_CMD);
+		if (!ret_code && read_news_via_nntp && !read_saved_news && strcasecmp(tinrc.inews_prog, INTERNAL_CMD)) {
+			if (prompt_yn(_(txt_post_via_builtin_inews), TRUE)) {
+				if ((ret_code = submit_inews(name, group, a_message_id)) == TRUE) {
+					if (prompt_yn(_(txt_post_via_builtin_inews_only), TRUE) == 1) {
+						FreeIfNeeded(tinrc.inews_prog);
+						tinrc.inews_prog = my_strdup(INTERNAL_CMD);
 					}
 				}
 			}
-#endif /* NNTP_INEWS */
 		}
+#endif /* NNTP_INEWS */
+	}
 	return ret_code;
 }
 
@@ -442,13 +452,13 @@ submit_news_file(
 #if defined(NNTP_INEWS) && !defined(FORGERY)
 static int
 sender_needed(
-	char *from,
-	char *sender,
+	const char *from,
+	const char *sender,
 	const char *charset)
 {
-	char *from_at_pos;
-	char *sender_at_pos;
-	char *sender_dot_pos;
+	const char *from_at_pos;
+	const char *sender_at_pos;
+	const char *sender_dot_pos;
 	char *p;
 	char from_addr[HEADER_LEN];
 	char sender_addr[HEADER_LEN];
