@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2023-04-29
+ *  Updated   : 2024-07-18
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -65,8 +65,11 @@ static int read_site_config(void);
 
 char active_times_file[PATH_LEN];
 char article_name[PATH_LEN];			/* ~/TIN_ARTICLE_NAME file */
-char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
-char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
+char *backup_article_name;			/* ~/TIN_ARTICLE_NAME[.pid].b[ak] file */
+#ifdef NNTP_ABLE
+	char bug_nntpserver1[PATH_LEN];		/* welcome message of NNTP server used */
+	char bug_nntpserver2[PATH_LEN];		/* welcome message of NNTP server used */
+#endif /* NNTP_ABLE */
 char cvers[LEN];
 char dead_article[PATH_LEN];		/* ~/dead.article file */
 char dead_articles[PATH_LEN];		/* ~/dead.articles file */
@@ -83,6 +86,7 @@ char inewsdir[PATH_LEN];
 char local_attributes_file[PATH_LEN];
 char local_config_file[PATH_LEN];
 char local_input_history_file[PATH_LEN];
+char local_motd_file[PATH_LEN];	/* local copy of NNTP MOTD message */
 char local_newsgroups_file[PATH_LEN];	/* local copy of NNTP newsgroups file */
 char local_newsrctable_file[PATH_LEN];
 char lock_file[PATH_LEN];		/* contains name of index lock file */
@@ -116,13 +120,17 @@ char userid[PATH_LEN];
 #endif /* !NNTP_ONLY */
 
 t_function last_search = GLOBAL_SEARCH_REPEAT;	/* for repeated search */
-int art_mark_width = 1;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	int art_mark_width = 1;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int system_status;
 int xmouse, xrow, xcol;			/* xterm button pressing information */
 
+long motd_hash = 0L;
+
 pid_t process_id;			/* Useful to have around for .suffixes */
 
 t_bool batch_mode;			/* update index files only mode */
@@ -332,6 +340,8 @@ struct t_config tinrc = {
 	0,		/* col_extquote (initialised later) */
 	0,		/* col_response (initialised later) */
 	0,		/* col_signature (initialised later) */
+	0,		/* col_score_neg (initialised later) */
+	0,		/* col_score_pos (initialised later) */
 	0,		/* col_urls (initialised later) */
 	0,		/* col_verbatim (initialised later) */
 	0,		/* col_subject (initialised later) */
@@ -346,6 +356,7 @@ struct t_config tinrc = {
 	TRUE,		/* word_highlight */
 	TRUE,		/* url_highlight */
 	0,		/* wrap_column */
+	FALSE,		/* dont_break_words */
 #ifdef HAVE_COLOR
 	FALSE,		/* use_color */
 #endif /* HAVE_COLOR */
@@ -392,6 +403,7 @@ struct t_config tinrc = {
 	TRUE,		/* show_only_unread_arts */
 	FALSE,		/* show_only_unread_groups */
 	TRUE,		/* show_signatures */
+	FALSE,		/* show_art_score */
 	TRUE,		/* sigdashes */
 	TRUE,		/* signature_repost */
 #ifndef USE_CURSES
@@ -431,10 +443,13 @@ struct t_config tinrc = {
 	FALSE,		/* use_slrnface */
 #endif /* XFACE_ABLE */
 	TRUE,		/* default_filter_select_global */
-	DEFAULT_SELECT_FORMAT,	/* select_format */
-	DEFAULT_GROUP_FORMAT,	/* group_format */
-	DEFAULT_THREAD_FORMAT,	/* thread_format */
-	DEFAULT_DATE_FORMAT,	/* date_format */
+	DEFAULT_SELECT_FORMAT,		/* select_format */
+	DEFAULT_GROUP_FORMAT,		/* group_format */
+	DEFAULT_THREAD_FORMAT,		/* thread_format */
+	DEFAULT_ATTACHMENT_FORMAT,	/* attachment_format */
+	DEFAULT_PAGE_MIME_FORMAT,	/* page_mime_format */
+	DEFAULT_PAGE_UUE_FORMAT,	/* page_uue_format */
+	DEFAULT_DATE_FORMAT,		/* date_format */
 #ifdef HAVE_UNICODE_NORMALIZATION
 	DEFAULT_NORMALIZE,		/* normalization form */
 #endif /* HAVE_UNICODE_NORMALIZATION */
@@ -444,6 +459,9 @@ struct t_config tinrc = {
 #ifdef CHARSET_CONVERSION
 	-1,		/* attrib_mm_network_charset, defaults to $MM_CHARSET */
 	"",		/* attrib_undeclared_charset */
+#	ifdef USE_ICU_UCSDET
+	FALSE,
+#	endif /* USE_ICU_UCSDET */
 #endif /* CHARSET_CONVERSION */
 	"",		/* attrib_editor_format */
 	"",		/* attrib_fcc */
@@ -507,6 +525,7 @@ struct t_config tinrc = {
 	FALSE,		/* attrib_prompt_followupto */
 	TRUE,		/* attrib_show_only_unread_arts */
 	TRUE,		/* attrib_show_signatures */
+	FALSE,		/* attrib_show_art_score */
 	TRUE,		/* attrib_sigdashes */
 	TRUE,		/* attrib_signature_repost */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -558,7 +577,8 @@ struct t_capabilities nntp_caps = {
 	FALSE, /* AUTHINFO USER/PASS */
 	FALSE, /* AUTHINFO SASL */
 	FALSE, /* AUTHINFO available but not in current state */
-	SASL_NONE, /* SASL CRAM-MD5 DIGEST-MD5 PLAIN GSSAPI EXTERNAL OTP NTLM LOGIN */
+	NULL, /* SASL mechs implemented on both sides */
+	NULL, /* SASL mech used after handshake */
 	FALSE, /* COMPRESS */
 	COMPRESS_NONE, /* COMPRESS_NONE, COMPRESS_DEFLATE */
 #if defined(MAXARTNUM) && defined(USE_LONG_ARTICLE_NUMBERS)
@@ -608,6 +628,8 @@ static const struct {
 	{ &tinrc.col_extquote,    5 },
 	{ &tinrc.col_response,    2 },
 	{ &tinrc.col_signature,   4 },
+	{ &tinrc.col_score_neg,   1 },
+	{ &tinrc.col_score_pos,   2 },
 	{ &tinrc.col_urls,       DFT_FORE },
 	{ &tinrc.col_verbatim,    5 },
 	{ &tinrc.col_subject,     6 },
@@ -665,12 +687,24 @@ init_selfinfo(
 {
 	FILE *fp;
 	char *ptr;
+	const char *p;
 	char tmp[PATH_LEN];
-	struct stat sb;
+	size_t space;
 	struct passwd *myentry;
-#if defined(DOMAIN_NAME) || defined(HAVE_GETHOSTBYNAME)
-	const char *cptr;
-#endif /* DOMAIN_NAME || HAVE_GETHOSTBYNAME */
+	struct stat sb;
+
+	process_id = getpid();
+
+	real_umask = umask(0);
+	(void) umask(real_umask);
+
+	if ((myentry = getpwuid(getuid())) == NULL) {
+		error_message(2, _(txt_error_passwd_missing));
+		free(tin_progname);
+		giveup();
+	}
+
+	my_strncpy(userid, myentry->pw_name, sizeof(userid) - 1);
 
 	domain_name[0] = '\0';
 
@@ -687,37 +721,21 @@ init_selfinfo(
 #endif /* HAVE_SYS_UTSNAME_H */
 
 #ifdef DOMAIN_NAME
-	if ((cptr = get_domain_name()) != NULL)
-		my_strncpy(domain_name, cptr, MAXHOSTNAMELEN);
+	if ((p = get_domain_name()) != NULL)
+		my_strncpy(domain_name, p, MAXHOSTNAMELEN);
 #endif /* DOMAIN_NAME */
 
 #ifdef HAVE_GETHOSTBYNAME
 	if (domain_name[0] == '\0') {
-		cptr = get_fqdn(get_host_name());
-		if (cptr != NULL)
-			my_strncpy(domain_name, cptr, MAXHOSTNAMELEN);
+		if ((p = get_fqdn(get_host_name())) != NULL)
+			my_strncpy(domain_name, p, MAXHOSTNAMELEN);
 	}
 #endif /* HAVE_GETHOSTBYNAME */
 
-	process_id = getpid();
-
-	real_umask = umask(0);
-	(void) umask(real_umask);
-
-	if ((myentry = getpwuid(getuid())) == NULL) {
-		error_message(2, _(txt_error_passwd_missing));
-		free(tin_progname);
-		giveup();
-	}
-
-	my_strncpy(userid, myentry->pw_name, sizeof(userid) - 1);
-
 	tmpdir = get_val("TMPDIR", _PATH_TMP);
 
-	if (((ptr = getenv("TIN_HOMEDIR")) != NULL) && strlen(ptr)) {
-		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
-	} else if (((ptr = getenv("HOME")) != NULL) && strlen(ptr)) {
-		my_strncpy(homedir, ptr, sizeof(homedir) - 1);
+	if ((p = get_val("TIN_HOMEDIR", get_val("HOME", NULL))) != NULL) {
+		my_strncpy(homedir, p, sizeof(homedir) - 1);
 	} else if (strlen(myentry->pw_dir)) {
 		strncpy(homedir, myentry->pw_dir, sizeof(homedir) - 1);
 	} else
@@ -727,8 +745,8 @@ init_selfinfo(
 	dangerous_signal_exit = FALSE;
 	disable_gnksa_domain_check = TRUE;
 	disable_sender = FALSE;	/* we set force_no_post=TRUE later on if we don't have a valid FQDN */
-	iso2asc_supported = atoi(get_val("ISO2ASC", DEFAULT_ISO2ASC));
-	if (iso2asc_supported >= NUM_ISO_TABLES || iso2asc_supported < 0) /* TODO: issue a warning here? */
+	iso2asc_supported = s2i(get_val("ISO2ASC", DEFAULT_ISO2ASC), -1, NUM_ISO_TABLES - 1);
+	if (errno) /* TODO: issue a warning here? */
 		iso2asc_supported = -1;
 	list_active = FALSE;
 	newsrc_active = FALSE;
@@ -752,17 +770,19 @@ init_selfinfo(
 	index_savedir[0] = '\0';
 	newsrc[0] = '\0';
 
-	snprintf(page_header, sizeof(page_header), "%s %s release %s (\"%s\") %s",
+	snprintf(page_header, sizeof(page_header), "%s %s release %s (\"%s\")%s",
 		PRODUCT, VERSION, RELEASEDATE, RELEASENAME,
-		(iso2asc_supported >= 0 ? "[ISO2ASC]" : ""));
+		(iso2asc_supported >= 0 ? " [ISO2ASC]" : ""));
 	snprintf(cvers, sizeof(cvers), txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
 
 	strncpy(bug_addr, BUG_REPORT_ADDRESS, sizeof(bug_addr) - 1);
 
+#ifdef NNTP_ABLE
 	bug_nntpserver1[0] = '\0';
 	bug_nntpserver2[0] = '\0';
+#endif /* NNTP_ABLE */
 
 #ifdef INEWSDIR
 	strncpy(inewsdir, INEWSDIR, sizeof(inewsdir) - 1);
@@ -804,15 +824,23 @@ init_selfinfo(
 		force_no_post = TRUE;
 	}
 
+#ifndef NNTP_ONLY
 	/*
 	 * only set the following variables if they weren't set from within
 	 * read_site_config()
 	 *
+	 * what if !*libdir - should be use some fallback other than '/' ?
+	 *
 	 * TODO: do we really want that read_site_config() overwrites
 	 * values given in env-vars? ($MM_CHARSET, $TIN_ACTIVEFILE)
 	 */
-	if (!*news_active_file) /* TODO: really prepend libdir here in case of $TIN_ACTIVEFILE is set? */
-		joinpath(news_active_file, sizeof(news_active_file), libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
+	if (!*news_active_file) {
+		p = get_val("TIN_ACTIVEFILE", ACTIVE_FILE);
+		if (*p != '/')
+			joinpath(news_active_file, sizeof(news_active_file), libdir, p);
+		else
+			my_strncpy(news_active_file, p, sizeof(news_active_file) - 1);
+	}
 	if (!*active_times_file)
 		joinpath(active_times_file, sizeof(active_times_file), libdir, ACTIVE_TIMES_FILE);
 	if (!*newsgroups_file)
@@ -823,18 +851,18 @@ init_selfinfo(
 		joinpath(overviewfmt_file, sizeof(overviewfmt_file), libdir, OVERVIEW_FMT);
 	if (!*default_organization) {
 		joinpath(tmp, sizeof(tmp), libdir, "organization");
-		if ((fp = fopen(tmp, "r")) != NULL) {
+		if ((fp = tin_fopen(tmp, "r")) != NULL) {
 			char buf[LEN];
 
 			if (fgets(buf, (int) sizeof(buf), fp) != NULL) {
-				ptr = strrchr(buf, '\n');
-				if (ptr != NULL)
+				if ((ptr = strrchr(buf, '\n')) != NULL)
 					*ptr = '\0';
 			}
 			fclose(fp);
 			my_strncpy(default_organization, buf, sizeof(default_organization) - 1);
 		}
 	}
+#endif /* NNTP_ONLY */
 
 	/*
 	 * Formerly get_mm_charset(), read_site_config() may set mm_charset
@@ -844,7 +872,7 @@ init_selfinfo(
 		STRCPY(tinrc.mm_charset, get_val("MM_CHARSET", MM_CHARSET));
 #else
 	if (tinrc.mm_network_charset < 0) {
-		size_t space = 255;
+		space = 255;
 
 		ptr = my_malloc(space + 1);
 
@@ -867,8 +895,12 @@ init_selfinfo(
 
 	joinpath(rcdir, sizeof(rcdir), homedir, RCDIR);
 	if (stat(rcdir, &sb) == -1) {
-		my_mkdir(rcdir, (mode_t) (S_IRWXU)); /* TODO: bail out? give error message? no_write = TRUE? */
-		created_rcdir = TRUE;
+		if (my_mkdir(rcdir, (mode_t) (S_IRWXU)) == -1) {
+			error_message(0, _(txt_cannot_create), rcdir);
+			free_all_arrays();
+			giveup();
+		} else
+			created_rcdir = TRUE;
 	}
 	strcpy(tinrc.mailer_format, MAILER_FORMAT);
 	my_strncpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER), sizeof(mailer) - 1);
@@ -881,6 +913,19 @@ init_selfinfo(
 #ifdef APPEND_PID
 	snprintf(article_name + strlen(article_name), sizeof(article_name) - strlen(article_name), ".%ld", (long) process_id);
 #endif /* APPEND_PID */
+
+#ifdef HAVE_LONG_FILE_NAMES
+	space = snprintf(NULL, 0, "%s.bak", article_name);
+#else
+	space = snprintf(NULL, 0, "%s.b", article_name);
+#endif /* HAVE_LONG_FILE_NAMES */
+	backup_article_name = my_malloc(++space);
+#ifdef HAVE_LONG_FILE_NAMES
+	snprintf(backup_article_name, space, "%s.bak", article_name);
+#else
+	snprintf(backup_article_name, space, "%s.b", article_name);
+#endif /* HAVE_LONG_FILE_NAMES */
+
 	joinpath(dead_article, sizeof(dead_article), homedir, "dead.article");
 	joinpath(dead_articles, sizeof(dead_articles), homedir, "dead.articles");
 	joinpath(tinrc.maildir, sizeof(tinrc.maildir), homedir, DEFAULT_MAILDIR);
@@ -891,11 +936,15 @@ init_selfinfo(
 	if (!index_newsdir[0])
 		joinpath(index_newsdir, sizeof(index_newsdir), get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
 	joinpath(index_maildir, sizeof(index_maildir), get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
-	if (stat(index_maildir, &sb) == -1)
-		my_mkdir(index_maildir, (mode_t) S_IRWXU);
+	if (stat(index_maildir, &sb) == -1) {
+		if (my_mkdir(index_maildir, (mode_t) S_IRWXU) == -1)
+			error_message(2, _(txt_cannot_create), index_maildir);
+	}
 	joinpath(index_savedir, sizeof(index_savedir), get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
-	if (stat(index_savedir, &sb) == -1)
-		my_mkdir(index_savedir, (mode_t) S_IRWXU);
+	if (stat(index_savedir, &sb) == -1) {
+		if (my_mkdir(index_savedir, (mode_t) S_IRWXU) == -1)
+			error_message(2, _(txt_cannot_create), index_savedir);
+	}
 	joinpath(local_attributes_file, sizeof(local_attributes_file), rcdir, ATTRIBUTES_FILE);
 	joinpath(local_config_file, sizeof(local_config_file), rcdir, CONFIG_FILE);
 	joinpath(filter_file, sizeof(filter_file), rcdir, FILTER_FILE);
@@ -904,11 +953,10 @@ init_selfinfo(
 #ifdef HAVE_MH_MAIL_HANDLING
 	joinpath(mail_active_file, sizeof(mail_active_file), rcdir, ACTIVE_MAIL_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
-	ptr = getenv("MAIL");
-	if (ptr == NULL || (*ptr == '\0'))
-		joinpath(mailbox, sizeof(mailbox), DEFAULT_MAILBOX, userid);
-	else
+	if ((ptr = getenv("MAIL")) != NULL && *ptr) {
 		STRCPY(mailbox, ptr);
+	} else
+		joinpath(mailbox, sizeof(mailbox), DEFAULT_MAILBOX, userid);
 #ifdef HAVE_MH_MAIL_HANDLING
 	joinpath(mailgroups_file, sizeof(mailgroups_file), rcdir, MAILGROUPS_FILE);
 #endif /* HAVE_MH_MAIL_HANDLING */
@@ -921,13 +969,23 @@ init_selfinfo(
 	joinpath(postponed_articles_file, sizeof(postponed_articles_file), rcdir, POSTPONED_FILE);
 	joinpath(save_active_file, sizeof(save_active_file), rcdir, ACTIVE_SAVE_FILE);
 
+	/*
+	 * TODO: why a lockfile in /tmp and not ${TIN_HOMEDIR:-"$HOME"}/[.tin].
+	 * if ${TIN_HOMEDIR:-"$HOME"} is on NFS the cache likely also is on NFS;
+	 * running multiple instanced of tin -u on different systems which do
+	 * share the cache will not be caught ...
+	 */
 	snprintf(tmp, sizeof(tmp), INDEX_LOCK, userid);
 	joinpath(lock_file, sizeof(lock_file), tmpdir, tmp);
 
 #ifdef NNTP_ABLE
-	nntp_tcp_default_port = (unsigned short) atoi(get_val("NNTPPORT", NNTP_TCP_PORT));
+	nntp_tcp_default_port = (unsigned short) s2i(get_val("NNTPPORT", NNTP_TCP_PORT), 0, 65535);
+	if (errno)
+		nntp_tcp_default_port = IPPORT_NNTP;
 #ifdef NNTPS_ABLE
-	nntps_tcp_default_port = (unsigned short) atoi(get_val("NNTPSPORT", NNTPS_TCP_PORT));
+	nntps_tcp_default_port = (unsigned short) s2i(get_val("NNTPSPORT", NNTPS_TCP_PORT), 0, 65535);
+	if (errno)
+		nntps_tcp_default_port = IPPORT_NNTPS;
 #endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 
@@ -979,7 +1037,7 @@ read_site_config(
 	 */
 	while (tin_defaults[i] != NULL) {
 		joinpath(filename, sizeof(filename), tin_defaults[i++], "tin.defaults");
-		if ((fp = fopen(filename, "r")) != NULL)
+		if ((fp = tin_fopen(filename, "r")) != NULL)
 			break;
 	}
 
