@@ -3,10 +3,10 @@
  *  Module    : mail.c
  *  Author    : I. Lea
  *  Created   : 1992-10-02
- *  Updated   : 2023-08-23
+ *  Updated   : 2024-11-25
  *  Notes     : Mail handling routines for creating pseudo newsgroups
  *
- * Copyright (c) 1992-2023 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1992-2025 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,7 @@
 #		include "tnntp.h"
 #	endif /* !TNNTP_H */
 #endif /* NNTP_ABLE */
+
 /*
  * local prototypes
  */
@@ -100,7 +101,7 @@ read_mail_active_file(
 	struct t_group *ptr;
 
 	if (!batch_mode)
-		wait_message(0, _(txt_reading_mail_active_file));
+		wait_message(0, _(txt_reading_mail_active_file), mail_active_file);
 
 	/*
 	 * Open the mail active file
@@ -188,7 +189,8 @@ write_mail_active_file(
 		return;
 
 	/* generate tmp-filename */
-	file_tmp = get_tmpfilename(mail_active_file);
+	if ((file_tmp = get_tmpfilename(mail_active_file)) == NULL)
+		return;
 
 	if (!backup_file(mail_active_file, file_tmp)) {
 		error_message(2, _(txt_filesystem_full_backup), mail_active_file);
@@ -214,11 +216,7 @@ write_mail_active_file(
 				clearerr(fp);
 				fclose(fp);
 			}
-			i = rename(file_tmp, mail_active_file);
-#	ifdef DEBUG
-			if ((debug & DEBUG_MISC) && i) /* TODO: is this the right debug-level? */
-				perror_message(_(txt_rename_error), file_tmp, mail_active_file);
-#	endif /* DEBUG */
+			rename_file(file_tmp, mail_active_file);
 		} else
 			unlink(file_tmp);
 	}
@@ -240,7 +238,7 @@ read_mailgroups_file(
 
 	if ((fp = open_mailgroups_fp()) != NULL) {
 		if (!batch_mode && verb)
-			wait_message(0, _(txt_reading_mailgroups_file));
+			wait_message(0, _(txt_reading_mailgroups_file), mailgroups_file);
 
 		read_groups_descriptions(fp, (FILE *) 0);
 
@@ -275,20 +273,15 @@ open_newsgroups_fp(
 
 	if (read_news_via_nntp && !read_saved_news) {
 		if (read_local_newsgroups_file) {
-			if ((result = fopen(local_newsgroups_file, "r")) != NULL) {
-				struct stat buf;
-
+			if ((result = tin_fopen(local_newsgroups_file, "r")) != NULL) {
 #	ifdef DEBUG
 				if ((debug & DEBUG_NNTP) && verbose > 1)
 					debug_print_file("NNTP", "open_newsgroups_fp Using local copy of newsgroups file");
 #	endif /* DEBUG */
-
-				if (!fstat(fileno(result), &buf)) {
-					if (buf.st_size > 0)
-						return result;
-				}
-				fclose(result);
-				unlink(local_newsgroups_file);
+				return result;
+			} else {
+				if (!errno) /* zero size file */
+					unlink(local_newsgroups_file);
 			}
 		}
 		/*
@@ -302,8 +295,8 @@ open_newsgroups_fp(
 			char line[NNTP_STRLEN];
 			char file[PATH_LEN];
 			char serverdir[PATH_LEN];
-			struct t_group *group;
 			int resp, i, j = 0;
+			struct t_group *group;
 
 			if (nntp_tcp_port != IPPORT_NNTP)
 				snprintf(file, sizeof(file), "%s:%u", nntp_server, nntp_tcp_port);
@@ -323,9 +316,9 @@ open_newsgroups_fp(
 										snprintf(buff + strlen(buff), sizeof(buff) - strlen(buff), ",%s", active[i].name);
 										continue;
 									} else {
-										put_server(buff);
+										put_server(buff, FALSE);
 										*buff = '\0';
-										j++;
+										++j;
 									}
 								}
 								if (!*buff) {
@@ -341,28 +334,28 @@ open_newsgroups_fp(
 								break;
 							}
 							while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
-#			ifdef DEBUG
+#			if defined(DEBUG) && defined(NNTP_ABLE)
 								if ((debug & DEBUG_NNTP) && verbose)
 									debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
-#			endif /* DEBUG */
+#			endif /* DEBUG && NNTP_ABLE */
 								fprintf(result, "%s\n", str_trim(ptr));
 							}
-#			ifdef DEBUG
+#			if defined(DEBUG) && defined(NNTP_ABLE)
 							if ((debug & DEBUG_NNTP) && !verbose)
 								debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
-#			endif /* DEBUG */
+#			endif /* DEBUG && NNTP_ABLE */
 
 #		else
-							put_server(buff);
+							put_server(buff, FALSE);
 							*buff = '\0';
-							j++;
+							++j;
 #		endif /* DISABLE_PIPELINING */
 						}
 					}
 				}
 				if (*buff) {
-					put_server(buff);
-					j++;
+					put_server(buff, FALSE);
+					++j;
 				}
 #		ifndef DISABLE_PIPELINING
 				while (j--) {
@@ -372,21 +365,23 @@ open_newsgroups_fp(
 						continue;
 					}
 					while ((ptr = tin_fgets(FAKE_NNTP_FP, FALSE)) != NULL) {
-#			ifdef DEBUG
+#			if defined(DEBUG) && defined(NNTP_ABLE)
 						if ((debug & DEBUG_NNTP) && verbose)
 							debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
-#			endif /* DEBUG */
+#			endif /* DEBUG && NNTP_ABLE */
 						fprintf(result, "%s\n", str_trim(ptr));
 					}
-#			ifdef DEBUG
+#			if defined(DEBUG) && defined(NNTP_ABLE)
 					if ((debug & DEBUG_NNTP) && !verbose)
 						debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
-#			endif /* DEBUG */
+#			endif /* DEBUG && NNTP_ABLE */
 				}
 				/* TODO: add 483 (RFC 3977) support */
 				if (no_more_wildmat == ERR_NOAUTH || no_more_wildmat == NEED_AUTHINFO) {
-					if (!authenticate(nntp_server, userid, FALSE))
-						tin_done(EXIT_FAILURE, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
+					if (!authenticate(nntp_server, userid, FALSE)) {
+						fclose(result);
+						tin_done(EXIT_FAILURE, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS); /* TODO: should we exit with NNTP_ERROR_EXIT? */
+					}
 				}
 #		endif /* !DISABLE_PIPELINING */
 				fclose(result);
@@ -462,6 +457,22 @@ read_descriptions(
  * of all groups if reading groups of type GROUP_TYPE_NEWS.
  * Aborting this early won't have any adverse affects, just some missing
  * descriptions.
+ *
+ * TODO: instead of matching the description to the group right away
+ *       we could collect (or fetch?) them on a per hierarchy base first
+ *       to guess the charset of the descriptions for that hierarchy for
+ *       conversion (descriptions SHOULD be in UTF-8, but in the real
+ *       world ...)
+ *
+ * TODO: add a tinrc-var to be able to en/disabale charset guessing?
+ *
+ * TODO: (maybe as a wildmat-list? suitable for match_group_list()
+ *        guess_descption_charset=*,!*.test
+ *        or
+ *        guess_descption_charset=ukr.*,cn.*,han.*,esp.*,ee.*,\
+ *          finet.*,fr.*,france.*,is.*,it.*,no.*,se.*,tw.*
+ *       ) results should still be passed per "scope" to charset-guessing
+ *       to have more data.
  */
 static void
 read_groups_descriptions(
@@ -475,10 +486,10 @@ read_groups_descriptions(
 	struct t_group *group;
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
-#ifdef DEBUG
+#if defined(DEBUG) && defined(NNTP_ABLE)
 		if ((debug & DEBUG_NNTP) && fp == FAKE_NNTP_FP && verbose)
 			debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
-#endif /* DEBUG */
+#endif /* DEBUG && NNTP_ABLE */
 		if (*ptr == '#' || *ptr == '\0')
 			continue;
 
@@ -508,24 +519,46 @@ read_groups_descriptions(
 		*q = '\0';
 
 		while (*p == '\t' || *p == ' ')
-			p++;
+			++p;
 
 		group = group_find(groupname, FALSE);
 
 		if (group != NULL && group->description == NULL) {
 			char *r;
 			size_t r_len;
+			t_bool conv_needed = FALSE;
 
 			q = p;
-			while ((q = strchr(q, '\t')) != NULL)
-				*q = ' ';
+			while (*q) {
+				if (*q == '\t') /* what about '\r', '\f', '\v', '\b', '\a'? */
+					*q = ' ';
+				else {
+					if (!conv_needed && (*q < 0x20 || (unsigned char) *q > 0x7f))
+						conv_needed = TRUE;
+				}
+				++q;
+			}
 
 			r = my_strdup(p);
 			r_len = strlen(r);
+
 			/*
 			 * Protect against invalid character sequences.
 			 */
-			process_charsets(&r, &r_len, "UTF-8", tinrc.mm_local_charset, FALSE);
+			if (conv_needed) {
+#ifdef CHARSET_CONVERSION
+#	ifdef USE_ICU_UCSDET
+				char *guessed_charset = guess_charset(r, 10);
+
+				process_charsets(&r, &r_len, guessed_charset ? guessed_charset : "UTF-8", tinrc.mm_local_charset, FALSE);
+				FreeAndNull(guessed_charset);
+#	else
+				process_charsets(&r, &r_len, "UTF-8", tinrc.mm_local_charset, FALSE);
+#	endif /* USE_ICU_UCSDET */
+#else
+				process_charsets(&r, &r_len, "UTF-8", tinrc.mm_local_charset, FALSE);
+#endif /* CHARSET_CONVERSION */
+			}
 			group->description = convert_to_printable(r, FALSE);
 		}
 
@@ -533,7 +566,7 @@ read_groups_descriptions(
 			spin_cursor();
 	}
 #	if defined(DEBUG) && defined(NNTP_ABLE)
-	if ((debug & DEBUG_NNTP) && !verbose)
+	if ((debug & DEBUG_NNTP) && fp == FAKE_NNTP_FP && !verbose)
 		debug_print_file("NNTP", "<<<%s%s", logtime(), txt_log_data_hidden);
 #	endif /* DEBUG && NNTP_ABLE */
 	FreeIfNeeded(groupname);
