@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-05-03
+ *  Updated   : 2024-09-01
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,12 @@
 #ifndef VERSION_H
 #	include "version.h"
 #endif /* !VERSION_H */
+#if defined(INET6) && defined(HAVE_INET_PTON)
+#	ifndef TNNTP_H
+#		include "tnntp.h"
+#	endif /* !TNNTP_H */
+#endif /* INET6 && HAVE_INET_PTON */
+
 
 signed long int read_newsrc_lines = -1;
 
@@ -65,13 +71,21 @@ static t_bool start_any_unread = FALSE;	/* only start if unread news */
 /*
  * Local prototypes
  */
+static char **read_cmd_line_options(int argc, char *argv[]);
 static void create_mail_save_dirs(void);
-static void read_cmd_line_options(int argc, char *argv[]);
 static void show_intro_page(void);
 _Noreturn static void update_index_files(void);
 static void usage(char *theProgname);
 
 
+#define FREE_ARGV_IF_NEEDED(orig, new) do { \
+		if (orig != new) { \
+			free(*(new + 1)); \
+			FreeAndNull(new); \
+		} \
+	} while (0)
+
+
 /*
  * OK lets start the ball rolling...
  */
@@ -80,6 +94,7 @@ main(
 	int argc,
 	char *argv[])
 {
+	char **argv_orig = argv;
 	int count, start_groupnum;
 	int num_cmd_line_groups = 0;
 	t_bool tmp_no_write;
@@ -93,8 +108,10 @@ main(
 		bindtextdomain(NLS_TEXTDOMAIN, LOCALEDIR);
 		textdomain(NLS_TEXTDOMAIN);
 #	endif /* ENABLE_NLS */
-	} else
-		error_message(4, txt_error_locale);
+	} else {
+		my_fprintf(stderr, "%s\n", txt_error_locale);
+		sleep(4);
+	}
 #endif /* HAVE_SETLOCALE && !NO_LOCALE */
 
 	/*
@@ -105,20 +122,22 @@ main(
 		const char *p;
 
 		if ((p = tin_nl_langinfo(CODESET)) != NULL) {
-			if (!strcasecmp(p, "ANSI_X3.4-1968"))
-				STRCPY(tinrc.mm_local_charset, "US-ASCII");
-			else
-				STRCPY(tinrc.mm_local_charset, p);
+			if (strcasecmp(p, "ANSI_X3.4-1968")) { /* !US-ASCII */
+				FreeIfNeeded(tinrc.mm_local_charset);
+				tinrc.mm_local_charset = my_strdup(p);
+			}
 		}
 	}
 #endif /* !NO_LOCALE */
 	/* always set a default value */
-	if (!*tinrc.mm_local_charset)
-		STRCPY(tinrc.mm_local_charset, "US-ASCII");
+	if (!tinrc.mm_local_charset || !*tinrc.mm_local_charset) {
+		FreeIfNeeded(tinrc.mm_local_charset);
+		tinrc.mm_local_charset = my_strdup("US-ASCII");
+	}
 
 	set_signal_handlers();
 
-	debug = 0;	/* debug OFF */
+	debug = 0;	/* can't use if (debug) before read_cmd_line_options() */
 
 	tin_progname = my_malloc(strlen(argv[0]) + 1);
 	base_name(argv[0], tin_progname);
@@ -133,7 +152,9 @@ main(
 #	ifdef NNTP_ABLE
 		read_news_via_nntp = TRUE;
 #	else
-		error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+		my_fprintf(stderr, _(txt_option_not_enabled), "-DNNTP_ABLE");
+		my_fprintf(stderr, "\n");
+		sleep(2);
 		free(tin_progname);
 		giveup();
 #	endif /* NNTP_ABLE */
@@ -161,7 +182,9 @@ main(
 	 * Process envargs & command line options
 	 * These override the configured in values
 	 */
-	read_cmd_line_options(argc, argv);
+	cmdargs = read_cmd_line_options(argc, argv);
+	if (cmdargs != argv_orig)
+		handle_cmdargs(TRUE);
 
 	/* preinit keybindings if interactive */
 	if (!batch_mode)
@@ -173,7 +196,8 @@ main(
 	 *
 	 * must be called before setup_screen()
 	 */
-	read_config_file(global_config_file, TRUE);
+	if (!read_config_file(global_config_file, TRUE))
+		global_config_file[0] = '\0';
 	read_config_file(local_config_file, FALSE);
 
 	tmp_no_write = no_write; /* keep no_write */
@@ -183,6 +207,7 @@ main(
 #ifndef USE_CURSES
 		if (!get_termcaps()) {
 			error_message(2, _(txt_screen_init_failed), tin_progname);
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			free_all_arrays();
 			giveup();
 		}
@@ -193,6 +218,7 @@ main(
 			no_write = TRUE;
 			/* TODO: looks ugly */
 			error_message(0, txt_info_nopostponed);
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			tin_done(EXIT_SUCCESS, NULL);
 		}
 
@@ -201,6 +227,7 @@ main(
 		 */
 		if (!InitScreen()) {
 			error_message(2, _(txt_screen_init_failed), tin_progname);
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			free_all_arrays();
 			giveup();
 		}
@@ -216,6 +243,7 @@ main(
 	if (!batch_mode || verbose) {
 		if (!batch_mode && (cLINES < MIN_LINES_ON_TERMINAL || cCOLS < MIN_COLUMNS_ON_TERMINAL)) {
 			ring_bell();
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			tin_done(EXIT_FAILURE, _(txt_screen_too_small_exiting), tin_progname);
 		}
 		wait_message(0, "%s\n", cvers);
@@ -228,10 +256,12 @@ main(
 		nntp_server = getserverbyfile(NNTP_SERVER_FILE);
 
 	open_msglog(); /* depends on nntp_server */
+	read_server_config();
 
 	if (read_news_via_nntp && !read_saved_news) {
 		if (use_nntps && tintls_init()) {
 			tintls_exit();
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			free_all_arrays();
 			giveup();
 		}
@@ -239,23 +269,26 @@ main(
 		if (nntp_open()) {
 			nntp_close(FALSE);
 			tintls_exit();
+			FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 			free_all_arrays();
 			giveup();
 		}
 	}
 
-	read_server_config();
-
 	/*
 	 * exit early - unfortunately we can't do that in read_cmd_line_options()
 	 * as nntp_caps.over_cmd is set in nntp_open()
 	 *
-	 * TODO: does the logic make sense? what
-	 * if (update_index && !nntp_caps.over_cmd && !tinrc.cache_overview_files)
-	 * no error message? why?
+	 * without nntp_caps.over_cmd we used HEAD to fetch the data
+	 * which is MUCH slower, inform the user about this (but he can't
+	 * do much about it anyway)?
+	 *
+	 * why evaluate tinrc.cache_overview_files in error_message, we know
+	 * it is false -> txt_batch_update_unavail could hold a fixed string.
 	 */
-	if (update_index && nntp_caps.over_cmd && !tinrc.cache_overview_files) {
+	if (update_index && !tinrc.cache_overview_files) {
 		error_message(2, _(txt_batch_update_unavail), tin_progname, print_boolean(tinrc.cache_overview_files));
+		FREE_ARGV_IF_NEEDED(argv_orig, cmdargs);
 		free_all_arrays();
 		giveup();
 	}
@@ -279,7 +312,7 @@ main(
 		/*
 		 * Read user specific keybindings and input history
 		 */
-		wait_message(0, _(txt_reading_keymap_file));
+		keymap_file[0] = '\0';
 		read_keymap_file();
 		read_input_history_file();
 
@@ -310,7 +343,8 @@ main(
 	 * $AUTOSUBSCRIBE groups
 	 */
 	no_write = tmp_no_write;
-	read_attributes_file(TRUE);
+	if (!read_attributes_file(TRUE))
+		global_attributes_file[0] = '\0';
 	read_attributes_file(FALSE);
 	start_groupnum = read_news_active_file();
 #ifdef DEBUG
@@ -396,15 +430,11 @@ main(
 	 * Check/start if any new/unread articles
 	 */
 	if (check_any_unread)
-		tin_done(check_start_save_any_news(CHECK_ANY_NEWS, catchup), NULL);
+		tin_done(check_start_save_any_news(CHECK_ANY_NEWS, catchup, num_cmd_line_groups), NULL);
 
 	if (start_any_unread) {
-		batch_mode = TRUE;			/* Suppress some unwanted on-screen garbage */
-		if ((start_groupnum = check_start_save_any_news(START_ANY_NEWS, catchup)) == -1) {
-			batch_mode = FALSE;
+		if ((start_groupnum = check_start_save_any_news(START_ANY_NEWS, catchup, num_cmd_line_groups)) == -1)
 			tin_done(EXIT_SUCCESS, NULL);
-		}
-		batch_mode = FALSE;
 	}
 
 	/*
@@ -417,7 +447,7 @@ main(
 	 *       for speed reasons?
 	 */
 	if (mail_news || save_news) {
-		check_start_save_any_news(mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup);
+		check_start_save_any_news(mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup, num_cmd_line_groups);
 		tin_done(EXIT_SUCCESS, NULL);
 	}
 
@@ -466,7 +496,7 @@ main(
 
 /*
  * process command line options
- * [01235789beEFijJKLOtyY] are unused
+ * [01235789beEFijJKOyY] are unused
  * [W] is reserved
  * [BPU] have been in use at some time, but now are unused:
  *   B BBS mode (M_AMIGA only)
@@ -475,31 +505,37 @@ main(
  * reused with different function:
  *   C was count articles, now is activate COMPRESS DEFLATE
  */
-#define OPTIONS "46aAcCdD:f:g:G:hHI:klm:M:nNop:qQrRs:STuvVwxXzZ"
+#define OPTIONS "46aAcCdD:f:g:G:hHI:klL:m:M:nNop:qQrRs:St:TuvVwxXzZ"
 
-static void
+static char **
 read_cmd_line_options(
 	int argc,
 	char *argv[])
 {
+	char **argv_orig = argv;
 	int ch;
+#if defined(NNTP_ABLE) || defined(DEBUG)
+	int i;
+#endif /* NNTP_ABLE || DEBUG */
+#ifdef NNTP_ABLE
 	t_bool newsrc_set = FALSE;
+#endif /* NNTP_ABLE */
 
 	envargs(&argc, &argv, "TINRC");
 
 	while ((ch = getopt(argc, argv, OPTIONS)) != -1) {
 		switch (ch) {
-
 			case '4':
 #if defined(NNTP_ABLE) && defined(INET6)
 				force_ipv4 = TRUE;
 				read_news_via_nntp = TRUE;
 #else
 #	ifdef NNTP_ABLE
-				error_message(2, _(txt_option_not_enabled), "-DENABLE_IPV6");
+				error_message(0, _(txt_option_not_enabled), "-DENABLE_IPV6");
 #	else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
 #	endif /* NNTP_ABLE */
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -513,10 +549,11 @@ read_cmd_line_options(
 				read_news_via_nntp = TRUE;
 #	else
 #	ifdef NNTP_ABLE
-				error_message(2, _(txt_option_not_enabled), "-DENABLE_IPV6");
+				error_message(0, _(txt_option_not_enabled), "-DENABLE_IPV6");
 #	else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
 #	endif /* NNTP_ABLE */
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -528,7 +565,8 @@ read_cmd_line_options(
 #ifdef HAVE_COLOR
 				cmdline.args |= CMDLINE_USE_COLOR;
 #else
-				error_message(2, _(txt_option_not_enabled), "-DHAVE_COLOR");
+				error_message(0, _(txt_option_not_enabled), "-DHAVE_COLOR");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -541,7 +579,8 @@ read_cmd_line_options(
 				force_auth_on_conn_open = TRUE;
 				read_news_via_nntp = TRUE;
 #else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -559,14 +598,16 @@ read_cmd_line_options(
 #	ifdef USE_ZLIB
 				use_compress = TRUE;
 #	else
-				error_message(2, _(txt_option_not_enabled), "-DUSE_ZLIB");
+				error_message(0, _(txt_option_not_enabled), "-DUSE_ZLIB");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
 #	endif /* USE_ZLIB */
 #else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -575,15 +616,30 @@ read_cmd_line_options(
 				break;
 
 			case 'd':
+				cmdline.args |= CMDLINE_NO_DESCRIPTION;
 				show_description = FALSE;
 				break;
 
 			case 'D':		/* debug mode */
 #ifdef DEBUG
-				debug = atoi(optarg) & 0xff;
-				debug_delete_files();
+				i = s2i(optarg, 0, 2 * DEBUG_REMOVE - 1);
+				switch (errno) {
+					case EINVAL: /* TODO: extra error message */
+					case ERANGE:
+						error_message(0, _(txt_value_out_of_range), "-D ", (int) strtol(optarg, NULL, 10), 2 * DEBUG_REMOVE - 1);
+						debug = 0;
+						if (!batch_mode)
+							sleep(2);
+						break;
+
+					default:
+						break;
+				}
+				if ((debug = (unsigned short) i))
+					debug_delete_files();
 #else
-				error_message(2, _(txt_option_not_enabled), "-DDEBUG");
+				error_message(0, _(txt_option_not_enabled), "-DDEBUG");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -593,40 +649,137 @@ read_cmd_line_options(
 
 			case 'f':	/* newsrc file */
 				my_strncpy(newsrc, optarg, sizeof(newsrc) - 1);
+#ifdef NNTP_ABLE
 				newsrc_set = TRUE;
+#endif /* NNTP_ABLE */
 				break;
 
 			case 'G':
-				cmdline.getart_limit = atoi(optarg);
-				cmdline.args |= CMDLINE_GETART_LIMIT;
+				cmdline.getart_limit = s2i(optarg, INT_MIN, INT_MAX);
+				/* TODO: add EINVAL/ERANGE error messages here */
+				if (cmdline.getart_limit != 0)
+					cmdline.args |= CMDLINE_GETART_LIMIT;
 				break;
 
 			case 'g':	/* select alternative NNTP-server, implies -r */
 #ifdef NNTP_ABLE
-				my_strncpy(cmdline.nntpserver, optarg, sizeof(cmdline.nntpserver) - 1);
-				{ /* ":port" suffix - no IPv6-address support yet */
+				cmdline.nntpserver= my_strdup(optarg);
+				str_trim(cmdline.nntpserver);
+				{
 					char *p;
-					unsigned short i;
 
 					if ((p = strchr(cmdline.nntpserver, ':')) != NULL) {
-						if (strrchr(cmdline.nntpserver, ':') == p) {
+						if (*cmdline.nntpserver != '[' && strrchr(cmdline.nntpserver, ':') == p) { /* exact 1 x ':' must be name:port or ipv4:port */
 							*p++ = '\0';
+							i = s2i(p, 0, 65535);
+							switch (errno) {
+								case EINVAL:
+									error_message(0, _(txt_port_not_numeric), cmdline.nntpserver, p);
+									FREE_ARGV_IF_NEEDED(argv_orig, argv);
+									free_all_arrays();
+									giveup();
+									/* keep lint quiet: */
+									/* FALLTHROUGH */
+									break;
 
-							if ((i = (unsigned short) atoi(p)) != 0)
-								nntp_tcp_port = i;
-#	ifdef DEBUG
-							else {
-								if (debug & DEBUG_MISC) /* -> lang.c and _()? */
-									wait_message(3, "Port isn't numeric: %s:%s\n", cmdline.nntpserver, p);
+								case ERANGE:
+									error_message(0, _(txt_value_out_of_range), "-g ", (int) strtol(optarg, NULL, 10), 65535);
+									if (!batch_mode)
+										sleep(2);
+									break;
+
+								default:
+									nntp_tcp_port = (unsigned short) i;
+									break;
 							}
-#	endif /* DEBUG */
+						} else /* "[ipv6]"[:port] */
+#	ifndef INET6
+						{
+							error_message(0, _(txt_option_not_enabled), "-DENABLE_IPV6");
+							FREE_ARGV_IF_NEEDED(argv_orig, argv);
+							free_all_arrays();
+							giveup();
 						}
+#	else
+						{
+							char *q;
+							int j = 0;
+
+							if (*cmdline.nntpserver == '[' && (q = strrchr(cmdline.nntpserver, ']')) != NULL) {
+								if ((p = strchr(cmdline.nntpserver, ':')) != NULL) {
+									if (p > q) { /* not an IPv6 literal (e.g. [host.name]:port or [i.p.v.4]:port, must not be in []) */
+										error_message(0, _(txt_error_not_ipv6_literal), cmdline.nntpserver);
+										FREE_ARGV_IF_NEEDED(argv_orig, argv);
+										free_all_arrays();
+										giveup();
+									}
+								}
+								if ((p = strchr(q, ':')) != NULL) {
+									if (p == q + 1) {
+										i = s2i(++p, 0, 65535);
+										switch (errno) {
+											case EINVAL:
+												*++q = '\0'; /* remove tailing ':' */
+												error_message(0, _(txt_port_not_numeric), cmdline.nntpserver, p);
+												FREE_ARGV_IF_NEEDED(argv_orig, argv);
+												free_all_arrays();
+												giveup();
+												/* keep lint quiet: */
+												/* FALLTHROUGH */
+												break;
+
+											case ERANGE:
+												error_message(0, _(txt_value_out_of_range), "-g ", (int) strtol(optarg, NULL, 10), 65535);
+												if (!batch_mode)
+													sleep(2);
+												break;
+
+											default:
+												nntp_tcp_port = (unsigned short) i;
+												break;
+										}
+									}
+								}
+								*q = '\0'; /* remove tailing ']' */
+								for (i = 1; cmdline.nntpserver[i] != '\0'; i++) { /* remove leading '[' */
+									cmdline.nntpserver[i - 1] = cmdline.nntpserver[i];
+									/* minimal IPv6 syntax checking - just if we didn't find inet_pton() */
+									if (cmdline.nntpserver[i] == ':')
+										j++;
+									else {
+										if (!IS_XDIGIT(cmdline.nntpserver[i]))
+											j += 1000;
+									}
+								}
+								cmdline.nntpserver[i - 1] = '\0';
+								str_trim(cmdline.nntpserver);
+								{
+#		if defined(HAVE_INET_PTON) && defined(AF_INET6)
+									struct in_addr inaddr;
+
+									(void) j; /* silence unused-but-set */
+									if (inet_pton(AF_INET6, cmdline.nntpserver, &inaddr) != 1)
+#		else
+									if (j < 2 || j > 7)
+#		endif /* HAVE_INET_PTON && AF_INET6 */
+									{
+										error_message(0, _(txt_error_not_ipv6_literal), cmdline.nntpserver);
+										FREE_ARGV_IF_NEEDED(argv_orig, argv);
+										free_all_arrays();
+										giveup();
+									}
+								}
+							}
+						}
+#	endif /* !INET6 */
 					}
 				}
-				cmdline.args |= CMDLINE_NNTPSERVER;
+				if (!*cmdline.nntpserver)
+					FreeAndNull(cmdline.nntpserver);
 				read_news_via_nntp = TRUE;
 #else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -636,10 +789,12 @@ read_cmd_line_options(
 
 			case 'H':
 				show_intro_page();
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				exit(EXIT_SUCCESS);
 				/* keep lint quiet: */
-				/* FALLTHROUGH */
+				/* NOTREACHED */
+				break;
 
 			case 'I':
 				joinpath(index_newsdir, sizeof(index_newsdir), optarg, INDEX_NEWSDIR);
@@ -650,7 +805,8 @@ read_cmd_line_options(
 				insecure_nntps = TRUE;
 				use_nntps = TRUE;
 #else
-				error_message(2, _(txt_option_not_enabled), "--with-nntps");
+				error_message(0, _(txt_option_not_enabled), "--with-nntps");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -662,9 +818,22 @@ read_cmd_line_options(
 				list_active = TRUE;
 				break;
 
+			case 'L':
+#ifdef NNTP_ABLE
+				cmdline.msgid = my_strdup(optarg);
+				str_trim(cmdline.msgid);
+#else
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NNTP_ABLE */
+				break;
+
 			case 'm':
-				my_strncpy(cmdline.maildir, optarg, sizeof(cmdline.maildir) - 1);
-				cmdline.args |= CMDLINE_MAILDIR;
+				cmdline.maildir = my_strdup(optarg);
 				break;
 
 			case 'M':	/* mail new news to specified user */
@@ -692,7 +861,8 @@ read_cmd_line_options(
 				post_postponed_and_exit = TRUE;
 				check_for_new_newsgroups = FALSE;
 #else
-				error_message(2, _(txt_option_not_enabled), "-UNO_POSTING");
+				error_message(0, _(txt_option_not_enabled), "-UNO_POSTING");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -702,11 +872,32 @@ read_cmd_line_options(
 
 			case 'p': /* implies -r */
 #ifdef NNTP_ABLE
+				i = s2i(optarg, 0, 65535);
+
+				switch (errno) {
+					case EINVAL:
+						error_message(0, _(txt_port_not_numeric), "", optarg);
+						FREE_ARGV_IF_NEEDED(argv_orig, argv);
+						free_all_arrays();
+						giveup();
+						/* keep lint quiet: */
+						/* FALLTHROUGH */
+						break;
+
+					case ERANGE:
+						error_message(0, _(txt_value_out_of_range), "-p ", (int) strtol(optarg, NULL, 10), 65535);
+						if (!batch_mode)
+							sleep(2);
+						break;
+
+					default:
+						nntp_tcp_port = (unsigned short) i;
+						break;
+				}
 				read_news_via_nntp = TRUE;
-				if (atoi(optarg) != 0)
-					nntp_tcp_port = (unsigned short) atoi(optarg);
 #else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -722,13 +913,15 @@ read_cmd_line_options(
 				newsrc_active = TRUE;
 				check_for_new_newsgroups = FALSE;
 				show_description = FALSE;
+				cmdline.args |= CMDLINE_NO_DESCRIPTION;
 				break;
 
 			case 'r':	/* read news remotely from default NNTP server */
 #ifdef NNTP_ABLE
 				read_news_via_nntp = TRUE;
 #else
-				error_message(2, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -745,8 +938,7 @@ read_cmd_line_options(
 				break;
 
 			case 's':
-				my_strncpy(cmdline.savedir, optarg, sizeof(cmdline.savedir) - 1);
-				cmdline.args |= CMDLINE_SAVEDIR;
+				cmdline.savedir = my_strdup(optarg);
 				break;
 
 			case 'S':	/* save new news to dir structure */
@@ -754,11 +946,38 @@ read_cmd_line_options(
 				batch_mode = TRUE;
 				break;
 
+			case 't':
+#if defined(NNTP_ABLE) && defined(HAVE_ALARM) && defined(SIGALRM)
+				cmdline.nntp_timeout = s2i(optarg, 0, TIN_NNTP_TIMEOUT_MAX);
+				switch (errno) {
+					case EINVAL: /* TODO: extra error message */
+					case ERANGE:
+						error_message(0, _(txt_value_out_of_range), "-t ", (int) strtol(optarg, NULL, 10), TIN_NNTP_TIMEOUT_MAX);
+						if (!batch_mode)
+							sleep(2);
+						break;
+
+					default:
+						break;
+				}
+				if (cmdline.nntp_timeout)
+					cmdline.args |= CMDLINE_NNTP_TIMEOUT;
+#else
+				error_message(0, _(txt_option_not_enabled), "-DNNTP_ABLE");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
+				free_all_arrays();
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NNTP_ABLE && HAVE_ALARM && SIGALRM */
+				break;
+
 			case 'T':
 #ifdef NNTPS_ABLE
 				use_nntps = TRUE;
 #else
-				error_message(2, _(txt_option_not_enabled), "--with-nntps");
+				error_message(0, _(txt_option_not_enabled), "--with-nntps");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -776,18 +995,21 @@ read_cmd_line_options(
 				break;
 
 			case 'V':
-				tin_version_info(stderr);
+				tin_version_info(stderr, verbose);
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				exit(EXIT_SUCCESS);
 				/* keep lint quiet: */
-				/* FALLTHROUGH */
+				/* NOTREACHED */
+				break;
 
 			case 'w':	/* post article & exit */
 #ifndef NO_POSTING
 				post_article_and_exit = TRUE;
 				check_for_new_newsgroups = FALSE;
 #else
-				error_message(2, _(txt_option_not_enabled), "-UNO_POSTING");
+				error_message(0, _(txt_option_not_enabled), "-UNO_POSTING");
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				giveup();
 				/* keep lint quiet: */
@@ -821,6 +1043,7 @@ read_cmd_line_options(
 			case '?':
 			default:
 				usage(tin_progname);
+				FREE_ARGV_IF_NEEDED(argv_orig, argv);
 				free_all_arrays();
 				exit(EXIT_SUCCESS);
 		}
@@ -828,36 +1051,31 @@ read_cmd_line_options(
 
 #ifdef NNTP_ABLE
 	if (nntp_tcp_port == 0) {
-#ifdef NNTPS_ABLE
+#	ifdef NNTPS_ABLE
 		if (use_nntps)
 			nntp_tcp_port = nntps_tcp_default_port;
 		else
-#endif /* NNTPS_ABLE */
+#	endif /* NNTPS_ABLE */
 			nntp_tcp_port = nntp_tcp_default_port;
 	}
-#endif /* NNTP_ABLE*/
+#endif /* NNTP_ABLE */
 
-	cmdargs = argv;
+	/* cmdargs = argv; */
 	num_cmdargs = optind;
 	max_cmdargs = argc;
-	if (!newsrc_set) {
+
+#ifdef NNTP_ABLE
+	if (!newsrc_set) { /* this gives "-f" a higher priority (in contrast to -p) */
 		if (read_news_via_nntp) {
 			nntp_server = getserverbyfile(NNTP_SERVER_FILE);
 			get_newsrcname(newsrc, sizeof(newsrc), nntp_server);
 		} else {
-#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
-			struct utsname uts;
-			(void) uname(&uts);
-			get_newsrcname(newsrc, sizeof(newsrc), uts.nodename);
-#else
-			char nodenamebuf[256]; /* SUSv2 limit; better use HOST_NAME_MAX */
-#	ifdef HAVE_GETHOSTNAME
-			(void) gethostname(nodenamebuf, sizeof(nodenamebuf));
-#	endif /* HAVE_GETHOSTNAME */
+			const char *nodenamebuf = get_host_name();
+
 			get_newsrcname(newsrc, sizeof(newsrc), nodenamebuf);
-#endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
 		}
 	}
+#endif /* NNTP_ABLE */
 
 	/*
 	 * Sort out option conflicts
@@ -884,7 +1102,15 @@ read_cmd_line_options(
 	 *       -NM
 	 *       -oN, -oM (at this stage we no longer know if -N or -M was given)
 	 *       -wN, -wM (at this stage we no longer know if -N or -M was given)
+	 *       -NZ, -MZ (at this stage we no longer know if -N or -M was given)
+	 *       -LM, -LN (at this stage we no longer know if -N or -M was given)
+	 *       (extend t_cmdlineopts with a flag indicating which of -M/-N was given)
+	 *       -uo, -uw, -uX, ...
 	 */
+	if (post_postponed_and_exit && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-o", "-L", "-L");
+		FreeAndNull(cmdline.msgid);
+	}
 	if (post_postponed_and_exit && force_no_post) {
 		wait_message(2, _(txt_useless_combination), "-o", "-x", "-x");
 		force_no_post = FALSE;
@@ -901,6 +1127,10 @@ read_cmd_line_options(
 		wait_message(2, _(txt_useless_combination), "-w", "-x", "-x");
 		force_no_post = FALSE;
 	}
+	if (post_article_and_exit && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-w", "-L", "-w");
+		post_article_and_exit = FALSE;
+	}
 	if (post_article_and_exit && start_any_unread) {
 		wait_message(2, _(txt_useless_combination), "-w", "-z", "-z");
 		start_any_unread = FALSE;
@@ -917,29 +1147,57 @@ read_cmd_line_options(
 		wait_message(2, _(txt_useless_combination), "-c", "-Z", "-c");
 		catchup = FALSE;
 	}
+	if (catchup && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-c", "-L", "-c");
+		catchup = FALSE;
+	}
+	if (update_index && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-u", "-L", "-u");
+		update_index = FALSE;
+	}
 	if (newsrc_active && read_saved_news) {
 		wait_message(2, _(txt_useless_combination), "-n", "-R", "-n");
 		newsrc_active = read_news_via_nntp = FALSE;
 	}
+	if (read_saved_news && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-R", "-L", "-R");
+		read_saved_news = FALSE;
+	}
+	if (save_news && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-S", "-L", "-S");
+		save_news = FALSE;
+	}
 	if (start_any_unread && save_news) {
 		wait_message(2, _(txt_useless_combination), "-z", "-S", "-z");
 		start_any_unread = FALSE;
 	}
+	if (start_any_unread && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-z", "-L", "-z");
+		start_any_unread = FALSE;
+	}
 	if (save_news && check_any_unread) {
 		wait_message(2, _(txt_useless_combination), "-S", "-Z", "-S");
 		save_news = FALSE;
 	}
-	if (start_any_unread && check_any_unread) {
+	if (check_any_unread && start_any_unread) {
 		wait_message(2, _(txt_useless_combination), "-Z", "-z", "-Z");
 		check_any_unread = FALSE;
 	}
-#	ifdef DEBUG
-	if ((debug & DEBUG_NNTP) && !read_news_via_nntp) {
-		/* TODO: lang.c */
-		wait_message(3, _(txt_useless_combination), _("reading from local spool"), "-D nntp", "-D nntp");
-		debug &= ~DEBUG_NNTP;
+	if (check_any_unread && cmdline.msgid) {
+		wait_message(2, _(txt_useless_combination), "-Z", "-L", "-Z");
+		check_any_unread = FALSE;
 	}
-#	endif /* DEBUG */
+#ifdef DEBUG
+#	ifdef NNTP_ABLE
+	if ((debug & DEBUG_NNTP) && !read_news_via_nntp)
+#		else
+	if (debug & DEBUG_NNTP)
+#	endif /* NNTP_ABLE */
+	{
+		wait_message(3, _(txt_useless_combination), _(txt_reading_from_spool), "-D nntp", "-D nntp");
+		debug ^= DEBUG_NNTP;
+	}
+#endif /* DEBUG */
 
 #if defined(NNTP_ABLE) && defined(INET6)
 	if (force_ipv4 && force_ipv6) {
@@ -978,6 +1236,8 @@ read_cmd_line_options(
 	 */
 	if (!list_active && !newsrc_active)
 		list_active = newsrc_active = TRUE;
+
+	return argv;
 }
 
 
@@ -988,92 +1248,102 @@ static void
 usage(
 	char *theProgname)
 {
-	error_message(2, _(txt_usage_tin), theProgname);
+	error_message(0, _(txt_usage_tin), theProgname);
 
 #if defined(NNTP_ABLE) && defined(INET6)
-	error_message(2, _(txt_usage_force_ipv4));
-	error_message(2, _(txt_usage_force_ipv6));
+	error_message(0, _(txt_usage_force_ipv4));
+	error_message(0, _(txt_usage_force_ipv6));
 #endif /* NNTP_ABLE && INET6 */
 
 #ifdef HAVE_COLOR
-	error_message(2, _(txt_usage_toggle_color));
+	error_message(0, _(txt_usage_toggle_color));
 #endif /* HAVE_COLOR */
 #ifdef NNTP_ABLE
-	error_message(2, _(txt_usage_force_authentication));
+	error_message(0, _(txt_usage_force_authentication));
 #endif /* NNTP_ABLE */
 
-	error_message(2, _(txt_usage_catchup));
+	error_message(0, _(txt_usage_catchup));
 #if defined(NNTP_ABLE) && defined(USE_ZLIB)
-	error_message(2, _(txt_usage_compress));
+	error_message(0, _(txt_usage_compress));
 #endif /* NNTP_ABLE && USE_ZLIB */
-	error_message(2, _(txt_usage_dont_show_descriptions));
+	error_message(0, _(txt_usage_dont_show_descriptions));
 
 #ifdef DEBUG
-	error_message(2, _(txt_usage_debug));
+	error_message(0, _(txt_usage_debug));
 #endif /* DEBUG */
 
-	error_message(2, _(txt_usage_newsrc_file), newsrc);
-	error_message(2, _(txt_usage_getart_limit));
+	error_message(0, _(txt_usage_newsrc_file), newsrc);
+	error_message(0, _(txt_usage_getart_limit));
 
 #ifdef NNTP_ABLE
 #	ifdef NNTP_DEFAULT_SERVER
-	error_message(2, _(txt_usage_newsserver), get_val("NNTPSERVER", NNTP_DEFAULT_SERVER));
+	error_message(0, _(txt_usage_newsserver), get_val("NNTPSERVER", NNTP_DEFAULT_SERVER));
 #	else
-	error_message(2, _(txt_usage_newsserver), get_val("NNTPSERVER", "news"));
+	error_message(0, _(txt_usage_newsserver), get_val("NNTPSERVER", "news"));
 #	endif /* NNTP_DEFAULT_SERVER */
 #endif /* NNTP_ABLE */
 
-	error_message(2, _(txt_usage_help_message));
-	error_message(2, _(txt_usage_help_information), theProgname);
-	error_message(2, _(txt_usage_index_newsdir), index_newsdir);
+	error_message(0, _(txt_usage_help_message));
+	error_message(0, _(txt_usage_help_information), theProgname);
+	error_message(0, _(txt_usage_index_newsdir), index_newsdir);
 
 #ifdef NNTP_ABLE
 #	ifdef NNTPS_ABLE
-	error_message(2, _(txt_usage_use_insecure_nntps));
+	error_message(0, _(txt_usage_use_insecure_nntps));
 #	endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 
-	error_message(2, _(txt_usage_read_only_active));
-	error_message(2, _(txt_usage_maildir), tinrc.maildir);
-	error_message(2, _(txt_usage_mail_new_news_to_user));
-	error_message(2, _(txt_usage_read_only_subscribed));
-	error_message(2, _(txt_usage_mail_new_news));
-	error_message(2, _(txt_usage_post_postponed_arts));
+	error_message(0, _(txt_usage_read_only_active));
+
+#ifdef NNTP_ABLE
+	error_message(0, _(txt_usage_lookup_id));
+#endif /* NNTP_ABLE */
+
+	error_message(0, _(txt_usage_maildir), tinrc.maildir);
+	error_message(0, _(txt_usage_mail_new_news_to_user));
+	error_message(0, _(txt_usage_read_only_subscribed));
+	error_message(0, _(txt_usage_mail_new_news));
+	error_message(0, _(txt_usage_post_postponed_arts));
 
 #ifdef NNTP_ABLE
 #	ifdef NNTPS_ABLE
-	error_message(2, _(txt_usage_port), use_nntps ? nntps_tcp_default_port : nntp_tcp_default_port);
+	error_message(0, _(txt_usage_port), use_nntps ? nntps_tcp_default_port : nntp_tcp_default_port);
 #	else
-	error_message(2, _(txt_usage_port), nntp_tcp_default_port);
+	error_message(0, _(txt_usage_port), nntp_tcp_default_port);
 #	endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
 
-	error_message(2, _(txt_usage_dont_check_new_newsgroups));
-	error_message(2, _(txt_usage_quickstart));
+	error_message(0, _(txt_usage_dont_check_new_newsgroups));
+	error_message(0, _(txt_usage_quickstart));
 
 #ifdef NNTP_ABLE
 	if (!read_news_via_nntp)
-		error_message(2, _(txt_usage_read_news_remotely));
+		error_message(0, _(txt_usage_read_news_remotely));
 #endif /* NNTP_ABLE */
 
-	error_message(2, _(txt_usage_read_saved_news));
-	error_message(2, _(txt_usage_savedir), tinrc.savedir);
-	error_message(2, _(txt_usage_save_new_news));
+	error_message(0, _(txt_usage_read_saved_news));
+	error_message(0, _(txt_usage_savedir), tinrc.savedir);
+	error_message(0, _(txt_usage_save_new_news));
+
 #ifdef NNTP_ABLE
+#	if defined(HAVE_ALARM) && defined(SIGALRM)
+	error_message(0, _(txt_usage_nntp_timeout), TIN_NNTP_TIMEOUT);
+#	endif /* HAVE_ALARM && SIGALRM */
 #	ifdef NNTPS_ABLE
-	error_message(2, _(txt_usage_use_nntps));
+	error_message(0, _(txt_usage_use_nntps));
 #	endif /* NNTPS_ABLE */
 #endif /* NNTP_ABLE */
-	error_message(2, _(txt_usage_update_index_files));
-	error_message(2, _(txt_usage_verbose));
-	error_message(2, _(txt_usage_version));
-	error_message(2, _(txt_usage_post_article));
-	error_message(2, _(txt_usage_no_posting));
-	error_message(2, _(txt_usage_dont_save_files_on_quit));
-	error_message(2, _(txt_usage_start_if_unread_news));
-	error_message(2, _(txt_usage_check_for_unread_news));
 
-	error_message(2, _(txt_usage_mail_bugreport), bug_addr);
+	error_message(0, _(txt_usage_update_index_files));
+	error_message(0, _(txt_usage_verbose));
+	error_message(0, _(txt_usage_version));
+	error_message(0, _(txt_usage_post_article));
+	error_message(0, _(txt_usage_no_posting));
+	error_message(0, _(txt_usage_dont_save_files_on_quit));
+	error_message(0, _(txt_usage_start_if_unread_news));
+	error_message(0, _(txt_usage_check_for_unread_news));
+
+	error_message(0, _(txt_usage_mail_bugreport), bug_addr);
 }
 
 
@@ -1146,9 +1416,11 @@ read_cmd_line_groups(
 					if (my_group_add(active[i].name, TRUE) != -1) {
 						matched++;
 						if (post_article_and_exit) {
-							my_strncpy(tinrc.default_post_newsgroups, active[i].name, sizeof(tinrc.default_post_newsgroups) - 1);
+							FreeIfNeeded(tinrc.default_post_newsgroups);
+							tinrc.default_post_newsgroups = my_strdup(active[i].name);
 							break;
 						}
+						active[i].read_during_session = TRUE; /* misuse for "-[zZMN] grp" */
 					}
 				}
 			}
@@ -1171,14 +1443,39 @@ create_mail_save_dirs(
 	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL, FALSE))
 		joinpath(path, sizeof(path), homedir, DEFAULT_MAILDIR);
 
-	if (stat(path, &sb) == -1)
-		my_mkdir(path, (mode_t) (S_IRWXU));
+	if (stat(path, &sb) == -1) {
+		if (my_mkdir(path, (mode_t) (S_IRWXU)) == -1)
+			error_message(2, _(txt_cannot_create), path);
+	}
 
 	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL, FALSE))
 		joinpath(path, sizeof(path), homedir, DEFAULT_SAVEDIR);
 
-	if (stat(path, &sb) == -1)
-		my_mkdir(path, (mode_t) (S_IRWXU));
+	if (stat(path, &sb) == -1) {
+		if (my_mkdir(path, (mode_t) (S_IRWXU)) == -1)
+			error_message(2, _(txt_cannot_create), path);
+	}
+}
+
+
+/*
+ * TODO: find a better solution to free() cmdargs from outside main.c
+ */
+void
+handle_cmdargs(
+	t_bool init)
+{
+	static t_bool argv_modified = FALSE;
+
+	if (init)
+		argv_modified = TRUE;
+	else if (argv_modified) {
+		if (cmdargs) {
+			free(*(cmdargs + 1));
+			free(cmdargs);
+			cmdargs = NULL;
+		}
+	}
 }
 
 
