@@ -3,10 +3,10 @@
  *  Module    : makecfg.c
  *  Author    : Thomas E. Dickey
  *  Created   : 1997-08-23
- *  Updated   : 2023-04-09
+ *  Updated   : 2024-03-30
  *  Notes     : #defines and structs for options_menu.c
  *
- * Copyright (c) 1997-2023 Thomas E. Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2024 Thomas E. Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -40,9 +40,12 @@
 /* replace tin.h {{ */
 
 #ifdef HAVE_CONFIG_H
-#include <autoconf.h>
+#include "autoconf.h"
 #endif /* HAVE_CONFIG_H */
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif /* HAVE_STDDEF_H */
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif /* HAVE_CONFIG_H */
@@ -169,12 +172,12 @@ parse_tbl(
 	/* strip leading/trailing blanks */
 	do {
 		t--;
-		if (isspace ((int)*t))
+		if (isspace((unsigned char) *t))
 			*t = '\0';
 		else
 			break;
 	} while (t > s);
-	while (isspace ((int)*s))
+	while (isspace((unsigned char) *s))
 		s++;
 	buffer = s;
 
@@ -187,9 +190,9 @@ parse_tbl(
 			 * otherwise the data consists of 2 blank
 			 * separated columns (name, type).
 			 */
-			while (*s && !isspace ((int)*s))
+			while (*s && !isspace((unsigned char) *s))
 				s++;
-			while (isspace ((int)*s))
+			while (isspace((unsigned char) *s))
 				*s++ = '\0';
 			store_data(buffer, s);
 		}
@@ -244,12 +247,13 @@ typename_of(
 {
 	if (!strcmp(p->type, "OPT_STRING"))
 		return "char *";
-	if (!strcmp(p->type, "OPT_CHAR"))
+	if (!strcmp(p->type, "OPT_CHAR")) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		return "wchar_t *";
 #else
 		return "char *";
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	}
 	if (!strcmp(p->type, "OPT_ON_OFF"))
 		return "t_bool *";
 	return "int *";
@@ -270,12 +274,12 @@ generate_tbl(
 		,"#define OPT_CHAR      5"
 		,""
 		,"struct t_option option_table[]={"
-		,0
+		,NULL
 	};
 	static const char *const table_2[] =
 	{
 		"};"
-		,0
+		,NULL
 	};
 	static const char prefix[] = "  { ";
 	static const char suffix[] = "},";
@@ -305,7 +309,7 @@ generate_tbl(
 			if (is_title(p)) {
 				fprintf(ofp, "0, NULL, ");
 			} else if (!is_int) {
-				fprintf(ofp, "OINX_%s, 0, ", p->name);
+				fprintf(ofp, "OINX_%s, NULL, ", p->name);
 			} else {
 				fprintf(ofp, "0, &tinrc.%s, ", p->name);
 			}
@@ -330,13 +334,13 @@ generate_enum(
 
 	static const char *const table_1[] = {
 		"enum option_enum {",
-		0
+		NULL
 	};
 	static const char *const table_2[] = {
 		"\tSIGNAL_HANDLER };",
 		"",
 		"#define LAST_OPT (SIGNAL_HANDLER - 1)",
-		0
+		NULL
 	};
 
 	/* generate enumerated type */
@@ -370,7 +374,12 @@ generate_ptr(
 {
 	MYDATA *p, *q;
 	int after;
+#ifdef __KEFIRCC__ /* && KEFIR_VERSION_MAJOR == 0 && KEFIR_VERSION_MINOR < 4 && *//* workaround a bug in kefir (0.3.1) */
+	const char *addr = "&";
+	const char *sqrbr = !strcmp(opt_type, "OPT_STRING") ? "[0]" : "";
+#else
 	const char *addr = !strcmp(opt_type, "OPT_STRING") ? "" : "&";
+#endif /* __KEFIRCC__ */
 
 	switch (mode) {
 	case 0:
@@ -404,13 +413,24 @@ generate_ptr(
 			}
 			switch (mode) {
 			case 0:
-				fprintf(ofp, "\t%stinrc.%s,%*s/* %2d: %s__ */\n",
+#ifdef __KEFIRCC__ /* see above */
+				fprintf(ofp, "\t%stinrc.%s%s,%*s/* %2d: %s__ */\n",
 					addr,
 					p->name,
-					MAXNAME - (int)(strlen(addr) + strlen(p->name)),
+					sqrbr,
+					MAXNAME - (int) (strlen(addr) - strlen(sqrbr) + strlen(p->name)),
 					" ",
 					index_of(p),
 					p->name);
+#else
+				fprintf(ofp, "\t%stinrc.%s,%*s/* %2d: %s__ */\n",
+					addr,
+					p->name,
+					MAXNAME - (int) (strlen(addr) + strlen(p->name)),
+					" ",
+					index_of(p),
+					p->name);
+#endif /* __KEFIRCC__ */
 				break;
 			case 1:
 				fprintf(ofp, "\tOVAL(oinx_%.*s, %s__)\n",
@@ -484,7 +504,7 @@ makecfg(
 		,"#\t\tdefine OTYP struct"
 		,"#\tendif /* CPP_DOES_CONCAT */"
 		,"#endif /* lint */"
-		,0
+		,NULL
 	};
 	static const char *const table_2[] =
 	{
@@ -496,7 +516,7 @@ makecfg(
 		,"#undef OTYP"
 		,""
 		,"#endif /* TINCFG_H */"
-		,0
+		,NULL
 	};
 	static const char *const table_3[] = {
 		"/* This file is generated by MAKECFG */",
@@ -504,12 +524,12 @@ makecfg(
 		"#ifndef OPTIONS_MENU_H",
 		"#define OPTIONS_MENU_H 1",
 		"",
-		0
+		NULL
 	};
 	static const char *const table_4[] = {
 		"",
 		"#endif /* OPTIONS_MENU_H */",
-		0
+		NULL
 	};
 
 	/*
