@@ -3,10 +3,10 @@
  *  Module    : mimetypes.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-31
- *  Updated   : 2022-04-09
+ *  Updated   : 2024-05-05
  *  Notes     : mime.types handling
  *
- * Copyright (c) 2000-2023 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2024 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,13 +60,13 @@ _lookup_mimetype(
 	const char *ext,
 	t_part *part)
 {
-	FILE *fp;
+	FILE *fp = NULL;
 	const char *exts;
 	char *ptr;
 	char buf[PATH_LEN];
 	int i;
 
-	if ((fp = fopen(file, "r")) == NULL)
+	if ((fp = tin_fopen(file, "r")) == NULL)
 		return FALSE;
 
 	while ((fgets(buf, sizeof(buf), fp)) != NULL) {
@@ -137,8 +137,9 @@ _lookup_extension(
 	char *p;
 	char buf[8192];
 
-	if ((fp = fopen(file, "r")) == NULL)
+	if ((fp = tin_fopen(file, "r")) == NULL)
 		return FALSE;
+
 	while ((fgets(buf, sizeof(buf), fp)) != NULL) {
 		if (buf[0] == '#' || buf[0] == '\n' || strncmp(buf, type, strlen(type)))
 			continue;
@@ -174,16 +175,25 @@ lookup_extension(
 {
 	char *type;
 	char buf[PATH_LEN];
+	int n;
+	size_t tlen;
 
 	if (!major || !minor) {
 		*extension = '\0';
 		return FALSE;
 	}
 
-	type = my_malloc(strlen(major) + 1 + strlen(minor) + 1);
-	strcpy(type, major);
-	strcat(type, "/");
-	strcat(type, minor);
+	if ((n = snprintf(NULL, 0, "%s/%s", major, minor)) < 0) {
+		*extension = '\0';
+		return FALSE;
+	}
+	tlen = (size_t) n + 1;
+	type = my_malloc(tlen);
+	if (snprintf(type, tlen, "%s/%s", major, minor) != n) {
+		*extension = '\0';
+		free(type);
+		return FALSE;
+	}
 
 	joinpath(buf, sizeof(buf), homedir, ".mime.types");
 	if (_lookup_extension(extension, ext_len, buf, type)) {
