@@ -3,10 +3,10 @@
  *  Module    : newsrc.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-07-29
+ *  Updated   : 2024-10-31
  *  Notes     : ArtCount = (ArtMax - ArtMin) + 1  [could have holes]
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -51,7 +51,10 @@
 #	include "newsrc.h"
 #endif /* !NEWSRC_H */
 
-static mode_t newsrc_mode = 0;
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	static mode_t newsrc_mode = 0;
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
+
 
 /*
  * Local prototypes
@@ -62,7 +65,7 @@ static char *parse_subseq(struct t_group *group, char *seq, t_artnum *low, t_art
 static char *parse_get_seq(char *seq, t_artnum *low, t_artnum *high);
 static int write_newsrc_line(FILE *fp, char *line);
 static t_bool create_newsrc(char *newsrc_file);
-static void auto_subscribe_groups(char *newsrc_file);
+static void auto_subscribe_groups(const char *newsrc_file);
 static void get_subscribe_info(struct t_group *grp);
 static void parse_bitmap_seq(struct t_group *group, char *seq);
 static void print_bitmap_seq(FILE *fp, struct t_group *group);
@@ -97,25 +100,55 @@ read_newsrc(
 	 * make a .newsrc if none exist & auto subscribe to set groups
 	 */
 	if ((fp = fopen(newsrc_file, "r")) == NULL) {
-		if (!create_newsrc(newsrc_file))
-			return -1L; /* ouch */
+		if (errno != ENOENT) { /* ENOENT is "ok" here */
+			perror_message(_(txt_cannot_open), newsrc_file);
+			if (!batch_mode)
+				sleep(2);
+		}
+		switch (errno) {
+			case EACCES:
+				return -1L;
 
+			default:	/* e.g. ENOENT */
+				if (!create_newsrc(newsrc_file))
+					return -1L;
+				break;
+		}
 		auto_subscribe_groups(newsrc_file);
 	}
+	/* see if create_newsrc() created the file for us */
 	if (!fp) {
-		if ((fp = fopen(newsrc_file, "r")) == NULL)
+		if ((fp = fopen(newsrc_file, "r")) == NULL) {
+			perror_message(_(txt_cannot_open), newsrc_file);
+			if (!batch_mode)
+				sleep(2);
 			return -1L;
+		}
 	}
 
-	if (fstat(fileno(fp), &statbuf) != -1)
+	if (fstat(fileno(fp), &statbuf) == -1) {
+		perror_message(_(txt_cannot_open), newsrc_file);
+		if (!batch_mode)
+			sleep(2);
+		fclose(fp);
+		return -1L;
+	}
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	else
 		newsrc_mode = statbuf.st_mode;
-	else {
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
+
+	if (S_ISDIR(statbuf.st_mode)) {	/* TODO: die hard? */
+		errno = EISDIR;
+		perror_message(_(txt_cannot_open), newsrc_file);
+		if (!batch_mode)
+			sleep(2);
 		fclose(fp);
 		return -1L;
 	}
 
 	if (!batch_mode || verbose)
-		wait_message(0, _(txt_reading_newsrc));
+		wait_message(0, _(txt_reading_newsrc), newsrc_file);
 
 	while ((grp = tin_fgets(fp, FALSE)) != NULL) {
 		strip_line(grp);
@@ -215,36 +248,32 @@ write_newsrc(
 	FILE *fp_op;
 	char *line;
 	signed long int tot = 0L;
-	struct stat note_stat_newsrc;
 	t_bool write_ok = FALSE;
 	int err;
 
 	if (no_write)
 		return 0L;
 
-	if ((fp_ip = fopen(newsrc, "r")) == NULL)
-		return -1L; /* can't open newsrc */
-
-	/* get size of original newsrc */
-	if (fstat(fileno(fp_ip), &note_stat_newsrc) != 0) {
-		fclose(fp_ip);
-		return -1L; /* can't access newsrc */
-	}
-
-	if (!note_stat_newsrc.st_size) {
-		fclose(fp_ip);
-		return 0L; /* newsrc is empty */
+	if ((fp_ip = tin_fopen(newsrc, "r")) == NULL) {
+		if (errno == ENOENT)
+			perror_message(_(txt_cannot_open), newsrc);
+		if (!batch_mode && errno)
+			sleep(2);
+		return (errno ? -1L : 0L);
 	}
 
 	if ((fp_op = fopen(newnewsrc, "w")) != NULL) {
-		if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+		if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 			fchmod(fileno(fp_op), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 			chmod(newnewsrc, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+		}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 		while ((line = tin_fgets(fp_ip, FALSE)) != NULL)
 			tot += write_newsrc_line(fp_op, line);
@@ -253,7 +282,7 @@ write_newsrc(
 		 * Don't rename if either fclose() fails or ferror() is set
 		 */
 		if ((err = ferror(fp_op)) || fclose(fp_op)) {
-			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), newsrc);
 			unlink(newnewsrc);
 			if (err) {
 				clearerr(fp_op);
@@ -261,12 +290,16 @@ write_newsrc(
 			}
 		} else
 			write_ok = TRUE;
+	} else {
+		perror_message(_(txt_cannot_open_for_saving), newnewsrc);
+		fclose(fp_ip);
+		return 0L; /* o we don't get prompted to try again */
 	}
 
 	fclose(fp_ip);
 
-	if (tot < 1) {
-		error_message(2, _(txt_newsrc_nogroups));
+	if (tot < 1L) {
+		error_message(2, _(txt_newsrc_nogroups), newsrc);
 		unlink(newnewsrc);
 		return 0L;		/* So we don't get prompted to try again */
 	}
@@ -289,13 +322,13 @@ create_newsrc(
 	int i;
 
 	if ((fp = fopen(newsrc_file, "w")) != NULL) {
-		wait_message(0, _(txt_creating_newsrc));
+		wait_message(0, _(txt_creating_newsrc), newsrc);
 
 		for_each_group(i)
 			fprintf(fp, "%s!\n", active[i].name);
 
 		if ((i = ferror(fp)) || fclose(fp)) {
-			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), newsrc);
 			if (i) {
 				clearerr(fp);
 				fclose(fp);
@@ -304,6 +337,11 @@ create_newsrc(
 		}
 		return TRUE; /* newsrc created */
 	}
+
+	perror_message(_(txt_cannot_open), newsrc_file);
+	if (!batch_mode)
+		sleep(2);
+
 	return FALSE;
 }
 
@@ -338,7 +376,7 @@ open_subscription_fp(
  */
 static void
 auto_subscribe_groups(
-	char *newsrc_file)
+	const char *newsrc_file)
 {
 	FILE *fp_newsrc;
 	FILE *fp_subs;
@@ -360,14 +398,17 @@ auto_subscribe_groups(
 		return;
 	}
 
-	if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 		fchmod(fileno(fp_newsrc), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 		chmod(newsrc_file, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+	}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 	/* TODO: test me! */
 	while ((ptr = tin_fgets(fp_subs, FALSE)) != NULL) {
@@ -380,7 +421,7 @@ auto_subscribe_groups(
 	/* We ignore user 'q'uits here. They will get them next time in any case */
 
 	if ((err = ferror(fp_newsrc)) || fclose(fp_newsrc)) {
-		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), newsrc);
 		if (err) {
 			clearerr(fp_newsrc);
 			fclose(fp_newsrc);
@@ -420,7 +461,7 @@ backup_newsrc(
 	}
 
 	if (!backup_file(newsrc, filebuf))
-		error_message(2, _(txt_filesystem_full_backup), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full_backup), newsrc);
 }
 
 
@@ -431,15 +472,13 @@ backup_newsrc(
  */
 int
 group_get_art_info(
-	char *tin_spooldir,
-	char *groupname,
+	const char *tin_spooldir,
+	const char *groupname,
 	int grouptype,
 	t_artnum *art_count,
 	t_artnum *art_max,
 	t_artnum *art_min)
 {
-	DIR *dir;
-	DIR_BUF *direntry;
 	t_artnum artnum;
 
 	if (read_news_via_nntp && grouptype == GROUP_TYPE_NEWS) {
@@ -451,7 +490,7 @@ group_get_art_info(
 		if ((debug & DEBUG_NNTP) && verbose > 1)
 			debug_print_file("NNTP", "group_get_art_info %s", line);
 #	endif /* DEBUG */
-		put_server(line);
+		put_server(line, FALSE);
 
 		switch (get_respcode(line, sizeof(line))) {
 			case OK_GROUP:
@@ -483,11 +522,14 @@ group_get_art_info(
 				return -1;
 		}
 #else
-		my_fprintf(stderr, _("Unreachable?\n")); /* TODO: -> lang.c */
+		my_fprintf(stderr, "%s", _(txt_error_unreachable));
 		return 0;
 #endif /* NNTP_ABLE */
 	} else {
 		char group_path[PATH_LEN];
+		DIR *dir;
+		DIR_BUF *direntry;
+
 		*art_count = T_ARTNUM_CONST(0);
 		*art_min = T_ARTNUM_CONST(0);
 		*art_max = T_ARTNUM_CONST(0);
@@ -572,20 +614,20 @@ subscribe(
 	int sub;
 	t_bool found = FALSE;
 
-	if (no_write)
+	if (no_write || (newfp = fopen(newnewsrc, "w")) == NULL)
 		return;
 
-	if ((newfp = fopen(newnewsrc, "w")) == NULL)
-		return;
-
-	if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 		fchmod(fileno(newfp), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 		chmod(newnewsrc, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+	}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 	if ((fp = fopen(newsrc, "r")) != NULL) {
 		while ((line = tin_fgets(fp, FALSE)) != NULL) {
@@ -622,7 +664,94 @@ subscribe(
 	}
 
 	if ((sub = ferror(newfp)) || fclose(newfp)) {
-		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), newsrc);
+		if (sub) {
+			clearerr(newfp);
+			fclose(newfp);
+		}
+		unlink(newnewsrc);
+	} else
+		rename_file(newnewsrc, newsrc);
+}
+
+
+/*
+ * Like subscribe() but for a bunch of groups.
+ */
+void
+bulk_subscribe(
+	struct t_group **groups,
+	int groups_cnt,
+	int sub_state,
+	t_bool get_info)
+{
+	FILE *fp;
+	FILE *newfp;
+	char *line;
+	char *seq;
+	int i, sub;
+	t_bool found;
+
+	if (no_write || (newfp = fopen(newnewsrc, "w")) == NULL)
+		return;
+
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
+		fchmod(fileno(newfp), newsrc_mode);
+#	else
+#		ifdef HAVE_CHMOD
+		chmod(newnewsrc, newsrc_mode);
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+	}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
+
+	if ((fp = fopen(newsrc, "r")) != NULL) {
+		while ((line = tin_fgets(fp, FALSE)) != NULL) {
+			if ((seq = parse_newsrc_line(line, &sub))) {
+				found = FALSE;
+				for (i = 0; i < groups_cnt; i++) {
+					if (groups[i] && STRCMPEQ(line, groups[i]->name)) {
+						fprintf(newfp, "%s%c %s\n", line, sub_state, seq);
+						groups[i]->subscribed = SUB_BOOL(sub_state);
+
+						/* If previously subscribed to in .newsrc, load up any existing information */
+						if (sub_state == SUBSCRIBED)
+							parse_bitmap_seq(groups[i], seq);
+
+						groups[i] = NULL;
+						found = TRUE;
+						break;
+					}
+				}
+				if (!found)
+					fprintf(newfp, "%s%c %s\n", line, sub, seq);
+			}
+		}
+
+		fclose(fp);
+
+		for (i = 0; i < groups_cnt; i++) {
+			if (groups[i]) {
+				wait_message(0, _(txt_subscribing));
+				spin_cursor();
+				groups[i]->subscribed = SUB_BOOL(sub_state);
+				if (sub_state == SUBSCRIBED) {
+					fprintf(newfp, "%s%c ", groups[i]->name, sub_state);
+					if (get_info) {
+						get_subscribe_info(groups[i]);
+						print_bitmap_seq(newfp, groups[i]);
+					} else /* we are not allowed to issue NNTP cmds during AUTOSUBSCRIBE loop */
+						fprintf(newfp, "1\n");
+				} else
+					fprintf(newfp, "%s%c\n", groups[i]->name, sub_state);
+			}
+		}
+	}
+
+	if ((sub = ferror(newfp)) || fclose(newfp)) {
+		error_message(2, _(txt_filesystem_full), newsrc);
 		if (sub) {
 			clearerr(newfp);
 			fclose(newfp);
@@ -643,14 +772,17 @@ reset_newsrc(
 	int sub, i;
 
 	if (!no_write && (newfp = fopen(newnewsrc, "w")) != NULL) {
-		if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+		if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 			fchmod(fileno(newfp), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 			chmod(newnewsrc, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+		}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 		if ((fp = fopen(newsrc, "r")) != NULL) {
 			while ((line = tin_fgets(fp, FALSE)) != NULL) {
@@ -660,7 +792,7 @@ reset_newsrc(
 			fclose(fp);
 		}
 		if ((sub = ferror(newfp)) || fclose(newfp)) {
-			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), newsrc);
 			if (sub) {
 				clearerr(newfp);
 				fclose(newfp);
@@ -692,14 +824,17 @@ delete_group(
 		return;
 
 	if ((newfp = fopen(newnewsrc, "w")) != NULL) {
-		if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+		if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 			fchmod(fileno(newfp), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 			chmod(newnewsrc, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+	}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 		if ((fp = fopen(newsrc, "r")) != NULL) {
 			while ((line = tin_fgets(fp, FALSE)) != NULL) {
@@ -712,7 +847,7 @@ delete_group(
 		}
 
 		if ((sub = ferror(newfp)) || fclose(newfp)) {
-			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), newsrc);
 			if (sub) {
 				clearerr(newfp);
 				fclose(newfp);
@@ -1191,7 +1326,6 @@ print_bitmap_seq(
 
 					if (group->newsrc.xmin > 2)
 						fprintf(fp, "-%"T_ARTNUM_PFMT, group->newsrc.xmin - 1);
-
 				}
 			}
 			if (group->newsrc.xmax == i)
@@ -1231,11 +1365,11 @@ pos_group_in_newsrc(
 	char *newsgroup = NULL;
 	char *line;
 	char filename[PATH_LEN];
-	char sub[PATH_LEN];
-	char unsub[PATH_LEN];
+	char *sub = NULL;
+	char *unsub = NULL;
 	int subscribed_pos = 1;
-	int err;
-	size_t group_len;
+	int err, n;
+	size_t len;
 	t_bool found = FALSE;
 	t_bool newnewsrc_created = FALSE;
 	t_bool option_line = FALSE;
@@ -1245,10 +1379,7 @@ pos_group_in_newsrc(
 	t_bool unsub_created = FALSE;
 	t_bool fs_error = FALSE;
 
-	if (no_write)
-		goto rewrite_group_done;
-
-	if ((fp_in = fopen(newsrc, "r")) == NULL)
+	if (no_write || (fp_in = fopen(newsrc, "r")) == NULL)
 		goto rewrite_group_done;
 
 	if ((fp_out = fopen(newnewsrc, "w")) == NULL)
@@ -1256,20 +1387,35 @@ pos_group_in_newsrc(
 
 	newnewsrc_created = TRUE;
 
-	if (newsrc_mode)
-#ifdef HAVE_FCHMOD
+#if defined(HAVE_FCHMOD) || defined(HAVE_CHMOD)
+	if (newsrc_mode) {
+#	ifdef HAVE_FCHMOD
 		fchmod(fileno(fp_out), newsrc_mode);
-#else
-#	ifdef HAVE_CHMOD
+#	else
+#		ifdef HAVE_CHMOD
 		chmod(newnewsrc, newsrc_mode);
-#	endif /* HAVE_CHMOD */
-#endif /* HAVE_FCHMOD */
+#		endif /* HAVE_CHMOD */
+#	endif /* HAVE_FCHMOD */
+	}
+#endif /* HAVE_FCHMOD || HAVE_CHMOD */
 
 	joinpath(filename, sizeof(filename), tmpdir, ".subrc");
-	snprintf(sub, sizeof(sub), "%s.%ld", filename, (long) process_id);
+	if ((n = snprintf(NULL, 0, "%s.%ld", filename, (long) process_id)) < 0)
+		goto rewrite_group_done;
+
+	len = (size_t) n + 1;
+	sub = my_malloc(len);
+	if (snprintf(sub, len, "%s.%ld", filename, (long) process_id) != n)
+		goto rewrite_group_done;
 
 	joinpath(filename, sizeof(filename), tmpdir, ".unsubrc");
-	snprintf(unsub, sizeof(unsub), "%s.%ld", filename, (long) process_id);
+	if ((n = snprintf(NULL, 0, "%s.%ld", filename, (long) process_id)) < 0)
+		goto rewrite_group_done;
+
+	len = (size_t) n + 1;
+	unsub = my_malloc(len);
+	if (snprintf(unsub, len, "%s.%ld", filename, (long) process_id) != n)
+		goto rewrite_group_done;
 
 	if ((fp_sub = fopen(sub, "w")) == NULL)
 		goto rewrite_group_done;
@@ -1284,10 +1430,10 @@ pos_group_in_newsrc(
 	/*
 	 * split newsrc into subscribed and unsubscribed to files
 	 */
-	group_len = strlen(group->name);
+	len = strlen(group->name);
 
 	while ((line = tin_fgets(fp_in, FALSE)) != NULL) {
-		if (STRNCMPEQ(group->name, line, group_len) && line[group_len] == SUBSCRIBED) {
+		if (STRNCMPEQ(group->name, line, len) && line[len] == SUBSCRIBED) {
 			FreeIfNeeded(newsgroup);
 			newsgroup = my_strdup(line);		/* Take a copy of this line */
 			found = TRUE;
@@ -1303,7 +1449,7 @@ pos_group_in_newsrc(
 	}
 
 	if ((err = ferror(fp_sub)) || fclose(fp_sub)) {
-		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), newsrc);
 		if (err) {
 			clearerr(fp_sub);
 			fclose(fp_sub);
@@ -1312,7 +1458,7 @@ pos_group_in_newsrc(
 	}
 	if ((err = ferror(fp_unsub)) || fclose(fp_unsub)) {
 		if (!fs_error) /* avoid repeatd error message */
-			error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+			error_message(2, _(txt_filesystem_full), newsrc);
 		if (err) {
 			clearerr(fp_unsub);
 			fclose(fp_unsub);
@@ -1374,7 +1520,7 @@ pos_group_in_newsrc(
 	 * Try and cleanly close out the newnewsrc file
 	 */
 	if ((err = ferror(fp_out)) || fclose(fp_out)) {
-		error_message(2, _(txt_filesystem_full), NEWSRC_FILE);
+		error_message(2, _(txt_filesystem_full), newsrc);
 		if (err) {
 			clearerr(fp_out);
 			fclose(fp_out);
@@ -1409,6 +1555,8 @@ rewrite_group_done:
 		unlink(unsub);
 
 	FreeIfNeeded(newsgroup);
+	FreeIfNeeded(sub);
+	FreeIfNeeded(unsub);
 
 	return ret_code;
 }
@@ -1571,8 +1719,7 @@ expand_bitmap(
 			debug_print_comment("expand_bitmap: group->newsrc.bitlen != (group->max-group->min)+1 and need full copy");
 #endif /* DEBUG */
 	} else if (max != group->newsrc.xmax || first != group->newsrc.xmin) {
-		t_bitmap *newbitmap;
-		newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
+		t_bitmap *newbitmap = my_malloc(BITS_TO_BYTES(bitlen));
 
 		/* Copy over old bitmap */
 
@@ -1643,7 +1790,7 @@ art_mark(
 					if (group->newsrc.num_unread)
 						group->newsrc.num_unread--;
 
-					if (group->attribute->show_only_unread_arts)
+					if (group->attribute && group->attribute->show_only_unread_arts)
 						art->keep_in_base = TRUE;
 				}
 
@@ -1657,7 +1804,7 @@ art_mark(
 				if (group != NULL) {
 					group->newsrc.num_unread++;
 
-					if (group->attribute->show_only_unread_arts)
+					if (group->attribute && group->attribute->show_only_unread_arts)
 						art->keep_in_base = FALSE;
 				}
 
