@@ -3,9 +3,9 @@
  *  Module    : nntps.c
  *  Author    : E. Berkhan
  *  Created   : 2022-09-10
- *  Updated   : 2023-09-26
+ *  Updated   : 2024-05-10
  *  Notes     : simple abstraction for various TLS implementations
- *  Copyright : (c) Copyright 2022-2023 Enrik Berkhan <Enrik.Berkhan@inka.de>
+ *  Copyright : (c) Copyright 2022-2024 Enrik Berkhan <Enrik.Berkhan@inka.de>
  *              Permission is hereby granted to copy, reproduce, redistribute
  *              or otherwise use this software  as long as: there is no
  *              monetary  profit  gained  specifically  from the use or
@@ -54,6 +54,9 @@ static void show_errors(const char *msg_fmt);
 #	endif /* USE_OPENSSL */
 #endif /* USE_GNUTLS */
 
+#if (defined(USE_LIBTLS) && defined(HAVE_LIB_CRYPTO)) || defined(USE_OPENSSL)
+static char **get_cert_info(const X509 *cert);
+#endif /* (USE_LIBTLS && HAVE_LIB_CRYPTO) || USE_OPENSSL */
 
 static char ca_cert_file_expanded[PATH_LEN];
 
@@ -66,7 +69,7 @@ tintls_init(
 	void)
 {
 	int result = 0;
-	const char *ca_cert_file = ca_cert_file_expanded;
+	char *ca_cert_file = ca_cert_file_expanded;
 
 	if (tinrc.tls_ca_cert_file[0] != '\0') {
 		result = strfpath(tinrc.tls_ca_cert_file, ca_cert_file_expanded, sizeof(ca_cert_file_expanded), NULL, FALSE);
@@ -97,9 +100,9 @@ tintls_init(
 	if (ca_cert_file[0] != '\0') {
 		result = tls_config_set_ca_file(libtls_config, ca_cert_file);
 		if (result != 0) {
+			error_message(2, "%s!\n", tls_config_error(libtls_config));
 			tls_config_free(libtls_config);
 			libtls_config = NULL;
-			error_message(2, "tls_config_set_ca_file: %d!\n", result);
 			return -EINVAL;
 		}
 	}
@@ -136,17 +139,17 @@ tintls_init(
 	if (ca_cert_file[0] == '\0') {
 		result = gnutls_certificate_set_x509_system_trust(tls_xcreds);
 		if (result < 0) {
+			error_message(2, "gnutls_certificate_set_x509_system_trust: %s (%d)!\n", gnutls_strerror(result), result);
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
-			error_message(2, "gnutls_certificate_set_x509_system_trust: %d!\n", result);
 			return -EINVAL;
 		}
 	} else {
 		result = gnutls_certificate_set_x509_trust_file(tls_xcreds, ca_cert_file, GNUTLS_X509_FMT_PEM);
 		if (result < 0) {
+			error_message(2, "gnutls_certificate_set_x509_trust_file: %s (%d): \"%s\"!\n", gnutls_strerror(result), result, ca_cert_file);
 			gnutls_certificate_free_credentials(tls_xcreds);
 			tls_xcreds = NULL;
-			error_message(2, "gnutls_certificate_set_x509_trust_file: %d!\n", result);
 			return -EINVAL;
 		}
 	}
@@ -157,13 +160,13 @@ tintls_init(
 
 	result = RAND_status();
 	if (result != 1) {
-		show_errors(_("RAND_status: %s!\n"));
+		show_errors("RAND_status: %s!\n");
 		return -EINVAL;
 	}
 
 	openssl_ctx = SSL_CTX_new(TLS_method());
 	if (!openssl_ctx) {
-		show_errors(_("SSL_CTX_new: %s!\n"));
+		show_errors("SSL_CTX_new: %s!\n");
 		return -ENOMEM;
 	}
 
@@ -180,7 +183,7 @@ tintls_init(
 		if (result != 1) {
 			SSL_CTX_free(openssl_ctx);
 			openssl_ctx = NULL;
-			show_errors(_("SSL_CTX_set_default_verify_paths: %s!\n"));
+			show_errors("SSL_CTX_set_default_verify_paths: %s!\n");
 			return -EINVAL;
 		}
 	} else {
@@ -188,7 +191,7 @@ tintls_init(
 		if (result != 1) {
 			SSL_CTX_free(openssl_ctx);
 			openssl_ctx = NULL;
-			show_errors(_("SSL_CTX_load_verify_locations: %s!\n"));
+			show_errors("SSL_CTX_load_verify_locations: %s!\n");
 			return -EINVAL;
 		}
 	}
@@ -245,8 +248,8 @@ tintls_open(
 	int fd,
 	void **session_ctx)
 {
-#ifdef USE_LIBTLS
 	int result;
+#ifdef USE_LIBTLS
 	struct tls *client;
 
 	if (!session_ctx)
@@ -267,6 +270,7 @@ tintls_open(
 	result = tls_connect_socket(client, fd, servername);
 	if (result == -1) {
 		tls_free(client);
+		tintls_exit();
 		return -ENOMEM;
 	}
 
@@ -274,7 +278,6 @@ tintls_open(
 
 #else
 #	ifdef USE_GNUTLS
-	int result;
 	gnutls_session_t client;
 	size_t servername_len;
 
@@ -317,7 +320,6 @@ tintls_open(
 
 #	else
 #		ifdef USE_OPENSSL
-	int result;
 	int long_result;
 	SSL *ssl;
 	BIO *sock;
@@ -332,14 +334,14 @@ tintls_open(
 
 	sock = BIO_new_socket(fd, 1);
 	if (!sock) {
-		show_errors(_("BIO_new_socket: %s!\n"));
+		show_errors("BIO_new_socket: %s!\n");
 		return -ENOMEM;
 	}
 
 	client = BIO_new_ssl(openssl_ctx, 1);
 	if (!client) {
 		BIO_free(sock);
-		show_errors(_("BIO_new_ssl: %s!\n"));
+		show_errors("BIO_new_ssl: %s!\n");
 		return -ENOMEM;
 	}
 
@@ -347,7 +349,7 @@ tintls_open(
 	if (long_result != 1) {
 		BIO_free(client);
 		BIO_free(sock);
-		show_errors(_("BIO_get_ssl: %s!\n"));
+		show_errors("BIO_get_ssl: %s!\n");
 		return -EINVAL;
 	}
 
@@ -355,7 +357,7 @@ tintls_open(
 	if (result != 1) {
 		BIO_free(client);
 		BIO_free(sock);
-		show_errors(_("SSL_set_tlsext_host_name: %s!\n"));
+		show_errors("SSL_set_tlsext_host_name: %s!\n");
 		return -EINVAL;
 	}
 
@@ -363,7 +365,7 @@ tintls_open(
 	if (result != 1) {
 		BIO_free(client);
 		BIO_free(sock);
-		show_errors(_("SSL_set1_host: %s!\n"));
+		show_errors("SSL_set1_host: %s!\n");
 		return -EINVAL;
 	}
 
@@ -387,6 +389,11 @@ tintls_handshake(
 	int result;
 	struct tls *client = session_ctx;
 	const char *subject, *issuer, *version, *cipher;
+#	ifdef HAVE_LIB_CRYPTO
+	BIO *io_buf = NULL;
+	const uint8_t *chain;
+	size_t chain_size;
+#	endif /* HAVE_LIB_CRYPTO */
 
 	do {
 		result = tls_handshake(client);
@@ -395,30 +402,57 @@ tintls_handshake(
 	if (result < 0) {
 		const char *err = tls_error(client);
 
-		error_message(2, "TLS handshake failed: %s!\n", err ? err : "unknown error");
+		error_message(2, _(txt_tls_handshake_failed), err ? err : _(txt_unknown_error));
 		return -EPROTO;
 	}
 
-	subject = tls_peer_cert_subject(client);
-	issuer = tls_peer_cert_issuer(client);
-	version = tls_conn_version(client);
-	cipher = tls_conn_cipher(client);
+#	ifdef HAVE_LIB_CRYPTO
+	if ((chain = tls_peer_cert_chain_pem(client, &chain_size)))
+		io_buf = BIO_new(BIO_s_mem());
 
-	if (!subject)
-		subject = "<failed to retrieve subject>";
-	if (!issuer)
-		issuer = "<failed to retrieve issuer>";
-	if (!version)
-		version = "<failed to retrieve version>";
-	if (!cipher)
-		cipher = "<failed to retrieve cipher>";
+	if (chain && io_buf) {
+		X509 *cert;
+		char **cert_info;
 
-	if (!batch_mode || verbose) {
-		wait_message(0, "subject: %s\n", subject);
-		wait_message(0, " issuer: %s\n", issuer);
-		wait_message(0, "%s handshake done: %s\n", version, cipher);
+		if (chain_size > 0 && BIO_write(io_buf, chain, chain_size) > 0) {
+			cert = PEM_read_bio_X509(io_buf, NULL, 0, NULL);
+			if (cert && ((cert_info = get_cert_info(cert)))) {
+				wait_message(0, _(txt_conninfo_subject), cert_info[0] ? cert_info[0] : _(txt_retr_subject_failed));
+				wait_message(0, _(txt_conninfo_issuer), cert_info[1] ? cert_info[1] : _(txt_retr_issuer_failed));
+				FreeIfNeeded(cert_info[0]);
+				FreeIfNeeded(cert_info[1]);
+				free(cert_info);
+				X509_free(cert);
+			}
+		}
+		BIO_free(io_buf);
+	} else
+#	endif /* HAVE_LIB_CRYPTO */
+	{ /* Fallback if access to the certificate chain has failed */
+		if (!batch_mode || verbose) {
+			subject = tls_peer_cert_subject(client);
+			issuer = tls_peer_cert_issuer(client);
+
+			if (!subject)
+				subject = _(txt_retr_subject_failed);
+			if (!issuer)
+				issuer = _(txt_retr_issuer_failed);
+			wait_message(0, _(txt_conninfo_subject), subject);
+			wait_message(0, _(txt_conninfo_issuer), issuer);
+		}
 	}
 
+	if (!batch_mode || verbose) {
+		version = tls_conn_version(client);
+		cipher = tls_conn_cipher(client);
+
+		if (!version)
+			version = _(txt_retr_version_failed);
+		if (!cipher)
+			cipher = _(txt_retr_cipher_failed);
+
+		wait_message(0, _(txt_tls_handshake_done), version, cipher);
+	}
 #else
 #	ifdef USE_GNUTLS
 	int result;
@@ -438,14 +472,14 @@ tintls_handshake(
 			status_result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
 
 			if (status_result == 0)
-				wait_message(0, _("TLS peer verification failed: %s\n"), msg.data);
+				wait_message(0, _(txt_tls_peer_verify_failed), msg.data);
 			else
-				wait_message(0, _("TLS peer verification failed: %s\n"), "<unable to retrieve status>");
+				wait_message(0, _(txt_tls_peer_verify_failed), _(txt_tls_unable_to_get_status));
 
 			gnutls_free(msg.data);
 		}
 
-		error_message(2, "TLS handshake failed: %s (%d)\n", gnutls_strerror(result), result);
+		error_message(2, _(txt_tls_handshake_failed_with_err_num), gnutls_strerror(result), result);
 
 		return -EPROTO;
 	} else {
@@ -458,18 +492,17 @@ tintls_handshake(
 			gnutls_datum_t msg;
 
 			if (!insecure_nntps) {
-				error_message(2, "unexpected certificate verification status!");
+				error_message(2, _(txt_tls_unexpected_status));
 				return -EPROTO;
 			}
 
 			type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
 			result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
 
-			if (result == 0) {
-				wait_message(0, _("TLS peer verification failed, continuing anyway as requested: %s\n"), msg.data);
-			} else {
-				wait_message(0, _("TLS peer verification failed, continuing anyway as requested: %s\n"), "<unable to retrieve status>");
-			}
+			if (result == 0)
+				wait_message(0, _(txt_tls_peer_verify_failed_continuing), msg.data);
+			else
+				wait_message(0, _(txt_tls_peer_verify_failed_continuing), _(txt_tls_unable_to_get_status));
 
 			gnutls_free(msg.data);
 
@@ -508,8 +541,22 @@ tintls_handshake(
 			}
 
 			if (!batch_mode || verbose) {
-				wait_message(0, "subject: %s\n", subject.data);
-				wait_message(0, " issuer: %s\n", issuer.data);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				char *sub = my_strdup((char *) subject.data);
+				char *iss = my_strdup((char *) issuer.data);
+				size_t len_s = strlen(sub);
+				size_t len_i = strlen(iss);
+
+				process_charsets(&sub, &len_s, "UTF-8", tinrc.mm_local_charset, FALSE);
+				process_charsets(&iss, &len_i, "UTF-8", tinrc.mm_local_charset, FALSE);
+				wait_message(0, _(txt_conninfo_subject), sub);
+				wait_message(0, _(txt_conninfo_issuer), iss);
+				free(sub);
+				free(iss);
+#else
+				wait_message(0, _(txt_conninfo_subject), subject.data);
+				wait_message(0, _(txt_conninfo_issuer), issuer.data);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			}
 
 err_cert:
@@ -521,54 +568,59 @@ err_cert:
 				gnutls_x509_crt_deinit(servercert);
 		}
 
-		desc = gnutls_session_get_desc(client);
-		if (!batch_mode || verbose)
-			wait_message(0, "TLS handshake done: %s\n", desc);
-		gnutls_free(desc);
+		if (!batch_mode || verbose) {
+			if ((desc = gnutls_session_get_desc(client)) != NULL) {
+				wait_message(0, _(txt_tls_handshake_done), desc);
+				gnutls_free(desc);
+			}
+		}
 	}
-
 #	else
 #		ifdef USE_OPENSSL
 	long long_result;
 	BIO *client = session_ctx;
 	SSL *ssl;
 	X509 *peer;
-	char name[128];
 
 	ERR_clear_error();
 
 	long_result = BIO_get_ssl(client, &ssl);
 	if (long_result != 1) {
-		show_errors(_("BIO_get_ssl: %s!\n"));
+		show_errors("BIO_get_ssl: %s!\n");
 		return -EINVAL;
 	}
 
 	long_result = BIO_do_handshake(client);
 	if (long_result != 1) {
 		long_result = SSL_get_verify_result(ssl);
-		if (long_result != X509_V_OK) {
-			error_message(0, _("TLS handshake failed: %s\n"), X509_verify_cert_error_string(long_result));
-		} else
-			show_errors(_("TLS handshake failed: %s\n"));
+		if (long_result != X509_V_OK)
+			error_message(2, _(txt_tls_handshake_failed), X509_verify_cert_error_string(long_result));
+		else
+			show_errors(_(txt_tls_handshake_failed));
 
 		return -EPROTO;
 	} else if (insecure_nntps) {
 		long_result = SSL_get_verify_result(ssl);
 		if (long_result != X509_V_OK && (!batch_mode || verbose))
-			wait_message(0, _("TLS peer verification failed: %s.\nContinuing anyway as requested.\n"), X509_verify_cert_error_string(long_result));
+			wait_message(0, _(txt_tls_peer_verify_failed_continuing), X509_verify_cert_error_string(long_result));
 	}
 
-	peer = SSL_get_peer_certificate(ssl);
-	if (peer) {
-		if (!batch_mode || verbose) {
-			wait_message(0, "subject: %s\n", X509_NAME_oneline(X509_get_subject_name(peer), name, sizeof(name)));
-			wait_message(0, " issuer: %s\n", X509_NAME_oneline(X509_get_issuer_name(peer), name, sizeof(name)));
+	if (!batch_mode || verbose) {
+		if ((peer = SSL_get_peer_certificate(ssl))) {
+			char **cert_info;
+
+			if ((cert_info = get_cert_info(peer))) {
+				wait_message(0, _(txt_conninfo_subject), BlankIfNull(cert_info[0]));
+				wait_message(0, _(txt_conninfo_issuer), BlankIfNull(cert_info[1]));
+				FreeIfNeeded(cert_info[0]);
+				FreeIfNeeded(cert_info[1]);
+				free(cert_info);
+			}
+			X509_free(peer);
 		}
-		X509_free(peer);
-	}
 
-	if (!batch_mode || verbose)
-		wait_message(0, "TLS handshake done: %s\n", SSL_get_cipher_name(ssl));
+		wait_message(0, _(txt_tls_handshake_done), SSL_get_cipher_name(ssl));
+	}
 #		endif /* USE_OPENSSL */
 #	endif /* USE_GNUTLS */
 #endif /* USE_LIBTLS */
@@ -609,9 +661,9 @@ tintls_read(
 	/* NOTREACHED */
 #	else
 #		ifdef USE_OPENSSL
+	int result;
 	size_t bytes_read;
 	BIO *client = session_ctx;
-	int result;
 
 	ERR_clear_error();
 
@@ -726,96 +778,170 @@ tintls_close(
 	return 0;
 }
 
+/* TODO: make date-format configurable? */
+#define PRINT_VALID_AFTER(ts, what) do { \
+		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) \
+			fprintf(fp, _(txt_valid_not_after), (what)); \
+		else \
+			fprintf(fp, "%s", txt_conninfo_fmt_error); \
+	} while (0)
+
+#define PRINT_VALID_BEFORE(ts, what) do { \
+		if (my_strftime(what, sizeof(what), "%Y-%m-%dT%H:%M%z", ts)) \
+			fprintf(fp, _(txt_valid_not_before), (what)); \
+		else \
+			fprintf(fp, "%s", txt_conninfo_fmt_error); \
+	} while (0)
+
 
-/*
- * TODO: -> lang.c
- */
 int
 tintls_conninfo(
 	void *session_ctx,
 	FILE *fp)
 {
-	int result;
 #ifdef USE_LIBTLS
 	struct tls *client = session_ctx;
-	time_t t;
-	struct tm *tm;
-	char fmt_time[64]; /* time zone name could long... */
+	char fmt_time[22]; /* %Y-%m-%dT%H:%M%z */
+#	ifdef HAVE_LIB_CRYPTO
+	BIO *io_buf = NULL;
+	const uint8_t *chain;
+	size_t chain_size;
 
-	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp, "----------------\n");
-	fprintf(fp, "%s %s (strength %d)\n", tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
-	fprintf(fp, "\nServer certificate information:\n");
-	fprintf(fp, "-------------------------------\n");
-	fprintf(fp, "Subject: %s\n", tls_peer_cert_subject(client));
-	fprintf(fp, "Issuer : %s\n", tls_peer_cert_issuer(client));
+	fprintf(fp, "%s", _(txt_conninfo_tls_info));
+	fprintf(fp, _(txt_conninfo_libtls_info), tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
 
-	t = tls_peer_cert_notbefore(client);
-	tm = localtime(&t);
-	result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm); /* make format configurable? */
-	if (result < 0)
-		my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-	fprintf(fp, txt_valid_not_before, fmt_time);
+	if ((chain = tls_peer_cert_chain_pem(client, &chain_size)))
+		io_buf = BIO_new(BIO_s_mem());
 
-	t = tls_peer_cert_notafter(client);
-	tm = localtime(&t);
-	result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm);
-	if (result < 0)
-		my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-	fprintf(fp, txt_valid_not_after, fmt_time);
+	if (chain && io_buf) {
+		X509 *cert;
+		char **cert_info;
+		const ASN1_TIME *asn1;
+		char *wchain, *cptr;
+		int i;
+		size_t cl;
+		struct tm tm;
 
+		fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
+
+		/* string copy of chain */
+		i = snprintf(NULL, 0, "%.*s", (int) chain_size, chain);
+		cl = (size_t) i + 1;
+		wchain = my_malloc(cl);
+		snprintf(wchain, cl, "%.*s", (int) chain_size, chain);
+		cptr = wchain;
+		i = 0;
+
+		while ((cptr = strstr(cptr, "-----BEGIN CERTIFICATE-----"))) {
+			chain_size = strlen(cptr);
+			if (chain_size > 0 && BIO_write(io_buf, cptr, chain_size) > 0) {
+				cert = PEM_read_bio_X509(io_buf, NULL, 0, NULL);
+				if (cert && ((cert_info = get_cert_info(cert)))) {
+					if (i)
+						fputs("\n", fp);
+					fprintf(fp, _(txt_conninfo_cert), i++);
+
+					fprintf(fp, _(txt_conninfo_subject), BlankIfNull(cert_info[0]));
+					fprintf(fp, _(txt_conninfo_issuer), BlankIfNull(cert_info[1]));
+					FreeIfNeeded(cert_info[0]);
+					FreeIfNeeded(cert_info[1]);
+					free(cert_info);
+
+					if ((asn1 = X509_get0_notBefore(cert)) != NULL) {
+						if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
+							PRINT_VALID_BEFORE(&tm, fmt_time);
+						}
+					}
+					if ((asn1 = X509_get0_notAfter(cert)) != NULL) {
+						if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
+							PRINT_VALID_AFTER(&tm, fmt_time);
+						}
+					}
+					X509_free(cert);
+				}
+			}
+			BIO_reset(io_buf);
+			cptr += 26; /* "-----BEGIN CERTIFICATE-----" */
+		}
+		free(wchain);
+		BIO_free(io_buf);
+	} else /* Fallback if access to the certificate chain has failed */
+#	else
+	fprintf(fp, "%s", _(txt_conninfo_tls_info));
+	fprintf(fp, _(txt_conninfo_libtls_info), tls_conn_version(client), tls_conn_cipher(client), tls_conn_cipher_strength(client));
+	fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
+#	endif /* HAVE_LIB_CRYPTO */
+	{
+		struct tm *tm;
+		time_t t;
+
+		fprintf(fp, _(txt_conninfo_subject), tls_peer_cert_subject(client));
+		fprintf(fp, _(txt_conninfo_issuer), tls_peer_cert_issuer(client));
+
+		if ((t = tls_peer_cert_notbefore(client)) != -1) {
+			tm = gmtime(&t);
+			PRINT_VALID_BEFORE(tm, fmt_time);
+		}
+
+		if ((t = tls_peer_cert_notafter(client)) != -1) {
+			tm = gmtime(&t);
+			PRINT_VALID_AFTER(tm, fmt_time);
+		}
+	}
 #else
-
 #	ifdef USE_GNUTLS
-	int retval = -1;
-	gnutls_session_t client = session_ctx;
 	char *desc;
+	char fmt_time[22]; /* %Y-%m-%dT%H:%M%z */
+	int retval = -1;
+	int result;
+	gnutls_session_t client = session_ctx;
 	gnutls_datum_t msg;
 	const gnutls_datum_t *raw_servercert_chain;
 	unsigned int servercert_chainlen;
 	unsigned int i;
 	time_t t;
 	struct tm *tm;
-	char fmt_time[64]; /* time zone name could long... */
 
 	desc = gnutls_session_get_desc(client);
-	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp, "----------------\n");
+	fprintf(fp, "%s", _(txt_conninfo_tls_info));
 	fprintf(fp, "%s\n", desc);
 	gnutls_free(desc);
 
 	msg.data = NULL;
 
 	if (gnutls_verification_status != 0) {
-		int type;
+		int type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
 
-		type = gnutls_certificate_type_get2(client, GNUTLS_CTYPE_SERVER);
 		result = gnutls_certificate_verification_status_print(gnutls_verification_status, type, &msg, 0);
 
-		if (result == 0) {
-			fprintf(fp, "Server certificate verification FAILED:\n\t%s (%s)\n", msg.data,
-					insecure_nntps ? "tolerated as \"-k\" (insecure) requested" : "UNEXPECTED, possible BUG");
-		} else
-			fprintf(fp, "Server certificate verification FAILED: <can't get reason>\n");
+		if (result == 0)
+			fprintf(fp, _(txt_conninfo_verify_failed), msg.data,
+					insecure_nntps ? _(txt_conninfo_error_tolerated) : _(txt_conninfo_error_unexpected));
+		else
+			fprintf(fp, "%s", _(txt_conninfo_verify_failed_no_reason));
 
 		gnutls_free(msg.data);
 	} else
-		fprintf(fp, "Server certificate verified successfully.\n");
+		fprintf(fp, "%s", _(txt_conninfo_verify_successful));
 
 	raw_servercert_chain = gnutls_certificate_get_peers(client, &servercert_chainlen);
-	if (servercert_chainlen > 0) {
-		fprintf(fp, "\nServer certificate information:\n");
-		fprintf(fp, "-------------------------------\n");
-	}
+	if (servercert_chainlen > 0)
+		fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
 
 	for (i = 0; i < servercert_chainlen; i++) {
 		gnutls_x509_crt_t servercert = NULL;
 		gnutls_datum_t subject = { NULL, 0 };
 		gnutls_datum_t issuer = { NULL, 0 };
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		char *sub;
+		char *iss;
+		size_t len_s;
+		size_t len_i;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-		if (i > 0)
+		if (i)
 			fputs("\n", fp);
-		fprintf(fp, "Certificate #%d\n", i);
+		fprintf(fp, _(txt_conninfo_cert), i);
 
 		result = gnutls_x509_crt_init(&servercert);
 		if (result < 0)
@@ -828,34 +954,44 @@ tintls_conninfo(
 		result = gnutls_x509_crt_get_dn3(servercert, &subject, 0);
 		if (result < 0)
 			goto err_cert;
-
-		fprintf(fp, "Subject: %s\n", subject.data);
+		else {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			sub = (char *) subject.data;
+			len_s = strlen(sub);
+			process_charsets(&sub, &len_s, "UTF-8", tinrc.mm_local_charset, FALSE);
+			fprintf(fp, _(txt_conninfo_subject), sub);
+#else
+			fprintf(fp, _(txt_conninfo_subject), subject.data);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		}
 
 		result = gnutls_x509_crt_get_issuer_dn3(servercert, &issuer, 0);
 		if (result < 0)
 			goto err_cert;
-
-		fprintf(fp, "Issuer : %s\n", issuer.data);
+		else {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			iss = (char *) issuer.data;
+			len_i = strlen(iss);
+			process_charsets(&iss, &len_i, "UTF-8", tinrc.mm_local_charset, FALSE);
+			fprintf(fp, _(txt_conninfo_issuer), iss);
+#else
+			fprintf(fp, _(txt_conninfo_issuer), issuer.data);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		}
 
 		t = gnutls_x509_crt_get_activation_time(servercert);
 		if (t == -1)
 			goto err_cert;
 
 		tm = localtime(&t);
-		result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm); /* make format configurable? */
-		if (result < 0)
-			my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-		fprintf(fp, txt_valid_not_before, fmt_time);
+		PRINT_VALID_BEFORE(tm, fmt_time);
 
 		t = gnutls_x509_crt_get_expiration_time(servercert);
 		if (t == -1)
 			goto err_cert;
 
 		tm = localtime(&t);
-		result = my_strftime(fmt_time, sizeof(fmt_time), "%Y-%m-%dT%H:%M%z (%Z)", tm);
-		if (result < 0)
-			my_strncpy(fmt_time, "<formatting error>", sizeof(fmt_time) - 1);
-		fprintf(fp, txt_valid_not_after, fmt_time);
+		PRINT_VALID_AFTER(tm, fmt_time);
 
 		retval = 0;
 
@@ -882,20 +1018,18 @@ err_cert:
 	if (long_result != 1)
 		return -1;
 
-	fprintf(fp, "\nTLS information:\n");
-	fprintf(fp, "----------------\n");
+	fprintf(fp, "%s", _(txt_conninfo_tls_info));
 	fprintf(fp, "%s %s\n", SSL_get_version(ssl), SSL_get_cipher_name(ssl));
 
 	verification_result = SSL_get_verify_result(ssl);
 	if (verification_result != X509_V_OK)
-		fprintf(fp, "Server certificate verification FAILED:\n\t%s (%s)\n",
+		fprintf(fp, _(txt_conninfo_verify_failed),
 			X509_verify_cert_error_string(verification_result),
-			insecure_nntps ? "tolerated as \"-k\" (insecure) requested" : "UNEXPECTED, possible BUG");
+			insecure_nntps ? _(txt_conninfo_error_tolerated) : _(txt_conninfo_error_unexpected));
 	else
-		fprintf(fp, "Server certificate verified successfully.\n");
+		fprintf(fp, "%s", _(txt_conninfo_verify_successful));
 
-	fprintf(fp, "\nServer certificate information:\n");
-	fprintf(fp, "-------------------------------\n");
+	fprintf(fp, "%s", _(txt_conninfo_server_cert_info));
 
 	if (verification_result == X509_V_OK)
 		chain = SSL_get_peer_cert_chain(ssl);
@@ -903,7 +1037,8 @@ err_cert:
 		chain = SSL_get0_verified_chain(ssl);
 
 	if (chain) {
-		char name[128];
+		char name[22]; /* %Y-%m-%dT%H:%M%z */
+		char **cert_info;
 		const ASN1_TIME *asn1;
 		int i;
 		struct tm tm;
@@ -911,28 +1046,28 @@ err_cert:
 		for (i = 0; i < sk_X509_num(chain); i++) {
 			X509* cert = sk_X509_value(chain, i);
 
-			if (i > 0)
+			if (i)
 				fputs("\n", fp);
-			fprintf(fp, "Certificate #%d\n", i);
-			fprintf(fp, "Subject: %s\n", X509_NAME_oneline(X509_get_subject_name(cert), name, sizeof(name)));
-			fprintf(fp, "Issuer : %s\n", X509_NAME_oneline(X509_get_issuer_name(cert), name, sizeof(name)));
+			fprintf(fp, _(txt_conninfo_cert), i);
 
-			asn1 = X509_get0_notBefore(cert);
-			result = ASN1_TIME_to_tm(asn1, &tm);
-			if (result == 1) {
-				result = my_strftime(name, sizeof(name), "%Y-%m-%dT%H:%M%z", &tm); /* make format configurable? */
-				if (result < 0)
-					my_strncpy(name, "<formatting error>", sizeof(name) - 1);
-				fprintf(fp, txt_valid_not_before, name);
+			if ((cert_info = get_cert_info(cert))) {
+				fprintf(fp, _(txt_conninfo_subject), BlankIfNull(cert_info[0]));
+				fprintf(fp, _(txt_conninfo_issuer), BlankIfNull(cert_info[1]));
+				FreeIfNeeded(cert_info[0]);
+				FreeIfNeeded(cert_info[1]);
+				free(cert_info);
 			}
 
-			asn1 = X509_get0_notAfter(cert);
-			result = ASN1_TIME_to_tm(asn1, &tm);
-			if (result == 1) {
-				result = my_strftime(name, sizeof(name), "%Y-%m-%dT%H:%M%z", &tm);
-				if (result < 0)
-					my_strncpy(name, "<formatting error>", sizeof(name) - 1);
-				fprintf(fp, txt_valid_not_after, name);
+			if ((asn1 = X509_get0_notBefore(cert)) != NULL) {
+				if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
+					PRINT_VALID_BEFORE(&tm, name);
+				}
+			}
+
+			if ((asn1 = X509_get0_notAfter(cert)) != NULL) {
+				if (ASN1_TIME_to_tm(asn1, &tm) == 1) {
+					PRINT_VALID_AFTER(&tm, name);
+				}
 			}
 		}
 	}
@@ -943,6 +1078,58 @@ err_cert:
 }
 
 
+#undef PRINT_VALID_AFTER
+#undef PRINT_VALID_BEFORE
+
+
+#if (defined(USE_LIBTLS) && defined(HAVE_LIB_CRYPTO)) || defined(USE_OPENSSL)
+static char **
+get_cert_info(
+	const X509 *cert)
+{
+	BIO *io_buf;
+	char **res = NULL;
+	char *tmp, *subject = NULL, *issuer = NULL;
+	long len;
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	unsigned long flags = XN_FLAG_ONELINE & ~ASN1_STRFLGS_ESC_MSB & ~XN_FLAG_SPC_EQ;
+#	else
+	unsigned long flags = XN_FLAG_ONELINE & ~XN_FLAG_SPC_EQ;
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
+	if (cert && (io_buf = BIO_new(BIO_s_mem()))) {
+		res = my_malloc(sizeof(char *) * 2);
+		if (X509_NAME_print_ex(io_buf, X509_get_subject_name(cert), 0, flags) != -1) {
+			len = BIO_get_mem_data(io_buf, &tmp);
+			if (len > 0) {
+				subject = my_malloc((size_t) len + 1);
+				memcpy(subject, tmp, (size_t) len);
+				subject[len] = '\0';
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				process_charsets(&subject, (size_t *) &len, "UTF-8", tinrc.mm_local_charset, FALSE);
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			}
+		}
+		if (BIO_reset(io_buf) != -1 && X509_NAME_print_ex(io_buf, X509_get_issuer_name(cert), 0, flags) != -1) {
+			len = BIO_get_mem_data(io_buf, &tmp);
+			if (len > 0) {
+				issuer = my_malloc((size_t) len + 1);
+				memcpy(issuer, tmp, (size_t) len);
+				issuer[len] = '\0';
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				process_charsets(&issuer, (size_t *) &len, "UTF-8", tinrc.mm_local_charset, FALSE);
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			}
+		}
+		res[0] = subject;
+		res[1] = issuer;
+		BIO_free(io_buf);
+	}
+	return res;
+}
+#endif /* (USE_LIBTLS && HAVE_LIB_CRYPTO) || USE_OPENSSL */
+
+
 #ifdef USE_OPENSSL
 static void
 show_errors(
