@@ -3,13 +3,13 @@
  *  Module    : nrctbl.c
  *  Author    : Sven Paulus <sven@tin.org>
  *  Created   : 1996-10-06
- *  Updated   : 2020-04-23
+ *  Updated   : 2024-03-30
  *  Notes     : This module does the NNTP server name lookup in
  *              ~/.tin/newsrctable and returns the real hostname
  *              and the name of the newsrc file for a given
  *              alias of the server.
  *
- * Copyright (c) 1996-2022 Sven Paulus <sven@tin.org>
+ * Copyright (c) 1996-2024 Sven Paulus <sven@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,8 +50,11 @@
 #ifndef VERSION_H
 #	include "version.h"
 #endif /* !VERSION_H */
+#ifndef TNNTP_H
+#	include "tnntp.h"
+#endif /* !TNNTP_H */
 
-
+#ifdef NNTP_ABLE
 /*
  * local prototypes
  */
@@ -145,43 +148,67 @@ get_newsrcname(
 	FILE *fp;
 	char *line_entry;
 	char line[LEN];
-	char name_found[PATH_LEN];
+	char name_found[PATH_LEN] = { '\0' };
 	int line_entry_counter;
 	int found = 0;
 	t_bool do_cpy = FALSE;
 
 	if ((fp = fopen(local_newsrctable_file, "r")) != NULL) {
-		while ((fgets(line, (int) sizeof(line), fp) != NULL) && (found != 1)) {
-			line_entry_counter = 0;
+		char *ns, *nsp, *p, *q;
+		size_t l = strlen(nntpserver_name) + 8; /* []:65535\0 */
 
-			if (!strchr("# ;", line[0])) {
-				while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
-					line_entry_counter++;
+		/* add [] and/or port if required */
+		ns = my_strdup(nntpserver_name);
+		nsp = my_malloc(l);
+		sprintf(nsp, "%s:%d", nntpserver_name, nntp_tcp_port);
 
-					if ((line_entry_counter == 1) && (!strcasecmp(line_entry, nntpserver_name))) {
-						found = 1;
-						do_cpy = TRUE;
-					}
-
-					if ((line_entry_counter == 1) && ((!strcasecmp(line_entry, "default")) || (!strcmp(line_entry, "*")))) {
-						found = 2;
-						do_cpy = TRUE;
-					}
-					if (do_cpy && (line_entry_counter == 2)) {
-						STRCPY(name_found, line_entry);
-						do_cpy = FALSE;
-					}
+		if ((p = strchr(nntpserver_name, ':')) != NULL) {
+			if ((q = strrchr(nntpserver_name, ':')) != NULL) {
+				if (p != q) {
+					free(ns);
+					free(nsp);
+					ns = my_malloc(l);
+					nsp = my_malloc(l);
+					sprintf(ns, "[%s]", nntpserver_name);
+					sprintf(nsp, "[%s]:%d", nntpserver_name, nntp_tcp_port);
 				}
 			}
 		}
+
+		while ((fgets(line, (int) sizeof(line), fp) != NULL) && (found != 1)) {
+			if (strchr("# ;", line[0])) /* comment? */
+				continue;
+
+			line_entry_counter = 0;
+			while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
+				line_entry_counter++;
+
+				if ((line_entry_counter == 1) && ((!strcasecmp(line_entry,nsp)) || (!strcasecmp(line_entry, ns)))) {
+					found = 1;
+					do_cpy = TRUE;
+				}
+
+				if (!found && (!strcasecmp(line_entry, "default") || !strcmp(line_entry, "*"))) {
+					found = 2;
+					do_cpy = TRUE;
+				}
+
+				if (do_cpy && (line_entry_counter == 2)) {
+					STRCPY(name_found, line_entry);
+					do_cpy = FALSE;
+				}
+			}
+		}
+		free(ns);
+		free(nsp);
 		fclose(fp);
 		if (found) {
-			char dir[PATH_LEN];
+			char dir[PATH_LEN] = { '\0' };
 			char tmp_newsrc[PATH_LEN];
 			int error = 0;
 
 			if (!strfpath(name_found, tmp_newsrc, sizeof(tmp_newsrc), NULL, FALSE)) {
-				my_fprintf(stderr, _("couldn't expand %s\n"), name_found); /* TODO: -> lang.c */
+				my_fprintf(stderr, _(txt_error_couldnt_expand), name_found);
 				error = 1;
 			} else {
 				if (tmp_newsrc[0] == '/') {
@@ -196,17 +223,14 @@ get_newsrcname(
 						*line_entry-- = '\0';
 				}
 
-				/*
-				 * TODO: shall we create a missing dir?
-				 *       currently something like
-				 *       ~/.tin/${NNTPSERVER-localhost}/.newsrc
-				 *       in newsrctable usually ends with
-				 *       "No permissions to go into /home/urs/.tin/${NNTPSERVER}"
-				 */
-				/* FIXME - write a global permission check routine */
 				if (access(dir, X_OK)) {
-					my_fprintf(stderr, _(txt_error_no_enter_permission), dir);
-					error = 1;
+					if (errno != ENOENT) {
+						my_fprintf(stderr, _(txt_error_no_enter_permission), dir);
+						error = 1;
+					} else {
+						my_fprintf(stderr, _(txt_error_no_such_file), dir);
+						error = 2;
+					}
 				} else if (access(newsrc_name, F_OK)) {
 					my_fprintf(stderr, _(txt_error_no_such_file), newsrc_name);
 					error = 2;
@@ -232,24 +256,38 @@ get_newsrcname(
 					/* very ugly code, but curses is not initialized yet */
 					if (error >= 2) {
 						default_ch = 'c';
-						printf("%s%c\b", _(txt_nrctbl_create), default_ch);
+						my_printf("%s%c\b", _(txt_nrctbl_create), default_ch);
 					} else
-						printf("%s%c\b", _(txt_nrctbl_default), default_ch);
+						my_printf("%s%c\b", _(txt_nrctbl_default), default_ch);
 
 					if ((ch = (char) ReadCh()) == '\r' || ch == '\n')
 						ch = default_ch;
 				} while (ch != ESC && ch != 'a' && ch != 'c' && ch != 'd' && ch != 'q');
-				printf("%c\n", ch);
+				my_printf("%c\n", ch);
 
 				/* NOTE: these keys can not be remapped */
 				switch (ch) {
 					case 'c':
-						/* FIXME this doesn't check if we could create the file */
-						return TRUE;
+						if (*dir) {
+							errno = 0;
+							(void) my_mkdir(dir, (mode_t) (S_IRWXU));
+							if (!errno || errno == EEXIST)
+								return TRUE;
+							my_fprintf(stderr, "mkdir(%s): Error: %s\n", dir, strerror(errno));
+						}
+
+						no_write = TRUE;
+						tin_done(EXIT_FAILURE, _(txt_cannot_create), *dir ? dir : "NULL");
+						/* keep lint quiet: */
+						/* NOTREACHED */
+						break;
 
 					case 'd':
 						joinpath(newsrc_name, newsrc_name_len, homedir, ".newsrc");
 						return TRUE;
+						/* keep lint quiet: */
+						/* NOTREACHED */
+						break;
 
 					case 'a':
 						/*
@@ -259,6 +297,9 @@ get_newsrcname(
 						snprintf(name_found, sizeof(name_found), ".newsrc-%s", nntpserver_name);
 						joinpath(newsrc_name, newsrc_name_len, homedir, name_found);
 						return TRUE;
+						/* keep lint quiet: */
+						/* NOTREACHED */
+						break;
 
 					case 'q':
 						exit(EXIT_SUCCESS);
@@ -277,3 +318,11 @@ get_newsrcname(
 
 	return FALSE;
 }
+#else
+static void no_newsrctable(void);	/* proto-type */
+static void
+no_newsrctable(	/* ANSI C requires non-empty source file */
+	void)
+{
+}
+#endif /* NNTP_ABLE */
