@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-11-03
+ *  Updated   : 2024-07-30
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -62,9 +62,9 @@ static t_url *url_list;
 
 t_openartinfo pgart =	/* Global context of article open in the pager */
 	{
-		{ NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, FALSE, NULL},
+		{ NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, FALSE, NULL },
 		FALSE, 0,
-		NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL, NULL,
 	};
 
 int last_resp;			/* previous & current article # in arts[] for '-' command */
@@ -83,6 +83,7 @@ static int reveal_ctrl_l_lines;	/* number of lines (from top) with de-activated
 static int rotate;				/* 0=normal, 13=rot13 decode */
 static int scroll_region_top;	/* first screen line for displayed message */
 static int search_line;			/* Line to commence next search from */
+static void show_article_info_page(void);
 static t_lineinfo *infoline = (t_lineinfo *) 0;
 
 static t_bool show_all_headers;	/* all headers <-> headers in news_headers_to[_not]_display */
@@ -92,6 +93,7 @@ static t_bool reveal_ctrl_l;	/* set when ^L hiding is off */
 /*
  * Local prototypes
  */
+static char *build_from_line(void);
 static int build_url_list(void);
 static int load_article(int new_respnum, struct t_group *group);
 static int prompt_response(int ch, int curr_respnum);
@@ -162,6 +164,11 @@ scroll_page(
 			case -2:
 				i >>= 1;
 				break;
+
+			default: /* should not happen */
+				/* CONSTANTCONDITION */
+				assert(0 != 0);
+				break;
 		}
 	}
 
@@ -252,11 +259,12 @@ deactivate_next_ctrl_l(
 		return FALSE;
 	if (end > artlines)
 		end = artlines;
-	for (i = reveal_ctrl_l_lines + 1; i < end; i++)
+	for (i = reveal_ctrl_l_lines + 1; i < end; i++) {
 		if (artline[i].flags & C_CTRLL) {
 			reveal_ctrl_l_lines = i;
 			return TRUE;
 		}
+	}
 	reveal_ctrl_l_lines = end - 1;
 	return FALSE;
 }
@@ -278,11 +286,12 @@ activate_last_ctrl_l(
 
 	if (reveal_ctrl_l)
 		return FALSE;
-	for (i = reveal_ctrl_l_lines; i >= curr_line; i--)
+	for (i = reveal_ctrl_l_lines; i >= curr_line; i--) {
 		if (artline[i].flags & C_CTRLL) {
 			reveal_ctrl_l_lines = i - 1;
 			return TRUE;
 		}
+	}
 	reveal_ctrl_l_lines = curr_line - 1;
 	return FALSE;
 }
@@ -680,7 +689,8 @@ page_goto_next_unread:
 				if (hide_uue && curr_line + ARTLINES > artlines)
 					curr_line = artlines - ARTLINES;
 				draw_page(0);
-				/* TODO: info_message()? */
+				/* TODO: use translateable sentences */
+				info_message("%s: %s", _(txt_hide_uue.opt), _(txt_hide_uue_type[hide_uue]));
 				break;
 
 			case PAGE_REVEAL:			/* toggle hiding after ^L */
@@ -725,18 +735,22 @@ page_goto_next_unread:
 				break;
 
 			case GLOBAL_EDIT_FILTER:
-				XFACE_CLEAR();
-				if (invoke_editor(filter_file, filter_file_offset, NULL)) {
-					old_artnum = arts[this_resp].artnum;
-					unfilter_articles(group);
-					(void) read_filter_file(filter_file);
-					filter_articles(group);
-					make_threads(group, FALSE);
-					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
-						return GRP_KILLED;
-					this_resp = n;
+				if (no_write)
+					info_message(_(txt_info_no_write));
+				else {
+					XFACE_CLEAR();
+					if (invoke_editor(filter_file, filter_file_offset, NULL)) {
+						old_artnum = arts[this_resp].artnum;
+						unfilter_articles(group);
+						(void) read_filter_file(filter_file);
+						filter_articles(group);
+						make_threads(group, FALSE);
+						if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
+							return GRP_KILLED;
+						this_resp = n;
+					}
+					draw_page(0);
 				}
-				draw_page(0);
 				break;
 
 			case GLOBAL_REDRAW_SCREEN:		/* redraw current page of article */
@@ -820,7 +834,7 @@ page_goto_next_unread:
 
 			case GLOBAL_CONNECTION_INFO:
 				XFACE_CLEAR();
-				show_connection_page(PAGE_LEVEL, _(txt_connection_info));
+				show_connection_page();
 				draw_page(0);
 				break;
 
@@ -1024,6 +1038,12 @@ return_to_index:
 				}
 				break;
 
+			case PAGE_ARTICLE_INFO:
+				XFACE_CLEAR();
+				show_article_info_page();
+				draw_page(0);
+				break;
+
 			default:
 				info_message(_(txt_bad_command), PrintFuncKey(key, GLOBAL_HELP, page_keys));
 		}
@@ -1111,26 +1131,29 @@ print_message_page(
 		 * Highlight URL's and mail addresses
 		 */
 		if (tinrc.url_highlight) {
-			if (curr->flags & C_URL)
+			if (curr->flags & C_URL) {
 #ifdef HAVE_COLOR
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &url_regex, use_color ? tinrc.col_urls : -1);
 #else
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &url_regex, -1);
 #endif /* HAVE_COLOR */
+			}
 
-			if (curr->flags & C_MAIL)
+			if (curr->flags & C_MAIL) {
 #ifdef HAVE_COLOR
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &mail_regex, use_color ? tinrc.col_urls : -1);
 #else
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &mail_regex, -1);
 #endif /* HAVE_COLOR */
+			}
 
-			if (curr->flags & C_NEWS)
+			if (curr->flags & C_NEWS) {
 #ifdef HAVE_COLOR
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &news_regex, use_color ? tinrc.col_urls : -1);
 #else
 				highlight_regexes((int) (i + (size_t) scroll_region_top), &news_regex, -1);
 #endif /* HAVE_COLOR */
+			}
 		}
 
 		/*
@@ -1242,6 +1265,43 @@ draw_page(
 	} else
 		draw_percent_mark(curr_line + ARTLINES, artlines);
 
+	if (CURR_GROUP.attribute->show_art_score && arts[this_resp].score != 0) {
+		char *buf;
+#ifdef HAVE_COLOR
+		int tmp_col = tinrc.col_invers_bg;
+#endif /* HAVE_COLOR */
+		size_t buflen = strlen(_(txt_art_score)) + 3;
+
+		buf = my_malloc(buflen + 1);
+		snprintf(buf, buflen, _(txt_art_score), tin_ltoa(arts[this_resp].score, 4));
+		MoveCursor(cLINES, 0);
+#ifdef HAVE_COLOR
+		fcol(tinrc.col_normal);
+		if (tinrc.inverse_okay) {
+			if (arts[this_resp].score < 0)
+				tinrc.col_invers_bg = tinrc.col_score_neg;
+			else
+				tinrc.col_invers_bg = tinrc.col_score_pos;
+		} else {
+			if (arts[this_resp].score < 0)
+				fcol(tinrc.col_score_neg);
+			else
+				fcol(tinrc.col_score_pos);
+		}
+#endif /* HAVE_COLOR */
+		StartInverse();
+		my_fputs(buf, stdout);
+		EndInverse();
+		my_flush();
+#ifdef HAVE_COLOR
+		if (tinrc.inverse_okay)
+			tinrc.col_invers_bg = tmp_col;
+		else
+			fcol(tinrc.col_normal);
+#endif /* HAVE_COLOR */
+		free(buf);
+	}
+
 #ifdef XFACE_ABLE
 	if (tinrc.use_slrnface && !show_raw_article)
 		slrnface_display_xface(note_h->xface);
@@ -1279,7 +1339,7 @@ invoke_metamail(
 	Raw(FALSE);
 
 #ifdef DONT_HAVE_PIPING
-	if ((fd_mime = my_tmpfile(mimefile, sizeof(mimefile) - 1, homedir)) == -1) {
+	if ((fd_mime = my_mktmp(mimefile, sizeof(mimefile) - 1, homedir)) == -1) {
 		perror_message(_(txt_command_failed), ptr);
 		return;
 	}
@@ -1288,17 +1348,47 @@ invoke_metamail(
 	if ((mime_fp = popen(ptr, "w")))
 #endif /* DONT_HAVE_PIPING */
 	{
+		t_bool seek_error = FALSE;
+
 		rewind(fp);
 		while (fgets(buf, (int) sizeof(buf), fp) != NULL)
 			fputs(buf, mime_fp);
 
 		fflush(mime_fp);
 		/* This is needed if we are viewing the raw art */
-		fseek(fp, offset, SEEK_SET);	/* goto old position */
+		if (fseek(fp, offset, SEEK_SET) != -1)	{ /* goto old position */
+		}
+#ifdef DEBUG
+		else {
+			int e = errno;
+			/*
+			 * TODO: always show to user?
+			 *       then use something less technical and move to lang.c
+			 */
+			perror_message("%s:%d invoke_metamail(fseek(fp)) failed", __FILE__, __LINE__);
+			seek_error = TRUE;
+			errno = e;
+		}
+#endif /* DEBUG */
+
+		if (!seek_error) {
+#ifdef DONT_HAVE_PIPING
+			char *pbuf;
+			size_t len;
+
+			len = snprintf(NULL, 0, "%s %s", tinrc.metamail_prog, mimefile);
+			pbuf= my_malloc(++len);
+			snprintf(pbuf, len, "%s %s", tinrc.metamail_prog, mimefile);
+			invoke_cmd(pbuf);
+			free(pbuf);
+#endif /* DONT_HAVE_PIPING */
+		}
+#ifdef DEBUG
+		else
+			perror_message(_(txt_command_failed), ptr);
+#endif /* DEBUG */
 
 #ifdef DONT_HAVE_PIPING
-		snprintf(buf, sizeof(buf) - 1, "%s %s", tinrc.metamail_prog, mimefile);
-		invoke_cmd(buf);
 		fclose(mime_fp);
 		unlink(mimefile);
 #else
@@ -1319,22 +1409,66 @@ invoke_metamail(
 }
 
 
+static char *
+build_from_line(
+	void)
+{
+	char *tmp_from, *curr_from, *next_from, *p;
+	char addr[HEADER_LEN];
+	char name[HEADER_LEN];
+	char single_from[HEADER_LEN + 3]; /*" <>"*/
+	char *from = NULL;
+	int type, c_needed = 0;
+
+	curr_from = tmp_from = my_strdup(note_h->from);
+
+	do {
+		next_from = split_mailbox_list(curr_from);
+		if (gnksa_split_from(curr_from, addr, name, &type) == GNKSA_OK) {
+			buffer_to_ascii(addr);
+			p = idna_decode(addr);
+
+			if (*name)
+				snprintf(single_from, sizeof(single_from), "%s <%s>", name, p);
+			else {
+				STRCPY(single_from, p);
+			}
+			free(p);
+		} else {
+			STRCPY(single_from, str_trim(curr_from));
+		}
+		if (c_needed++)
+			from = append_to_string(from, ", ");
+		from = append_to_string(from, single_from);
+		curr_from = next_from;
+	} while (curr_from);
+
+	free(tmp_from);
+	return from;
+}
+
+
 /*
  * PAGE_HEADER defines the size in lines of this header
+ *
+ * TODO: what about page_header_format{1,PAGE_HEADER-1}
+ *       to be able to customize the shown data and if all
+ *       are set to NULL use the full screen for ARTLINES?
+ *       (headers then could be seen via news_headers_to_display)
  */
 static void
 draw_page_header(
 	void)
 {
-	char *buf, *tmp;
+	char *buf, *from, *tmp, *tmp2;
 	int i;
 	int whichresp, x_resp;
 	int len, right_len, center_pos, cur_pos;
 	size_t line_len;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *fmt_resp = NULL, *fmt_thread, *wtmp, *wtmp2, *wbuf;
-#else
-	char *tmp2;
+	int n;
+	size_t tlen;
+	wchar_t *fmt_resp = NULL, *fmt_thread, *wtmp, *wtmp2, *wtmp3, *wbuf;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	whichresp = which_response(this_resp);
@@ -1383,7 +1517,8 @@ draw_page_header(
 	/* date */
 	if ((wtmp = char2wchar_t(buf)) != NULL) {
 		my_fputws(wtmp, stdout);
-		cur_pos += wcswidth(wtmp, wcslen(wtmp));
+		if ((n = wcswidth(wtmp, wcslen(wtmp))) > 0)
+			cur_pos += n;
 		free(wtmp);
 	}
 
@@ -1391,10 +1526,11 @@ draw_page_header(
 	 * determine max len for centered group name
 	 * allow one space before and after group name
 	 */
-	len = cCOLS - 2 * MAX(cur_pos, right_len) - 3;
+	if ((len = cCOLS - 2 * MAX(cur_pos, right_len) - 3) < 0)
+		len = 0;
 
 	/* group name */
-	if ((wtmp = char2wchar_t(curr_group->name)) != NULL) {
+	if ((wtmp = char2wchar_t(curr_group->name)) != NULL && len > 0) {
 		/* wconvert_to_printable(wtmp, FALSE); */
 		if (tinrc.abbreviate_groupname)
 			wtmp2 = abbr_wcsgroupname(wtmp, len);
@@ -1413,8 +1549,8 @@ draw_page_header(
 		my_fputws(wtmp2, stdout);
 		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
 		free(wtmp2);
-		free(wtmp);
 	}
+	FreeIfNeeded(wtmp);
 
 	/* pad out to right */
 	for (; cur_pos < cCOLS - right_len - 1; cur_pos++)
@@ -1422,13 +1558,32 @@ draw_page_header(
 
 	/* thread info */
 	/* can't eval tin_ltoa() more than once in a statement due to statics */
-	strcpy(buf, tin_ltoa(which_thread(this_resp) + 1, 4));
-	tmp = strunc(_(txt_thread_x_of_n), cCOLS / 3 - 1);
-	my_printf(tmp, buf, tin_ltoa(grpmenu.max, 4));
-	free(tmp);
+	{
+		char *buf2 = my_calloc(1, 5);
 
-	my_fputs(cCRLF, stdout);
+		strcpy(buf, tin_ltoa(which_thread(this_resp) + 1, 4));
+		strcpy(buf2, tin_ltoa(grpmenu.max, 4));
 
+		if ((n = snprintf(NULL, 0, _(txt_thread_x_of_n), buf, buf2)) < 0) {
+			free(buf2);
+			goto skip;
+		}
+
+		tlen = (size_t) n + 1;
+		tmp2 = my_malloc(tlen);
+		if (snprintf(tmp2, tlen, _(txt_thread_x_of_n), buf, buf2) != n) {
+			free(tmp2);
+			free(buf2);
+			goto skip;
+		}
+		tmp = strunc(tmp2, cCOLS / 3 - 1);
+		my_printf("%s", tmp);
+		free(tmp2);
+		free(buf2);
+		free(tmp);
+skip:
+		my_fputs(cCRLF, stdout);
+	}
 #	if 0
 	/* display a ruler for layout checking purposes */
 	my_fputs("....|....3....|....2....|....1....|....0....|....1....|....2....|....3....|....\n", stdout);
@@ -1537,8 +1692,18 @@ draw_page_header(
 		my_fputc(' ', stdout);
 
 	if (whichresp) {
-		tmp = strunc(_(txt_art_x_of_n), cCOLS / 3 - 1);
-		my_printf(tmp, whichresp + 1, x_resp + 1);
+		if ((n = snprintf(NULL, 0, _(txt_art_x_of_n), whichresp + 1, x_resp + 1)) < 0)
+			goto shrug;
+
+		tlen = (size_t) n + 1;
+		tmp2 = my_malloc(tlen);
+		if (snprintf(tmp2, tlen, _(txt_art_x_of_n), whichresp + 1, x_resp + 1) != n) {
+			free(tmp2);
+			goto shrug;
+		}
+		tmp = strunc(tmp2, cCOLS / 3 - 1);
+		my_printf("%s", tmp);
+		free(tmp2);
 		free(tmp);
 	} else {
 		/* TODO: ngettext */
@@ -1549,11 +1714,22 @@ draw_page_header(
 			tmp = strunc(_(txt_1_resp), cCOLS / 3 - 1);
 			my_printf("%s", tmp);
 		} else {
-			tmp = strunc(_(txt_x_resp), cCOLS / 3 - 1);
-			my_printf(tmp, x_resp);
+			if ((n = snprintf(NULL, 0, _(txt_x_resp), x_resp)) < 0)
+				goto shrug;
+
+			tlen = (size_t) n + 1;
+			tmp2 = my_malloc(tlen);
+			if (snprintf(tmp2, tlen, _(txt_x_resp), x_resp) != n) {
+				free(tmp2);
+				goto shrug;
+			}
+			tmp = strunc(tmp2, cCOLS / 3 - 1);
+			my_printf("%s", tmp);
+			free(tmp2);
 		}
 		free(tmp);
 	}
+shrug:
 	my_fputs(cCRLF, stdout);
 
 	/*
@@ -1565,41 +1741,55 @@ draw_page_header(
 	fcol(tinrc.col_from);
 #	endif /* HAVE_COLOR */
 	/* from */
-	/*
-	 * TODO: don't use arts[this_resp].name/arts[this_resp].from
-	 *       split up note_h->from and use that instead as it might
-	 *       be different _if_ the overviews are broken
-	 *
-	 *       add BiDi handling
-	 */
-	{
-		char *p = idna_decode(arts[this_resp].from);
+	from = build_from_line();
+	snprintf(buf, line_len, "%s", from);
+	free(from);
 
-		if (arts[this_resp].name)
-			snprintf(buf, line_len, "%s <%s>", arts[this_resp].name, p);
-		else {
-			strncpy(buf, p, line_len);
-			buf[line_len - 1] = '\0';
+	/* precalculate min. organization length */
+	wtmp3 = my_calloc(1, sizeof(wchar_t));
+	if (note_h->org) {
+		char *tb;
+		char *tail = NULL;
+
+		if (tinrc.utf8_graphics)
+			tail = wchar_t2char(WTRUNC_TAIL);
+
+		if ((n = snprintf(NULL, 0, _(txt_at_s), tail ? tail : TRUNC_TAIL)) > 0) {
+			size_t olen = (size_t) n + 1;
+
+			tb = my_malloc(olen);
+			if (snprintf(tb, olen, _(txt_at_s), tail ? tail : TRUNC_TAIL) == n) {
+				if ((wtmp = char2wchar_t(tb)) != NULL) {
+					wbuf = wexpand_tab(wtmp, tabwidth);
+					free(wtmp3);
+					wtmp3 = my_wcsdup(wbuf);
+					free(wtmp);
+					free(wbuf);
+				}
+			}
+			free(tb);
 		}
-		free(p);
+		FreeIfNeeded(tail);
 	}
 
+	/* from */
 	if ((wtmp = char2wchar_t(buf)) != NULL) {
-		wtmp2 = wstrunc(wtmp, cCOLS - 1);
+		wtmp2 = wstrunc(wtmp, cCOLS - 1 - wcslen(wtmp3));
 		my_fputws(wtmp2, stdout);
 		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
 		free(wtmp2);
 		free(wtmp);
 	}
+	free(wtmp3);
 
 	/*
 	 * Organization
 	 *
-	 * TODO: IDNA decoding, see also comment in
-	 *       cook.c:cook_article()
-	 *       add BiDi handling
+	 * TODO: - IDNA decoding, see also comment in
+	 *         cook.c:cook_article()
+	 *       - add BiDi handling
 	 */
-	if (note_h->org) {
+	if (note_h->org && cur_pos < cCOLS - 1) {
 		snprintf(buf, line_len, _(txt_at_s), note_h->org);
 
 		if ((wtmp = char2wchar_t(buf)) != NULL) {
@@ -1770,17 +1960,9 @@ draw_page_header(
 	fcol(tinrc.col_from);
 #	endif /* HAVE_COLOR */
 	/* from */
-	/*
-	 * TODO: don't use arts[this_resp].name/arts[this_resp].from
-	 *       split up note_h->from and use that instead as it might
-	 *       be different _if_ the overviews are broken
-	 */
-	if (arts[this_resp].name)
-		snprintf(buf, line_len, "%s <%s>", arts[this_resp].name, arts[this_resp].from);
-	else {
-		strncpy(buf, arts[this_resp].from, line_len);
-		buf[line_len - 1] = '\0';
-	}
+	from = build_from_line();
+	snprintf(buf, line_len, "%s", from);
+	free(from);
 
 	tmp = strunc(buf, cCOLS - 1);
 	my_fputs(tmp, stdout);
@@ -1934,7 +2116,7 @@ load_article(
 	}
 
 	XFACE_SUPPRESS();
-	if (strcmp(tinrc.metamail_prog, INTERNAL_CMD) == 0)	/* Use internal viewer */
+	if (STRCMPEQ(tinrc.metamail_prog, INTERNAL_CMD))	/* Use internal viewer */
 		decode_save_mime(&pgart, FALSE);
 	else
 		invoke_metamail(pgart.raw);
@@ -1976,7 +2158,8 @@ process_search(
 	size_t screen_lines,
 	int help_level)
 {
-	int i, start, end;
+	int i;
+	REGEX_SIZE start, end;
 
 	if ((i = get_search_vectors(&start, &end)) == -1)
 		return;
@@ -2002,7 +2185,9 @@ process_search(
 			display_info_page(0);
 			break;
 
-		default:
+		default: /* unknown level, should not happen */
+			/* CONSTANTCONDITION */
+			assert(0 != 0);
 			break;
 	}
 	search_line = i;								/* draw_page() resets this to 0 */
@@ -2526,6 +2711,15 @@ url_page(
 				scroll_up();
 				break;
 
+#ifdef HAVE_COLOR
+			case GLOBAL_TOGGLE_COLOR:
+				if (toggle_color()) {
+					show_url_page();
+					show_color_status();
+				}
+				break;
+#endif /* HAVE_COLOR */
+
 			case GLOBAL_TOGGLE_HELP_DISPLAY:
 				toggle_mini_help(URL_LEVEL);
 				show_url_page();
@@ -2536,6 +2730,16 @@ url_page(
 				show_url_page();
 				break;
 
+			case GLOBAL_TOGGLE_INVERSE_VIDEO:
+				toggle_inverse_video();
+				show_url_page();
+				show_inverse_video_status();
+				break;
+
+			case GLOBAL_VERSION:
+				info_message(cvers);
+				break;
+
 			case URL_SELECT:
 				if (urlmenu.max) {
 					if (process_url(urlmenu.curr))
@@ -2629,6 +2833,7 @@ process_url(
 	int n)
 {
 	char *url, *url_esc;
+	int l;
 	size_t len;
 	t_url *lptr;
 
@@ -2640,11 +2845,18 @@ process_url(
 			free(url);
 			return FALSE;
 		}
-		wait_message(2, _(txt_url_open), url);
 		url_esc = escape_shell_meta(url, no_quote);
-		len = strlen(url_esc) + strlen(tinrc.url_handler) + 2;
+		if ((l = snprintf(NULL, 0, "%s %s", tinrc.url_handler, url_esc)) < 0) {
+			free(url);
+			return FALSE;
+		}
+		wait_message(2, _(txt_url_open), url);
+		len = (size_t) l + 1;
 		url = my_realloc(url, len);
-		snprintf(url, len, "%s %s", tinrc.url_handler, url_esc);
+		if (snprintf(url, len, "%s %s", tinrc.url_handler, url_esc) != l) {
+			free(url);
+			return FALSE;
+		}
 		invoke_cmd(url);
 		free(url);
 		cursoroff();
@@ -2681,11 +2893,11 @@ build_url_list(
 		 */
 		forever {
 			/* any matches left? */
-			if (match_regex_ex(ptr, (int) strlen(ptr), 0, 0, &url_regex) >= 0) {
+			if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &url_regex) >= 0) {
 				offsets = regex_get_ovector_pointer(&url_regex);
-			} else if (match_regex_ex(ptr, (int) strlen(ptr), 0, 0, &mail_regex) >= 0) {
+			} else if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &mail_regex) >= 0) {
 				offsets = regex_get_ovector_pointer(&mail_regex);
-			} else if (match_regex_ex(ptr, (int) strlen(ptr), 0, 0, &news_regex) >= 0) {
+			} else if (match_regex_ex(ptr, (REGEX_SIZE) strlen(ptr), 0, 0, &news_regex) >= 0) {
 				offsets = regex_get_ovector_pointer(&news_regex);
 			} else
 				break;
@@ -2724,13 +2936,127 @@ free_url_list(
 }
 
 
+static void
+show_article_info_page(
+	void)
+{
+	if (!(pgart.log))
+		return;
+
+	info_pager(pgart.log, _(txt_article_info_page), FALSE); /* all other pagers do wrap */
+	info_pager(NULL, NULL, TRUE); /* free mem */
+}
+
+
+#define MIME_FLAG(what) (ptr->mime_hints.flags & what)
+/* TODO: strings to lang.c and _() ? */
+void
+log_article_info(
+	t_openartinfo *artinfo)
+{
+	const char *charset;
+	int i = 1;
+	FILE *fp;
+	t_part *ptr;
+#ifdef DEBUG
+	const char *name;
+#endif /* DEBUG */
+
+	if (!(fp = artinfo->log))
+		return;
+
+	fprintf(fp, "Message-ID: %s\n", BlankIfNull(artinfo->hdr.messageid));
+
+	for (ptr = artinfo->hdr.ext; ptr; ptr = ptr->next, i++) {
+		fprintf(fp, "\nPart %d:\n", i);
+
+		if (i == 1) {
+			fprintf(fp, "\tMIME Version:\n");
+			fprintf(fp, "\t\tArticle:\t%s", MIME_FLAG(MIME_VERSION_MISSING) ? "Missing" : MIME_FLAG(MIME_VERSION_UNSUPPORTED) ? "Unsupported" : MIME_SUPPORTED_VERSION);
+			fprintf(fp, "\n\t\tUsed:\t\t%s\n", MIME_SUPPORTED_VERSION);
+		}
+
+		fprintf(fp, "\tContent Type:\n");
+		fprintf(fp, "\t\tArticle:\t");
+		if (MIME_FLAG(MIME_TYPE_MISSING))
+			fprintf(fp, "%s", "Missing");
+		else if (MIME_FLAG(MIME_TYPE_UNKNOWN))
+			fprintf(fp, "%s: %s", "Unsupported", ptr->mime_hints.type);
+		else
+			fprintf(fp, "%s", ptr->mime_hints.type);
+		if (MIME_FLAG(MIME_SUBTYPE_MISSING))
+			fprintf(fp, " / %s", "Missing");
+		else if (MIME_FLAG(MIME_SUBTYPE_UNKNOWN))
+			fprintf(fp, " / %s: %s", "Unsupported", ptr->mime_hints.subtype);
+		else
+			fprintf(fp, " / %s", ptr->mime_hints.subtype);
+		fprintf(fp, "\n\t\tUsed:\t\t%s / %s", content_types[ptr->type], ptr->subtype);
+
+		charset = get_param(ptr->params, "charset");
+		if (!charset || !*charset)
+			charset = "US-ASCII";
+		fprintf(fp, "\n\tCharset:\n");
+		fprintf(fp, "\t\tArticle:\t");
+		if (MIME_FLAG(MIME_CHARSET_MISSING)) {
+			fprintf(fp, "%s", "Missing");
+			if (MIME_FLAG(MIME_CHARSET_UNDECLARED))
+				fprintf(fp, ", %s: %s", "undeclared_charset", charset);
+#if defined(CHARSET_CONVERSION) && defined(USE_ICU_UCSDET)
+			else if (MIME_FLAG(MIME_CHARSET_GUESSED)) {
+				charset = get_param(ptr->params, "guessed_charset");
+				fprintf(fp, ", %s: %s", "charset guessed", charset);
+			}
+#endif /* CHARSET_CONVERSION && USE_ICU_UCSDET */
+		} else if (MIME_FLAG(MIME_CHARSET_UNSUPPORTED)) {
+			fprintf(fp, "%s: %s", "Unsupported", ptr->mime_hints.charset);
+			charset = "None";
+		} else
+			fprintf(fp, "%s", charset);
+		if (MIME_FLAG(MIME_TRANSFER_ENCODING_UNKNOWN))
+			charset = "None";
+		fprintf(fp, "\n\t\tUsed:\t\t%s", charset);
+
+		fprintf(fp, "\n\tContent Transfer Encoding:\n");
+		fprintf(fp, "\t\tArticle:\t");
+		if (MIME_FLAG(MIME_TRANSFER_ENCODING_MISSING))
+			fprintf(fp, "%s", "Missing");
+		else if (MIME_FLAG(MIME_TRANSFER_ENCODING_UNKNOWN))
+			fprintf(fp, "%s: %s", "Unsupported", ptr->mime_hints.encoding);
+		else
+			fprintf(fp, "%s", content_encodings[ptr->encoding]);
+		fprintf(fp, "\n\t\tUsed:\t\t%s", content_encodings[ptr->encoding]);
+
+#ifdef DEBUG
+		if ((name = get_param(ptr->params, "name")) != NULL) {
+			fprintf(fp, "\n\tName:\n");
+			fprintf(fp, "\t\t%s", name);
+		}
+		if (ptr->description != NULL) {
+			fprintf(fp, "\n\tContent Description:\n");
+			fprintf(fp, "\t\t%s", ptr->description);
+		}
+		if (ptr->language != NULL) {
+			fprintf(fp, "\n\tContent Language:\n");
+			fprintf(fp, "\t\t%s", ptr->language);
+		}
+		if (ptr->disposition != DISP_NONE) {
+			fprintf(fp, "\n\tContent Disposition:\n");
+			fprintf(fp, "\t\t%s", ptr->disposition == DISP_INLINE ? content_disposition[DISP_INLINE] : content_disposition[DISP_ATTACHMENT]);
+		}
+#endif /* DEBUG */
+		fprintf(fp, "\n");
+	}
+}
+
+
 static void
 draw_percent_mark(
 	long cur_num,
 	long max_num)
 {
-	char buf[32]; /* FIXME: may get truncated with long localized _(txt_more) ... */
+	char *buf;
 	int len;
+	size_t sz;
 
 	if (NOTESLINES <= 0)
 		return;
@@ -2738,9 +3064,18 @@ draw_percent_mark(
 	if (cur_num <= 0 && max_num <= 0)
 		return;
 
-	clear_message();
-	snprintf(buf, sizeof(buf), "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num);
+	if ((len = snprintf(NULL, 0, "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num)) < 0)
+		return;
+
+	sz = (size_t) len + 1;
+	buf = my_malloc(sz);
+	if (snprintf(buf, sz, "%s(%d%%) [%ld/%ld]", _(txt_more), (int) (cur_num * 100 / max_num), cur_num, max_num) != len) {
+		free(buf);
+		return;
+	}
+
 	len = strwidth(buf);
+	clear_message();
 	MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_normal);
@@ -2749,4 +3084,5 @@ draw_percent_mark(
 	my_fputs(buf, stdout);
 	EndInverse();
 	my_flush();
+	free(buf);
 }
