@@ -3,9 +3,9 @@
  *  Module    : read.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1997-04-10
- *  Updated   : 2022-09-19
+ *  Updated   : 2024-04-10
  *
- * Copyright (c) 1997-2022 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1997-2025 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -152,7 +152,6 @@ wait_for_input(
 					if (prompt_yn(_(txt_read_exit), FALSE) == 1)
 						tin_done(EXIT_SUCCESS, NULL);
 				}
-
 			}
 
 #	if 0
@@ -163,7 +162,6 @@ wait_for_input(
 				return TRUE;
 #	endif /* 0 */
 		}
-
 	}
 }
 #endif /* HAVE_SELECT */
@@ -201,7 +199,7 @@ tin_read(
 #ifdef NNTP_ABLE
 #	ifdef HAVE_SELECT
 	if (wait_for_input()) {			/* Check if okay to read */
-		info_message(_("Aborting read, please wait..."));
+		info_message(_(txt_abort_reading));
 		drain_buffer(fp);
 		clear_message();
 		tin_errno = TIN_ABORT;
@@ -209,21 +207,19 @@ tin_read(
 		return NULL;
 	}
 #	endif /* HAVE_SELECT */
+#endif /* NNTP_ABLE */
 
 	errno = 0;		/* To check errno after read, clear it here */
 
+#ifdef NNTP_ABLE
 	/*
 	 * Initially try and fit into supplied buffer
 	 */
 	if (fp == FAKE_NNTP_FP)
 		ptr = get_server(buffer, (int) len);
 	else
-		ptr = fgets(buffer, (int) len, fp);
-#else
-	errno = 0;		/* To check errno after read, clear it here */
-
-	ptr = fgets(buffer, len, fp);
 #endif /* NNTP_ABLE */
+		ptr = fgets(buffer, (int) len, fp);
 
 /* TODO: develop this next line? */
 #ifdef DEBUG
@@ -241,7 +237,6 @@ tin_read(
 	 */
 	i = (int) strlen(buffer);
 	if (i >= 1 && buffer[i - 1] == '\n') {
-
 		if (i >= 2 && buffer[i - 2] == '\r') {
 			buffer[i - 2] = '\0';
 			offset = i -= 2;
