@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2023-07-31
+ *  Updated   : 2024-11-25
  *  Notes     : Caching of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2023 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2025 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,6 @@ static char *_get_references(struct t_msgid *refptr, int depth);
 static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
 static struct t_msgid *find_next(struct t_msgid *ptr);
 static struct t_msgid *parse_references(char *r);
-static t_bool valid_msgid(char *msgid);
 static unsigned int hash_msgid(const char *key);
 static void add_to_parent(struct t_msgid *ptr);
 static void build_thread(struct t_msgid *ptr);
@@ -76,7 +75,7 @@ static void rearrange_siblings(void);
 /*
  * The msgids are all hashed into a big array, with overspill
  */
-static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
+static struct t_msgid *msgids[MSGID_HASH_SIZE] = { NULL };
 
 /*
  * This part of the code deals with the caching and retrieval
@@ -159,14 +158,15 @@ add_to_parent(
 
 
 /*
- * Checks if Message-ID has valid format
+ * Checks if a single Message-ID has valid format
  * Returns TRUE if it does, FALSE if it does not
  * modifies *msgid
  *
  * see also post.c:damaged_id(), which does not modify its input,
  * has swapped return values and does not allow trailing spaces
+ * but doesn't error on whitespace in between.
  */
-static t_bool
+t_bool
 valid_msgid(
 	char *msgid)
 {
@@ -177,8 +177,11 @@ valid_msgid(
 	str_trim(msgid);
 	mlen = strlen(msgid);
 
-	/* must start with '<' and have exactly one '>' (at the very end) */
-	if (!mlen || mlen < 5 /* || mlen > 250 */ || *(msgid + mlen -1) != '>' || strchr(msgid, '>') != (msgid + mlen - 1) || *msgid++ != '<')
+	/*
+	 * must start with '<' and have exactly one '>' (at the very end)
+	 *
+	 */
+	if (mlen < 8 /* || mlen > 250 */ || *(msgid + mlen - 1) != '>' || strchr(msgid, '>') != (msgid + mlen - 1) || *msgid++ != '<')
 		return FALSE;
 
 	while (*msgid) {
@@ -190,19 +193,19 @@ valid_msgid(
 				if (bracket != 0 || !at_present || *(msgid - 1) != '@')
 					return FALSE;
 				else
-					bracket++;
+					++bracket;
 				break;
 
 			case ']':
 				if (bracket != 1 || !at_present || *(msgid + 1) != '>')
 					return FALSE;
 				else
-					bracket--;
+					--bracket;
 				break;
 
 			case '@':
 				if (!at_present) {
-					if (*(msgid + 1) == '.')
+					if (*(msgid + 1) == '.' || *(msgid - 1) == '<')
 						return FALSE;
 
 					at_present = TRUE;
@@ -230,7 +233,7 @@ valid_msgid(
 			default:
 				break;
 		}
-		msgid++;
+		++msgid;
 	}
 
 	if (bracket != 0 || !at_present)
@@ -284,7 +287,7 @@ add_msgid(
 
 #ifdef DEBUG
 	if (debug & DEBUG_REFS)
-		DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _("unchanged") : newparent->txt));
+		DEBUG_PRINT((dbgfd, "---------------- Add %s %s with parent %s\n", (key == MSGID_REF) ? "MSG" : "REF", msgid, (newparent == NULL) ? _(txt_unchanged) : newparent->txt));
 #endif /* DEBUG */
 
 	/*
@@ -340,7 +343,7 @@ add_msgid(
 			add_to_parent(i);
 #ifdef DEBUG
 			if (debug & DEBUG_REFS)
-				DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _("None")));
+				DEBUG_PRINT((dbgfd, "set: %s -> %s\n", i->txt, newparent ? newparent->txt : _(txt_none)));
 #endif /* DEBUG */
 			return i;
 		}
@@ -614,10 +617,8 @@ free_msgids(
 		*msgptr++ = NULL;			/* declare list empty */
 
 		while (ptr != NULL) {	/* for each node in the list */
-
 			next = ptr->next;		/* grab ptr before we free node */
 			free(ptr);			/* release node */
-
 			ptr = next;			/* hop down the chain */
 		}
 	}
@@ -648,7 +649,7 @@ dump_thread(
 	i = cCOLS - len - 20;
 
 	if (msgid->article >= 0)
-		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].name) ? arts[msgid->article].name : arts[msgid->article].from);
+		sprintf(ptr + len, "%-*.*s   %-17.17s", i, i, arts[msgid->article].subject, (arts[msgid->article].mailbox->name) ? arts[msgid->article].mailbox->name : arts[msgid->article].mailbox->from);
 	else
 		sprintf(ptr + len, "%-*.*s", i, i, _("[- Unavailable -]"));
 
@@ -678,7 +679,6 @@ dump_msgids(
 				my_fprintf(stderr, " -> %s", ptr->txt);
 
 			my_fprintf(stderr, "\n");
-
 		}
 	}
 }
@@ -813,7 +813,6 @@ find_next(
 	 * got something in arts[]
 	 */
 	while (ptr != NULL) {
-
 		/*
 		 * Children first, unless bottom is set
 		 */
@@ -976,7 +975,7 @@ collate_subjects(
 		/*
 		 * Get the contents of the magic marker in the hashnode
 		 */
-		h = (struct t_hashnode *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
+		h = (void *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
 		j = h->aptr;
 
 		if (j != -1 && j < i) {
@@ -1031,7 +1030,7 @@ build_references(
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
+	if (group->attribute && group->attribute->sort_article_type != SORT_ARTICLES_BY_NOTHING)
 		sort_arts(group->attribute->sort_article_type);
 
 #ifdef DEBUG
