@@ -3,10 +3,10 @@
  *  Module    : rfc1524.c
  *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  *  Created   : 2000-05-15
- *  Updated   : 2021-02-23
+ *  Updated   : 2024-05-10
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2023 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 2000-2024 Urs Janssen <urs@tin.org>, Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -94,7 +94,7 @@ get_mailcap_entry(
 	while (nptr != NULL) {
 		/* expand ~ and/or $HOME etc. */
 		if (strfpath(nptr, filename, sizeof(filename) - 1, &CURR_GROUP, FALSE)) {
-			if ((fp = fopen(filename, "r")) != NULL) {
+			if ((fp = tin_fopen(filename, "r")) != NULL) {
 				while ((fgets(ptr, (int) (sizeof(buf) - strlen(buf)), fp)) != NULL) {
 					if (*ptr == '#' || *ptr == '\n')		/* skip comments & blank lines */
 						continue;
@@ -221,7 +221,7 @@ parse_mailcap_line(
 			ptr += strlen(ptr) + 1;
 		}
 		if (!strncasecmp(ptr, "textualnewlines=", 16)) {
-			tmailcap->textualnewlines = atoi(ptr + 16);
+			tmailcap->textualnewlines = s2i(ptr + 16, 0, 1);
 			ptr += strlen(ptr) + 1;
 		}
 		if (!strncasecmp(ptr, "compose=", 8)) {
