@@ -3,10 +3,10 @@
  *  Module    : save.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-10-29
+ *  Updated   : 2024-10-17
  *  Notes     :
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -105,10 +105,6 @@ static void untag_all_parts(void);
 static void untag_part(int n);
 static void uudecode_line(const char *buf, FILE *fp);
 static void view_file(const char *path, const char *file);
-#ifndef HAVE_LIBUU
-	static void sum_file(const char *path, const char *file);
-#endif /* !HAVE_LIBUU */
-
 static int num_of_tagged_parts, info_len;
 static t_menu attmenu = { 0, 0, 0, show_attachment_page, draw_attachment_arrow, build_attachment_line };
 static t_partl *part_list;
@@ -212,9 +208,9 @@ check_start_save_any_news(
 		}
 
 		if (function == MAIL_ANY_NEWS || function == SAVE_ANY_NEWS) {
-			if (!group->attribute->batch_save) { /* TODO: string -> lang.c */
+			if (!group->attribute->batch_save) {
 				if (verbose > 1 && function == MAIL_ANY_NEWS)
-					fprintf(fp_log, "Skipped %s", group->name);
+					fprintf(fp_log, _(txt_skipped_group), group->name);
 
 				continue;
 			}
@@ -230,7 +226,7 @@ check_start_save_any_news(
 				char *group_path = my_malloc(strlen(group->name) + 2); /* trailing "/\0" */
 
 				make_group_path(group->name, group_path);
-				if (!strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : tinrc.savedir, tmp, sizeof(tmp), group, FALSE))
+				if (!strfpath(cmdline.savedir ? cmdline.savedir : tinrc.savedir, tmp, sizeof(tmp), group, FALSE))
 					joinpath(tmp, sizeof(tmp), homedir, DEFAULT_SAVEDIR);
 
 				joinpath(path, sizeof(path), tmp, group_path);
@@ -288,7 +284,7 @@ check_start_save_any_news(
 						continue;
 					}
 
-					if ((function == MAIL_ANY_NEWS) && ((INTERACTIVE_NONE == tinrc.interactive_mailer) || (INTERACTIVE_WITH_HEADERS == tinrc.interactive_mailer))) {
+					if ((function == MAIL_ANY_NEWS) && (tinrc.interactive_mailer == INTERACTIVE_NONE || tinrc.interactive_mailer == INTERACTIVE_WITH_HEADERS)) {
 						fprintf(savefp, "To: %s\n", mail_news_user);
 						fprintf(savefp, "Subject: %s\n", arts[j].subject);
 						/*
@@ -337,9 +333,12 @@ check_start_save_any_news(
 
 		if (art_count) {
 			unread_news = TRUE;
-			if (verbose)
-				wait_message(0, _(txt_saved_group), art_count, hot_count,
+			if (verbose) {
+				char *tmp_hot_count = my_strdup(tin_ltoa(hot_count, 4));
+				wait_message(0, _(txt_saved_group), tin_ltoa(art_count, 4), tmp_hot_count,
 					PLURAL(art_count, txt_article), group->name);
+				free(tmp_hot_count);
+			}
 		}
 	}
 
@@ -399,6 +398,7 @@ open_save_filename(
 {
 	FILE *fp;
 	char keyappend[MAXKEYLEN], keyoverwrite[MAXKEYLEN], keyquit[MAXKEYLEN];
+	int i;
 	struct stat st;
 	t_function func;
 
@@ -407,11 +407,12 @@ open_save_filename(
 		return NULL;
 	}
 
-	if (fstat(fileno(fp), &st) != -1) {
+	i = fileno(fp);
+	if (i == -1 || fstat(i, &st) != -1) {
 		/*
 		 * Admittedly a special case hack, but it saves failing later on
 		 */
-		if (S_ISDIR(st.st_mode)) {
+		if (i == -1 || S_ISDIR(st.st_mode)) {
 			wait_message(2, _(txt_cannot_write_to_directory), path);
 			fclose(fp);
 			return NULL;
@@ -430,8 +431,22 @@ open_save_filename(
 			switch (func) {
 				case SAVE_OVERWRITE_FILE:
 					tinrc.default_save_mode = 'o';
-					if (!ftruncate(fileno(fp), 0L))
-						(void) fseek(fp, 0L, SEEK_SET);
+#ifdef HAVE_FTRUNCATE
+					errno = 0;
+					if (i == -1 || ftruncate(i, 0L)) {
+#	ifdef DEBUG
+						if (debug & DEBUG_MISC)
+							perror_message("ftruncate(%d)", i);
+#	endif /* DEBUG */
+					}
+#endif /* HAVE_FTRUNCATE */
+					errno = 0;
+					if (i == -1 || fseek(fp, 0L, SEEK_SET) == -1) {
+						perror_message("fseek(%d)", i);
+						fclose(fp);
+						wait_message(1, _(txt_art_not_saved));
+						return NULL;
+					}
 					break;
 
 				case GLOBAL_ABORT:
@@ -532,7 +547,7 @@ save_and_process_art(
 	 *       then at least in the (is_mailbox && !post_process && !mmdf)
 	 *       case it should be done.
 	 */
-	if (copy_fp(artinfo->raw, fp)) /* Write tailing newline or MMDF-mailbox separator */
+	if (copy_fp(artinfo->raw, fp) == 0) /* Write tailing newline or MMDF-mailbox separator */
 		print_art_separator_line(fp, is_mailbox);
 	else {
 		fclose(fp);
@@ -567,6 +582,8 @@ save_and_process_art(
 		scrollok(stdscr, FALSE);
 #	endif /* USE_CURSES */
 	}
+#else /* silence compiler warning (unused parameter) */
+	(void) post_process;
 #endif /* !HAVE_LIBUU */
 
 	return TRUE;
@@ -700,7 +717,7 @@ expand_save_filename(
 	base_name(buf_path, base_filename);
 
 	/* Build default path to save to */
-	if (!(ret = strfpath((cmdline.args & CMDLINE_SAVEDIR) ? cmdline.savedir : curr_group->attribute->savedir, buf, sizeof(buf), curr_group, FALSE)))
+	if (!(ret = strfpath(cmdline.savedir ? cmdline.savedir : (curr_group->attribute->savedir && *curr_group->attribute->savedir) ? *curr_group->attribute->savedir : NULL, buf, sizeof(buf), curr_group, FALSE)))
 		joinpath(buf, sizeof(buf), homedir, DEFAULT_SAVEDIR);
 
 	/* Join path and filename */
@@ -786,8 +803,9 @@ post_process_uud(
 {
 	FILE *fp_in;
 	char file_out_dir[PATH_LEN];
+	char path[PATH_LEN];
 	const char *eptr;
-	int i;
+	int i, r;
 	int count;
 	int errors = 0;
 	uulist *item;
@@ -801,7 +819,7 @@ post_process_uud(
 
 	UUSetOption(UUOPT_SAVEPATH, 0, file_out_dir);
 	for (i = 0; i < num_save; i++) {
-		if ((fp_in = fopen(save[i].path, "r")) != NULL) {
+		if ((fp_in = tin_fopen(save[i].path, "r")) != NULL) {
 			UULoadFile(save[i].path, NULL, 0);	/* Scans file for encoded data */
 			fclose(fp_in);
 		}
@@ -824,8 +842,7 @@ post_process_uud(
 	my_printf(cCRLF);
 
 	while (item != NULL) {
-		if (UUDecodeFile(item, NULL) == UURET_OK) {
-			char path[PATH_LEN];
+		if ((r = UUDecodeFile(item, NULL)) == UURET_OK) {
 
 /* TODO: test for multiple things per article decoded okay? */
 			count++;
@@ -847,7 +864,7 @@ post_process_uud(
 				eptr = _(txt_uu_error_no_end);
 			else if (item->state & UUFILE_NODATA)
 				eptr = _(txt_libuu_error_no_data);
-			else
+			else /* add UUstrerror(r) here?*/
 				eptr = _(txt_libuu_error_unknown);
 
 			my_printf(_(txt_uu_error_decode), (item->filename) ? item->filename : item->subfname, eptr);
@@ -862,9 +879,9 @@ post_process_uud(
 	my_printf(cCRLF);
 	UUCleanUp();
 }
-
 #else
 
+
 /*
  * Open and read all the files in save[]
  * Scan for uuencode BEGIN lines, decode input as we go along
@@ -894,13 +911,13 @@ post_process_uud(
 	u[0] = '\0';
 
 	for (i = 0; i < num_save; i++) {
-		if ((fp_in = fopen(save[i].path, "r")) == NULL)
+		if ((fp_in = tin_fopen(save[i].path, "r")) == NULL)
 			continue;
 
 		while (fgets(s, (int) sizeof(s), fp_in) != NULL) {
 			switch (state) {
 				case INITIAL:
-					if (strncmp("begin ", s, 6) == 0) {
+					if (STRNCMPEQ("begin ", s, 6)) {
 						char fmt[15];
 						char name[PATH_LEN];
 						char buf[PATH_LEN];
@@ -990,7 +1007,6 @@ post_process_uud(
 
 				my_printf(_(txt_uu_success), filename);
 				my_printf(cCRLF);
-				sum_file(path, filename);
 				if (curr_group->attribute->post_process_view)
 					view_file(path, filename);
 				state = INITIAL;
@@ -1015,47 +1031,6 @@ post_process_uud(
 		my_printf(cCRLF);
 	}
 }
-
-
-/*
- * Sum file - why do we bother to do this?
- * nuke code or add DONT_HAVE_PIPING -tree
- */
-static void
-sum_file(
-	const char *path,
-	const char *file)
-{
-#	if defined(HAVE_SUM) && !defined(DONT_HAVE_PIPING)
-	FILE *fp_in;
-	char *ext;
-	char buf[LEN];
-
-	sh_format(buf, sizeof(buf), "%s \"%s\"", DEFAULT_SUM, path);
-	if ((fp_in = popen(buf, "r")) != NULL) {
-		buf[0] = '\0';
-
-		/*
-		 * You can't do this with (fgets != NULL)
-		 */
-		while (!feof(fp_in)) {
-			fgets(buf, (int) sizeof(buf), fp_in);
-			if ((ext = strchr(buf, '\n')) != NULL)
-				*ext = '\0';
-		}
-		fflush(fp_in);
-		pclose(fp_in);
-
-		my_printf(_(txt_checksum_of_file), file, file_size(path), _("bytes"));
-		my_printf(cCRLF);
-		my_printf("\t%s%s", buf, cCRLF);
-	} else {
-		my_printf(_(txt_command_failed), buf);
-		my_printf(cCRLF);
-	}
-	my_flush();
-#	endif /* HAVE SUM && !DONT_HAVE_PIPING */
-}
 #endif /* HAVE_LIBUU */
 
 
@@ -1102,9 +1077,7 @@ uudecode_line(
 {
 	const char *p = buf;
 	char ch;
-	int n;
-
-	n = DEC(*p);
+	int n = DEC(*p);
 
 	for (++p; n > 0; p += 4, n -= 3) {
 		if (n >= 3) {
@@ -1140,25 +1113,28 @@ post_process_sh(
 {
 	FILE *fp_in, *fp_out = NULL;
 	char buf[LEN];
-	char file_out[PATH_LEN];
+	char *file_out;
 	char file_out_dir[PATH_LEN];
 	int i;
+	size_t len;
 
 	/*
 	 * Grab the dirname portion
 	 */
 	my_strncpy(file_out_dir, save[0].path, (size_t) (save[0].file - save[0].path));
-	snprintf(file_out, sizeof(file_out), "%ssh%ld", file_out_dir, (long) process_id);
+	len = snprintf(NULL, 0, "%ssh%ld", file_out_dir, (long) process_id);
+	file_out = my_malloc(++len);
+	snprintf(file_out, len, "%ssh%ld", file_out_dir, (long) process_id);
 
 	for (i = 0; i < num_save; i++) {
-		if ((fp_in = fopen(save[i].path, "r")) == NULL)
+		if ((fp_in = tin_fopen(save[i].path, "r")) == NULL)
 			continue;
 
 		wait_message(0, _(txt_extracting_shar), save[i].path);
 
 		while (fgets(buf, (int) sizeof(buf), fp_in) != NULL) {
 			/* find #!/bin/sh style patterns */
-			if ((fp_out == NULL) && match_regex_ex(buf, (int) strlen(buf), 0, 0, &shar_regex) >= 0)
+			if ((fp_out == NULL) && match_regex_ex(buf, (REGEX_SIZE) strlen(buf), 0, 0, &shar_regex) >= 0)
 				fp_out = fopen(file_out, "w");
 
 			/* write to temp file */
@@ -1180,6 +1156,7 @@ post_process_sh(
 		invoke_cmd(buf);			/* Handles its own errors */
 		unlink(file_out);
 	}
+	free(file_out);
 }
 
 
@@ -1264,7 +1241,7 @@ decode_save_one(
 	/*
 	 * Decode this message part if appropriate
 	 */
-	if (!(check_save_mime_type(part, curr_group->attribute->mime_types_to_save))) {
+	if (curr_group->attribute->mime_types_to_save && *curr_group->attribute->mime_types_to_save && !(check_save_mime_type(part, BlankIfNull(*curr_group->attribute->mime_types_to_save)))) {
 		/* TODO: skip message if saving multiple files (e.g. save 't'agged) */
 		wait_message(1, "Skipped %s/%s", content_types[part->type], part->subtype);	/* TODO: better msg */
 		return TRUE;
@@ -1284,21 +1261,33 @@ decode_save_one(
 	if (part->encoding == ENCODING_BASE64)
 		mmdecode(NULL, 'b', 0, NULL);				/* flush */
 
-	fseek(rawfp, part->offset, SEEK_SET);
+	if (fseek(rawfp, part->offset, SEEK_SET) == -1) {
+#ifdef DEBUG
+		/*
+		 * TODO: always show to user?
+		 *       then use something less technical and move to lang.c
+		 *
+		 */
+		perror_message("%s:%d decode_save_one(fseek(rawfp,)) failed", __FILE__, __LINE__);
+#endif /* DEBUG */
+		free(savepath);
+		fclose(fp);
+		return FALSE;
+	}
 
 	for (i = 0; i < part->line_count; i++) {
 		if ((fgets(buf, sizeof(buf), rawfp)) == NULL)
 			break;
 
 		/* This should catch cases where people illegally append text etc */
-		if (buf[0] == '\0')
+		if (!*buf)
 			break;
 
 		switch (part->encoding) {
 			case ENCODING_QP:
 			case ENCODING_BASE64:
-				count = mmdecode(buf, part->encoding == ENCODING_QP ? 'q' : 'b', '\0', buf2);
-				fwrite(buf2, (size_t) count, 1, fp);
+				if ((count = mmdecode(buf, part->encoding == ENCODING_QP ? 'q' : 'b', '\0', buf2)) > 0)
+					fwrite(buf2, (size_t) count, 1, fp);
 				break;
 
 			case ENCODING_UUE:
@@ -1395,7 +1384,7 @@ match_content_type(
 		return NO;
 
 	/* Try and match major */
-	if (strcmp(type, "*") == 0)
+	if (STRCMPEQ(type, "*"))
 		found = TRUE;
 	else if (((typeindex = content_type(type)) != -1) && typeindex == part->type)
 		found = TRUE;
@@ -1405,9 +1394,9 @@ match_content_type(
 
 	/* Try and match subtype */
 	found = FALSE;
-	if (strcmp(subtype, "*") == 0)
+	if (STRCMPEQ(subtype, "*"))
 		found = TRUE;
-	else if (strcmp(subtype, part->subtype) == 0)
+	else if (STRCMPEQ(subtype, part->subtype))
 		found = TRUE;
 
 	if (!found)
@@ -1519,9 +1508,10 @@ show_attachment_page(
 	void)
 {
 	char *attach_line;
-	char buf[BUFSIZ];
+	char *buf;
 	const char *charset;
 	int i, tmp_len, max_depth;
+	size_t blen;
 	t_part *part;
 
 	signal_context = cAttachment;
@@ -1532,11 +1522,20 @@ show_attachment_page(
 		attmenu.curr = 0;
 
 	info_len = max_depth = 0;
+
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	buf = my_malloc(MB_CUR_MAX * (size_t) (cCOLS + 1));
+	blen = MB_CUR_MAX * (size_t) cCOLS;
+#	else
+	buf = my_malloc(cCOLS + 1);
+	blen = (size_t) cCOLS;
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 	for (i = 0; i < attmenu.max; ++i) {
 		part = get_part(i);
 		charset = get_param(part->params, "charset");
 		attach_line = build_attach_line(part, 0, cCOLS - 2, 0, NULL, charset);
-		snprintf(buf, sizeof(buf), "  %s", attach_line);
+		snprintf(buf, blen, "  %s", attach_line);
 		FreeIfNeeded(attach_line);
 		tmp_len = strwidth(buf);
 		if (tmp_len > info_len)
@@ -1546,6 +1545,7 @@ show_attachment_page(
 		if (tmp_len > max_depth)
 			max_depth = tmp_len;
 	}
+	free(buf);
 	tmp_len = cCOLS - 13 - MIN((cCOLS - 13) / 2 + 10, max_depth * 2 + 1 + strwidth(_(txt_attachment_no_name)));
 	if (info_len > tmp_len)
 		info_len = tmp_len;
@@ -1670,6 +1670,10 @@ attachment_page(
 				show_attachment_page();
 				break;
 
+			case GLOBAL_VERSION:
+				info_message(cvers);
+				break;
+
 			case ATTACHMENT_SAVE:
 				if (attmenu.max) {
 					part = get_part(attmenu.curr);
@@ -1688,9 +1692,8 @@ attachment_page(
 
 			case ATTACHMENT_TAG:
 				if (attmenu.max) {
-					t_bool tagged;
+					t_bool tagged = tag_part(attmenu.curr);
 
-					tagged = tag_part(attmenu.curr);
 					show_attachment_page();
 					if (attmenu.curr + 1 < attmenu.max)
 						move_down();
@@ -1814,9 +1817,9 @@ build_attachment_line(
 	 * make it the same size like in !USE_CURSES case to simplify some code
 	 */
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		sptr = my_malloc(cCOLS * MB_CUR_MAX + 2);
+		sptr = my_malloc((size_t) cCOLS * MB_CUR_MAX + 2);
 #	else
-		sptr = my_malloc(cCOLS + 2);
+		sptr = my_malloc((size_t) cCOLS + 2);
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #else
 	sptr = screen[INDEX2SNUM(i)].col;
@@ -1981,7 +1984,7 @@ tag_pattern(
 	void)
 {
 	char buf[BUFSIZ];
-	char pat[128];
+	char pat[LEN];
 	char *prompt;
 	const char *name;
 	const char *charset;
@@ -1995,7 +1998,7 @@ tag_pattern(
 #endif /* 0 */
 
 	prompt = fmt_string(_(txt_select_pattern), tinrc.default_select_pattern);
-	if (!(prompt_string_default(prompt, tinrc.default_select_pattern, _(txt_info_no_previous_expression), HIST_SELECT_PATTERN))) {
+	if (!(prompt_string_ptr_default(prompt, &tinrc.default_select_pattern, _(txt_info_no_previous_expression), HIST_SELECT_PATTERN))) {
 		free(prompt);
 		return;
 	}
@@ -2205,6 +2208,8 @@ process_parts(
 			else {
 				savepath = get_tmpfilename(tmppath);
 				free(tmppath);
+				if (!savepath)
+					return;
 			}
 			if ((fp = open_save_filename(savepath, FALSE)) == NULL) {
 				free(savepath);
@@ -2213,6 +2218,7 @@ process_parts(
 			process_part(part, art, fp, savepath, what);
 			break;
 	}
+
 	switch (what) {
 		case SAVE_TAGGED:
 			wait_message(2, _(txt_attachments_saved), saved_parts, num_of_tagged_parts);
@@ -2247,8 +2253,7 @@ process_part(
 {
 	FILE *infile;
 	char buf[2048], buf2[2048];
-	int count;
-	int i, line_count;
+	int count, i, line_count;
 #ifdef CHARSET_CONVERSION
 	char *conv_buf;
 	const char *network_charset;
@@ -2267,7 +2272,17 @@ process_part(
 	if (what != PIPE_RAW && part->encoding == ENCODING_BASE64)
 		mmdecode(NULL, 'b', 0, NULL);				/* flush */
 
-	fseek(infile, part->offset, SEEK_SET);
+	if (fseek(infile, part->offset, SEEK_SET) == -1) {
+#ifdef DEBUG
+		/*
+		 * TODO: always show to user?
+		 *       then use something less technical and move to lang.c
+		 */
+		perror_message("%s:%d process_part(fseek(infile)) failed", __FILE__, __LINE__);
+#endif /* DEBUG */
+		fclose(outfile);
+		return;
+	}
 
 	line_count = part->line_count;
 
@@ -2276,7 +2291,7 @@ process_part(
 			break;
 
 		/* This should catch cases where people illegally append text etc */
-		if (buf[0] == '\0')
+		if (!*buf)
 			break;
 
 		/*
@@ -2292,15 +2307,13 @@ process_part(
 			switch (part->encoding) {
 				case ENCODING_QP:
 				case ENCODING_BASE64:
-#ifdef CHARSET_CONVERSION
 					memset(buf2, '\0', sizeof(buf2));
-#endif /* CHARSET_CONVERSION */
 					if ((count = mmdecode(buf, part->encoding == ENCODING_QP ? 'q' : 'b', '\0', buf2)) > 0) {
 #ifdef CHARSET_CONVERSION
-						if (what != SAVE && what != SAVE_TAGGED && !strncmp(content_types[part->type], "text", 4)) {
+						if (what != SAVE && what != SAVE_TAGGED && part->type == TYPE_TEXT) {
+							network_charset = validate_charset(get_param(part->params, "charset"));
 							line_len = (size_t) count;
 							conv_buf = my_strdup(buf2);
-							network_charset = validate_charset(get_param(part->params, "charset"));
 							process_charsets(&conv_buf, &line_len, network_charset ? network_charset : "US-ASCII", tinrc.mm_local_charset, FALSE);
 							strncpy(buf2, conv_buf, sizeof(buf2) - 1);
 							count = (int) strlen(buf2);
@@ -2312,9 +2325,12 @@ process_part(
 					break;
 
 				case ENCODING_UUE:
-					/* TODO: if postproc, don't decode these since the traditional uudecoder will get them */
 					/*
-					 * x-uuencode attachments have all the header info etc which we must ignore
+					 * TODO: if postproc, don't decode these since the
+					 *       traditional uudecoder will get them
+					 *
+					 * x-uuencode attachments have all the header info etc
+					 * which we must ignore
 					 */
 					if (strncmp(buf, "begin ", 6) != 0 && strncmp(buf, "end\n", 4) != 0 && buf[0] != '\n')
 						uudecode_line(buf, outfile);
@@ -2322,7 +2338,7 @@ process_part(
 
 				default:
 #ifdef CHARSET_CONVERSION
-						if (what != SAVE && what != SAVE_TAGGED && !strncmp(content_types[part->type], "text", 4)) {
+						if (what != SAVE && what != SAVE_TAGGED && part->type == TYPE_TEXT) {
 							conv_buf = my_strdup(buf);
 							line_len = strlen(conv_buf);
 							network_charset = validate_charset(get_param(part->params, "charset"));
@@ -2366,13 +2382,12 @@ pipe_part(
 	char *prompt;
 
 	prompt = fmt_string(_(txt_pipe_to_command), (size_t) cCOLS - (strlen(_(txt_pipe_to_command)) + 30), tinrc.default_pipe_command);
-	if (!(prompt_string_default(prompt, tinrc.default_pipe_command, _(txt_no_command), HIST_PIPE_COMMAND))) {
+	if (!(prompt_string_ptr_default(prompt, &tinrc.default_pipe_command, _(txt_no_command), HIST_PIPE_COMMAND))) {
 		free(prompt);
 		return;
 	}
 	free(prompt);
-	if ((fp = fopen(savepath, "r")) == NULL)
-		/* TODO: error message? */
+	if ((fp = tin_fopen(savepath, "r")) == NULL) /* TODO: error message? */
 		return;
 	EndWin();
 	Raw(FALSE);
