@@ -3,7 +3,7 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2022-11-01
+ *  Updated   : 2024-10-19
  *  Notes     :
  *
  * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -142,6 +142,7 @@ wait_message(
 	...)
 {
 	char *buf, *msg;
+	size_t len = cCOLS - 1;
 	va_list ap;
 
 	va_start(ap, fmt);
@@ -154,17 +155,21 @@ wait_message(
 	buf = fmt_message(fmt, ap);
 	/* test for multiline messages */
 	if (strrchr(buf, '\n')) {
-		char *from, *to;
+		char *from, *to, *tmp;
 
 		for (from = buf; *from && (to = strchr(from, '\n')); from = ++to) {
 			*to = '\0';
-			msg = strunc(from, cCOLS - 1);
+			/* expand tabs in multiline msg, as we wrap anyway */
+			tmp = expand_tab(from, tabwidth);
+			/* and strunc would replace them */
+			msg = strunc(tmp, len);
+			free(tmp);
 			my_fputs(msg, stdout);
 			my_fputc('\n', stdout);
 			free(msg);
 		}
 	} else {
-		msg = strunc(buf, cCOLS - 1);
+		msg = strunc(buf, len);
 		my_fputs(msg, stdout);
 		free(msg);
 	}
@@ -209,12 +214,13 @@ wait_message(
 		}
 
 		if (nfds > 0) {
-			if (FD_ISSET(STDIN_FILENO, &readfds))
+			if (FD_ISSET(STDIN_FILENO, &readfds)) {
 #	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				ReadWch();
 #	else
 				ReadCh();
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+			}
 		}
 	}
 #else
@@ -310,9 +316,7 @@ center_line(
 {
 	char *ln;
 	int pos;
-	int len;
-
-	len = strwidth(str);
+	int len = strwidth(str);
 
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	if (tinrc.render_bidi && IS_LOCAL_CHARSET("UTF-8") && len > 1) {
@@ -337,14 +341,15 @@ center_line(
 		}
 	}
 
-	if (len >= cCOLS) {
-		char *buffer;
+    if (len >= cCOLS) {
+		char *buffer = strunc(ln, cCOLS - 2);
 
-		buffer = strunc(ln, cCOLS - 2);
-		my_fputs(buffer, stdout);
-		free(buffer);
-	} else
-		my_fputs(ln, stdout);
+		if (buffer != ln)
+			free(ln);
+
+		ln = buffer;
+	}
+	my_fputs(ln, stdout);
 
 	if (cmd_line)
 		my_flush();
@@ -375,12 +380,20 @@ draw_arrow_mark(
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			my_fputs("->", stdout);
 	} else {
+		char *s;
 #ifdef USE_CURSES
-		char buffer[BUFSIZ];
-		char *s = screen_contents(line, 0, buffer);
+		char *buffer;
+
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		buffer = my_malloc(MB_CUR_MAX * (size_t) (cCOLS + 1));
+#	else
+		buffer = my_malloc(cCOLS + 1);
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		s = screen_contents(line, 0, buffer);
 #else
-		char *s = screen[line - INDEX_TOP].col;
+		s = screen[line - INDEX_TOP].col;
 #endif /* USE_CURSES */
+
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		if ((wtmp = char2wchar_t(s)) != NULL) {
 			StartInverse();
@@ -403,6 +416,9 @@ draw_arrow_mark(
 			my_fputc(s[mark_offset], stdout);
 		}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#ifdef USE_CURSES
+		free(buffer);
+#endif /* USE_CURSES */
 	}
 	stow_cursor();
 }
@@ -425,11 +441,18 @@ erase_arrow(
 	if (tinrc.draw_arrow)
 		my_fputs("  ", stdout);
 	else {
-#ifdef USE_CURSES
-		char buffer[BUFSIZ];
-		char *s = screen_contents(line, 0, buffer);
-#else
 		char *s;
+#ifdef USE_CURSES
+		char *buffer;
+
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+		buffer = my_malloc(MB_CUR_MAX * (size_t) (cCOLS + 1));
+#	else
+		buffer = my_malloc(cCOLS + 1);
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+		s = screen_contents(line, 0, buffer);
+#else
+		s = screen[line - INDEX_TOP].col;
 
 		if (line - INDEX_TOP < 0) /* avoid underruns */
 			line = INDEX_TOP;
@@ -457,6 +480,9 @@ erase_arrow(
 			EndInverse();
 		}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+#ifdef USE_CURSES
+		free(buffer);
+#endif /* USE_CURSES */
 	}
 }
 
@@ -468,7 +494,7 @@ show_title(
 	char keyhelp[MAXKEYLEN];
 	char *helps;
 	const char *sign;
-	int col;
+	size_t col;
 
 	if (tinrc.show_help_mail_sign != SHOW_SIGN_NONE) {
 		col = MAXKEYLEN + strlen(_(txt_type_h_for_help)) + 1;
@@ -508,9 +534,9 @@ show_title(
 		}
 
 		if (sign) {
-			col = cCOLS - strwidth(sign);
-			if (col > 0) {
-				MoveCursor(0, col);
+			if (cCOLS > strwidth(sign)) {
+				col = cCOLS - strwidth(sign);
+				MoveCursor(0, (int) col);
 #ifdef HAVE_COLOR
 				fcol(tinrc.col_title);
 #endif /* HAVE_COLOR */
@@ -789,8 +815,9 @@ log_formatted_msg(
 	const char *tag,
 	const char *msg)
 {
+	size_t len;
 
-	if (msglog == NULL || msg == NULL || strlen(msg) == 0)
+	if (msglog == NULL || msg == NULL || (len = strlen(msg)) == 0)
 		return;
 
 	if (tag)
@@ -798,7 +825,7 @@ log_formatted_msg(
 	else
 		fprintf(msglog, "%s", msg);
 
-	if (msg[strlen(msg) - 1] != '\n')
+	if (msg[len - 1] != '\n')
 		fputc('\n', msglog);
 
 	fflush(msglog);
