@@ -3,7 +3,7 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2023-11-15
+ *  Updated   : 2024-04-19
  *  Notes     :
  *
  * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -107,7 +107,8 @@ select_right(
 void
 selection_page(
 	int start_groupnum,
-	int num_cmd_line_groups)
+	int num_cmd_line_groups,
+	char *messageid)
 {
 	char buf[LEN];
 	char key[MAXKEYLEN];
@@ -126,6 +127,22 @@ selection_page(
 	Raw(TRUE);
 	ClearScreen();
 
+#ifdef NNTP_ABLE
+	if (cmdline.args & CMDLINE_MSGID) { /* -L cmd */
+		switch (show_article_by_msgid(messageid)) {
+			case LOOKUP_ART_UNAVAIL:
+				wait_message(2, _(txt_art_unavailable));
+				break;
+
+			default:
+				break;
+		}
+		cmdline.args ^= CMDLINE_MSGID; /* clear flag */
+	}
+#else
+	(void) messageid;
+#endif /* NNTP_ABLE */
+
 	/*
 	 * If user specified only 1 cmd line groupname (eg. tin -r alt.sources)
 	 * then go there immediately.
@@ -134,6 +151,7 @@ selection_page(
 		select_read_group();
 
 	cursoroff();
+
 	show_selection_page();	/* display group selection page */
 
 	forever {
@@ -616,9 +634,9 @@ show_selection_page(
 
 	if (use_nntps) {
 		if (insecure_nntps)
-			secflag=_(txt_selection_flag_insecure);
+			secflag = _(txt_selection_flag_insecure);
 		else
-			secflag=_(txt_selection_flag_secure);
+			secflag = _(txt_selection_flag_secure);
 	}
 
 	if (read_news_via_nntp)
@@ -861,7 +879,7 @@ build_gline(
 					buf = sptr + strlen(sptr);
 					for (j = 1; j <= sel_fmt.len_ucnt - (art_mark_width - (art_mark_width - wcwidth(tinrc.art_marked_inrange))); ++j)
 						*buf++ = ' ';
-					snprintf(tmp_buf, sizeof(tmp_buf), "%"T_CHAR_FMT, tinrc.art_marked_inrange);
+					snprintf(tmp_buf, sizeof(tmp_buf), "%"T_CHAR_FMT, (wint_t) tinrc.art_marked_inrange);
 					*buf-- = '\0';
 					strcat(buf, tmp_buf);
 #else
@@ -1141,7 +1159,7 @@ reposition_group(
 		return default_num;
 
 	if (*pos)
-		pos_num = ((pos[0] == '$') ? selmenu.max : atoi(pos));
+		pos_num = ((pos[0] == '$') ? selmenu.max : s2i(pos, 1, selmenu.max));
 	else {
 		if (tinrc.default_move_group)
 			pos_num = tinrc.default_move_group;
@@ -1657,6 +1675,7 @@ show_article_by_msgid(
 	char id[NNTP_STRLEN];	/* still way too big; RFC 3977 3.6 & RFC 5536 3.1.3 limit Message-ID to max 250 octets */
 	char *idptr = NULL;
 	char *newsgroups = NULL;
+	char *ngcpy;
 	int i, ret = 0;
 	struct t_article *art;
 	struct t_group *group = NULL;
@@ -1669,28 +1688,36 @@ show_article_by_msgid(
 		return LOOKUP_UNAVAIL;
 
 	if (messageid) {
-		idptr = messageid;
-		newsgroups = lookup_msgid(idptr);
-	} else {
-		if (prompt_string(_(txt_enter_message_id), id + 1, HIST_MESSAGE_ID) && id[1]) {
+		/* sizeof(id) - 2 to have space for '>' later on if necessary */
+		if (snprintf(id + 1, sizeof(id) - 2, "%s", messageid) > 0)
 			idptr = str_trim(id + 1);
-			if (id[1] != '<') {
-				id[0] = '<';
-				strcat(id, ">");
-				idptr = id;
-			}
-			newsgroups = lookup_msgid(idptr);
+	} else {
+		if (prompt_string(_(txt_enter_message_id), id + 1, HIST_MESSAGE_ID) && id[1])
+			idptr = str_trim(id + 1);
+	}
+
+	if (idptr) {
+		if (*idptr != '<') {
+			*(--idptr) = '<';
+			strcat(idptr, ">");
 		}
+		newsgroups = lookup_msgid(idptr);
 	}
 
 	if (!newsgroups)
 		return LOOKUP_ART_UNAVAIL;
 
+	ngcpy = my_strdup(newsgroups); /* take a copy as get_group_from_list() modifies newsgroups */
 	if ((group = get_group_from_list(newsgroups)) == NULL) {
-		info_message(strchr(newsgroups, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), newsgroups);
+		if (!(cmdline.args & CMDLINE_MSGID))
+			info_message(strchr(ngcpy, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), ngcpy);
+		else /* -L cmd. */
+			wait_message(2, strchr(ngcpy, ',') ? _(txt_lookup_show_groups) : _(txt_lookup_show_group), ngcpy);
 		free(newsgroups);
+		free(ngcpy);
 		return LOOKUP_FAILED;
 	}
+	free(ngcpy);
 
 	if (curr_group)
 		tmp_group = curr_group;
