@@ -3,7 +3,7 @@
  *  Module    : sigfile.c
  *  Author    : M. Gleason & I. Lea
  *  Created   : 1992-10-17
- *  Updated   : 2021-07-26
+ *  Updated   : 2024-09-07
  *  Notes     : Generate random signature for posting/mailing etc.
  *
  * Copyright (c) 1992-2024 Mike Gleason
@@ -70,7 +70,7 @@ msg_write_signature(
 #endif /* NNTP_INEWS */
 
 	if (thisgroup && !thisgroup->bogus) {
-		if (!strcmp(thisgroup->attribute->sigfile, "--none"))
+		if (!strcmp(*thisgroup->attribute->sigfile, "--none"))
 			return;
 
 		/*
@@ -78,17 +78,17 @@ msg_write_signature(
 		 *       use strfpath()?
 		 */
 #ifndef DONT_HAVE_PIPING
-		if (thisgroup->attribute->sigfile[0] == '!') {
+		if (*thisgroup->attribute->sigfile[0] == '!') {
 			FILE *pipe_fp;
 			char *sigcmd, *sigattr, *ptr;
 			char cmd[PATH_LEN];
 
 			fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "");
-			sigattr = thisgroup->attribute->sigfile + 1;
+			sigattr = *thisgroup->attribute->sigfile + 1;
 
 			if ((ptr = strstr(sigattr, "%G"))) {
 				char *to, *grpname;
-				int cnt = 1;
+				size_t cnt = 1;
 
 				/* check if %G occurs more than once */
 				while (strstr(++ptr, "%G"))
@@ -117,7 +117,7 @@ msg_write_signature(
 			}
 
 			if ((pipe_fp = popen(sigcmd, "r")) != NULL) {
-				while (fgets(cmd, PATH_LEN, pipe_fp))
+				while (fgets(cmd, sizeof(cmd), pipe_fp))
 					fputs(cmd, fp);
 				pclose(pipe_fp);
 			} /* else issue an error-message? */
@@ -128,7 +128,7 @@ msg_write_signature(
 #endif /* !DONT_HAVE_PIPING */
 		get_cwd(cwd);
 
-		if (!strfpath(thisgroup->attribute->sigfile, path, sizeof(path), thisgroup, FALSE)) {
+		if (!strfpath(*thisgroup->attribute->sigfile, path, sizeof(path), thisgroup, FALSE)) {
 			if (!strfpath(tinrc.sigfile, path, sizeof(path), thisgroup, FALSE))
 				joinpath(path, sizeof(path), homedir, ".Sig");
 		}
@@ -138,7 +138,7 @@ msg_write_signature(
 		 * generate a random signature from sigs in sigdir. If
 		 * the file path/.sigfixed or ~/.sigfixed exists (fixed
 		 * part of random sig) then read it in first and append
-		 * the random sig part onto the end.
+		 * the random sig part to the end.
 		 */
 		if ((sigfp = open_random_sig(path)) != NULL) {
 #ifdef DEBUG
@@ -151,7 +151,7 @@ msg_write_signature(
 			if (debug & DEBUG_MISC)
 				error_message(2, "TRYING fixed sig=[%s]", pathfixed);
 #endif /* DEBUG */
-			if ((fixfp = fopen(pathfixed, "r")) != NULL) {
+			if ((fixfp = tin_fopen(pathfixed, "r")) != NULL) {
 				copy_fp(fixfp, fp);
 				fclose(fixfp);
 			} else {
@@ -160,7 +160,7 @@ msg_write_signature(
 				if (debug & DEBUG_MISC)
 					error_message(2, "TRYING fixed sig=[%s]", pathfixed);
 #endif /* DEBUG */
-				if ((fixfp = fopen(pathfixed, "r")) != NULL) {
+				if ((fixfp = tin_fopen(pathfixed, "r")) != NULL) {
 					copy_fp(fixfp, fp);
 					fclose(fixfp);
 				}
@@ -169,15 +169,14 @@ msg_write_signature(
 			fclose(sigfp);
 			if (chdir(cwd) == -1) {
 #ifdef DEBUG
-				int e = errno;
 				if (debug & DEBUG_MISC)
-					error_message(2, "chdir(%s): Error: %s", cwd, strerror(e));
+					perror_message("chdir(%s)", cwd);
 #endif /* DEBUG */
 			}
 			return;
 		}
 
-		if ((sigfp = fopen(path, "r")) != NULL) {
+		if ((sigfp = tin_fopen(path, "r")) != NULL) {
 			fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "");
 			copy_fp(sigfp, fp);
 			fclose(sigfp);
@@ -188,7 +187,7 @@ msg_write_signature(
 		 * Use ~/.signature as a last resort, but only if mailing or
 		 * using internal inews (external inews appends it automagically).
 		 */
-		if ((sigfp = fopen(default_signature, "r")) != NULL) {
+		if ((sigfp = tin_fopen(default_signature, "r")) != NULL) {
 			if (include_dot_signature) {
 				fprintf(fp, "\n%s", thisgroup->attribute->sigdashes ? SIGDASHES : "");
 				copy_fp(sigfp, fp);
@@ -203,7 +202,7 @@ static FILE *
 open_random_sig(
 	char *sigdir)
 {
-	srand((unsigned int) time(NULL));
+	srndm();
 
 	if (chdir(sigdir) == 0) {
 		if (thrashdir(sigdir) || !*sigfile) {
@@ -277,7 +276,7 @@ thrashdir(
 			return 1;
 		}
 #endif /* HAVE_REWINDDIR */
-		pick = rand() % numentries + 1;
+		pick = rndm() % numentries + 1;
 		while (--pick >= 0) {
 			if ((dp = readdir(dirp)) == NULL)
 				break;
@@ -309,9 +308,8 @@ thrashdir(
 							dp = NULL;
 							if (chdir(cwd) == -1) {
 #ifdef DEBUG
-								int e = errno;
 								if (debug & DEBUG_MISC)
-									error_message(2, "chdir(%s): Error: %s", cwd, strerror(e));
+									perror_message("chdir(%s)", cwd);
 #endif /* DEBUG */
 							}
 						}
