@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2023-06-22
+ *  Updated   : 2023-11-12
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2023 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2024 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -428,11 +428,38 @@ signal_handler(
  */
 #if defined(HAVE_ALARM) && defined(SIGALRM)
 		case SIGALRM:
+#	ifdef NNTP_ABLE
 #	ifdef DEBUG
 			if ((debug & DEBUG_NNTP) && verbose > 1)
-				debug_print_file("NNTP", "get_server() %d sec elapsed without response", tinrc.nntp_read_timeout_secs);
+				debug_print_file("NNTP", "get_server() %d sec elapsed without response", TIN_NNTP_TIMEOUT);
 #	endif /* DEBUG */
-			tin_done(NNTP_ERROR_EXIT, _("NNTP connection error. Exiting..."));
+
+#	ifdef USE_ZLIB
+			/*
+			 * response compression from the server may take a while
+			 * when running interactively and not being in connection
+			 * phase give the user a chance to go on instead of exiting
+			 */
+			{
+				char *prompt;
+				size_t len;
+
+				len = strlen(_(txt_read_timeout_quit)) + snprintf(NULL, 0, "%d", tinrc.nntp_read_timeout_secs) - 1;
+				prompt = my_malloc(len);
+				snprintf(prompt, len, _(txt_read_timeout_quit), tinrc.nntp_read_timeout_secs);
+				if (signal_context == cReconnect || batch_mode || !use_compress || !nntp_caps.compress || prompt_yn(prompt, FALSE) == 1) {
+					free(prompt);
+#	endif /* USE_ZLIB */
+					tin_done(NNTP_ERROR_EXIT, _(txt_connection_error));
+#	ifdef USE_ZLIB
+				} else {
+					free(prompt);
+					RESTORE_HANDLER(sig, signal_handler);
+					wait_message(0, _(txt_continuing));
+				}
+			}
+#	endif /* USE_ZLIB */
+#	endif /* NNTP_ABLE */
 			return;
 #endif /* HAVE_ALARM && SIGALRM */
 
