@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2023-08-03
+ *  Updated   : 2024-10-28
  *  Notes     :
  *
- * Copyright (c) 1997-2023 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2024 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,13 +61,6 @@
  * this file needs some work
  */
 
-/*
- * local prototypes
- */
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	static wchar_t *my_wcsdup(const wchar_t *wstr);
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-
 
 /*
  * special ltoa()
@@ -79,35 +72,36 @@
 char *
 tin_ltoa(
 	t_artnum value,
-	int digits)
+	size_t digits)
 {
 	static char buffer[64];
-	static const char power[] = { ' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', '\0' };
-	int len;
+	static const char power[] = { 'e', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', '\0' };
+	size_t len;
 	size_t i = 0;
 
-	if (digits <= 0) {
-		buffer[0] = 'e';
-		return buffer;
-	}
+	if (digits >= sizeof(buffer))
+		digits = sizeof(buffer) - 1;
+
+	if (!digits)
+		digits++;
 
 	snprintf(buffer, sizeof(buffer), "%"T_ARTNUM_PFMT, value);
-	len = (int) strlen(buffer);
+	len = strlen(buffer);
 
 	/*
 	 * only shorten if necessary,
 	 * then ensure that the metric prefix fits into the buffer
 	 */
-	if (len > digits) {
+	if (len > digits && digits >= 3) {
 		while (len >= digits) {
 			len -= 3;
 			i++;
 		}
 	}
 
-	if (i >= strlen(power)) {	/* buffer is to small */
-		buffer[(digits & 0x7f) - 1] = 'e';
-		buffer[(digits & 0x7f)] = '\0';
+	if (i >= strlen(power) || len > digits) {	/* buffer is to small */
+		buffer[digits - 1] = power[0];
+		buffer[digits] = '\0';
 		return buffer;
 	}
 
@@ -118,11 +112,11 @@ tin_ltoa(
 		if (digits > len)
 			buffer[digits - 1] = power[i];
 		else /* overflow */
-			buffer[digits - 1] = 'e';
+			buffer[digits - 1] = power[0];
 
 		buffer[digits] = '\0';
 	} else
-		snprintf(buffer, sizeof(buffer), "%*"T_ARTNUM_PFMT, digits, value);
+		snprintf(buffer, sizeof(buffer), "%*"T_ARTNUM_PFMT, (int) digits, value);
 
 	return buffer;
 }
@@ -139,15 +133,20 @@ char *
 my_strdup(
 	const char *str)
 {
-	size_t len = strlen(str) + 1;
-	void *ptr = my_malloc(len);
+	size_t len = 1;
+	void *ptr;
+
+	if (str)
+		len += strlen(str);
+
+	ptr = my_malloc(len);
 
 #	if 0 /* as my_malloc exits on error, ptr can't be NULL */
 	if (ptr == NULL)
 		return NULL;
 #	endif /* 0 */
 
-	memcpy(ptr, str, len);
+	memcpy(ptr, BlankIfNull(str), len);
 	return (char *) ptr;
 }
 #endif /* !USE_DMALLOC || (USE_DMALLOC && !HAVE_STRDUP) */
@@ -237,6 +236,7 @@ strcasestr(
 #endif /* !HAVE_STRCASESTR */
 
 
+/* strcat() returning the number of appended octets */
 size_t
 mystrcat(
 	char **t,
@@ -287,6 +287,9 @@ my_toupper(
 }
 
 
+/*
+ * TODO: add a wide char variant "wchar *wstr_lwr(wchar *wc)"
+ */
 void
 str_lwr(
 	char *str)
@@ -399,7 +402,7 @@ atol(
 		s++;
 
 	while (*s) {
-		if (isdigit(*s))
+		if (isdigit((unsigned char) *s))
 			ret = ret * 10 + (*s - '0');
 		else
 			return -1;
@@ -410,7 +413,6 @@ atol(
 #endif /* !HAVE_ATOL */
 
 #ifndef HAVE_STRTOL
-/* fix me - put me in tin.h */
 #	define DIGIT(x) (isdigit((unsigned char) x) ? ((x) - '0') : (10 + my_tolower((unsigned char) x) - 'a'))
 #	define MBASE 36
 long
@@ -422,15 +424,20 @@ strtol(
 	long val = 0L;
 	int xx = 0, sign = 1;
 
-	if (use_base < 0 || use_base > MBASE)
+	if (use_base < 0 || use_base == 1 || use_base > MBASE) {
+		errno = EINVAL;
 		goto OUT;
+	}
+
 	while (isspace((unsigned char) *str))
 		++str;
+
 	if (*str == '-') {
 		++str;
 		sign = -1;
 	} else if (*str == '+')
 		++str;
+
 	if (use_base == 0) {
 		if (*str == '0') {
 			++str;
@@ -453,6 +460,7 @@ strtol(
 		val = use_base * val - xx;
 		++str;
 	}
+
 OUT:
 	if (ptr != NULL)
 		*ptr = str;
@@ -464,7 +472,6 @@ OUT:
 #endif /* !HAVE_STRTOL */
 
 #if !defined(HAVE_STRCASECMP) || !defined(HAVE_STRNCASECMP)
-	/* fix me - put me in tin.h */
 #	define FOLD_TO_UPPER(a)	(my_toupper((unsigned char) (a)))
 #endif /* !HAVE_STRCASECMP || !HAVE_STRNCASECMP */
 /*
@@ -477,6 +484,7 @@ strcasecmp(
 	const char *q)
 {
 	int r;
+
 	for (; (r = FOLD_TO_UPPER (*p) - FOLD_TO_UPPER (*q)) == 0; ++p, ++q) {
 		if (*p == '\0')
 			return 0;
@@ -550,12 +558,12 @@ str_trim(
 
 	/* remove training spaces */
 	ep = string + s - 1;
-    while (ep >= string && isspace((int) *ep))
-        ep--;
-    *(ep + 1) = '\0';
+	while (ep >= string && isspace((unsigned char) *ep))
+		ep--;
+	*(ep + 1) = '\0';
 
 	/* skip leading space */
-	for (rp = wp = string; isspace((int) *rp); rp++)
+	for (rp = wp = string; isspace((unsigned char) *rp); rp++)
 		;
 
 	/* copy if required to keep address */
@@ -624,9 +632,7 @@ wexpand_tab(
 
 	return wbuf;
 }
-
-
-#else /* !MULTIBYTE_ABLE || NO_LOCALE */
+#endif /* !MULTIBYTE_ABLE || NO_LOCALE */
 
 
 char *
@@ -634,8 +640,24 @@ expand_tab(
 	char *str,
 	size_t tab_width)
 {
+	char *buf = my_calloc(1, LEN);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wchar_t *wmessage, *wbuf;
+
+	if ((wmessage = char2wchar_t(str)) != NULL) {
+		if ((wbuf = wexpand_tab(wmessage, tab_width))) {
+			free(buf);
+			if ((buf = wchar_t2char(wbuf)) != NULL) {
+				free(wbuf);
+				free(wmessage);
+				return buf;
+			}
+			free(wbuf);
+		}
+		free(wmessage);
+	}
+#else
 	size_t cur_len = LEN, i = 0, tw;
-	char *buf = my_malloc(cur_len);
 	char *c = str;
 
 	while (*c) {
@@ -652,10 +674,10 @@ expand_tab(
 		c++;
 	}
 	buf[i] = '\0';
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	return buf;
 }
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 
 /*
@@ -692,9 +714,7 @@ sh_format(
 	va_start(ap, fmt);
 
 	while (*fmt != 0) {
-		int ch;
-
-		ch = *fmt++;
+		int ch = *fmt++;
 
 		if (ch == '\\') {
 			SH_FORMAT(ch);
@@ -730,11 +750,6 @@ sh_format(
 			while (*src != '\0') {
 				t_bool fix;
 
-				/*
-				 * This logic works for Unix. Non-Unix systems may require a
-				 * different set of problem chars, and may need quotes around
-				 * the whole string rather than escaping individual chars.
-				 */
 				if (quote == '"') {
 					fix = (strchr(SH_DOUBLE, *src) != NULL);
 				} else if (quote == '\'') {
@@ -807,7 +822,7 @@ strrstr(
 
 		if ((plen != 0) && (plen <= slen)) {
 			for (ptr = str + (slen - plen); ptr > str; --ptr) {
-				if (*ptr == *pat && strncmp(ptr, pat, plen) == 0)
+				if (*ptr == *pat && STRNCMPEQ(ptr, pat, plen))
 					return (char *) ptr;
 			}
 		}
@@ -841,8 +856,10 @@ char2wchar_t(
 		return NULL;
 	}
 
-	wstr = my_calloc(1, sizeof(wchar_t) * (len + 1));
-	mbstowcs(wstr, test, len + 1);
+	/* if ((len = mbstowcs(NULL, test, 0) == (size_t) (-1)) { free(test); return NULL; } */
+	wstr = my_calloc(len + 1, sizeof(wchar_t));
+	/* pos = */ mbstowcs(wstr, test, len);
+	/* wstr[pos == (size_t) (-1) ? 0 : pos] = '\0'; */
 	free(test);
 
 	return wstr;
@@ -857,9 +874,8 @@ wchar_t2char(
 	const wchar_t *wstr)
 {
 	char *str;
-	size_t len;
+	size_t len = wcstombs(NULL, wstr, 0);
 
-	len = wcstombs(NULL, wstr, 0);
 	if (len == (size_t) (-1))
 		return NULL;
 
@@ -876,7 +892,7 @@ wchar_t2char(
 char *
 spart(
 	const char *str,
-	int columns,
+	size_t columns,
 	t_bool pad)
 {
 	char *buf = NULL;
@@ -900,10 +916,10 @@ spart(
 wchar_t *
 wcspart(
 	const wchar_t *wstr,
-	int columns,
+	size_t columns,
 	t_bool pad)
 {
-	int used = 0;
+	size_t used = 0;
 	wchar_t *ptr, *wbuf;
 
 	wbuf = my_wcsdup(wstr);
@@ -917,11 +933,9 @@ wcspart(
 
 	/* pad with spaces */
 	if (pad) {
-		int gap;
+		size_t gap = columns - used;
 
-		gap = columns - wcswidth(wbuf, wcslen(wbuf) + 1);
-		assert(gap >= 0);
-		wbuf = my_realloc(wbuf, sizeof(wchar_t) * (wcslen(wbuf) + (size_t) gap + 1));
+		wbuf = my_realloc(wbuf, sizeof(wchar_t) * (wcslen(wbuf) + (size_t) (gap + 1)));
 		ptr = wbuf + wcslen(wbuf); /* set ptr again to end of wbuf */
 
 		while (gap-- > 0)
@@ -948,7 +962,7 @@ abbr_wcsgroupname(
 
 	dest = new_grpname = my_wcsdup(grpname);
 
-	if (wcswidth(grpname, wcslen(grpname)) > len) {
+	if (len > 0 && wcswidth(grpname, wcslen(grpname)) > len) {
 		if ((src = wcschr(grpname, (wchar_t) '.')) != NULL) {
 			tmplen = wcwidth(*dest++);
 
@@ -985,9 +999,9 @@ abbr_wcsgroupname(
 
 	return new_grpname;
 }
-
 #else /* !MULTIBYTE_ABLE || NO_LOCALE */
 
+
 /*
  * Abbreviate a groupname like this:
  * 	foo.bar.baz
@@ -1068,7 +1082,6 @@ strwidth(
 }
 
 
-#define TRUNC_TAIL	"..."
 /*
  * shortens 'mesg' that it occupies at most 'len' screen positions.
  * If it was necessary to truncate 'mesg', " ..." is appended to the
@@ -1078,13 +1091,13 @@ strwidth(
 char *
 strunc(
 	const char *message,
-	int len)
+	size_t len)
 {
 	char *tmp;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wmessage, *wbuf;
 
-	if ((wmessage = char2wchar_t(message)) != NULL) {
+	if ((wmessage = char2wchar_t(BlankIfNull(message))) != NULL) {
 		wbuf = wstrunc(wmessage, len);
 		free(wmessage);
 
@@ -1098,11 +1111,11 @@ strunc(
 	/* something went wrong using wide-chars, default back to normal chars */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-	if ((int) strlen(message) <= len)
-		tmp = my_strdup(message);
+	if (strlen(BlankIfNull(message)) <= len)
+		tmp = my_strdup(BlankIfNull(message));
 	else {
 		tmp = my_malloc(len + 1);
-		snprintf(tmp, (size_t) (len + 1), "%-.*s%s", len - 3, message, TRUNC_TAIL);
+		snprintf(tmp, len + 1, "%-.*s%s", (int) (len - 3), message, TRUNC_TAIL);
 	}
 
 	return tmp;
@@ -1114,7 +1127,7 @@ strunc(
 wchar_t *
 wstrunc(
 	const wchar_t *wmessage,
-	int len)
+	size_t len)
 {
 	wchar_t *wtmp;
 
@@ -1122,29 +1135,33 @@ wstrunc(
 	wtmp = my_wcsdup(wmessage);
 	wconvert_to_printable(wtmp, FALSE);
 
-	if (wcswidth(wtmp, wcslen(wtmp)) > len) {
+	if (wcswidth(wtmp, wcslen(wtmp)) > (int) len) {
 		/* wtmp must be truncated */
 		size_t len_tail;
 		wchar_t *wtmp2, *tail;
 
-		if (tinrc.utf8_graphics) {
-			/*
-			 * use U+2026 (HORIZONTAL ELLIPSIS) instead of "..."
-			 * we gain two additional screen positions
-			 */
-			tail = my_calloc(2, sizeof(wchar_t));
-			tail[0] = 8230; /* U+2026 */
-		} else
+		if (tinrc.utf8_graphics)
+			tail = my_wcsdup(WTRUNC_TAIL);
+		else
 			tail = char2wchar_t(TRUNC_TAIL);
 
 		len_tail = tail ? wcslen(tail) : 0;
-		wtmp2 = wcspart(wtmp, (int) ((size_t) len - len_tail), FALSE);
+		if (len_tail > len) {
+			FreeAndNull(tail);
+			len_tail = 0;
+		}
+		wtmp2 = wcspart(wtmp, len - len_tail, FALSE);
 		free(wtmp);
-		wtmp = my_realloc(wtmp2, sizeof(wchar_t) * (wcslen(wtmp2) + len_tail + 1));	/* wtmp2 isn't valid anymore and doesn't have to be free()ed */
-		if (!tail)
-			tail = my_calloc(1, sizeof(wchar_t));
-		wcscat(wtmp, tail);
-		free(tail);
+
+		if (wtmp2)
+			wtmp = my_realloc(wtmp2, sizeof(wchar_t) * (wcslen(wtmp2) + len_tail + 1));	/* wtmp2 isn't valid anymore and doesn't have to be free()ed */
+		else
+			wtmp = my_calloc(len_tail + 1, sizeof(wchar_t));
+
+		if (tail && *tail)
+			wcscat(wtmp, tail);
+
+		FreeIfNeeded(tail);
 	}
 
 	return wtmp;
@@ -1154,14 +1171,19 @@ wstrunc(
 /*
  * duplicates a wide-char string
  */
-static wchar_t *
+wchar_t *
 my_wcsdup(
 	const wchar_t *wstr)
 {
-	size_t len = wcslen(wstr) + 1;
-	void *ptr = my_malloc(sizeof(wchar_t) * len);
+	size_t len = 1;
+	void *ptr;
 
-	memcpy(ptr, wstr, sizeof(wchar_t) * len);
+	if (wstr)
+		len += wcslen(wstr);
+
+	ptr = (wchar_t *) my_calloc(len, sizeof(wchar_t));
+	if (wstr && *wstr && len > 1)
+		memcpy(ptr, wstr, sizeof(wchar_t) * len);
 	return (wchar_t *) ptr;
 }
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -1249,7 +1271,7 @@ normalize(
 		UChar *ustr, *norm;
 		UErrorCode status = U_ZERO_ERROR;
 #		ifdef HAVE_UNICODE_UNORM2_H
-		static const char *uni_name[] = {"nfc", "nfkc", "nfkc_cf"}; /* */
+		static const char *uni_name[] = { "nfc", "nfkc", "nfkc_cf" };
 		const char *uni_namep;
 		UNormalization2Mode mode;
 #		else
@@ -1442,9 +1464,10 @@ parse_format_string(
 	struct t_fmt *fmt)
 {
 	char tmp_date_str[LEN];
+	char tmp_str[LEN];
 	char *out, *d_fmt, *buf;
 	const char *in;
-	int min_cols;
+	int min_cols, max_cols, cond_len;
 	size_t cnt = 0;
 	size_t len, len2, tmplen;
 	time_t tmptime;
@@ -1464,7 +1487,8 @@ parse_format_string(
 		SCORE			= 1 << 11,
 		SUBJECT			= 1 << 12,
 		THREAD_TREE		= 1 << 13,
-		U_CNT			= 1 << 14
+		U_CNT			= 1 << 14,
+		FROM_FIRST		= 1 << 15
 	};
 	int flags = NO_FLAGS;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -1495,10 +1519,12 @@ parse_format_string(
 	fmt->g_before_f = FALSE;
 	fmt->d_before_u = FALSE;
 	fmt->g_before_u = FALSE;
-	fmt->date_str[0] = '\0';
+	FreeAndNull(fmt->str);
+	FreeAndNull(fmt->date_str);
 	tmp_date_str[0] = '\0';
+	tmp_str[0] = '\0';
 	in = fmtstr;
-	out = fmt->str;
+	out = tmp_str;
 
 	if (tinrc.draw_arrow)
 		cnt += 2;
@@ -1512,6 +1538,8 @@ parse_format_string(
 		*out++ = *in++;
 		len = 0;
 		len2 = 0;
+		cond_len = 0;
+		max_cols = cCOLS + 1;
 		min_cols = 0;
 		tmp_date_str[0] = '\0';
 		d_fmt = tmp_date_str;
@@ -1534,6 +1562,21 @@ parse_format_string(
 					;
 			}
 		}
+		if (*in == '<') {
+			if (*++in > '0' && *in <= '9') {
+				max_cols = atoi(in);
+				for (; *in >= '0' && *in <= '9'; in++)
+					;
+			}
+		}
+		if (*in == ':') {
+			if (*++in > '0' && *in <= '9') {
+				cond_len = atoi(in);
+				for (; *in >= '0' && *in <= '9'; in++)
+					;
+			} else
+				cond_len = -1;
+		}
 		if (*in == '(') {
 			char *tmpp;
 			const char *endp = NULL;
@@ -1567,7 +1610,7 @@ parse_format_string(
 
 			case 'd':
 				/* Newsgroup description */
-				if (cCOLS > min_cols && !(flags & GRP_DESC) && signal_context == cSelect) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & GRP_DESC) && signal_context == cSelect) {
 					flags |= GRP_DESC;
 					fmt->show_grpdesc = TRUE;
 					if (len)
@@ -1578,12 +1621,12 @@ parse_format_string(
 
 			case 'D':
 				/* Date */
-				if (cCOLS > min_cols && (!(flags & DATE) && (signal_context == cGroup || signal_context == cThread))) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & DATE) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= DATE;
 					if (*tmp_date_str)
-						strcpy(fmt->date_str, tmp_date_str);
+						fmt->date_str = my_strdup(tmp_date_str);
 					else
-						STRCPY(fmt->date_str, curr_group->attribute->date_format);
+						fmt->date_str = my_strdup(curr_group->attribute->date_format ? BlankIfNull(*curr_group->attribute->date_format) : "");
 					buf = my_malloc(LEN);
 					(void) time(&tmptime);
 					if (my_strftime(buf, LEN - 1, fmt->date_str, localtime(&tmptime))) {
@@ -1608,7 +1651,7 @@ parse_format_string(
 
 			case 'f':
 				/* Newsgroup flags */
-				if (cCOLS > min_cols && !(flags & GRP_FLAGS) && signal_context == cSelect) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & GRP_FLAGS) && signal_context == cSelect) {
 					flags |= GRP_FLAGS;
 					fmt->flags_offset = cnt;
 					if (flags & GRP_NAME)
@@ -1622,18 +1665,19 @@ parse_format_string(
 
 			case 'F':
 				/* From */
-				if (!(flags & FROM) && (signal_context == cGroup || signal_context == cThread)) {
+				if (((cCOLS > min_cols && cCOLS < max_cols) || cond_len) && !(flags & FROM) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= FROM;
-					if (len)
-						fmt->len_from = len;
+					if (!(flags & (SUBJECT | THREAD_TREE)))
+						flags |= FROM_FIRST;
 
+					fmt->len_from = (cCOLS > min_cols && cCOLS < max_cols) ? len : cond_len > 0 ? cond_len : 0;
 				} else
 					out -= 2;
 				break;
 
 			case 'G':
 				/* Newsgroup name */
-				if (cCOLS > min_cols && !(flags & GRP_NAME) && signal_context == cSelect) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & GRP_NAME) && signal_context == cSelect) {
 					flags |= GRP_NAME;
 					if (len)
 						fmt->len_grpname = len;
@@ -1645,7 +1689,7 @@ parse_format_string(
 
 			case 'I':
 				/* Initials */
-				if (cCOLS > (int) min_cols && !(flags & INITIALS) && (signal_context == cGroup || signal_context == cThread)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & INITIALS) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= INITIALS;
 					fmt->len_initials = (len ? len : 3);
 					cnt += fmt->len_initials;
@@ -1655,7 +1699,7 @@ parse_format_string(
 
 			case 'L':
 				/* Lines */
-				if (cCOLS > min_cols && !(flags & LINE_CNT) && (signal_context == cGroup || signal_context == cThread)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & LINE_CNT) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= LINE_CNT;
 					fmt->len_linecnt = (len ? len : 4);
 					cnt += fmt->len_linecnt;
@@ -1665,7 +1709,7 @@ parse_format_string(
 
 			case 'm':
 				/* Article marks */
-				if (cCOLS > (int) min_cols && !(flags & ART_MARKS) && (signal_context == cGroup || signal_context == cThread)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & ART_MARKS) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= ART_MARKS;
 					cnt += 3;
 				} else
@@ -1674,7 +1718,7 @@ parse_format_string(
 
 			case 'M':
 				/* Message-ID */
-				if (cCOLS > min_cols && !(flags & MSGID) && (signal_context == cGroup || signal_context == cThread)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & MSGID) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= MSGID;
 					fmt->len_msgid = (len ? len : 10);
 					cnt += fmt->len_msgid;
@@ -1684,7 +1728,7 @@ parse_format_string(
 
 			case 'n':
 				/* Number in the menu */
-				if (cCOLS > min_cols && !(flags & LINE_NUMBER)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & LINE_NUMBER)) {
 					flags |= LINE_NUMBER;
 					fmt->len_linenumber = (len ? len : 4);
 					cnt += fmt->len_linenumber;
@@ -1694,7 +1738,7 @@ parse_format_string(
 
 			case 'R':
 				/* Number of responses in the thread */
-				if (cCOLS > min_cols && !(flags & RESP_COUNT) && signal_context == cGroup) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & RESP_COUNT) && signal_context == cGroup) {
 					flags |= RESP_COUNT;
 					fmt->len_respcnt = (len ? len : 3);
 					cnt += fmt->len_respcnt;
@@ -1704,17 +1748,16 @@ parse_format_string(
 
 			case 's':
 				/* Subject */
-				if (cCOLS > min_cols && !(flags & SUBJECT) && signal_context == cGroup) {
+				if (((cCOLS > min_cols && cCOLS < max_cols) || cond_len) && !(flags & SUBJECT) && signal_context == cGroup) {
 					flags |= SUBJECT;
-					if (len)
-						fmt->len_subj = len;
+					fmt->len_subj = (cCOLS > min_cols && cCOLS < max_cols) ? len : cond_len > 0 ? cond_len : 0;
 				} else
 					out -= 2;
 				break;
 
 			case 'S':
 				/* Score */
-				if (cCOLS > min_cols && !(flags & SCORE) && (signal_context == cGroup || signal_context == cThread)) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & SCORE) && (signal_context == cGroup || signal_context == cThread)) {
 					flags |= SCORE;
 					fmt->len_score = (len ? len : 6);
 					cnt += fmt->len_score;
@@ -1724,18 +1767,17 @@ parse_format_string(
 
 			case 'T':
 				/* Thread tree */
-				if (cCOLS > min_cols && !(flags & THREAD_TREE) && signal_context == cThread) {
+				if (((cCOLS > min_cols && cCOLS < max_cols) || cond_len) && !(flags & THREAD_TREE) && signal_context == cThread) {
 					flags |= THREAD_TREE;
 					show_subject = TRUE;
-					if (len)
-						fmt->len_subj = len;
+					fmt->len_subj = (cCOLS > min_cols && cCOLS < max_cols) ? len : cond_len > 0 ? cond_len : 0;
 				} else
 					out -= 2;
 				break;
 
 			case 'U':
 				/* Unread count */
-				if (cCOLS > min_cols && !(flags & U_CNT) && signal_context == cSelect) {
+				if ((cCOLS > min_cols && cCOLS < max_cols) && !(flags & U_CNT) && signal_context == cSelect) {
 					flags |= U_CNT;
 					fmt->len_ucnt = (len ? len : 5);
 					fmt->ucnt_offset = cnt;
@@ -1757,6 +1799,7 @@ parse_format_string(
 	}
 
 	*out = '\0';
+	fmt->str = my_strdup(tmp_str);
 
 	/*
 	 * check the given values against the screen width, fallback
@@ -1767,13 +1810,27 @@ parse_format_string(
 	 * if we draw no thread tree %F can use the entire space - otherwise
 	 * %F will use one third of the space
 	 */
-	if (cnt > (size_t) cCOLS - 1) {
+	if (!*fmt->str || cnt > (size_t) cCOLS - 1) {
 		flags = NO_FLAGS;
 		fmt->len_linenumber = 4;
 		switch (signal_context) {
+			case cGroup:
+				if (!*fmt->str)
+					error_message(3, _(txt_error_empty_format_string), DEFAULT_GROUP_FORMAT);
+				else
+					error_message(3, _(txt_error_format_string), DEFAULT_GROUP_FORMAT);
+				free(fmt->str);
+				fmt->str = my_strdup(DEFAULT_GROUP_FORMAT);
+				flags = (LINE_NUMBER | ART_MARKS | RESP_COUNT | LINE_CNT | SUBJECT | FROM);
+				cnt = tinrc.draw_arrow ? 23 : 21;
+				fmt->len_linecnt = 4;
+				fmt->len_respcnt = 3;
+				break;
+
 			case cSelect:
-				error_message(2, _(txt_error_format_string), DEFAULT_SELECT_FORMAT);
-				STRCPY(fmt->str, DEFAULT_SELECT_FORMAT);
+				error_message(3, _(txt_error_format_string), DEFAULT_SELECT_FORMAT);
+				free(fmt->str);
+				fmt->str = my_strdup(DEFAULT_SELECT_FORMAT);
 				flags = (GRP_FLAGS | LINE_NUMBER | U_CNT | GRP_NAME | GRP_DESC);
 				cnt = tinrc.draw_arrow ? 18 : 16;
 				fmt->show_grpdesc = TRUE;
@@ -1784,18 +1841,13 @@ parse_format_string(
 				fmt->len_ucnt = 5;
 				break;
 
-			case cGroup:
-				error_message(2, _(txt_error_format_string), DEFAULT_GROUP_FORMAT);
-				STRCPY(fmt->str, DEFAULT_GROUP_FORMAT);
-				flags = (LINE_NUMBER | ART_MARKS | RESP_COUNT | LINE_CNT | SUBJECT | FROM);
-				cnt = tinrc.draw_arrow ? 23 : 21;
-				fmt->len_linecnt = 4;
-				fmt->len_respcnt = 3;
-				break;
-
 			case cThread:
-				error_message(2, _(txt_error_format_string), DEFAULT_THREAD_FORMAT);
-				STRCPY(fmt->str, DEFAULT_THREAD_FORMAT);
+				if (!*fmt->str)
+					error_message(3, _(txt_error_empty_format_string), DEFAULT_THREAD_FORMAT);
+				else
+					error_message(3, _(txt_error_format_string), DEFAULT_THREAD_FORMAT);
+				free(fmt->str);
+				fmt->str = my_strdup(DEFAULT_THREAD_FORMAT);
 				flags = (LINE_NUMBER | ART_MARKS | LINE_CNT | THREAD_TREE | FROM);
 				cnt = tinrc.draw_arrow ? 22 : 20;
 				fmt->len_linecnt = 4;
@@ -1805,37 +1857,88 @@ parse_format_string(
 				break;
 		}
 
-		if (flags & SUBJECT)
+		if (flags & (SUBJECT | THREAD_TREE))
 			fmt->len_from = ((size_t) cCOLS - cnt - 1) / 3;
 		else
 			fmt->len_from = ((size_t) cCOLS - cnt - 1);
 
 		fmt->len_subj = (size_t) cCOLS - fmt->len_from - cnt - 1;
 	} else {
+		int len_cnt, len_tmp;
+		size_t *len_first, *len_last;
+		t_bool flags_first, flags_last;
+
+		len_cnt = (cCOLS - (int) cnt - 1) < 0 ? 0 : cCOLS - cnt - 1;
+
 		if (flags & (GRP_NAME | GRP_DESC))
-			fmt->len_grpname_max = (size_t) cCOLS - cnt - 1;
+			fmt->len_grpname_max = (size_t) len_cnt;
 
 		if (!show_description && !(flags & GRP_NAME))
 			fmt->len_grpname_max = 0;
 
-		if (flags & DATE && fmt->len_date > ((size_t) cCOLS - cnt - 1))
-			fmt->len_date = ((size_t) cCOLS - cnt - 1);
+		if ((flags & DATE) && fmt->len_date > (size_t) len_cnt)
+			fmt->len_date = (size_t) len_cnt;
 
-		if (flags & DATE && (!fmt->len_date_max || fmt->len_date_max > ((size_t) cCOLS - cnt - 1)))
+		if ((flags & DATE) && (!fmt->len_date_max || fmt->len_date_max > (size_t) len_cnt))
 			fmt->len_date_max = fmt->len_date;
 
-		if (flags & FROM && (!fmt->len_from || fmt->len_from > ((size_t) cCOLS - fmt->len_date_max - cnt - 1))) {
-			if (flags & (SUBJECT | THREAD_TREE)) {
-				if (fmt->len_subj)
-					fmt->len_from = (size_t) cCOLS - fmt->len_date_max - fmt->len_subj - cnt - 1;
-				else
-					fmt->len_from = ((size_t) cCOLS - fmt->len_date_max - cnt - 1) / 3;
-			} else
-				fmt->len_from = ((size_t) cCOLS - fmt->len_date_max - cnt - 1);
+		/*
+		 * For %s and %F, the item that comes last is normally truncated
+		 * according to the available space first. If no length is specified
+		 * for the item that comes first, but a length is specified for the
+		 * item that comes last, the logic is reversed. In this case, the item
+		 * that comes first is truncated according to the available space
+		 * first.
+		 */
+		if ((flags & FROM_FIRST)) {
+			if (!fmt->len_from && fmt->len_subj)
+				flags &= ~FROM_FIRST;
+		} else {
+			if (fmt->len_from && !fmt->len_subj)
+				flags |= FROM_FIRST;
 		}
 
-		if (flags & (SUBJECT | THREAD_TREE) && (!fmt->len_subj || fmt->len_subj > ((size_t) cCOLS - fmt->len_from - fmt->len_date_max - cnt - 1)))
-			fmt->len_subj = ((size_t) cCOLS - fmt->len_from - fmt->len_date_max - cnt - 1);
+		if ((flags & FROM_FIRST)) {
+			flags_first = (flags & (SUBJECT | THREAD_TREE));
+			flags_last = (flags & FROM);
+			len_first = &fmt->len_from;
+			len_last = &fmt->len_subj;
+		} else {
+			flags_first = (flags & FROM);
+			flags_last = (flags & (SUBJECT | THREAD_TREE));
+			len_first = &fmt->len_subj;
+			len_last = &fmt->len_from;
+		}
+
+		len_tmp = len_cnt - (int) fmt->len_date_max < 0 ? 0 : len_cnt - fmt->len_date_max;
+		if (flags_first) {
+			if (!*len_last || *len_last > (size_t) len_tmp) {
+				if (flags_last) {
+					if (*len_first) {
+						len_tmp = len_cnt - fmt->len_date_max - *len_first;
+						*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp;
+					} else {
+						len_tmp = len_cnt - fmt->len_date_max;
+						*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp / 3;
+					}
+					len_tmp = len_cnt - fmt->len_date_max - *len_last;
+					*len_first = len_tmp < 0 ? 0 : (size_t) len_tmp;
+				} else {
+					len_tmp = len_cnt - fmt->len_date_max;
+					*len_last = len_tmp < 0 ? 0 : (size_t) len_tmp;
+				}
+			} else {
+				if (flags_last) {
+					if (*len_first) {
+						len_tmp = len_cnt - fmt->len_date_max - *len_first;
+						*len_last = len_tmp < 0 ? 0 : MIN(*len_last, (size_t) len_tmp);
+					}
+					len_tmp = len_cnt - fmt->len_date_max - *len_last;
+					*len_first = len_tmp < 0 ? 0 : MIN(*len_first, (size_t) len_tmp);
+				}
+			}
+		} else if (flags_last && (!*len_first || *len_first > (size_t) len_tmp))
+				*len_first = (size_t) len_tmp;
 	}
 }
 
@@ -1912,3 +2015,56 @@ render_bidi(
 	return tmp;
 }
 #endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && HAVE_UNICODE_UBIDI_H && !NO_LOCALE */
+
+
+/*
+ * string to int conversion with lower/upper limits
+ * clears/sets errno
+ */
+signed int
+s2i(
+	const char *s,
+	signed int min,
+	signed int max)
+{
+	char *end;
+	signed long res;
+
+	errno = 0;
+	res = strtol(s, &end, 10);
+
+	if (res < min) {
+		res = min;
+		errno = ERANGE;
+	}
+
+	if (res > max) {
+		res = max;
+		errno = ERANGE;
+	}
+
+	if (s == end)
+		errno = EINVAL;
+
+	return ((signed int) res);
+}
+
+
+char *
+append_to_string(
+	char *dst,
+	const char *src)
+{
+	if (!src || !*src)
+		return dst;
+
+	if (!dst) {
+		dst = my_strdup(src);
+		return dst;
+	}
+
+	dst = my_realloc(dst, strlen(dst) + strlen(src) + 1);
+	strcat(dst, src);
+
+	return dst;
+}
