@@ -3,11 +3,11 @@
  *  Module    : tcurses.c
  *  Author    : Thomas Dickey <dickey@invisible-island.net>
  *  Created   : 1997-03-02
- *  Updated   : 2023-08-14
+ *  Updated   : 2024-05-13
  *  Notes     : This is a set of wrapper functions adapting the termcap
  *	             interface of tin to use SVr4 curses (e.g., ncurses).
  *
- * Copyright (c) 1997-2023 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2024 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -394,6 +394,10 @@ highlight_string(
 		}
 	}
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+	if (row < 0)
+		row = 0;
+	if (col < 0)
+		col = 0;
 
 	MoveCursor(row, col);
 	my_innstr(tmp, MIN(size, (LEN / 2) - 1));
@@ -416,6 +420,7 @@ word_highlight_string(
 	int size,
 	int color)
 {
+	char tmp[LEN] = { '\0' };
 	/*
 	 * Mapping of the tinrc.mono_mark* values to the ncurses attributes
 	 */
@@ -428,14 +433,13 @@ word_highlight_string(
 		A_DIM,
 		A_BOLD
 	};
-	char tmp[LEN] =  { '\0' };
 	int wsize = size;
 #		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	wchar_t *wtmp;
 
 	/*
 	 * In a multibyte locale we get byte offsets instead of character offsets
-	 * calculate now the correct correct starting column
+	 * calculate now the correct starting column
 	 */
 	if (col > 0 && col < (LEN / 2)) {
 		MoveCursor(row, 0);
@@ -447,7 +451,10 @@ word_highlight_string(
 		}
 	}
 #		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-
+	if (row < 0)
+		row = 0;
+	if (col < 0)
+		col = 0;
 	MoveCursor(row, col);
 	my_innstr(tmp, MIN(size, (LEN / 2) - 1));
 
@@ -460,10 +467,10 @@ word_highlight_string(
 #		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	/* safegurad against bogus regexps */
-	if ((tmp[0] == '*' && tmp[size - 1] == '*') ||
+	if (size > 1 && ((tmp[0] == '*' && tmp[size - 1] == '*') ||
 		 (tmp[0] == '/' && tmp[size - 1] == '/') ||
 		 (tmp[0] == '_' && tmp[size - 1] == '_') ||
-		 (tmp[0] == '-' && tmp[size - 1] == '-')) {
+		 (tmp[0] == '-' && tmp[size - 1] == '-'))) {
 
 		switch (tinrc.word_h_display_marks) {
 			case 0:
@@ -857,10 +864,13 @@ my_innstr(
 	size_t len = 0;
 	wchar_t *buffer;
 
-	buffer = my_malloc(sizeof(wchar_t) * (n + 1));
+	if (n < 0)
+		return len;
+
+	buffer = my_malloc(sizeof(wchar_t) * (size_t) (n + 1));
 
 	if (innwstr(buffer, n) != ERR) {
-		if ((len = wcstombs(str, buffer, 2 * n)) == (size_t) (-1))
+		if ((len = wcstombs(str, buffer, (size_t) (2 * n))) == (size_t) (-1))
 			len = 0;
 		str[len] = '\0';
 	}
