@@ -4,7 +4,7 @@
 # signs the article and posts it.
 #
 #
-# Copyright (c) 2002-2022 Urs Janssen <urs@tin.org>,
+# Copyright (c) 2002-2024 Urs Janssen <urs@tin.org>,
 #                         Marc Brockschmidt <marc@marcbrockschmidt.de>
 #
 # Redistribution and use in source and binary forms, with or without
@@ -49,15 +49,20 @@
 #           $ENV{'GPG_TTY'}=$tty if($tty =~ m/^\//)}}
 #         for gpg?
 #       - option to break long header lines?
-#         use Text::Wrap; $Text::Wrap::columns=998; wrap("","\t",$_);
 #       - option to trim References
 #       ...
+#
+# cmd-line options used in other inews:
+# inews-xt (Olaf Titz):
+#    -C accepted for historic reasons and errors out
+# inews (inn)
+#    -P don't add Sender
 
 use strict;
 use warnings;
 
 # version Number
-my $version = "1.1.59";
+my $version = "1.1.64";
 
 my %config;
 
@@ -192,6 +197,7 @@ GetOptions('A|V|W|h|headers' => [], # do nothing
 	'control|c=s'	=> \$config{'control'},
 	'canlock-algorithm=s'	=> \$config{'canlock-algorithm'},
 	'distribution|d=s'	=> \$config{'distribution'},
+	'discard-empty|E'	=> \$config{'discard-empty'},
 	'expires|e=s'	=> \$config{'expires'},
 	'from|f=s'	=> \$config{'from'},
 	'ignore-headers|i=s'	=> \$config{'ignore-headers'},
@@ -208,11 +214,12 @@ GetOptions('A|V|W|h|headers' => [], # do nothing
 	'help|H'	=> \$config{'help'},
 	'transform'	=> \$config{'transform'},
 	'verbose|v'	=> \$config{'verbose'},
-	'version'	=> \$config{'version'}
+	'version'	=> \$config{'version'},
+	'man'	=>	\$config{'man'}
 );
 
 foreach (@ARGV) {
-	print STDERR "Unknown argument $_.";
+	print STDERR "Unknown argument $_.\n";
 	usage();
 }
 
@@ -223,6 +230,18 @@ if ($config{'version'}) {
 
 usage() if ($config{'help'});
 
+# not listed in usage() or man-page as it may not work
+if ($config{'man'}) {
+	eval "use Pod::Usage";
+	if ($@) {
+		$config{'man'} = 0;
+		print STDERR "Unknown option: man.\n";
+		usage();
+	} else {
+		pod2usage(-verbose => 2, -exit => 0);
+	}
+}
+
 # check if SSL support is available
 if ($config{'ssl'}) {
 	eval "Net::NNTP->can_ssl";
@@ -314,6 +333,12 @@ if (${config{'ignore-headers'}}) {
 # Read the message and split the header
 readarticle(\%Header, \@Body);
 
+# empty @Body
+if (scalar @Body == 0) {
+	warn("Empty article\n") if ($config{'verbose'});
+	exit 0 if ($config{'discard-empty'});
+}
+
 # Add signature if there is none
 if (!$config{'no-signature'}) {
 	if ($config{'add-signature'} && !grep {/^-- /} @Body) {
@@ -349,12 +374,23 @@ foreach ('DISTRIBUTION', 'ORGANIZATION') {
 # overwrite headers if specified via cmd-line
 foreach ('Approved', 'Control', 'Distribution', 'Expires',
 	'From', 'Followup-To', 'Message-ID', 'Newsgroups',' Reply-To',
-	'Subject', 'References', 'Organization', 'Path') {
+	'Subject', 'References', 'Organization') {
 	next if (!defined($config{lc($_)}));
 	chomp($Header{lc($_)} = $_ . ": " . $config{lc($_)});
 	$Header{lc($_)} .= "\n";
 }
 
+# -x doesn't overwrite but prefixes
+if (defined($config{'path'})) {
+	if (defined($Header{'path'})) {
+		(my $pbody = $Header{'path'}) =~ s#^Path: ##i;
+		chomp($Header{'path'} = "Path: " . $config{'path'} . "!" . $pbody);
+	} else {
+		chomp($Header{'path'} = "Path: " . $config{'path'});
+	}
+	$Header{'path'} .= "\n";
+}
+
 # verify/add/remove headers
 foreach ('From', 'Subject') {
 	die("$0: No $_:-header defined.") if (!defined($Header{lc($_)}));
@@ -383,7 +419,8 @@ if ($config{'debug'} || $config{'verbose'}) {
 	foreach (keys %Header) {
 		warn "Raw 8-bit data in the following header:\n$Header{$_}\n" if ($Header{$_} =~ m/[\x80-\xff]/o);
 	}
-	if (!defined($Header{'mime-version'}) || !defined($Header{'content-type'}) || !defined($Header{'content-transfer-encoding'})) {
+	# do not check for CTE as it's not required for miltipart/*
+	if (!defined($Header{'mime-version'}) || !defined($Header{'content-type'})) {
 		warn "8bit body without MIME-headers\n" if (grep {/[\x80-\xff]/} @Body);
 	}
 }
@@ -395,7 +432,7 @@ if (!$config{'nntp-pass'}) {
 		open (my $NEWSAUTH, '<', (glob("~/.newsauth"))[0]) or die("Can't open ~/.newsauth: $!");
 		while ($l = <$NEWSAUTH>) {
 			chomp $l;
-			next if ($l =~ m/^[#\s]/);
+			next if ($l =~ m/(^[#\s]|)/);
 			($server, $pass, $user) = split(/\s+\b/, $l);
 			last if ($server =~ m/\Q$config{'nntp-server'}\E/);
 		}
@@ -413,7 +450,7 @@ if (!$config{'nntp-pass'}) {
 			open (my $NNTPAUTH, '<', (glob("~/.nntpauth"))[0]) or die("Can't open ~/.nntpauth: $!");
 			while ($l = <$NNTPAUTH>) {
 				chomp $l;
-				next if ($l =~ m/^[#\s]/);
+				next if ($l =~ m/(^[#\s]|)/);
 				($server, $user, $pass) = split(/\s+\b/, $l);
 				last if ($server =~ m/\Q$config{'nntp-server'}\E/);
 			}
@@ -996,8 +1033,9 @@ sub usage {
 	print "  -t string  set Subject:-header to string\n";
 	print "  -v         show warnings about missing/disabled features\n";
 	print "  -w string  set Followup-To:-header to string\n";
-	print "  -x string  set Path:-header to string\n";
+	print "  -x string  prepend Path:-header with string\n";
 	print "  -D         enable debugging\n";
+	print "  -E         silently discard empty article\n";
 	print "  -F string  set References:-header to string\n";
 	print "  -H         show help\n";
 	print "  -I         do not add Injection-Date: header\n";
@@ -1138,7 +1176,7 @@ Set the article header field Followup-To: to the given value.
 =item -B<x> C<Path> | --B<path> C<Path>
 X<-x> X<--path>
 
-Set the article header field Path: to the given value.
+Prepend the article header field Path: with the given value.
 
 =item -B<D> | -B<N> | --B<debug>
 X<-D> X<-N> X<--debug>
@@ -1147,6 +1185,16 @@ Set L<Net::NNTP(3pm)> to debug mode, enable warnings about raw 8-bit data,
 warn about disabled options due to lacking perl-modules or executables and
 unreadable files.
 
+=item -B<E> | --B<discard-empty>
+X<-E> X<--discard-empty>
+
+Silently discard an empty article.
+
+=item -B<F> | --B<references>
+X<-F> X<--references>
+
+Set the article header field References: to the given value.
+
 =item -B<H> | --B<help>
 X<-H> X<--help>
 
